Ext.define('monitor.widget.abstract.mixins.MonitorSettingsMixin', {getMonitorStatusColumn:function() {
  var me = this;
  return me.getMonitorSetting(monitor.assets.ConfigKeys.columnStatus);
}, getMonitorTimestampColumn:function() {
  var me = this;
  return me.getMonitorSetting(monitor.assets.ConfigKeys.columnTimeseries);
}, getAllMonitorColumns:function() {
  var me = this;
  var allColumnsObject = me.getMonitorSetting(monitor.assets.ConfigKeys.columnList);
  if (!Ext.isObject(allColumnsObject)) {
    return [];
  }
  var columnsList = Ext.Object.getAllKeys(allColumnsObject);
  return columnsList.map(function(el) {
    return {label:el, value:el};
  });
}, getMonitorColumnsConfig:function() {
  var me = this;
  return me.getMonitorSetting(monitor.assets.ConfigKeys.columnConfig);
}, getMonitorColumnConfig:function(columnName) {
  var me = this;
  var monitorColumnConfig = me.getMonitorColumnsConfig();
  var columnConfig = Ext.Array.findBy(monitorColumnConfig, function(colConfig) {
    return colConfig.id === columnName;
  });
  if (!me.columnConfig) {
    columnConfig = Ext.Array.findBy(monitorColumnConfig, function(colConfig) {
      return colConfig.dataIndex === columnName;
    });
  }
  return columnConfig || undefined;
}, getMonitorIdColumn:function() {
  var me = this;
  return me.getMonitorSetting(monitor.assets.ConfigKeys.columnId);
}, isMonitorMultipleGridActive:function() {
  var me = this;
  return me.getMonitorSetting(monitor.assets.ConfigKeys.multipleGrids);
}, getMonitorMultipleGridsConfig:function() {
  var me = this;
  return me.isMonitorMultipleGridActive() ? me.getMonitorSetting(monitor.assets.ConfigKeys.multipleGridsConfig) : undefined;
}, getMonitorMultipleGridViews:function() {
  var me = this;
  var monitorMultipleGridsConfig = me.getMonitorMultipleGridsConfig();
  if (monitorMultipleGridsConfig && monitorMultipleGridsConfig.views) {
    return monitorMultipleGridsConfig.views;
  }
  return [];
}, getMonitorStateViews:function() {
  var me = this;
  var moduleId = me.getSetting('moduleId');
  var monitorViews = monitor.Module.monitorViewStatesController.getMonitorViewStateStore(moduleId).getRange();
  return monitorViews.map(function(el) {
    return {value:el.get('id'), name:el.get('name'), iconCls:el.get('iconCls')};
  });
}, isMonitorAvailable:function() {
  var me = this;
  var moduleId = me.getSetting('moduleId');
  return BpcCommon.Api.getModuleModel(moduleId) && BpcCommon.Api.getModuleModel(moduleId).getModuleConfiguration().get('moduleId') === 'monitor';
}, getMonitorSetting:function(key) {
  var me = this;
  var moduleId = me.getSetting('moduleId');
  return BpcCommon.Api.getSetting(moduleId, key, false);
}, getCustomRenderer:function(columnName) {
  var me = this;
  var columnConfig = me.getMonitorColumnConfig(columnName);
  if (!columnConfig) {
    return Ext.emptyFn;
  }
  var pluginName = 'monitorCustomRenderer';
  if (!me.pluginHelperCmp) {
    me.loadCustomRenderer();
  }
  var plugin = me.pluginHelperCmp.findPlugin(pluginName);
  if (!plugin) {
    return Ext.emptyFn;
  }
  return Ext.clone(plugin.getColumnRendererByColumnConfig(columnConfig));
}, getCustomRendererValueConfig:function(columnName, value) {
  var me = this;
  var columnConfig = me.getMonitorColumnConfig(columnName);
  if (!columnConfig) {
    return {};
  }
  var pluginName = 'monitorCustomRenderer';
  if (!me.pluginHelperCmp) {
    me.loadCustomRenderer();
  }
  var plugin = me.pluginHelperCmp.findPlugin(pluginName);
  if (!plugin) {
    return {};
  }
  var rendererId = plugin.getRendererIdByColumnConfig(columnConfig);
  return plugin.getValueConfig(rendererId, value);
}, loadCustomRenderer:function() {
  var me = this;
  me.pluginHelperCmp = Ext.create({xtype:'box', plugins:{monitorCustomRenderer:true}, moduleId:me.getSetting('moduleId')});
}});
Ext.define('monitor.widget.abstract.mixins.ViewStateMixin', {setMonitorViewState:function(state) {
  var me = this;
  me.setWidgetSetting('selectedView', state.id, true);
  me.settingStore.commitChanges();
}, resetMonitorViewState:function() {
  var me = this;
  me.setWidgetSetting('selectedView', '', true);
  me.settingStore.commitChanges();
}, loadMonitorViewState:function() {
  var me = this;
  var moduleId = me.getSetting('moduleId');
  return new Promise(function(resolve) {
    monitor.Module.monitorViewStatesController.getMonitorViewState(moduleId, me.getSetting('selectedView')).then(function(state) {
      me.activeMonitorViewState = state;
      me.fireEvent('monitorViewStateLoaded');
      resolve(state);
    });
  });
}, deleteMonitorViewState:function() {
  var me = this;
  delete me.activeMonitorViewState;
}, getStoreStateObjectFromStateView:function(gridViewName) {
  var me = this;
  var moduleId = me.getSetting('moduleId');
  var storeState = {filters:[], sorters:[]};
  if (!me.activeMonitorViewState) {
    return storeState;
  }
  var actualViewStateObject = me.activeMonitorViewState.get('views')[moduleId + '_' + gridViewName];
  if (actualViewStateObject && actualViewStateObject.storeState) {
    storeState = Ext.apply(storeState, actualViewStateObject.storeState);
  }
  return storeState;
}, getStoreFilterFromStateView:function(gridViewName) {
  var me = this;
  var timeSeriesColumn = me.getSetting('timeseriesColumn');
  var filters = this.getStoreStateObjectFromStateView(gridViewName).filters;
  var mappedFilters = filters.map(function(el) {
    return {invert:el.invert, property:el.property, source:el.source, value:el.value, operator:el.operator};
  });
  return mappedFilters.filter(function(el) {
    return el.property !== timeSeriesColumn;
  });
}, getStoreSortersFromStateView:function(gridViewName) {
  var sorters = this.getStoreStateObjectFromStateView(gridViewName).sorters;
  return sorters.map(function(el) {
    return {property:el.property, direction:el.direction};
  });
}});
Ext.define('monitor.widget.abstract.mixins.WidgetSettingsMixin', {loadConfigOrDefaults:function() {
  var me = this;
  var config = this.mixins.bpcWidget.getWidgetConfiguration.call(this);
  var clonedDefaultConfig = Ext.clone(this.mixins.bpcWidget.getDefaultWidgetConfiguration.call(this));
  var widgetSettingConfiguration = Ext.apply(clonedDefaultConfig, config);
  var localStorageConfiguration = me.getState();
  this.widgetConfiguration = Ext.apply(widgetSettingConfiguration, localStorageConfiguration);
}, initSettingStore:function() {
  var me = this;
  me.loadConfigOrDefaults();
  var storeData = monitor.assets.Helpers.configurationToArray(this.widgetConfiguration);
  me.settingStore = Ext.create('Ext.data.Store', {proxy:{type:'memory'}});
  me.settingStore.loadRawData(storeData);
  me.settingStore.on('endupdate', Ext.Function.createBuffered(this.onStoreEndUpdate, 100, me), me);
  me.settingStore.on('commit', Ext.Function.createBuffered(this.onStoreCommit, 100, me), me);
}, onStoreCommit:function() {
  var me = this;
  me.fireEvent('widgetConfigurationChange');
}, onStoreEndUpdate:function() {
  var me = this;
  var storeData = me.settingStore.getRange().map(function(el) {
    return el.getData();
  });
  me.widgetConfiguration = monitor.assets.Helpers.configurationToObject(storeData);
  me.renderWidget();
}, isFormValid:function() {
  var me = this;
  if (!me.settingPanel) {
    return true;
  } else if (me.settingPanel.down('[isValid]') && me.settingPanel.down('[isValid]').isValid()) {
    return true;
  }
  return false;
}, getWidgetSetting:function(property) {
  var me = this;
  var setting = me.settingStore.getById(property);
  if (!setting) {
    return '';
  }
  var settingValue = setting.get('value');
  if (typeof settingValue === 'string') {
    settingValue = settingValue.trim();
  }
  return settingValue;
}, setWidgetSetting:function(property, value, setPropertyField) {
  setPropertyField = setPropertyField === void 0 ? false : setPropertyField;
  var me = this;
  var setting = me.settingStore.getById(property);
  if (!setting) {
    return false;
  }
  setting.set('value', value);
  if (setPropertyField) {
    if (me.settingPanel && me.settingPanel.down('[property\x3d' + property + ']')) {
      me.settingPanel.down('[property\x3d' + property + ']').setValue(value);
    }
  }
}, restoreSettingsListToDefault:function(propertyList) {
  var me = this;
  if (!Ext.isArray(propertyList)) {
    return;
  }
  Ext.Array.each(propertyList, function(property) {
    me.restoreSettingToDefault(property);
  });
}, restoreSettingToDefault:function(property) {
  var me = this;
  if (property === 'timeseriesColumn') {
    me.setWidgetSetting(property, me.getMonitorTimestampColumn(), true);
  } else if (property === 'aggregationColumn') {
    me.setWidgetSetting(property, me.getMonitorStatusColumn(), true);
  } else if (property === 'column') {
    me.setWidgetSetting(property, me.getMonitorStatusColumn(), true);
  } else {
    me.setWidgetSetting(property, me.defaultWidgetConfiguration[property], true);
  }
}, restoreAllSettingsToDefault:function() {
  var me = this;
  Ext.Object.each(me.defaultWidgetConfiguration, function(key) {
    me.restoreSettingToDefault(key);
  }, me);
  return true;
}, getQueryId:function(gridViewName) {
  var userSession = BpcCommon.Api.getUserSession();
  var sessionId = userSession.get('loginName') + userSession.get('expirationDate').getTime();
  return this.widgetId + ':::' + gridViewName + ':::' + sessionId;
}, onNewIncomingDataFields:function(dataFields) {
  var me = this;
  var dataHasChanged = false;
  if (!Ext.isArray(dataFields) || Ext.isEmpty(dataFields)) {
    return;
  }
  Ext.Array.each(dataFields, function(dataField) {
    var loadedDataFields = me.getLoadedDataFields();
    if (!loadedDataFields.includes(dataField)) {
      loadedDataFields.push(dataField);
      me.setLoadedDataFields(loadedDataFields);
      dataHasChanged = true;
    }
  });
  if (dataHasChanged) {
    me.fireEvent('loadedDataFieldsChanged');
  }
}, getMappedLoadedDataFields:function() {
  var me = this;
  return me.getLoadedDataFields().map(function(el) {
    return {name:el};
  });
}, resetLoadedDataFields:function() {
  var me = this;
  me.setLoadedDataFields([]);
  me.fireEvent('loadedDataFieldsChanged');
}, enableGlobalDateRangeListeners:function() {
  var me = this;
  var vm = BpcCommon.Api.getGlobalViewModel();
  vm.bind({bindTo:'{global.dateRange}', deep:true}, Ext.Function.createBuffered(me.refreshWidget, 100, me));
}, getGlobalDateRangeIsActive:function() {
  var vm = BpcCommon.Api.getGlobalViewModel();
  return vm.get('global.dateRange.active');
}, getGlobalDateRangeStart:function() {
  var vm = BpcCommon.Api.getGlobalViewModel();
  var me = this;
  return me.getGlobalDateRangeIsActive() ? vm.get('global.dateRange.value.start') : undefined;
}, getGlobalDateRangeEnd:function() {
  var vm = BpcCommon.Api.getGlobalViewModel();
  var me = this;
  return me.getGlobalDateRangeIsActive() ? vm.get('global.dateRange.value.end') : undefined;
}, shouldRefreshOnGlobalDataLoadedEvent:function(dataObj, endpointName, queryId) {
  try {
    var doRefresh = dataObj.properties.componentId && dataObj.properties.componentId === endpointName && dataObj.properties.customId && dataObj.properties.customId === queryId;
    return !!doRefresh;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR shouldRefreshOnGlobalDataLoadedEvent', dump:e, stack:true, level:'error'});
    return false;
  }
}});
Ext.define('monitor.widget.abstract.mixins.WidgetRenderingMixin', {getWrongConfigPanel:function(type) {
  var monitorWidgetStatics = monitor.widget["abstract"].MonitorWidget;
  var configPanel = {xtype:'container', padding:'10 10 10 10', layout:{type:'vbox', pack:'center', align:'middle'}, items:[]};
  var messageLanguageKey;
  if (type === monitorWidgetStatics.MONITOR_ID_MISSING) {
    messageLanguageKey = 'MONITOR_WIDGET_MONITOR_ID_MISSING';
  } else if (type === monitorWidgetStatics.COLUMN_CONFIG_MISSING) {
    messageLanguageKey = 'MONITOR_WIDGET_INCORRECT_COLUMN_CONFIG';
  } else {
    messageLanguageKey = 'MONITOR_WIDGET_INCOMPLETE_CONFIG';
  }
  var messageItem = {xtype:'box', html:'\x3ch3\x3e' + BpcCommon.Api.getTranslation(messageLanguageKey) + '\x3c/h3\x3e'};
  configPanel.items = [{xtype:'button', localized:{text:'MONITOR_WIDGET_SHOW_CONFIG'}, iconCls:'x-fal fa-cog', handler:function() {
    var widget = this.up('monitorBaseWidget');
    widget.configureWidget();
  }}, messageItem];
  return configPanel;
}, configureWidget:function() {
  var me = this;
  try {
    if (me.settingPanel) {
      me.settingPanel.removeAll();
      me.settingPanel.destroy();
      delete me.settingPanel;
    }
    var accessAllowedItems = [{xtype:me.settingsPanelCmp, moduleId:me.getSetting('moduleId'), settingStore:me.settingStore, defaultWidgetConfiguration:me.defaultWidgetConfiguration, widgetReference:me}];
    var accessDeniedItems = [{xtype:'bpcCenterMessageCmp', message:'MONITOR_WIDGET_NO_RIGHTS_TO_EDIT', messageIconCls:'x-fal fa-exclamation-triangle'}];
    var isUserAllowed = BpcCommon.Api.hasUserRight('bpcDashboard_editDashboard');
    var renderItems = isUserAllowed ? accessAllowedItems : accessDeniedItems;
    me.settingPanel = Ext.create({xtype:'bpcWindow', layout:'fit', closeAction:'method-hide', items:renderItems, closable:false, width:700, iconCls:monitor.assets.Icons.WIDGET_SETTINGS, title:BpcCommon.lang.Current.MONITOR_WIDGET_CONFIG, bodyPadding:10, tools:[{type:'close', handler:function(e, owner, source) {
      me.settingStore.rejectChanges();
      me.closeWindow(source);
    }.bind(me)}], buttons:[{iconCls:BpcCommon.Icons.RESET, itemId:'monitorWidgetConfigResetBtn', ui:'secondary', localized:{text:'MONITOR_PROCESS_DEFAULT_VALUES'}, handler:function() {
      me.restoreAllSettingsToDefault();
    }.bind(me)}, {iconCls:BpcCommon.Icons.CANCEL, itemId:'monitorWidgetConfigCanceltBtn', ui:'secondary', localized:{text:'CORE_CANCEL'}, handler:function(source) {
      me.settingStore.rejectChanges();
      me.refreshFormPanelItems(source);
      me.closeWindow(source);
    }.bind(me)}, {iconCls:BpcCommon.Icons.OK, itemId:'monitorWidgetConfigApplyBtn', ui:'primary', localized:{text:'CORE_APPLY'}, handler:function(source) {
      me.settingStore.commitChanges();
      me.closeWindow(source);
    }.bind(me)}]});
    me.settingPanel.show();
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR configureWidget', dump:e, stack:true, level:'error'});
  }
}, refreshFormPanelItems:function(source) {
  var formPanel = source.up('bpcWindow').down('monitorWidgetFormPanel');
  if (formPanel && formPanel.refreshItems) {
    formPanel.refreshItems();
  }
}, closeWindow:function(src) {
  if (src.up('bpcWindow') && src.up('bpcWindow').close) {
    src.up('bpcWindow').close();
  }
}, renderInitialLoading:function() {
  var me = this;
  var msg = 'MONITOR_WIDGET_IS_LOADING';
  var icon = 'fa fa-spinner fa-spin fa-3x fa-fw margin-bottom';
  me.renderCenterMessage(msg, icon);
}, renderCenterMessage:function(message, icon) {
  message = message === void 0 ? '' : message;
  icon = icon === void 0 ? '' : icon;
  var me = this;
  var cmp = {xtype:'bpcCenterMessageCmp', message:message, messageIconCls:icon};
  me.renderCmp(cmp);
}, renderWrongConfigPanel:function(message, icon) {
  message = message === void 0 ? 'MONITOR_WIDGET_INCOMPLETE_CONFIG' : message;
  icon = icon === void 0 ? 'x-fal fa-cog' : icon;
  var me = this;
  var configPanel = {xtype:'container', padding:'10 10 10 10', layout:{type:'vbox', pack:'center', align:'middle'}, items:[{xtype:'box', html:'\x3ch3\x3e' + BpcCommon.Api.getTranslation(message) + '\x3c/h3\x3e'}, {xtype:'button', localized:{text:'MONITOR_WIDGET_SHOW_CONFIG'}, iconCls:icon, handler:function() {
    me.configureWidget();
  }.bind(me)}]};
  me.renderCmp(configPanel);
}, renderCmp:function(cmp) {
  var me = this;
  if (!cmp) {
    return;
  }
  if (me.contentPanel) {
    me.remove(this.contentPanel);
    delete me.contentPanel;
  }
  me.contentPanel = this.add(cmp);
  me.getLayout().setActiveItem(me.contentPanel);
}, createMultipleGridPanel:function() {
  var me = this;
  var components;
  if (me.isMonitorMultipleGridActive()) {
    var multiConf = me.getMonitorMultipleGridsConfig();
    var enabledViewStr = this.getSetting('views') || '';
    var showAllViews = enabledViewStr.length === 0;
    var widgetViews = [];
    var enabledViewArr = monitor.assets.Helpers.getArrayFromString(enabledViewStr);
    enabledViewArr = enabledViewArr.map(Ext.String.trim);
    var tabPanel = {xtype:'tabpanel', tabPosition:Ext.isDefined(multiConf.tabPosition) ? multiConf.tabPosition : 'top', activeTab:0, items:[]};
    Ext.Array.each(multiConf.views, function(view) {
      if (BpcCommon.Api.isUserAllowedToSee(view)) {
        if (showAllViews || Ext.Array.contains(enabledViewArr, view.id)) {
          var widgetView = me.createWidgetView({viewObj:view, viewId:view.id, localized:{title:view.title ? view.title : view.id}, tabConfig:{tooltip:view.id}});
          widgetViews.push(widgetView);
        }
      }
    });
    if (widgetViews.length > 1) {
      tabPanel.items = widgetViews;
      var activeTab = 0;
      var initialView = this.getSetting('initialView');
      if (initialView) {
        var viewId = String(initialView).split(':')[0];
        if (viewId) {
          Ext.Array.each(widgetViews, function(viewConf, index) {
            if (viewConf.viewObj.id === viewId) {
              activeTab = index;
              return false;
            }
          });
        }
      }
      tabPanel.activeTab = activeTab;
    } else if (widgetViews.length === 1) {
      delete widgetViews[0].title;
      tabPanel = widgetViews[0];
    }
    components = tabPanel;
  } else {
    components = me.createWidgetView({viewId:'mainGrid'});
  }
  return components;
}, createWidgetView:function(additionalConfig) {
  var me = this;
  var moduleId = this.getSetting('moduleId');
  var widgetId = this.getWidgetId();
  var additionalAttributes = Ext.isObject(additionalConfig) ? additionalConfig : {};
  var queryId = me.getQueryId(additionalConfig.viewId);
  var bpcGridViewName = additionalConfig.viewId;
  return Ext.apply({xtype:me.contentPanelCmp, moduleId:moduleId, widgetId:widgetId, queryId:queryId, creationTimeStamp:(new Date()).getTime(), bpcGridViewName:bpcGridViewName, widgetReference:me}, additionalAttributes);
}, refreshWidget:function() {
  var me = this;
  var isVisible = me.isVisible(true);
  if (!isVisible) {
    me.pendingUpdate = true;
    return;
  }
  me.pendingUpdate = false;
  var tabs = this.query(this.contentPanelCmp);
  if (tabs.length > 0) {
    Ext.Array.each(tabs, function(tab) {
      if (Ext.isFunction(tab.refreshWidget)) {
        tab.refreshWidget();
      }
    });
  }
}, generalErrorHandling:function() {
  var me = this;
  var moduleId = me.getSetting('moduleId');
  if (Ext.isEmpty(moduleId)) {
    me.renderCenterMessage('No monitor selected', 'x-fal fa-fire');
    return false;
  }
  if (!me.isMonitorAvailable()) {
    me.renderCenterMessage('Monitor does not exist or no rights', 'x-fal fa-fire');
    return false;
  }
  return true;
}});
Ext.define('monitor.widget.abstract.mixins.LocalStorageHiddenGraphManager', {getLocalStorageConsistencyWidgetConfigurationList:function() {
  var me = this;
  var $jscomp$destructuring$var0 = me.getWidgetConfiguration() || {};
  var moduleId = $jscomp$destructuring$var0.moduleId;
  var aggregationColumn = $jscomp$destructuring$var0.aggregationColumn;
  var excludedValues = $jscomp$destructuring$var0.excludedValues;
  var hiddenGraphs = $jscomp$destructuring$var0.hiddenGraphs;
  return [moduleId, aggregationColumn, Ext.Array.sort(excludedValues), Ext.Array.sort(hiddenGraphs)];
}, getLocalStorageConsistencyKey:function() {
  var me = this;
  var widgetId = me.widgetId;
  return 'monitorWidgetConsistencyConfiguration_' + widgetId;
}, isWidgetConfigurationConsistent:function() {
  var me = this;
  var actualConfig = me.getLocalStorageConsistencyWidgetConfigurationList();
  var savedConfig = me.readLocalStorageConsistencyList();
  if (Ext.isEmpty(savedConfig)) {
    me.writeLocalStorageConsistencyList();
    return false;
  }
  return BpcCommon.Util.compareObject(actualConfig, savedConfig);
}, getStatefulHiddenGraphs:function(viewId) {
  var me = this;
  var defaultValue = [];
  if (!me.isWidgetConfigurationConsistent()) {
    me.initLocalStorageItem(me.getLocalStorageHiddenGraphKey());
  }
  var savedValue = me.readLocalStorageHiddenGraphs();
  var foundEntryIdx = savedValue.findIndex(function(el) {
    return el.viewId === viewId;
  });
  if (foundEntryIdx === -1) {
    return defaultValue;
  } else {
    return savedValue[foundEntryIdx].value;
  }
}, setStatefulHiddenGraphs:function(viewId, hiddenGraphsList) {
  var me = this;
  var localStorageEntry = {viewId:viewId, value:hiddenGraphsList};
  me.updateHiddenGraphs(localStorageEntry);
}, updateHiddenGraphs:function(newEntry) {
  var me = this;
  var hiddenGraphs = me.readLocalStorageHiddenGraphs();
  var foundEntryIdx = hiddenGraphs.findIndex(function(el) {
    return el.viewId === newEntry.viewId;
  });
  if (foundEntryIdx === -1) {
    hiddenGraphs.push(newEntry);
  } else {
    hiddenGraphs[foundEntryIdx] = newEntry;
  }
  me.writeLocalStorageHiddenGraphs(hiddenGraphs);
}, getLocalStorageHiddenGraphKey:function() {
  var me = this;
  var widgetId = me.widgetId;
  return 'monitorWidgetChartSeriesHiddenConfig_' + widgetId;
}, writeLocalStorageConsistencyList:function() {
  var me = this;
  me.writeLocalStorageEntry(me.getLocalStorageConsistencyKey(), me.getLocalStorageConsistencyWidgetConfigurationList());
}, readLocalStorageConsistencyList:function() {
  var me = this;
  return me.readLocalStorageEntry(me.getLocalStorageConsistencyKey());
}, writeLocalStorageHiddenGraphs:function(value) {
  var me = this;
  me.writeLocalStorageEntry(me.getLocalStorageHiddenGraphKey(), value);
  var widgetSettingsConsistencyList = me.getLocalStorageConsistencyWidgetConfigurationList();
  me.writeLocalStorageEntry(me.getLocalStorageConsistencyKey(), widgetSettingsConsistencyList);
}, readLocalStorageHiddenGraphs:function() {
  var me = this;
  return me.readLocalStorageEntry(me.getLocalStorageHiddenGraphKey());
}, readLocalStorageEntry:function(key) {
  var me = this;
  var defaultValue = [];
  var localStorageValue = Ext.state.Manager.get(key);
  if (localStorageValue === null) {
    me.initLocalStorageItem(key);
    return defaultValue;
  }
  try {
    return JSON.parse(localStorageValue);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ':readLocalStorage', dump:e, stack:true, level:'error'});
    return defaultValue;
  }
}, writeLocalStorageEntry:function(key, value) {
  try {
    Ext.state.Manager.set(key, JSON.stringify(value));
  } catch (e) {
    Ext.log({msg:this.self.getName() + ':writeLocalStorage', dump:e, stack:true, level:'error'});
  }
}, initLocalStorageItem:function(key) {
  var me = this;
  var initValue = [];
  me.writeLocalStorageEntry(key, initValue);
}});
Ext.define('monitor.widget.abstract.MonitorBaseWidget', {extend:'Ext.container.Container', alias:'widget.monitorBaseWidget', mixins:['BpcCommon.mixin.Widget', 'monitor.widget.abstract.mixins.MonitorSettingsMixin', 'monitor.widget.abstract.mixins.ViewStateMixin', 'monitor.widget.abstract.mixins.WidgetSettingsMixin', 'monitor.widget.abstract.mixins.WidgetRenderingMixin', 'monitor.widget.abstract.mixins.LocalStorageHiddenGraphManager'], controller:'monitorWidgetAbstractController', config:{widgetConfiguration:{}, 
loadedDataFields:[]}, stateEvents:['widgetConfigurationChange'], statics:{MONITOR_ID_MISSING:'monitorIdMissing', COLUMN_CONFIG_MISSING:'columnConfigMissing'}, pendingUpdate:false, activeMonitorViewState:undefined, stateful:true, dbModuleId:'', dbViewModel:{}, initComponent:function() {
  var me = this;
  me.stateId = 'monitorDashboardWidget' + me.widgetId;
  me.getState = function() {
    return this.widgetConfiguration;
  };
  me.callParent();
  me.setLocalizedDateFormat();
  me.renderInitialLoading();
}, setLocalizedDateFormat:function() {
  var me = this;
  var activeBpcLanguage = BpcCommon.Api.getGlobalViewModel().get('global.language');
  var dateFormat = 'dd.MM.yyyy';
  if (activeBpcLanguage === 'en') {
    dateFormat = 'MM/dd/yyyy';
  }
  me.defaultWidgetConfiguration['localizedDateFormat'] = dateFormat;
}, widgetJumpToModule:function() {
  var selectedView = this.getSetting('selectedView');
  var moduleId = this.getSetting('moduleId');
  var selectedViewIsEmpty = Ext.isEmpty(selectedView);
  if (selectedViewIsEmpty) {
    BpcCommon.Api.showModule(moduleId, false, {state:{}});
  } else {
    BpcCommon.Api.showModule(moduleId, false, {stateId:selectedView});
  }
}, getSetting:function(property) {
  var me = this;
  return me.getWidgetSetting(property);
}});
Ext.define('monitor.widget.components.form.stateView.StateSelectionMenu', {extend:'Ext.button.Button', alias:'widget.monitorStateSelectionMenu', iconCls:BpcCommon.Icons.FAV, moduleId:undefined, widget:undefined, stateResetFn:Ext.emptyFn, stateSelectFn:Ext.emptyFn, showOnlyFavorites:false, showResetItem:true, initComponent:function() {
  var me = this;
  me.menu = Ext.create('Ext.menu.Menu');
  me.widget = me.up('dbWidgetWrapper').down('monitorBaseWidget');
  me.moduleId = me.widget.getSetting('moduleId');
  me.on('menushow', me.setSelectedState, me);
  me.on('afterrender', me.setActiveState, me);
  me.mon(me.widget, 'selectedViewChange', me.setActiveState, me);
  me.mon(me.widget, 'monitorViewStateLoaded', me.updateCmp, me);
  me.mon(me.widget, 'moduleIdChange', me.updateCmp, me);
  me.callParent(arguments);
  var stateStore = monitor.Module.monitorViewStatesController.getMonitorViewStateStore(me.moduleId);
  if (stateStore.isLoading()) {
    me.mon(stateStore, 'load', me.updateCmp, me, {single:true});
  } else {
    me.updateCmp();
  }
}, updateCmp:function() {
  var me = this;
  var bufferedCreateMenu = Ext.Function.createBuffered(me.createMenu, 100, me);
  bufferedCreateMenu();
}, createMenu:function() {
  var me = this;
  var stateStore = monitor.Module.monitorViewStatesController.getMonitorViewStateStore(me.moduleId);
  me.getMenu().removeAll();
  stateStore.each(function(monitorViewState) {
    if (me.showOnlyFavorites === false || monitorViewState.get('favorite')) {
      me.getMenu().add({xtype:'menucheckitem', group:'state', text:monitorViewState.get('name'), iconCls:monitorViewState.get('iconCls'), state:monitorViewState, stateId:monitorViewState.get('id'), moduleId:me.moduleId, handler:function() {
        me.onStateSelect(monitorViewState);
      }});
    }
  });
  if (me.showResetItem) {
    me.getMenu().add([{xtype:'menuseparator'}, {iconCls:BpcCommon.Icons.RESET, localized:{text:'MONITOR_RESET_VIEW'}, itemId:'monitorViewStateResetBtn', handler:me.onStateReset.bind(me)}]);
  }
}, onStateSelect:function(selectedState) {
  var me = this;
  me.widget.setMonitorViewState(selectedState);
  this.showAsActive();
  this.selectMenuItem(selectedState.stateId);
}, onStateReset:function() {
  var me = this;
  me.showAsInactive();
  me.unselectAllMenuItems();
  me.widget.resetMonitorViewState();
}, setActiveState:function() {
  var me = this;
  var stateId = me.widget.getSetting('selectedView');
  var defaultStatedId = me.widget.getDefaultWidgetConfiguration()['selectedView'];
  if (Ext.isEmpty(stateId) || stateId === defaultStatedId) {
    this.showAsInactive();
  } else {
    this.showAsActive();
  }
}, setSelectedState:function() {
  var me = this;
  var stateId = me.widget.getSetting('selectedView');
  if (Ext.isEmpty(stateId)) {
    me.unselectAllMenuItems();
  } else {
    me.selectMenuItem(stateId);
  }
}, showAsActive:function() {
  this.setIconCls('x-fas fa-star');
}, showAsInactive:function() {
  this.setIconCls('x-fal fa-star');
}, selectMenuItem:function(stateId) {
  var menuItem = this.down('menucheckitem[stateId\x3d' + stateId + ']');
  if (menuItem) {
    menuItem.setChecked(true, true);
  }
}, unselectAllMenuItems:function() {
  var me = this;
  var menu = me.getMenu();
  menu.query('menucheckitem').forEach(function(el) {
    return el.setChecked(false);
  });
}});
Ext.define('monitor.widget.timeSeries.Main', {extend:'monitor.widget.abstract.MonitorBaseWidget', alias:'widget.monitorWidgetTimeSeriesMain', alternateClassName:['monitor.widget.timeseriesAggregation.TimeseriesAggregation'], settingsPanelCmp:'monitorWidgetProcessOverTimeConfigurationPanel', contentPanelCmp:'monitorWidgetTimeSeriesView', layout:'card', activeItem:0, title:false, statics:{WIDGET_NAME:BpcCommon.lang.Current.MONITOR_WIDGET_PROCESS_OVERVIEW_TIMEFRAME, WIDGET_DESCRIPTION:BpcCommon.lang.Current.MONITOR_WIDGET_PROCESS_OVERVIEW_TIMEFRAME_DESC, 
WIDGET_ICON_CLS:'x-fal fa-chart-bar', WIDGET_ADDITIONAl_ACTIONS:[{xtype:'monitorStateSelectionMenu'}]}, defaultWidgetConfiguration:{iconCls:'x-fal fa-chart-bar', title:BpcCommon.lang.Current.MONITOR_WIDGET_PROCESS_OVERVIEW_TIMEFRAME, name:BpcCommon.lang.Current.MONITOR_WIDGET_PROCESS_OVERVIEW_TIMEFRAME, moduleId:'auditlog', aggregationColumn:'level', selectedView:0, startDate:'now-7d/d', endDate:'now', timeseriesColumn:'timestamp', dateFormat:'d.m.', localizedDateFormat:'', excludedValues:[], graphOrder:[], 
hiddenGraphs:[], dataRefreshBuffer:10, interval:'day', viewMode:'tabbed', hideEmptyRows:false, hideZeroValues:true, showPreviousData:false, showFollowingData:false, cellAlign:'center', chartConfiguration:{'categoryAxis':{'renderer':{'labels':{'template':{'rotation':315}}}, 'gridIntervals':[{'timeUnit':'day', 'count':1}], 'dateFormats':{'day':'dd.MM.'}}, 'valueAxes':{}, 'legend':{'type':'Legend', 'position':'right', 'labels':{'fontSize':14}, 'useDefaultMarker':true, 'markers':{'children':[{'cornerRadiusTopLeft':12, 
'cornerRadiusTopRight':12, 'cornerRadiusBottomRight':12, 'cornerRadiusBottomLeft':12, 'strokeWidth':2, 'strokeOpacity':1, 'stroke':'#ccc'}]}}, 'exporting':{'menu':{}}}, graphType:'column', dockSummary:'', views:[], initialView:''}, widgetConfiguration:undefined, renderWidget:function() {
  var me = this;
  if (me.specificErrorHandling()) {
    var components = me.createMultipleGridPanel();
    me.renderCmp(components);
  }
}, specificErrorHandling:function() {
  var me = this;
  var $jscomp$destructuring$var1 = me.getWidgetConfiguration() || {};
  var aggregationColumn = $jscomp$destructuring$var1.aggregationColumn;
  var timeseriesColumn = $jscomp$destructuring$var1.timeseriesColumn;
  var moduleId = $jscomp$destructuring$var1.moduleId;
  if (Ext.isEmpty(moduleId) || !me.isMonitorAvailable()) {
    me.renderWrongConfigPanel('MONITOR_WIDGET_INCORRECT_MODULE_ID');
    return false;
  }
  if (Ext.isEmpty(aggregationColumn) || !me.getMonitorColumnConfig(aggregationColumn)) {
    me.renderWrongConfigPanel('MONITOR_WIDGET_INCORRECT_COLUMN_CONFIG');
    return false;
  }
  if (Ext.isEmpty(timeseriesColumn) || !me.getMonitorColumnConfig(timeseriesColumn)) {
    me.renderWrongConfigPanel('MONITOR_SETTING_WIDGET_INCORRECT_TIMESERIES_COLUMN_CONFIG');
    return false;
  }
  return true;
}});
Ext.define('monitor.widget.processPreview.Main', {extend:'monitor.widget.abstract.MonitorBaseWidget', alias:'widget.monitorWidgetProcessPreviewMain', alternateClassName:['monitor.widget.monitorWidget.Table', 'monitor.widget.processPreview.Table'], settingsPanelCmp:'monitorWidgetProcessPreviewConfigurationPanel', contentPanelCmp:'monitorWidgetProcessPreviewView', statics:{WIDGET_NAME:BpcCommon.lang.Current.MONITOR_PROCESS_PREVIEW, WIDGET_DESCRIPTION:BpcCommon.lang.Current.MONITOR_PROCESS_PREVIEW_TABLE_DESCRIPTION, 
WIDGET_ICON_CLS:'x-fal fa-table', WIDGET_ADDITIONAl_ACTIONS:[{xtype:'monitorStateSelectionMenu'}]}, defaultWidgetConfiguration:{iconCls:'x-fal fa-table', title:BpcCommon.lang.Current.MONITOR_PROCESS_PREVIEW, name:BpcCommon.lang.Current.MONITOR_PROCESS_PREVIEW, moduleId:'auditlog', startDate:'now-1h', endDate:'now', columnList:[], timeseriesColumn:'timestamp', goTo:true, pageSize:25, selectedView:0, views:[], initialView:'', dataRefreshBuffer:10, openDetailView:false}, minHeight:100, layout:'card', 
activeItem:0, title:false, renderWidget:function() {
  var me = this;
  if (me.specificErrorHandling()) {
    var components = me.createMultipleGridPanel();
    me.renderCmp(components);
  }
}, specificErrorHandling:function() {
  var me = this;
  var $jscomp$destructuring$var2 = me.getWidgetConfiguration() || {};
  var moduleId = $jscomp$destructuring$var2.moduleId;
  var timeseriesColumn = $jscomp$destructuring$var2.timeseriesColumn;
  if (Ext.isEmpty(moduleId) || !me.isMonitorAvailable()) {
    me.renderWrongConfigPanel('MONITOR_WIDGET_INCORRECT_MODULE_ID');
    return false;
  }
  if (Ext.isEmpty(timeseriesColumn) || !me.getMonitorColumnConfig(timeseriesColumn)) {
    me.renderWrongConfigPanel('MONITOR_SETTING_WIDGET_INCORRECT_TIMESERIES_COLUMN_CONFIG');
    return false;
  }
  return true;
}});
Ext.define('monitor.widget.processDistribution.Main', {extend:'monitor.widget.abstract.MonitorBaseWidget', alias:'widget.monitorWidgetProcessDistributionMain', alternateClassName:['monitor.widget.columnAggregation.Circle'], layout:'card', activeItem:0, title:false, minHeight:100, settingsPanelCmp:'monitorWidgetProcessDistributionConfigurationPanel', contentPanelCmp:'monitorWidgetProcessDistributionView', statics:{WIDGET_NAME:BpcCommon.lang.Current.MONITOR_PROCESS_CIRCLE_TOTAL, WIDGET_DESCRIPTION:BpcCommon.lang.Current.MONITOR_PROCESS_CIRCLE_AGGREGATION_ALL_DATA, 
WIDGET_ICON_CLS:'x-fal fa-chart-pie', WIDGET_ADDITIONAl_ACTIONS:[{xtype:'monitorStateSelectionMenu'}]}, defaultWidgetConfiguration:{iconCls:'x-fal fa-chart-pie', title:BpcCommon.lang.Current.MONITOR_PROCESS_CIRCLE_TOTAL, name:BpcCommon.lang.Current.MONITOR_PROCESS_CIRCLE_TOTAL, moduleId:'auditlog', column:'level', timeseriesColumn:'timestamp', initialView:'', excludedValues:[], hiddenGraphs:[], views:[], viewMode:'tabbed', startDate:'now-7d/d', endDate:'now', selectedView:0, hideZeroValues:true, 
cellAlign:'center', aggregationLimit:10, '3d':true, dataRefreshBuffer:10, chartConfiguration:{'labelRadius':2, 'language':'de', 'export':{'language':'de', 'enabled':true}, 'marginTop':5}, dockSummary:''}, renderWidget:function() {
  var me = this;
  if (me.specificErrorHandling()) {
    var components = me.createMultipleGridPanel();
    components = me.addOverViewTab(components);
    me.renderCmp(components);
  }
}, specificErrorHandling:function() {
  var me = this;
  var $jscomp$destructuring$var3 = me.getWidgetConfiguration() || {};
  var column = $jscomp$destructuring$var3.column;
  var timeseriesColumn = $jscomp$destructuring$var3.timeseriesColumn;
  var moduleId = $jscomp$destructuring$var3.moduleId;
  if (Ext.isEmpty(moduleId) || !me.isMonitorAvailable()) {
    me.renderWrongConfigPanel('MONITOR_WIDGET_INCORRECT_MODULE_ID');
    return false;
  }
  if (Ext.isEmpty(column) || !me.getMonitorColumnConfig(column)) {
    me.renderWrongConfigPanel('MONITOR_WIDGET_INCORRECT_COLUMN_CONFIG');
    return false;
  }
  if (Ext.isEmpty(timeseriesColumn) || !me.getMonitorColumnConfig(timeseriesColumn)) {
    me.renderWrongConfigPanel('MONITOR_SETTING_WIDGET_INCORRECT_TIMESERIES_COLUMN_CONFIG');
    return false;
  }
  return true;
}, addOverViewTab:function(tabPanel) {
  var me = this;
  var widgetPanel = tabPanel;
  var moduleId = this.getSetting('moduleId');
  var shouldAddOverviewTab = tabPanel.xtype === 'tabpanel' && Ext.isArray(tabPanel.items) && tabPanel.items.length > 1;
  if (shouldAddOverviewTab) {
    var overviewTab = {xtype:'monitorWidgetProcessDistributionOverview', storeId:'overView' + moduleId, widgetReference:me, moduleId:moduleId};
    widgetPanel.items = [overviewTab].concat(tabPanel.items);
    var selectedTab = widgetPanel.items.findIndex(function(el) {
      return el.viewId === me.getSetting('initialView');
    });
    if (Ext.isNumber(selectedTab) && selectedTab !== -1) {
      widgetPanel.activeTab = selectedTab;
    }
  }
  return widgetPanel;
}, updateOverViewData:function(source) {
  try {
    this.moduleId = this.getSetting('moduleId');
    var target = this.down('monitorWidgetProcessDistributionOverview');
    if (target && source && source.moduleId && source.moduleId === this.moduleId) {
      var sourceCmps = this.query(this.contentPanelCmp);
      var dataArray = [];
      Ext.Array.each(sourceCmps, function(sourceCmp) {
        var view = '';
        if (sourceCmp.viewObj && sourceCmp.viewObj.title) {
          view = BpcCommon.Api.getTranslation(sourceCmp.viewObj.title);
        }
        var record = {view:view};
        sourceCmp.store.each(function(i) {
          record[i.get('value')] = i.get('count');
        });
        dataArray.push(record);
      }, this);
      target.store.loadData(dataArray);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateOverViewData', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.interaction.processAction.startProcess.Widget', {extend:'Ext.container.Container', alias:'widget.monitorStartProcessWidget', alternateClassName:['monitor.view.interaction.startprocess.Widget'], mixins:['BpcCommon.mixin.Widget'], statics:{WIDGET_NAME:'MONITOR_PROCESS_STARTER_WIDGET_TITLE', WIDGET_DESCRIPTION:'MONITOR_PROCESS_STARTER_WIDGET_DESCRIPTION', WIDGET_ICON_CLS:'x-fal fa-play'}, layout:'card', activeItems:0, widgetConfigSettingMap:{moduleId:{settingType:'moduleInstance', 
'_label':'MONITOR_SETTING_WIDGET_MODULE_ID', '_tooltip':'MONITOR_SETTING_WIDGET_MODULE_ID_TOOLTIP'}, processKey:{settingType:'text'}}, viewModel:{data:{vmDataUnmanagedSetting:'nothing1', vmDataManagedSettingWithDefault:'nothing2', vmDataManagedSettingWithOUTDefault:'nothing3'}}, initComponent:function() {
  var me = this;
  me.initWidgetSettings();
  me.callParent();
  me.addContent();
  me.settingPanel = me.add({xtype:'bpcSettingPanel', settings:me.settingStore, buttons:[{text:'OK', handler:me.setActiveItem.bind(this, 0)}]});
  me.initWidget();
}, addContent:function() {
  var me = this;
  if (me.content) {
    me.remove(me.content);
  }
  me.content = me.add({xtype:'monitorStartProcessPanel', moduleId:me.getSetting('moduleId'), preselectedProcess:{key:me.getSetting('processKey')}, header:false});
}, refreshWidget:function() {
  this.addContent();
}, configureWidget:function() {
  Ext.log({msg:this.self.getName() + ': configureWidget', level:'info'});
  this.setActiveItem(1);
}, initWidget:function() {
  Ext.log({msg:this.self.getName() + ': initWidget', level:'info'});
  var vm = this.getViewModel();
  vm.set('vmDataUnmanagedSetting', this.getSetting('unmanagedSetting'));
  vm.set('vmDataManagedSettingWithDefault', this.getSetting('managedSettingWithDefault'));
  vm.set('vmDataManagedSettingWithOUTDefault', this.getSetting('managedSettingWithOUTDefault'));
}});
Ext.define('monitor.BpcInterface', {extend:'BpcCommon.BpcInterface', requires:['monitor.widget.timeseriesAggregation.TimeseriesAggregation', 'monitor.widget.processPreview.Table', 'monitor.widget.columnAggregation.Circle', 'monitor.view.interaction.startprocess.Widget'], config:{moduleConfigurationComponents:[], defaultModuleInstanceConfigurationComponent:'bpcMonitorConfigurationMain', moduleInstanceConfigurationComponents:['monitorAdminMonitorViewStates', 'tableViewConfigurator', 'bpcMonitorConfigurationMain'], 
moduleInstanceCreationComponent:'bpcMonitorCreationMain', requiredModules:[], widgets:['monitor.widget.timeseriesAggregation.TimeseriesAggregation', 'monitor.widget.processPreview.Table', 'monitor.widget.columnAggregation.Circle', 'monitor.view.interaction.startprocess.Widget'], css:true}});
Ext.define('monitor.assets.ConfigKeys', {singleton:true, actionSeperator:'function_processActionSeparator', aggregationLimit:'aggregationLimit', appToolbarLeftContent:'appToolbarLeftContent', appToolbarRightContent:'appToolbarRightContent', autoUpdate:'function_autoUpdate', bpmBasePath:'inubit_baseUrl', bpmProxyId:'inubit_proxyId', bpmReferenceEndpoint:'inubit_referenceEndpoint', changeState:'function_changeState', changeStateConfig:'function_changeStateConfig', changeStateEndpoint:'inubit_changeStateEndpoint', 
columnActionPrefix:'function_processActionPrefix', columnConfig:'column_config', columnCustomRenderer:'column_customRenderer', columnDefaultMimetype:'column_default_mimetype', columnFile:'column_file', columnGroupField:'column_groupField', columnHistoryConfig:'column_historyConfig', columnHistoryId:'column_historyId', columnHistoryList:'column_historyList', columnHistorySubId:'column_historySubId', columnId:'column_id', columnList:'column_list', columnMimetype:'column_mimetype', columnStatus:'column_status', 
columnTimeseries:'column_timeseries', columnTaskView:'function_interactionColumn', columnTaskViewValue:'function_interactionFormId', customViewsEnabled:'function_saveView', countColumns:'gui_countFilteredColumns', dataRefreshBufferInSeconds:'data_refreshBuffer', dataCountLimit:'monitor_data_count_limit', dataExcludedFields:'data_excludedFields', dataFilter:'data_filter', dataFilterOperator:'data_filter_operator', dataIndex:'data_index', dataType:'data_type', dateFormat:'date_format', dataHistoryIndex:'data_historyIndex', 
dataHistoryType:'data_historyType', detailViewEastConfig:'detailView_eastConfig', detailViewEastEnabled:'detailView_east', detailViewEastInitialShow:'detailView_eastInitialShow', detailViewInlineConfig:'detailView_inlineConfig', detailViewInlineEnabled:'detailView_inline', detailViewPopupConfig:'detailView_popupConfig', detailViewPopupEnabled:'detailView_popup', dockSummary:'dockSummary', exportCsv:'function_exportConfigCsv', exportCsvInclHidden:'function_exportConfigCsvInclHidden', exportCsvSeparator:'function_exportConfigCsvSeparator', 
exportCsvUseCustomColumnHeaders:'function_exportConfigCsvUseCustomColumnHeaders', exportDialogDefaults:'function_exportDialogDefaults', exportEnabled:'function_exportEnable', exportJson:'function_exportConfigJson', exportJsonInclHidden:'function_exportConfigJsonInclHidden', exportJsonUseCustomColumnHeaders:'function_exportConfigJsonUseCustomColumnHeaders', exportJsonUseUTCDateFormat:'function_exportConfigJsonUseUTCDateFormat', exportLimit:'function_maxExportLimit', exportXlsx:'function_exportConfigXlsx', 
exportXlsxAutoResizeColumns:'function_exportConfigXlsxAutoResizeColumns', exportXlsxInclHidden:'function_exportConfigXlsxInclHidden', exportXlsxUseCustomColumnHeaders:'function_exportConfigXlsxUseCustomColumnHeaders', filterRespectOtherColumns:'function_filterRespectOtherColumns', gridEditable:'grid_textSelectionEnabled', gridMainSortDirection:'grid_mainSortDirection', gridSortingColumn:'grid_mainSortColumn', guiShowFilterBar:'gui_showFilterBar', guiStateBrowsing:'gui_stateBrowsing', hideDefaultTableHeaderTooltip:'hideDefaultTableHeaderTooltip', 
hideZeroValues:'hideZeroValues', instanceDataCountLimit:'instance_data_count_limit', instanceDataViewLimit:'instance_data_view_limit', legacyPmGridId:'inubit_dbGridId', legacyPmMandant:'inubit_pmMandant', legacyPmTablePrefix:'inubit_dbTablePrefix', moduleDescription:'module_description', moduleHeaderContent:'moduleHeaderContent', moduleHeaderEnabled:'moduleHeader_enabled', moduleIconCls:'module_iconCls', moduleName:'module_name', multipleGrids:'function_gridMultipleViews', multipleGridsConfig:'function_gridMultipleViewsConfig', 
parallelProcessActions:'function_processActionAllowParallel', processActions:'function_processActions', processActionsEndpoint:'inubit_actionEndpoint', processNotificationDisplayMode:'function_processNotificationDisplayMode', processStart:'function_processStart', processStartConfig:'function_processStartConfig', processStartEndpoint:'inubit_VpsEndpoint', resourceSavingMode:'resource_saving_mode', rowStyle:'row_style', rowStyleEnabled:'row_styleEnabled', showGridLoadMask:'grid_showLoadMask', sortMulti:'sort_multi', 
sortMultiColumnCount:'sort_multiColumnCount', sortMultiReversedOrder:'sort_multiReversedOrder', viewStates:'view_states'});
Ext.define('monitor.controller.RowStyleController', {extend:'Ext.app.Controller', init:function() {
  this.createStylesheets();
}, createStylesheets:function() {
  try {
    var monitorInstances = BpcCommon.Api.getModuleInstances('monitor');
    monitorInstances.each(this.createStylesheetForMonitor, this);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createStylesheets', dump:e, stack:true, level:'error'});
  }
}, createStylesheetForMonitor:function(monitorInstance) {
  var $jscomp$this = this;
  var monitorId = monitorInstance.id;
  try {
    if (!BpcCommon.Api.getSetting(monitorId, monitor.assets.ConfigKeys.rowStyleEnabled)) {
      return;
    }
    var rowStyleSettings = BpcCommon.Api.getSetting(monitorId, monitor.assets.ConfigKeys.rowStyle);
    if (rowStyleSettings && Ext.isArray(rowStyleSettings) && rowStyleSettings.length !== 0) {
      Ext.Array.each(rowStyleSettings, function(setting) {
        var rowClasses = setting.rowClasses;
        var applyToViews = setting.views;
        if (applyToViews && applyToViews.length > 0) {
          Ext.Array.each(applyToViews, function(viewId) {
            $jscomp$this.createStyleSheet(monitorId + viewId, rowClasses);
          });
        } else {
          $jscomp$this.createStyleSheet(monitorId, setting.rowClasses);
        }
      });
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createStylesheetForMonitor', dump:e, stack:true, level:'error'});
  }
}, createStyleSheet:function(styleId, styles) {
  try {
    styles = styles || '';
    var styleSheetId = styleId + 'rowStyle';
    Ext.util.CSS.removeStyleSheet(styleSheetId);
    styles = styles.replace(/\.-/g, '._' + styleId + '-');
    Ext.util.CSS.createStyleSheet(styles, styleId);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createStyleSheet', dump:e, stack:true, level:'error'});
  }
}, getRowClass:function(monitorId, viewId, record, rowIndex, rowParams, store) {
  var baseRowClass = 'monitorGrid-row ';
  var classes = baseRowClass;
  try {
    var grid = this;
    if (!BpcCommon.Api.getSetting(monitorId, monitor.assets.ConfigKeys.rowStyleEnabled)) {
      return baseRowClass;
    }
    var addClassFn = function(column, shouldApplyAllView) {
      column = column || '';
      var value = record.get(column) || '';
      value = value.toUpperCase();
      var splitArray = column.split('-');
      splitArray.push(store.bpcGridViewId);
      if (BpcCommon.Util.isStringEqual(splitArray[0], 'tpl') && splitArray[1]) {
        var columnId = splitArray.slice(1).join('-');
        var columnObject = grid.getColumnManager().getHeaderById(columnId);
        if (columnObject && Ext.isFunction(columnObject.renderer)) {
          value = String(columnObject.renderer(record.get(columnObject.dataIndex), {}, record)).toUpperCase();
        }
      }
      if (shouldApplyAllView) {
        classes = classes + ' _' + [id, column, value].join('-');
      } else {
        classes = classes + ' _' + [id + viewId, column, value].join('-');
      }
    };
    var rowStyleSettings = BpcCommon.Api.getSetting(monitorId, monitor.assets.ConfigKeys.rowStyle);
    var id = monitorId;
    if (rowStyleSettings && Ext.isArray(rowStyleSettings) && rowStyleSettings.length !== 0) {
      Ext.Array.each(rowStyleSettings, function(setting) {
        if (setting.columnList) {
          Ext.Array.each(setting.columnList, function(column) {
            var shouldApplyAllView = !setting.views || setting.views.length === 0;
            addClassFn(column, shouldApplyAllView);
          });
        }
      });
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR getRowClass', dump:e, stack:true, level:'error'});
  }
  return classes;
}});
Ext.define('monitor.controller.MonitorViewStates', {extend:'Ext.app.Controller', storeMap:{}, initMonitorViewStateStore:function(moduleId) {
  var me = this;
  if (me.isStoreCreationRequired(moduleId)) {
    me.storeMap[moduleId] = Ext.create('monitor.store.MonitorViewStates', {moduleId:moduleId, autoLoad:BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.customViewsEnabled) === true});
  }
  return me.storeMap[moduleId];
}, isStoreCreationRequired:function(moduleId) {
  var me = this;
  var viewStore = me.storeMap[moduleId];
  if (!Ext.isDefined(viewStore)) {
    return true;
  }
  if (!viewStore.isStore) {
    return true;
  }
  if (viewStore.isLoading()) {
    return false;
  }
  return false;
}, getMonitorViewStateStore:function(moduleId) {
  var me = this;
  return me.initMonitorViewStateStore(moduleId);
}, getMonitorViewState:function(moduleId, monitorViewStateId) {
  var $jscomp$async$this = this;
  var me;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    me = $jscomp$async$this;
    me.initMonitorViewStateStore(moduleId);
    return $jscomp$generator$context["return"](new Promise(function(resolve) {
      if (!me.storeMap[moduleId].isLoaded()) {
        me.storeMap[moduleId].load();
        me.storeMap[moduleId].on('load', function() {
          resolve(me.storeMap[moduleId].getById(monitorViewStateId));
        }, me, {single:true});
      } else {
        resolve(me.storeMap[moduleId].getById(monitorViewStateId));
      }
    }));
  });
}, getCurrentMonitorViewState:function(moduleId) {
  var me = this;
  var monitorViewCmp = BpcCommon.Api.getModuleInstanceComponent(moduleId);
  if (monitorViewCmp) {
    return me.cleanMonitorViewStateObject(monitorViewCmp.getState());
  }
  return null;
}, cleanMonitorViewStateObject:function(stateObject) {
  stateObject = stateObject || {};
  if (stateObject instanceof monitor.model.MonitorViewState) {
    stateObject = stateObject.getData();
  }
  stateObject.id = stateObject.id || (new Date()).getTime();
  delete stateObject.activeStateDirty;
  delete stateObject.local;
  return stateObject;
}, compareMonitorViewStates:function(state1, state2) {
  if (state1 instanceof monitor.model.MonitorViewState) {
    state1 = state1.getData();
  }
  if (state2 instanceof monitor.model.MonitorViewState) {
    state2 = state2.getData();
  }
  if (state1 === state2) {
    return true;
  } else if (!state1 && state2 || state1 && !state2) {
    return false;
  }
  var compareState1 = {views:state1 ? state1.views : []};
  var compareState2 = {views:state2 ? state2.views : []};
  return state1.searchQuery === state2.searchQuery && BpcCommon.Util.compareObject(compareState1, compareState2);
}, saveMonitorViewStates:function(moduleId) {
  var me = this;
  var deferred = new Ext.Deferred();
  me.initMonitorViewStateStore(moduleId);
  me.storeMap[moduleId].sync({success:function() {
    deferred.resolve();
    BpcCommon.Api.showNotification('MONITOR_VIEW_SAVED');
  }, failure:function() {
    deferred.reject();
    BpcCommon.Api.showNotification('MONITOR_ERROR_SAVING_VIEW');
  }});
  return deferred.promise;
}, addMonitorViewState:function(moduleId, state) {
  var me = this;
  me.initMonitorViewStateStore(moduleId);
  state = me.prepareMonitorViewStateModel(state);
  me.storeMap[moduleId].add(state);
  me.saveMonitorViewStates(moduleId);
  return state;
}, updateMonitorViewState:function(moduleId, stateId, state, save) {
  var me = this;
  delete state.id;
  me.getMonitorViewState(moduleId, stateId).then(function(monitorViewStateModel) {
    Ext.Object.each(state, function(key, value) {
      monitorViewStateModel.set(key, value);
    });
    if (save === true) {
      monitorViewStateModel.save();
    }
  });
}, applyState:function(moduleId, state) {
  var monitorViewCmp = BpcCommon.Api.getModuleInstanceComponent(moduleId);
  if (state instanceof monitor.model.MonitorViewState) {
    state = state.getData();
  }
  monitorViewCmp.applyState(state);
}, resetState:function(moduleId) {
  var monitorViewCmp = BpcCommon.Api.getModuleInstanceComponent(moduleId);
  if (monitorViewCmp) {
    monitorViewCmp.resetState();
  }
}, prepareMonitorViewStateModel:function(state) {
  if (!(state instanceof monitor.model.MonitorViewState)) {
    state = monitor.model.MonitorViewState.create(Ext.apply(state, {readRoles:['bpcuser'], writeRoles:['bpcuser']}));
  }
  return state;
}, resetDirtyChangesFromState:function(moduleId, stateId) {
  var me = this;
  me.getMonitorViewState(moduleId, stateId).then(function(stateModel) {
    if (stateModel) {
      stateModel.reject();
      me.applyState(moduleId, stateModel);
    }
  });
}});
Ext.define('monitor.store.NumericOperatorStore', {extend:'Ext.data.Store', storeId:'numericOperatorStore', fields:['id', 'text'], data:[{id:'lt', text:'\x3c'}, {id:'lte', text:'\x3c\x3d'}, {id:'\x3d', text:'\x3d'}, {id:'gt', text:'\x3e'}, {id:'gte', text:'\x3e\x3d'}]});
Ext.define('monitor.Module', {singleton:true, requires:['monitor.widget.timeseriesAggregation.TimeseriesAggregation', 'monitor.widget.columnAggregation.Circle', 'monitor.assets.ConfigKeys', 'monitor.controller.RowStyleController', 'monitor.controller.MonitorViewStates', 'monitor.store.NumericOperatorStore'], constructor:function() {
  try {
    this.rowStyleController = Ext.getApplication().getController('monitor.controller.RowStyleController');
    this.monitorViewStatesController = Ext.getApplication().getController('monitor.controller.MonitorViewStates');
    Ext.create('monitor.store.NumericOperatorStore');
    var showMonitorSearchFieldButton = false;
    if (Ext.state.Manager.get('monitorSearchFieldButton') && Ext.isBoolean(Ext.state.Manager.get('monitorSearchFieldButton').pressed)) {
      showMonitorSearchFieldButton = Ext.state.Manager.get('monitorSearchFieldButton').pressed;
    }
    BpcCommon.Api.getGlobalViewModel().set('global.monitor.showSearchField', showMonitorSearchFieldButton);
  } catch (e) {
    Ext.log({msg:'monitor.Module: ERROR constructor', dump:e, stack:true, level:'error'});
  }
}, testRenderer:function() {
  return function(value) {
    return 'Tested_' + value;
  };
}});
Ext.define('monitor.Util', {singleton:true, createFilterObj:function(property, value, anyMatch, type, source, operator) {
  var filter;
  try {
    if (!type) {
      type = 'text';
    }
    if (!source) {
      source = 'analyzed';
    }
    if (!anyMatch) {
      anyMatch = false;
    }
    if (!operator) {
      operator = 'analyzed';
    }
    if (Array.isArray(value)) {
      filter = {property:property, operator:operator, value:value, source:source};
    } else {
      filter = {property:property, value:value, type:type, source:source};
    }
  } catch (e) {
    Ext.log('Fehler beim Erstellen des Filters' + e);
    filter = {};
  }
  return filter;
}, ajaxGetCall:function(url, callback) {
  Ext.Ajax.request({url:url, method:'GET', callback:function(options, success, response) {
    var responseData = '';
    if (success) {
      try {
        responseData = Ext.JSON.decode(response.responseText);
      } catch (e) {
        Ext.log({msg:'Monitor: Invalid json configured: ' + e, level:'warn'});
      }
    }
    if (callback) {
      callback(success, responseData);
    }
  }});
}, toXML:function(value) {
  return Ext.util.Format.htmlEncode(value);
}, toAtomXML:function(name, value) {
  return '\x3c' + name + '\x3e' + this.toXML(value) + '\x3c/' + name + '\x3e';
}, toBeginXML:function(name, type) {
  if (type === 'atom' || Ext.isEmpty(type)) {
    type = '';
  } else {
    type = ' type\x3d"' + this.toXML(type) + '"';
  }
  return '\x3c' + name + type + '\x3e';
}, toEndXML:function(name) {
  return '\x3c/' + name + '\x3e';
}, parseXML:function(dataInString) {
  if (window.DOMParser) {
    return (new window.DOMParser()).parseFromString(dataInString, 'text/xml');
  } else if (typeof window.ActiveXObject !== 'undefined' && new window.ActiveXObject('Microsoft.XMLDOM')) {
    var xmlDoc = new window.ActiveXObject('Microsoft.XMLDOM');
    xmlDoc.async = 'false';
    xmlDoc.loadXML(dataInString);
    return xmlDoc;
  }
  return null;
}, isEmpty:function(element) {
  if (typeof element === 'undefined') {
    return true;
  }
  if (typeof element === 'boolean' && (element === true || element === false)) {
    return false;
  }
  if (element === null) {
    return true;
  }
  if (element.length > 0) {
    return false;
  }
  if (element.length === 0) {
    return true;
  }
  for (var key in element) {
    if (hasOwnProperty.call(element, key)) {
      return false;
    }
  }
  if (typeof element === 'number') {
    return isNaN(element);
  }
  return !element;
}, createRendererFn:function(key, values, globalIconOnly, labelOnly, basic) {
  return function(value, meta, record, testLabelOnly) {
    var styleStr = 'font-size: 18px; margin-right: 3px;';
    var htmlString, icon, color, label;
    var tooltip = '';
    var iconOnly = globalIconOnly ? globalIconOnly : false;
    var rawValue = value;
    testLabelOnly = testLabelOnly ? testLabelOnly : false;
    htmlString = Ext.util.Format.htmlEncode2(value);
    if (Ext.isDefined(value) && value !== null) {
      value = String(value);
      value = value.toLowerCase().trim();
      if (values[value]) {
        if (Ext.isDefined(values[value].iconOnly) && values[value].iconOnly) {
          iconOnly = true;
        }
        icon = values[value].iconCls;
        color = values[value].color;
        label = values[value].label;
      } else {
        if (values['*']) {
          if (Ext.isDefined(values['*'].iconOnly) && values[value].iconOnly) {
            iconOnly = true;
          }
          icon = values['*'].iconCls;
          color = values['*'].color;
          label = values['*'].label;
        }
      }
    } else {
      if (values['!']) {
        if (Ext.isDefined(values['!'].iconOnly) && values[value].iconOnly) {
          iconOnly = true;
        }
        icon = values['!'].iconCls;
        color = values['!'].color;
        label = values['!'].label;
      }
    }
    label = BpcCommon.Api.getTranslation(label);
    if (testLabelOnly === true) {
      return Ext.isDefined(label) ? label : rawValue;
    }
    if (basic) {
      iconOnly = false;
    }
    if (iconOnly) {
      htmlString = "\x3ci class\x3d'" + icon + "' style\x3d'color:" + color + '; ' + styleStr + "'\x3e\x3c/i\x3e";
      label = Ext.isDefined(label) ? label : rawValue;
    } else if (labelOnly) {
      htmlString = Ext.isDefined(label) ? label : rawValue;
    } else {
      if (value === null) {
        value = '';
      }
      label = Ext.isDefined(label) ? label : rawValue;
      htmlString = "\x3ci class\x3d'" + icon + "' style\x3d'color:" + color + '; ' + styleStr + "'\x3e\x3c/i\x3e" + label;
    }
    if (meta) {
      tooltip = Ext.isDefined(label) ? label : rawValue;
      meta.tdAttr = 'data-qtip\x3d"' + tooltip + '"';
    }
    return htmlString;
  };
}, nodeToJsonTree:function(node) {
  var jsonTree = {text:node.nodeName, draggable:false};
  var textValue = '';
  Ext.each(node.childNodes, function(item) {
    if (item.nodeType === 3 && item.textContent.trim() !== '') {
      textValue = textValue + item.textContent.trim() + ' ';
      textValue = textValue.replace(/</g, '\x26lt;');
      textValue = textValue.replace(/>/g, '\x26gt;');
    }
  }, this);
  var attributes = '';
  if (node.attributes) {
    for (var i = 0; i < node.attributes.length; i++) {
      attributes = attributes + node.attributes.item(i).name + '\x3d' + node.attributes.item(i).value + ' ';
    }
  }
  if (attributes.length > 0) {
    jsonTree.text = jsonTree.text + ' (' + attributes + ')';
  }
  if (textValue.length > 0) {
    jsonTree.text = jsonTree.text + ' : \x3cb\x3e' + textValue + '\x3c/b\x3e';
  }
  if (!node.childElementCount || node.childElementCount < 1) {
    jsonTree.leaf = true;
    jsonTree.iconCls = '';
    jsonTree.icon = Ext.BLANK_IMAGE_URL;
  } else {
    jsonTree.leaf = false;
    jsonTree.icon = Ext.BLANK_IMAGE_URL;
    jsonTree.children = [];
    Ext.each(node.childNodes, function(node) {
      if (node.nodeType !== 3) {
        jsonTree.children.push(this.nodeToJsonTree(node));
      }
    }, this);
  }
  return jsonTree;
}, mergeAmChartGraphConfiguration:function(configuration1, configuration2) {
  var valueMap = Ext.Array.toValueMap(configuration1, 'valueField');
  var newValueMap = Ext.Array.toValueMap(configuration2, 'valueField');
  Ext.Array.each(configuration2, function(config) {
    if (config.valueField && !valueMap[config.valueField]) {
      configuration1.push(config);
    }
  });
  Ext.Array.each(configuration1, function(config, index) {
    if (config.valueField && !newValueMap[config.valueField]) {
      configuration1[index].hidden = true;
      configuration1[index].visibleInLegend = false;
    } else {
      configuration1[index].visibleInLegend = true;
    }
  });
  return configuration1;
}, getViewModel:function(startCmpOrViewModel, alias) {
  alias = alias || 'monitorMainViewModel';
  var vm;
  if (startCmpOrViewModel instanceof Ext.Component) {
    vm = startCmpOrViewModel.lookupViewModel();
  } else if (startCmpOrViewModel instanceof Ext.app.ViewModel) {
    vm = startCmpOrViewModel;
  }
  if (vm) {
    if (vm.type === alias) {
      return vm;
    }
    return monitor.Util.getViewModel(vm.getParent(), alias);
  }
}, getStandardColumnConfig:function(columnConfig, monitorGrid) {
  if (Ext.Object.isEmpty(columnConfig)) {
    return {};
  }
  var mainCmp = monitorGrid.up('bpcMonitor');
  var plugin = mainCmp.findPlugin('monitorCustomRenderer');
  var newConfig = Ext.clone(columnConfig);
  delete newConfig.renderer;
  if (Ext.isString(newConfig.customRenderer) && newConfig.customRenderer !== '' || newConfig.customRenderer === true) {
    if (plugin) {
      newConfig.renderer = plugin.getColumnRendererByColumnConfig(columnConfig);
    }
  }
  if (newConfig.xtype === 'datecolumn' && !newConfig.format) {
    newConfig.format = BpcCommon.Api.getSetting(mainCmp.moduleId, monitor.assets.ConfigKeys.dateFormat);
  }
  if (newConfig.text) {
    newConfig.text = Ext.String.htmlEncode(newConfig.text);
  } else if (newConfig.text === '') {
    newConfig.text = '\x26nbsp;';
  }
  return newConfig;
}, getMimeType:function(record, monitorId) {
  var mimetypeDataField = BpcCommon.Api.getSetting(monitorId, monitor.assets.ConfigKeys.columnMimetype);
  var defaultMimetype = BpcCommon.Api.getSetting(monitorId, monitor.assets.ConfigKeys.columnDefaultMimetype);
  return record.get(mimetypeDataField) || defaultMimetype;
}, isBinaryMimeType:function(mimeType) {
  var binaryTypes = ['application/pdf', 'application/octet-stream'];
  if (mimeType.startsWith('image/') || mimeType.startsWith('audio/') || mimeType.startsWith('video/')) {
    return true;
  }
  return binaryTypes.includes(mimeType.toLowerCase());
}, binaryStringToObjectUrl:function(record, stringOrBinary, mimeType) {
  if (typeof stringOrBinary === 'string') {
    var length = stringOrBinary.length;
    var bytes$3 = new Uint8Array(length);
    for (var i = 0; i < length; i++) {
      bytes$3[i] = stringOrBinary.charCodeAt(i);
    }
  } else {
    bytes = stringOrBinary;
  }
  return URL.createObjectURL(new Blob([bytes], {type:mimeType}));
}});
Ext.define('monitor.assets.Colors', {singleton:true, virtimoColors:['#c63441', '#e18b50', '#96bebe', '#adbfc3', '#85909c', '#958f77', '#cbbc6b', '#dcd9d2', '#677488']});
Ext.define('monitor.assets.Helpers', {singleton:true, formatDate:function(date, dateFormat) {
  var verifiedDate = this.transformDate(date);
  return Ext.Date.format(verifiedDate, dateFormat);
}, transformDate:function(date) {
  return !isNaN(new Date(date)) ? new Date(date) : BpcCommon.util.EsDateMath.getCalculatedDateFromOffsetMap(BpcCommon.util.EsDateMath.getOffsetMapFromEsDateMathString(date));
}, getArrayFromString:function(inputValue) {
  if (Ext.isEmpty(inputValue)) {
    return [];
  }
  var arrayValue = Ext.isArray(inputValue) ? inputValue : inputValue.split(',');
  return arrayValue.map(Ext.String.trim);
}, configurationToArray:function(configuration) {
  var configList = [];
  Ext.Object.each(configuration, function(key, value) {
    configList.push({name:key, id:key, value:value});
  });
  return configList;
}, configurationToObject:function(settingsList) {
  var settingsObject = {};
  Ext.Array.each(settingsList, function(setting) {
    settingsObject[setting.name] = setting.value;
  });
  return settingsObject;
}});
Ext.define('monitor.assets.Icons', {singleton:true, PROCESS_START:'x-fal fa-play', TASKS:'x-fal fa-tasks', PROCESS_ACTION:'x-fal fa-indent', REFRESH:'x-fal fa-sync', VIEW:'x-fal fa-browser', DOWNLOAD:'x-fal fa-download', MONITOR:'x-fal fa-table', COLUMN_CONFIGURATOR:'fal fa-game-board-alt', WIDGET_SETTINGS:'x-fal fa-cogs'});
Ext.define('monitor.assets.ModuleRights', {singleton:true, editMonitorViews:'bpcmonitor_editmonitorviews', changeProcessState:'bpcmonitor_changeprocessstate', startProcess:'bpcmonitor_startprocess', processAction:'bpcmonitor_processaction'});
Ext.define('monitor.component.AccessControlFieldSet', {extend:'Ext.form.FieldSet', alias:'widget.monitorAccessControlFieldSet', localized:{title:'CORE_ACCESS_CONTROL'}, defaults:{labelWidth:135, anchor:'100%'}, items:[{xtype:'checkbox', itemId:'personalView', localized:{fieldLabel:'MONITOR_VIEW_PERSONAL'}, bind:'{personal}'}, {xtype:'fieldset', localized:{title:'MONITOR_VIEW_WRITE_ACCESS'}, defaults:{labelWidth:120, anchor:'100%'}, items:[{xtype:'bpcOrganisationSelect', itemId:'writeOrganisations', 
grow:true, growToLongestValue:true, matchFieldWidth:false, bind:{disabled:'{personal}', value:'{stateObject.writeOrganisations}'}}, {xtype:'bpcRoleSelect', itemId:'writeRoles', grow:true, growToLongestValue:true, matchFieldWidth:false, bind:{disabled:'{personal}', value:'{stateObject.writeRoles}'}, value:['bpcuser']}, {xtype:'bpcRightSelect', itemId:'writeRights', grow:true, growToLongestValue:true, matchFieldWidth:false, bind:{disabled:'{personal}', value:'{stateObject.writeRights}'}}]}, {xtype:'fieldset', 
localized:{title:'MONITOR_VIEW_READ_ACCESS'}, defaults:{labelWidth:120, anchor:'100%'}, items:[{xtype:'bpcOrganisationSelect', itemId:'readOrganisations', grow:true, growToLongestValue:true, matchFieldWidth:false, bind:{disabled:'{personal}', value:'{stateObject.readOrganisations}'}}, {xtype:'bpcRoleSelect', itemId:'readRoles', grow:true, growToLongestValue:true, matchFieldWidth:false, bind:{disabled:'{personal}', value:'{stateObject.readRoles}'}, value:['bpcuser']}, {xtype:'bpcRightSelect', itemId:'readRights', 
grow:true, growToLongestValue:true, matchFieldWidth:false, bind:{disabled:'{personal}', value:'{stateObject.readRights}'}}]}]});
Ext.define('BPC.component.grid.DistinctDataMixin', {extend:'Ext.Mixin', getStoreConfig:function() {
  var me = this;
  var newModel = Ext.define(null, {extend:'BPC.component.grid.DistinctDataModel', column:me.column});
  return {proxy:{type:'ajax', url:BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + this.moduleId + '/distinctFieldValues/' + this.column.dataIndex, extraParams:{source:this.source, size:100}, reader:{type:'json', rootProperty:'values'}}, model:newModel, sorters:[{property:'value', transform:Ext.util.Format.uppercase}], moduleId:this.moduleId, listeners:{beforeload:function(store) {
    if (BpcCommon.Api.getSetting(store.moduleId, monitor.assets.ConfigKeys.filterRespectOtherColumns) === true) {
      store.getProxy().setExtraParam('filter', me.getEncodedFilters());
      store.getProxy().setExtraParam('view', me.grid.bpcGridViewName);
    }
  }}};
}, getEncodedFilters:function() {
  var me = this;
  var grid = me.grid;
  var filters = Ext.clone(grid.store.getFilters().items);
  var columnId = me.column.columnId;
  Ext.each(filters, function(filter, index) {
    var filterId = filter.columnId || filter.getProperty();
    if (filterId === columnId) {
      filters.splice(index, 1);
    }
  });
  var encodedFilters = grid.store.proxy.encodeFilters(filters);
  if (encodedFilters && !Ext.isArray(encodedFilters)) {
    JSON.stringify(encodedFilters);
  }
  return encodedFilters;
}, getBoundListConfig:function() {
  var me = this;
  return {localized:{loadingText:'MONITOR_LOADING_VALUES', emptyText:'MONITOR_NO_DATA_FOUND'}, scrollable:'y', itemId:'distinctDataTagfieldPicker', cls:'distinctDataTagfieldPicker', maxWidth:500, itemTpl:new Ext.XTemplate('\x3ctpl\x3e', '\x3cdiv style\x3d" display: flex; justify-content: space-between; align-items: center;" ' + 'data-qtip\x3d"{[this.returnLabel(values.value)]}"\x3e' + '\x3cdiv class\x3d"tagfieldPicker-checkbox"\x3e\x3c/div\x3e' + '{[this.returnRenderer(values)]}\x3c/div\x3e', '\x3c/tpl\x3e', 
  {returnRenderer:function(values) {
    var returnVal = values.value;
    if (!(me.column instanceof Ext.grid.column.Template) && me.column.renderer) {
      var renderer = me.column.renderer;
      returnVal = renderer(returnVal, false, false, false) || returnVal;
      if (returnVal && values.value !== returnVal && Ext.util.Format.htmlEncode2(values.value) !== returnVal && BpcCommon.util.JsonPointer.get(me, '/column/filter/hideRawValue') !== true) {
        returnVal = returnVal + ' (' + values.value + ')';
      }
    }
    if (values['count']) {
      returnVal = '\x3cspan class\x3d"tagfieldPicker-text" style\x3d"width: 100%; margin-right:15px"\x3e' + returnVal + '\x3c/span\x3e';
      var counter = '\x3cspan class\x3d"tagfieldPicker-count" style\x3d"min-width:16px text-align: right"\x3e' + values['count'] + '\x3c/span\x3e';
      returnVal = returnVal + counter;
    }
    return returnVal;
  }, returnLabel:function(value) {
    if (!(me.column instanceof Ext.grid.column.Template) && me.column.renderer) {
      var renderer = me.column.renderer;
      value = renderer(value, false, false, true) || value;
    }
    return Ext.util.Format.htmlEncode(value);
  }}), tooltipItemTpl:new Ext.XTemplate('\x3ctpl\x3e', '\x3cdiv style\x3d" display: flex; justify-content: space-between; align-items: center;" ' + 'data-qtip\x3d"{[this.returnLabel(values.value)]}"\x3e' + '{[this.returnRenderer(values)]}\x3c/div\x3e', '\x3c/tpl\x3e', {returnRenderer:function(values) {
    var returnVal = values.value;
    if (me.column.renderer) {
      var renderer = me.column.renderer;
      returnVal = renderer(returnVal, false, false, false) || returnVal;
      if (returnVal && values.value !== returnVal && Ext.util.Format.htmlEncode2(values.value) !== returnVal && BpcCommon.util.JsonPointer.get(me, '/column/filter/hideRawValue') !== true) {
        returnVal = returnVal + ' (' + values.value + ')';
      }
    }
    if (values['count']) {
      returnVal = '\x3cspan style\x3d"margin-right:15px"\x3e' + returnVal + '\x3c/span\x3e';
      var counter = '\x3cspan style\x3d"text-align: right"\x3e' + values['count'] + '\x3c/span\x3e';
      returnVal = returnVal + counter;
    }
    return returnVal;
  }, returnLabel:function(value) {
    if (me.column.renderer) {
      var renderer = me.column.renderer;
      value = renderer(value, false, false, true) || value;
    }
    return Ext.util.Format.htmlEncode(value);
  }})};
}, getTooltipListener:function() {
  return {afterrender:function() {
    this.tooltip = Ext.create('Ext.tip.ToolTip', {target:this.id, html:BpcCommon.Api.getTranslation('MONITOR_NO_FILTER_SET'), emptyValue:BpcCommon.Api.getTranslation('MONITOR_NO_FILTER_SET'), listeners:{beforeshow:function(tooltip) {
      var field = tooltip.target.component;
      var store = field.store;
      var valueArray = field.getValue();
      if (valueArray && !Ext.isArray(valueArray)) {
        valueArray = [valueArray];
      } else if (!valueArray) {
        valueArray = [];
      }
      var getDataFn = function(id) {
        var record = store.getAt(store.find('value', id));
        var data;
        if (record) {
          data = record.getData();
        } else {
          data = {value:id, count:''};
        }
        return data;
      };
      if (valueArray.length === 0) {
        tooltip.setHtml(tooltip.emptyValue);
      } else if (valueArray.length === 1) {
        tooltip.setHtml(field.listConfig.itemTpl.apply(getDataFn(valueArray[0])));
      } else {
        var newToolTip = '';
        Ext.Array.each(valueArray, function(id) {
          newToolTip = newToolTip + '\x3cli style\x3d"list-style: none; margin: 10px 4px;"\x3e' + field.listConfig.tooltipItemTpl.apply(getDataFn(id)) + '\x3c/li\x3e';
        });
        tooltip.setHtml('\x3cspan\x3e' + BpcCommon.Api.getTranslation('MONITOR_SET_COLUMN_FILTER_ACTIVE') + ':\x3c/span\x3e\x3cbr\x3e\x3cul style\x3d"padding: 0"\x3e' + newToolTip + '\x3c/ul\x3e');
      }
    }}});
  }};
}, onExpand:function() {
  var me = this;
  me.callParent(arguments);
  if (me.store.loadCount === 0) {
    me.store.load();
    me.setFilterChanged(false);
    return;
  }
  if (me.getFilterChanged()) {
    if (!me.store.isLoading() || me.store.loadCount > 0) {
      me.store.reload();
    }
    me.setFilterChanged(false);
  }
}});
Ext.define('BPC.component.grid.DistinctDataCombobox', {extend:'Ext.form.field.ComboBox', alias:'widget.bpcMonitoDistinctDataCombobox', mixins:{distinctDataMixin:'BPC.component.grid.DistinctDataMixin'}, column:undefined, grid:undefined, moduleId:undefined, enableKeyEvents:true, valueField:'value', displayField:'value', triggerAction:'all', forceSelection:false, minChars:0, matchFieldWidth:false, minWidth:64, autoSelect:false, autoSelectMatches:false, config:{filterChanged:false, source:'noSource'}, 
initComponent:function() {
  var me = this;
  this.store = this.getStoreConfig();
  this.listConfig = this.getBoundListConfig();
  Ext.override(this, {onExpand:me.mixins.distinctDataMixin.onExpand});
  this.listeners = Ext.apply(this.listeners || {}, this.getTooltipListener());
  this.callParent();
  if (BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.filterRespectOtherColumns) === true) {
    me.grid.getStore().on({filterchange:function() {
      me.setFilterChanged(true);
    }});
  }
}});
Ext.define('BPC.component.grid.DistinctDataModel', {extend:'Ext.data.Model', fields:['value', 'count', {name:'renderedValue', calculate:function(data) {
  if (this.owner.prototype.column && !(this.owner.prototype.column instanceof Ext.grid.column.Template) && this.owner.prototype.column.renderer) {
    return this.owner.prototype.column.renderer(data.value, false, false, true);
  }
  return data.value;
}}], idProperty:'value', rendererFn:undefined});
Ext.define('BPC.component.grid.DistinctDataTagPicker', {extend:'Ext.panel.Panel', alias:'widget.bpcDistinctDataTagPicker', requires:['Ext.util.KeyNav'], plugins:['bpcFocusKeeper'], pickerColumnRenderer:Ext.util.Format.htmlEncode2, labelValueRenderer:Ext.util.Format.htmlEncode2, layout:'fit', resizable:true, viewModel:{data:{displayHiddenRecordsInfo:false}}, initComponent:function() {
  var me = this;
  me.column = me.pickerField.column;
  me.moduleId = me.pickerField.moduleId;
  me.value = me.pickerField.value;
  me.queryMode = me.pickerField.queryMode;
  me.maxQuerySize = me.pickerField.maxQuerySize || 100;
  me.gridStore = me.getStoreConfig();
  if (me.column.customRenderer) {
    me.pickerColumnRenderer = me.pickerField.grid.up('bpcMonitor').findPlugin('monitorCustomRenderer').getColumnRendererByColumnConfig(me.column.config, true, true);
    me.labelValueRenderer = me.pickerField.grid.up('bpcMonitor').findPlugin('monitorCustomRenderer').getColumnRendererByColumnConfig(me.column.config, false, true);
  } else if (me.column.renderer) {
    me.pickerColumnRenderer = me.column.renderer;
    me.labelValueRenderer = me.column.renderer;
  }
  me.items = [{xtype:'grid', cls:'distinctDataPickerGrid', itemId:'distinctDataPickerGrid', selModel:{selType:'checkboxmodel', checkOnly:false, mode:'MULTI'}, store:me.gridStore, columns:[{localized:{text:'MONITOR_VALUE'}, dataIndex:'value', flex:1, minWidth:100, cellWrap:true, renderer:me.pickerColumnRenderer}, {localized:{text:'MONITOR_VALUE_RAW'}, dataIndex:'value', cellWrap:true, flex:1, minWidth:100, hidden:true}, {localized:{text:'MONITOR_VALUE_LABEL'}, dataIndex:'renderedValue', cellWrap:true, 
  flex:1, minWidth:100, hidden:true}, {localized:{text:'MONITOR_COUNT'}, dataIndex:'count', minWidth:32, width:60}]}];
  me.dockedItems = [{dock:'top', xtype:me.queryMode === 'remote' ? 'bpcRemoteGridFilterField' : 'bpcLocalGridFilterField', itemId:'distinctDataPickerGridFilterField', emptyText:'Filter Tags', flex:1, source:me.pickerField.source, grid:'^ panel grid', maxQuerySize:me.maxQuerySize}, {xtype:'toolbar', dock:'bottom', ui:'footer', overflowHandler:'scroller', items:[{xtype:'bpcToolbarMsg', iconCls:BpcCommon.Icons.WARNING, iconColorCls:'warningColor', text:{key:'MONITOR_MISSING_FILTER_ENTRIES', data:{amount:me.maxQuerySize}}, 
  hidden:true, bind:{hidden:'{!displayHiddenRecordsInfo}'}}, {xtype:'tbfill', flex:0.1}, {text:BpcCommon.Api.getTranslation('MONITOR_CANCEL'), iconCls:BpcCommon.Icons.CANCEL, itemId:'cancelBtn', ui:'secondary', handler:me.pickerField.closePicker.bind(me.pickerField)}, {text:BpcCommon.Api.getTranslation('MONITOR_APPLY'), iconCls:BpcCommon.Icons.OK, itemId:'applyBtn', ui:'primary', handler:me.pickerField.setValueFromPicker.bind(me.pickerField)}]}];
  me.resizable = {listeners:{resize:{fn:me.pickerField.expand.bind(me.pickerField), buffer:50}}};
  me.callParent();
  var monitorGrid = me.pickerField.up('monitorGrid');
  if (monitorGrid) {
    monitorGrid.on('load', me.reloadStoreWhenVisible, me);
  }
}, reloadStoreWhenVisible:function() {
  var me = this;
  if (me.isVisible(true)) {
    delete me.flagReloadOnShow;
    me.down('#distinctDataPickerGrid').getStore().reload();
  } else if (!me.flagReloadOnShow) {
    me.flagReloadOnShow = true;
    me.on('show', me.reloadStoreWhenVisible, me, {single:true});
  }
}, afterRender:function() {
  var me = this;
  me.setInitialPickerSelection();
  me.callParent();
}, setInitialPickerSelection:function() {
  var me = this;
  var currentValue = me.pickerField.value ? me.pickerField.value : '';
  if (currentValue.length) {
    var grid = me.down('#distinctDataPickerGrid');
    var gridSelectionModel = grid.getSelectionModel();
    var selection = [];
    if (grid.store.isLoaded()) {
      currentValue.forEach(function(val) {
        grid.store.data.items.forEach(function(item) {
          if (val === item.data.value) {
            selection.push(item);
          }
        });
      });
      gridSelectionModel.select(selection);
    } else {
      grid.store.on('load', me.setInitialPickerSelection, me, {single:true});
    }
  }
}, deselectPickerGridAll:function() {
  var me = this;
  var currentValue = me.pickerField.value ? me.pickerField.value : '';
  if (currentValue.length === 0) {
    var grid = me.down('#distinctDataPickerGrid');
    var gridSelectionModel = grid.getSelectionModel();
    gridSelectionModel.deselectAll();
  }
}, getValue:function() {
  var me = this;
  var currentGridSelection = me.down('#distinctDataPickerGrid').getSelection();
  var selectionValues = [];
  currentGridSelection.forEach(function(item) {
    selectionValues.push(item.data.value);
  });
  me.value = selectionValues;
  return selectionValues;
}, getStoreConfig:function() {
  var me = this;
  var newModel = Ext.define(null, {extend:'BPC.component.grid.DistinctDataModel', column:me.column, fields:[{name:'renderedValue', calculate:function(data) {
    if (me.labelValueRenderer) {
      return me.labelValueRenderer(data.value);
    }
    return data.value;
  }}]});
  return {proxy:{type:'ajax', url:BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + me.moduleId + '/distinctFieldValues/' + this.column.dataIndex, extraParams:{source:me.pickerField.source, size:me.maxQuerySize}, reader:{type:'json', rootProperty:'values'}}, model:newModel, sorters:[{transform:Ext.util.Format.uppercase, sorterFn:function(item1, item2) {
    var me = this;
    var transform = me._transform;
    var root = me._root;
    var property = 'value';
    var lhs, rhs;
    if (root) {
      item1 = item1[root];
      item2 = item2[root];
    }
    lhs = item1[property];
    rhs = item2[property];
    if (lhs === '!') {
      return -1;
    } else if (rhs === '!') {
      return 1;
    }
    if (transform) {
      lhs = transform(lhs);
      rhs = transform(rhs);
    }
    return lhs > rhs ? 1 : lhs < rhs ? -1 : 0;
  }}], moduleId:me.moduleId, listeners:{beforeload:function(store) {
    if (BpcCommon.Api.getSetting(store.moduleId, monitor.assets.ConfigKeys.filterRespectOtherColumns) === true) {
      store.getProxy().setExtraParam('filter', me.getEncodedFilters());
      store.getProxy().setExtraParam('view', me.pickerField.grid.bpcGridViewName);
      store.getProxy().setExtraParam('query', me.getGlobalFilterValue());
    }
  }, load:function() {
    me.setHiddenRecordsInfo();
  }.bind(me)}, autoLoad:true};
}, setHiddenRecordsInfo:function() {
  var me = this;
  var amountRecords = this.down('#distinctDataPickerGrid').getStore().getRange().length;
  var vm = me.getViewModel();
  var displayHiddenRecordsInfo = amountRecords >= me.maxQuerySize;
  vm.set('displayHiddenRecordsInfo', displayHiddenRecordsInfo);
}, getEncodedFilters:function() {
  var me = this;
  var grid = me.pickerField.grid;
  var filters = Ext.clone(grid.store.getFilters().items);
  var columnId = me.column.columnId;
  var remainingFilters = filters.filter(function(filter) {
    var filterId = filter.columnId || filter.getProperty();
    return filterId !== columnId;
  });
  var encodedFilters = grid.store.proxy.encodeFilters(remainingFilters);
  if (encodedFilters && !Ext.isArray(encodedFilters)) {
    JSON.stringify(encodedFilters);
  }
  return encodedFilters;
}, onShow:function() {
  var me = this;
  me.callParent(arguments);
  Ext.defer(function() {
    me.down('#distinctDataPickerGridFilterField').focus();
  }, 100);
}, getGlobalFilterValue:function() {
  var me = this;
  var failureValue = '';
  try {
    return me.up('bpcMonitor').getViewModel().get('globalFilterValue');
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR get global filter value', dump:e, stack:true, level:'error'});
    return failureValue;
  }
}});
Ext.define('BPC.component.grid.DistinctDataPickerField', {extend:'Ext.form.field.Picker', alias:'widget.bpcMonitorDistinctDataPickerField', column:undefined, grid:undefined, moduleId:undefined, enableKeyEvents:true, matchFieldWidth:false, oldDisplayValue:undefined, editable:false, queryMode:'local', viewModel:{data:{mouseOver:false}}, config:{filterChanged:false}, initComponent:function() {
  var me = this;
  me.listeners = Ext.apply({afterrender:function() {
    this.tooltip = Ext.create('Ext.tip.ToolTip', {target:this.id, html:BpcCommon.Api.getTranslation('MONITOR_NO_FILTER_SET'), emptyValue:BpcCommon.Api.getTranslation('MONITOR_NO_FILTER_SET'), listeners:{beforeshow:function(tooltip) {
      var valueArray = me.getDisplayValues(me.value, false);
      if (me.value.length === 0) {
        tooltip.setHtml(tooltip.emptyValue);
      } else if (valueArray.length === 1) {
        tooltip.setHtml(valueArray[0]);
      } else {
        var newToolTip = '';
        Ext.Array.each(valueArray, function(item) {
          newToolTip = newToolTip + ' ' + item;
        });
        tooltip.setHtml('\x3cspan\x3e' + BpcCommon.Api.getTranslation('MONITOR_SET_COLUMN_FILTER_ACTIVE') + ':\x3c/span\x3e\x3cbr\x3e\n                                        \x3cul style\x3d"padding: 0"\x3e' + newToolTip + '\x3c/ul\x3e');
      }
    }}});
  }}, me.listeners);
  me.callParent();
  if (BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.filterRespectOtherColumns) === true) {
    me.grid.getStore().on({filterchange:function() {
      me.setFilterChanged(true);
    }});
  }
  me.on('select', me.updateLayout.bind(me));
}, initEvents:function() {
  var me = this;
  this.callParent();
  me.listWrapper.on({scope:me, click:me.onItemListClick});
}, fieldSubTpl:['\x3cdiv id\x3d"{cmpId}-listWrapper" data-ref\x3d"listWrapper" ' + (Ext.isGecko ? ' tabindex\x3d"-1"' : '') + ' class\x3d"' + Ext.baseCSSPrefix + 'tagfield {fieldCls} {typeCls} {typeCls}-{ui}"\x3e', '\x3cul id\x3d"{cmpId}-itemList" data-ref\x3d"itemList" role\x3d"presentation" class\x3d"' + Ext.baseCSSPrefix + 'tagfield-list{itemListCls}" style\x3d"overflow: hidden"\x3e', '\x3cli id\x3d"{cmpId}-inputElCt" data-ref\x3d"inputElCt" role\x3d"presentation" class\x3d"' + Ext.baseCSSPrefix + 
'tagfield-input"\x3e', '\x3cspan id\x3d"{cmpId}-inputElSelection" data-ref\x3d"inputElSelection" class\x3d"' + Ext.baseCSSPrefix + 'tagfield-selection" style\x3d"white-space: nowrap; position: absolute"\x3e\x3c/span\x3e', '\x3cinput id\x3d"{cmpId}-inputEl" data-ref\x3d"inputEl" type\x3d"{type}" class\x3d"' + Ext.baseCSSPrefix + 'tagfield-input-field {inputElCls} {emptyCls} {fixCls}"\x3e', '\x3c/li\x3e', '\x3c/ul\x3e', '\x3c/div\x3e', {disableFormats:true}], postSubTpl:['\x3clabel id\x3d"{cmpId}-placeholderLabel" data-ref\x3d"placeholderLabel" for\x3d"{cmpId}-inputEl" class\x3d"{placeholderCoverCls} {placeholderCoverCls}-{ui} {emptyCls}"\x3e{emptyText}\x3c/label\x3e', 
'\x3c/div\x3e', '\x3ctpl for\x3d"triggers"\x3e{[values.renderTrigger(parent)]}\x3c/tpl\x3e', '\x3c/div\x3e'], childEls:['listWrapper', 'itemList', 'inputEl', 'inputElCt', 'inputElSelection'], createPicker:function() {
  var me = this;
  me.filterPickList = false;
  var pickerCfg = Ext.create({xtype:'bpcDistinctDataTagPicker', floating:true, width:550, height:300, bodyPadding:8, hidden:true, displayField:me.displayField, preserveScrollOnRefresh:true, pickerField:me, keyMap:{ENTER:me.setValueFromPicker.bind(me)}, keyMapEnabled:true});
  return me.picker = Ext.widget(pickerCfg);
}, afterRender:function() {
  var me = this;
  me.callParent();
  if (me.value && me.value.length) {
    var displayValues = me.displayValues = me.getDisplayValues(me.value, true);
    me.setDisplayValue(displayValues);
  }
}, getDisplayValues:function(values, isInputValues) {
  var me = this;
  var returnVal;
  try {
    var valueArray = Ext.isArray(values) ? values : [values];
    if (me.column.renderer) {
      if (!me.tooltipRenderer) {
        var customRenderer = me.grid.up('bpcMonitor').findPlugin('monitorCustomRenderer').getColumnRendererByColumnConfig(me.column.config, true, true);
        me.tooltipRenderer = customRenderer || Ext.util.Format.htmlEncode2;
      }
      returnVal = Ext.Array.map(valueArray, function(item) {
        var fakeData = {};
        fakeData[me.column.dataIndex || 'unknown'] = item;
        var fakeRecord = Ext.data.Model.create(fakeData);
        var itemData = {value:me.column.renderer.call(me.column, item, {}, fakeRecord, false, 0, 0) || item, tooltip:me.tooltipRenderer(item, {}, fakeRecord, false, 0, 0) || item};
        if (isInputValues) {
          return '\x3cspan\x3e' + itemData.value + '\x3c/span\x3e';
        }
        return '\x3cli style\x3d"list-style: none; margin: 10px 4px;"\x3e\x3cspan\x3e' + itemData.tooltip + '\x3c/span\x3e\x3c/li\x3e';
      });
    } else {
      if (isInputValues) {
        returnVal = Ext.Array.map(valueArray, function(item) {
          return '\x3cspan\x3e' + item + '\x3c/span\x3e';
        });
      } else {
        returnVal = Ext.Array.map(valueArray, function(item) {
          return '\x3cli style\x3d"list-style: none; margin: 10px 4px;"\x3e\x3cspan\x3e' + item + '\x3c/span\x3e';
        });
      }
    }
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR getDisplayValues', dump:e, stack:true, level:'error'});
  }
  return returnVal;
}, setDisplayValue:function(values) {
  var me = this;
  var displayValues = values;
  if (!me.rendered) {
    me.on('afterrender', me.setDisplayValue.bind(me, values), me, {single:true});
    return;
  }
  if (Ext.isArray(displayValues)) {
    displayValues = Ext.Array.map(values, function(item) {
      if (item instanceof Ext.data.Model) {
        return Ext.String.htmlEncode(item);
      }
      return item;
    });
  }
  var newDisplayValue = displayValues.toString();
  me.inputElSelection.setHtml(newDisplayValue);
  me.inputEl.dom.value = '';
  me.oldDisplayValue = newDisplayValue;
}, getValue:function() {
  var me = this;
  var value = me.value;
  if (Ext.isArray(value)) {
    return Ext.Array.map(value, function(item) {
      if (item instanceof Ext.data.Model) {
        return item.get(me.valueField);
      }
      return item;
    });
  }
  return value;
}, setValueFromPicker:function() {
  var me = this;
  me.setValue(me.picker.getValue());
  me.closePicker();
  me.fireEvent('select', me);
}, setValue:function(value) {
  var me = this;
  value = this.callParent([value]);
  me.refreshEmptyText();
  if (!me.value || me.value.length === 0) {
    if (me.picker) {
      me.picker.deselectPickerGridAll();
    }
    me.setDisplayValue('');
  } else {
    me.displayValues = me.getDisplayValues(me.value, true);
    me.setDisplayValue(me.displayValues);
  }
  return value;
}, onFocusEnter:function() {
  var me = this;
  me.callParent(arguments);
  if (!me.readOnly && !me.disabled) {
    me.delayedExpandTask = new Ext.util.DelayedTask(function() {
      me.expand();
    });
    me.delayedExpandTask.delay(100);
  }
}, collapse:function() {
  var me = this;
  if (me.delayedExpandTask) {
    me.delayedExpandTask.cancel();
    delete me.delayedExpandTask;
  }
  me.callParent();
}, closePicker:function() {
  var me = this;
  me.collapse();
}, onFocus:function(e) {
  var me = this;
  var focusCls = me.focusCls;
  var itemList = me.itemList;
  if (focusCls && itemList && !itemList.hasCls('form-focus')) {
    itemList.addCls(focusCls);
  }
  me.callParent([e]);
}, onBlur:function() {
  var me = this;
  var focusCls = me.focusCls;
  var itemList = me.itemList;
  if (focusCls && itemList) {
    itemList.removeCls(focusCls);
  }
  me.callParent(arguments);
}, onItemListClick:function(e) {
  var me = this;
  if (me.readOnly || me.disabled) {
    return;
  }
  e.stopPropagation();
  me.inputEl.focus();
}, onItemMouseDown:function(e) {
  if (e.target !== this.inputEl.dom) {
    e.preventDefault();
  }
}});
Ext.define('BPC.component.grid.LocalGridFilterField', {extend:'BpcCommon.component.grid.GridFilterField', alias:'widget.bpcLocalGridFilterField', filterBuffer:250, selectedGridRecords:[], onFilterChange:function(textField, newVal) {
  var me = this;
  var store = me.grid.getStore();
  var filters = store.getFilters();
  me.rememberSelection();
  filters.removeAll();
  if (newVal && newVal.length > 0) {
    store.filterBy(function(item) {
      item.isSelected = false;
      var match = false;
      var matcher = Ext.String.createRegex(newVal, false, false, true);
      if (me.selectedGridRecords.length) {
        me.selectedGridRecords.forEach(function(selectionItem) {
          if (selectionItem.data.value === item.data.value) {
            item.isSelected = true;
          }
        });
      }
      if (me.filterFields.length === 0) {
        me.filterFields = Ext.Object.getKeys(store.getModel().getFieldsMap());
      }
      Ext.Array.each(me.filterFields, function(key) {
        var value = item.get(key);
        if (Ext.isObject(value) || Ext.isArray(value)) {
          value = JSON.stringify(value);
        }
        match = matcher.test(value);
        if (match === true) {
          return false;
        }
      });
      return match || item.isSelected;
    });
  }
}, rememberSelection:function() {
  var me = this;
  me.selectedGridRecords = me.grid.getSelectionModel().getSelection();
}});
Ext.define('BPC.component.grid.RemoteGridFilterField', {extend:'BpcCommon.component.grid.GridFilterField', alias:'widget.bpcRemoteGridFilterField', filterBuffer:500, selectedGridRecords:[], onFilterChange:function(textField, newVal) {
  var me = this;
  var querySize = me.maxQuerySize || 100;
  var store = me.grid.getStore();
  store.load({params:{query:newVal, size:querySize, source:me.source}});
  store.on('beforeload', this.rememberSelection, this);
  me.grid.getView().on('refresh', this.setCurrentSelection, me);
}, rememberSelection:function() {
  var me = this;
  me.selectedGridRecords = me.grid.getSelectionModel().getSelection();
}, setCurrentSelection:function(view) {
  var me = this;
  if (me.selectedGridRecords.length) {
    var gridSelectionModel = view.grid.getSelectionModel();
    var grid = view.grid;
    var itemsToSelect = [];
    var itemsToAdd = [];
    var selection = me.selectedGridRecords;
    selection.forEach(function(selectionItem) {
      if (!grid.store.getById(selectionItem.id)) {
        itemsToAdd.push(selectionItem);
        itemsToSelect.push(selectionItem);
      }
      grid.store.data.items.forEach(function(item) {
        if (selectionItem.data.value === item.data.value) {
          itemsToSelect.push(item);
        }
      });
    });
    if (itemsToAdd.length) {
      grid.store.insert(0, itemsToAdd);
    }
    gridSelectionModel.select(itemsToSelect);
  }
}});
Ext.define('monitor.component.grid.filter.Base', {extend:'Ext.grid.filters.filter.Base', mixin:['Ext.mixin.Bindable'], viewModel:{data:{dirty:false, source:undefined, mouseOver:false, operator:undefined, invert:false}}, emptyValue:'', viewModelDefaults:{}, columnId:undefined, dirty:false, menuDefaults:{plain:true}, menuOptionFieldDefaults:[], columHeaderFieldDefault:{xtype:'bpcMonitoDistinctDataCombobox', setTriggerPickerHidden:function(hidden) {
  this.getTrigger('picker').setVisible(!hidden);
}, getTriggerPickerHidden:function() {
  return !this.getTrigger('picker').isVisible();
}, setTriggerResetHidden:function(hidden) {
  this.getTrigger('reset').setVisible(!hidden);
}, getTriggerResetHidden:function() {
  return !this.getTrigger('reset').isVisible();
}, bind:{triggerPickerHidden:'{!dirty}', triggerResetHidden:'{!mouseOver}'}, triggers:{reset:{cls:BpcCommon.Icons.FILTER_RESET, extraCls:'resetTrigger', weight:-2, scope:'this', handler:'deactivate', width:'24px'}, picker:{cls:BpcCommon.Icons.OK, weight:-1, scope:'this', itemId:'submitColumHeader', handler:'submitColumHeaderField', width:'24px'}}, margin:0}, constructor:function(config) {
  var me = this;
  me.viewModel.data = Ext.apply(me.viewModel.data, me.viewModelDefaults);
  me.viewModel = Ext.Factory.viewModel(me.viewModel);
  me.source = config.source || me.source;
  me.viewModel.set('source', me.source);
  me.dataIndex = config.column.dataIndex;
  me.callParent(arguments);
  me.moduleId = me.grid.config.moduleId;
  me.column.addListener({mouseenter:{fn:function() {
    me.viewModel.set('mouseOver', true);
    me.column.onTitleMouseOver();
  }, element:'el'}, mouseleave:{fn:me.viewModel.set.bind(me.viewModel, 'mouseOver', false), element:'el'}});
  me.columHeaderFieldConfig = Ext.apply({viewModel:me.viewModel, submitColumHeaderField:me.submitColumHeaderField.bind(me), deactivate:me.deactivate.bind(me), column:me.column, moduleId:me.moduleId, grid:me.grid, source:me.source, operator:me.operator, ui:'columnHeaderField', listeners:{keyup:me.onValueChange.bind(me), change:me.checkDirty.bind(me), select:me.submitColumHeaderField.bind(me)}}, me.columHeaderFieldDefault, me.column.filter.fieldConfig);
  if (me.active) {
    me.columHeaderFieldConfig.value = me.getValue();
  }
  me.column.isContainer = true;
  me.focusable = true;
  me.column.setLayout({type:'vbox', align:'stretch'});
  me.columHeaderField = me.column.add(me.columHeaderFieldConfig);
  me.filterChangeListener = function() {
    me.onFilterChange();
  };
  me.getGridStore().on('filterchange', me.filterChangeListener);
}, getFilterConfig:function(config, key) {
  config = this.callParent(arguments);
  if (!config.columnId) {
    config.columnId = this.column.columnId;
  }
  if (!config.key) {
    config.key = key;
  }
  return config;
}, getValue:function(field) {
  return field ? field.getValue() : undefined;
}, setValue:function(value, silent) {
  silent = silent === true ? silent : false;
  this.value = value;
  if (this.columHeaderField.getValue() !== value) {
    this.columHeaderField.setValue(value);
  }
  if (this.inputItem && this.inputItem.getValue() !== value) {
    this.inputItem.setValue(value);
  }
  if (silent === false) {
    if (this.columHeaderField && this.columHeaderField.collapse) {
      this.columHeaderField.collapse();
    }
  }
}, clearValue:function() {
  this.setValue(this.emptyValue);
}, getStoreFilter:function(key) {
  var filters = this.getGridStore().getFilters();
  var filter;
  if (!key) {
    var filterIndex = filters.findIndex('columnId', this.column.columnId);
    if (filterIndex >= 0) {
      return filters.getAt(filterIndex);
    }
  } else {
    filter = filters.findBy(function(item) {
      if (item.config) {
        return item.config.columnId === this.column.columnId && item.config.key === key;
      }
    }, this);
    if (filter) {
      return filter;
    }
  }
  var id = this.getBaseIdPrefix();
  if (key) {
    id += '-' + key;
  }
  filter = filters.get(id);
  if (filter && filter.columnId) {
    filter = undefined;
  }
  if (!filter) {
    filter = filters.get(this.dataIndex);
  }
  return filter;
}, onFilterChange:function() {
  var me = this;
  if (me.destroyed) {
    return;
  }
  var storeFilter = me.getStoreFilter();
  if (!storeFilter) {
    me.setColumnActive(false);
    me.setValue(me.emptyValue, true);
    me.active = false;
  } else {
    me.filter = storeFilter;
    me.setColumnActive(true);
    me.setValue(storeFilter.getValue(), true);
    me.active = true;
  }
  this.checkDirty();
}, getMenuConfig:function() {
  return Ext.apply({viewModel:this.viewModel}, this.getMenuDefaults());
}, createMenu:function() {
  var me = this;
  me.callParent();
  me.menu.add(me.menuOptionFieldDefaults);
  me.itemDefaults = me.columHeaderFieldConfig;
  var config = Ext.apply({}, me.getItemDefaults());
  if (config.iconCls && !('labelClsExtra' in config)) {
    config.labelClsExtra = Ext.baseCSSPrefix + 'grid-filters-icon ' + config.iconCls;
  }
  delete config.iconCls;
  config.emptyText = config.emptyText || me.emptyText;
  me.inputItem = me.menu.add(config);
  me.inputItem.on({scope:me, keyup:me.onValueChange, el:{click:function(e) {
    e.stopPropagation();
  }}});
}, setColumnActive:function(active) {
  this.column[active ? 'addCls' : 'removeCls'](this.owner.filterCls);
}, onValueChange:function(field, e) {
  var me = this;
  var keyCode = e.getKey();
  var value;
  if (keyCode === e.TAB) {
    return;
  }
  if (field.isValid()) {
    if (keyCode === e.RETURN) {
      value = me.getValue(field);
      if (value === me.value) {
        return;
      }
      me.setValue(value);
    } else {
    }
  }
}, submitColumHeaderField:function(source) {
  var me = this;
  var sourceField = me.columHeaderField;
  if (source && source.getValue) {
    sourceField = source;
  }
  me.setValue(sourceField.getValue());
  sourceField.blur();
}, checkDirty:function() {
  var oldDirtyState = this.viewModel.get('dirty');
  var newDirtyState = oldDirtyState;
  if (this.isDirty() === true) {
    newDirtyState = true;
    this.viewModel.set('dirty', newDirtyState);
  } else {
    newDirtyState = false;
    this.viewModel.set('dirty', newDirtyState);
  }
  return newDirtyState;
}, isDirty:Ext.emptyFn, activateMenu:function() {
  this.inputItem.setValue(this.filter.getValue());
}, isValueFilterable:function(value) {
  return !!value;
}, destroy:function() {
  var me = this;
  var store = me.getGridStore();
  if (store && me.filterChangeListener) {
    store.un('filterchange', me.filterChangeListener);
    me.filterChangeListener = null;
  }
  me.callParent();
}});
Ext.define('monitor.component.grid.filter.MultiFilter', {extend:'monitor.component.grid.filter.Base', activeFilterArray:undefined, defaultFilterMap:undefined, inactiveFilterArray:undefined, filterMap:undefined, constructor:function(config) {
  var me = this;
  if (!me.defaultFilterMap) {
    me.defaultFilterMap = {};
  } else {
    me.defaultFilterMap = Ext.clone(me.defaultFilterMap);
  }
  me.filterMap = {};
  me.callParent([config]);
  var filters = me.getStoreFilters();
  if (filters.length > 0) {
    me.active = true;
    Ext.Array.each(filters, me.addStoreFilter.bind(me));
  }
  if (me.active) {
    me.setColumnActive(true);
  }
  me.activeFilterArray = filters;
}, getStoreFilters:function() {
  var filters = [];
  var me = this;
  me.inactiveFilterArray = [];
  Ext.Object.each(me.defaultFilterMap, function(key, value) {
    var filter = me.getStoreFilter(key);
    if (filter) {
      filters.push(filter);
    } else {
      filter = me.createFilter(value, key);
      me.inactiveFilterArray.push(filter);
    }
    me.filterMap[key] = filter;
  });
  return filters;
}, activate:function(showingMenu) {
  var me = this;
  if (showingMenu) {
    this.activateMenu();
  } else {
    Ext.Array.each(me.activeFilterArray, me.addStoreFilter.bind(me));
  }
}, deactivate:function() {
  var me = this;
  me.clearValue();
  Ext.Array.each(me.activeFilterArray, me.removeStoreFilter.bind(me));
}, onFilterRemove:function() {
  if (!this.menu || this.active) {
    this.active = false;
  }
}, activateMenu:function() {
  this.inputItem.setValue(this.getFilterValueMap());
}, getFilterValueMap:function() {
  var valueMap = {};
  Ext.Object.each(this.filterMap, function(key, filter) {
    valueMap[key] = filter.getValue();
  });
  return valueMap;
}});
Ext.define('monitor.component.grid.filter.Date', {extend:'monitor.component.grid.filter.MultiFilter', alias:['grid.filter.daterange'], columHeaderFieldDefault:{xtype:'bpcDateRangeField', matchFieldWidth:false, setTriggerPickerHidden:function(hidden) {
  this.getTrigger('picker').setVisible(!hidden);
}, getTriggerPickerHidden:function() {
  return !this.getTrigger('picker').isVisible();
}, setTriggerResetHidden:function(hidden) {
  this.getTrigger('reset').setVisible(!hidden);
}, getTriggerResetHidden:function() {
  return !this.getTrigger('reset').isVisible();
}, bind:{triggerPickerHidden:'{!dirty}', triggerResetHidden:'{!mouseOver}'}, triggers:{reset:{cls:BpcCommon.Icons.FILTER_RESET, extraCls:'resetTrigger', scope:'this', handler:function() {
  this.setValue({});
  this.submitColumHeaderField();
}}, picker:{cls:BpcCommon.Icons.OK, scope:'this', handler:'submitColumHeaderField'}}, listeners:{scope:'this', change:'submitColumHeaderField'}, margin:0}, defaultFilterMap:{start:{operator:'gte', invert:false}, end:{operator:'lt', invert:false}}, menuOptionFieldDefaults:[], constructor:function() {
  var me = this;
  me.callParent(arguments);
  me.startFilter = me.filterMap.start;
  me.endFilter = me.filterMap.end;
}, activate:function(showingMenu) {
  if (showingMenu) {
    this.activateMenu();
  } else {
    if (this.startFilter.getValue()) {
      this.addStoreFilter(this.startFilter);
    }
    if (this.endFilter.getValue()) {
      this.addStoreFilter(this.endFilter);
    }
  }
}, setValue:function(value, silent) {
  silent = silent === true ? silent : false;
  var me = this;
  if (me.preventFilterRemoval) {
    return;
  }
  me.preventFilterRemoval = true;
  this.value = value;
  if (this.columHeaderField.getValue() !== value) {
    this.columHeaderField.setValue(value);
  }
  if (this.inputItem && this.inputItem.getValue() !== value) {
    this.inputItem.setValue(value);
  }
  if (this.startFilter && this.startFilter.getValue() !== value.start) {
    this.startFilter.setValue(value.start);
  }
  if (this.endFilter && this.endFilter.getValue() !== value.end) {
    this.endFilter.setValue(value.end);
  }
  if (silent === false) {
    if (this.columHeaderField) {
      this.columHeaderField.collapse();
    }
    if ((value.start || value.end) && me.active) {
      var filterCollection = me.getGridStore().getFilters();
      filterCollection.beginUpdate();
      if (this.startFilter.getValue()) {
        this.addStoreFilter(this.startFilter);
      } else {
        this.removeStoreFilter(this.startFilter);
      }
      if (this.endFilter.getValue()) {
        this.addStoreFilter(this.endFilter);
      } else {
        this.removeStoreFilter(this.endFilter);
      }
      me.setActive(true);
      filterCollection.endUpdate();
    } else {
      this.removeStoreFilter(this.startFilter);
      this.removeStoreFilter(this.endFilter);
      me.setActive(!!(value.start || value.end));
    }
  }
  me.preventFilterRemoval = false;
}, onFilterChange:function() {
  var me = this;
  var startFilter = me.getStoreFilter('start');
  var endFilter = me.getStoreFilter('end');
  if (!startFilter && !endFilter) {
    me.setColumnActive(false);
    me.setValue({start:null, end:null}, true);
    me.active = false;
  } else {
    me.setColumnActive(true);
    me.startFilter = startFilter || me.startFilter;
    me.endFilter = endFilter || me.endFilter;
    me.setValue({start:startFilter ? startFilter.getValue() : null, end:endFilter ? endFilter.getValue() : null}, true);
    me.active = true;
  }
  this.checkDirty();
}});
Ext.define('monitor.component.grid.filter.Dummy', {extend:'Ext.grid.filters.filter.Base', alias:['grid.filter.dummy'], type:'dummy', activate:function() {
}, deactivate:function() {
}, getValue:function() {
  return 'dummy';
}, onFilterRemove:function() {
}});
Ext.define('monitor.component.grid.filter.SingleFilter', {extend:'monitor.component.grid.filter.Base', filter:undefined, constructor:function(config) {
  var me = this;
  var filter;
  me.callParent([config]);
  var value = me.value;
  filter = me.getStoreFilter();
  if (filter) {
    me.active = true;
  } else {
    me.active = me.getActiveState(config, value);
    filter = me.createFilter({value:value});
    if (me.active) {
      me.addStoreFilter(filter);
    }
  }
  if (me.active) {
    me.setColumnActive(true);
  }
  me.filter = filter;
  me.viewModel.bind('{operator}', me.filter.setOperator.bind(me.filter));
  me.viewModel.bind('{invert}', me.setInvert.bind(me));
}, setInvert:function(invertState) {
  var me = this;
  me.filter.setConfig('invert', invertState);
  me.onFilterChange();
}, isDirty:function() {
  var dirty = false;
  var fieldValue = this.columHeaderField.getValue() || '';
  var filterValue = this.filter.getValue() || '';
  if (Ext.isArray(filterValue)) {
    filterValue = filterValue.join(',');
  }
  if (this.viewModel.get('invert') !== this.filter.getConfig('invert')) {
    dirty = true;
  }
  if (fieldValue !== filterValue) {
    dirty = true;
  }
  return dirty;
}, activate:function(showingMenu) {
  if (showingMenu) {
    this.activateMenu();
  } else {
    this.addStoreFilter(this.filter);
  }
}, deactivate:function() {
  var me = this;
  me.clearValue();
  this.removeStoreFilter(this.filter);
}, onFilterRemove:function() {
  this.setInvert(false);
  if (!this.menu || this.active) {
    this.active = false;
  }
}});
Ext.define('monitor.component.grid.filter.Text', {extend:'monitor.component.grid.filter.SingleFilter', alias:['grid.filter.text'], operator:'\x3d*', invert:false, menuOptionFieldDefaults:[{xtype:'checkbox', localized:{fieldLabel:'MONITOR_FILTER_INVERT'}, itemId:'invertFilter', bind:'{invert}'}, {xtype:'combobox', fieldLabel:'Operator', itemId:'operator', bind:'{operator}', valueField:'id', store:{fields:['id', 'text'], data:[{id:'\x3d*', text:'enthält'}, {id:'\x3d', text:'gleich'}]}}], constructor:function() {
  var me = this;
  me.callParent(arguments);
  me.viewModel.set('operator', me.operator);
  me.viewModel.set('invert', me.invert);
  me.oldOperator = me.operator;
  me.oldInvert = me.invert;
  me.viewModel.set('source', me.source);
  me.viewModel.bind('{operator}', Ext.Function.createThrottled(me.updateFilterConfig, 100, me));
  me.viewModel.bind('{source}', Ext.Function.createThrottled(me.updateFilterConfig, 100, me));
  me.viewModel.bind('{invert}', Ext.Function.createThrottled(me.updateFilterConfig, 100, me));
}, updateFilterConfig:function() {
  var me = this;
  me.filter.config.source = me.viewModel.get('source') || me.source;
  me.filter.config.operator = me.viewModel.get('operator') || me.operator;
  me.filter.config.invert = me.viewModel.get('invert') || me.invert;
  me.filter.setConfig('source', me.filter.config.source);
  me.filter.setConfig('operator', me.filter.config.operator);
  me.filter.setConfig('invert', me.filter.config.invert);
  me.setValue(me.value);
}, getFilterConfig:function(config) {
  config = this.callParent(arguments);
  config.source = this.viewModel.get('source') || this.source;
  config.operator = this.viewModel.get('operator') || this.operator;
  config.invert = this.viewModel.get('invert') || this.invert;
  return config;
}, getValue:function(field) {
  return field ? field.getValue() : this.filter.getValue();
}, setValue:function(value, silent) {
  var me = this;
  silent = silent === true ? silent : false;
  me.value = value;
  me.callParent(arguments);
  if (me.filter && me.filter.getValue() !== value) {
    me.filter.setValue(value);
  } else if (me.filter && me.filter.getValue() === value && me.active === true && me.oldOperator === me.viewModel.get('operator') && me.oldInvert === me.viewModel.get('invert')) {
    silent = true;
  }
  if (silent === false) {
    if (Ext.isArray(value) && value.length === 0) {
      value = '';
    }
    if (me.isValueFilterable(value) && me.active) {
      me.oldOperator = me.viewModel.get('operator');
      me.oldInvert = me.viewModel.get('invert');
      me.updateStoreFilter();
    } else {
      me.setActive(me.isValueFilterable(value));
    }
  }
}, onFilterChange:function() {
  var me = this;
  var storeFilter = me.getStoreFilter();
  if (storeFilter) {
    me.viewModel.set('operator', storeFilter.getOperator());
    me.viewModel.set('invert', storeFilter.getConfig('invert'));
  }
  me.callParent();
}});
Ext.define('monitor.component.grid.filter.MultiText', {extend:'monitor.component.grid.filter.Text', alias:['grid.filter.dynamiclist'], operator:'\x3d', emptyValue:[], columHeaderFieldDefault:{xtype:'bpcMonitorDistinctDataPickerField', itemId:'bpcMonitorDistinctDataPickerField', setTriggerPickerHidden:function() {
  this.getTrigger('picker').hide();
}, setTriggerResetHidden:function(hidden) {
  this.getTrigger('reset').setVisible(!hidden);
}, getTriggerResetHidden:function() {
  return !this.getTrigger('reset').isVisible();
}, bind:{triggerResetHidden:'{!mouseOver}'}, triggers:{reset:{cls:BpcCommon.Icons.FILTER_RESET, weight:-2, scope:'this', extraCls:'resetTrigger', handler:'deactivate', width:'24px'}, picker:{hidden:true, width:'0px'}}, margin:0}, isDirty:function() {
  var newDirtyState = false;
  var fieldValue = this.columHeaderField.getValue();
  if (Ext.isArray(fieldValue) && fieldValue.length === 0) {
    fieldValue = '';
  }
  var filterValue = this.filter.getValue();
  if (Ext.isArray(filterValue) && filterValue.length === 0) {
    filterValue = '';
  }
  newDirtyState = fieldValue !== filterValue;
  return newDirtyState;
}, getFilterConfig:function(config) {
  config = this.callParent(arguments);
  if (!config.value) {
    config.value = [];
  }
  return config;
}});
Ext.define('monitor.component.grid.filter.check.Checkbox', {extend:'Ext.form.field.Text', alias:'widget.monitorFilterCheckbox', setTriggerResetHidden:function(hidden) {
  this.getTrigger('reset').setVisible(!hidden);
}, afterRender:function() {
  this.callParent(arguments);
  var resetTrigger = this.el.down('#' + this.getId() + '-trigger-reset');
  if (resetTrigger) {
    resetTrigger.setStyle({position:'absolute', right:'0px'});
  }
  Ext.create('Ext.tip.ToolTip', {target:resetTrigger, html:BpcCommon.Api.getTranslation('MONITOR_RESET_TOOLTIP'), dismissDelay:0, anchor:'top', anchorToTarget:true});
}, getTriggerResetHidden:function() {
  return !this.getTrigger('reset').isVisible();
}, bind:{triggerResetHidden:'{!mouseOver}'}, triggers:{reset:{cls:BpcCommon.Icons.FILTER_RESET, extraCls:'resetTrigger', weight:-2, scope:'this', handler:'deactivate'}}, preSubTpl:['\x3cdiv id\x3d"{cmpId}-triggerWrap" data-ref\x3d"triggerWrap"', '\x3ctpl if\x3d"ariaEl \x3d\x3d \'triggerWrap\'"\x3e', '\x3ctpl foreach\x3d"ariaElAttributes"\x3e {$}\x3d"{.}"\x3c/tpl\x3e', '\x3ctpl else\x3e', ' role\x3d"presentation"', '\x3c/tpl\x3e', ' class\x3d"{triggerWrapCls} {triggerWrapCls}-{ui}"\x3e', '\x3cdiv style\x3d"text-align: center;" id\x3d{cmpId}-inputWrap data-ref\x3d"inputWrap"', 
' role\x3d"presentation" class\x3d"{inputWrapCls} {inputWrapCls}-{ui}"\x3e'], fieldSubTpl:['\x3cspan style\x3d"display: inline-block; position: relative;" id\x3d"{cmpId}-displayEl" data-ref\x3d"displayEl" role\x3d"presentation" class\x3d" x-form-cb-checked x-form-field x-form-checkbox x-form-checkbox-default x-form-cb x-form-cb-default', '"\x3e', '\x3cinput id\x3d"{id}" data-ref\x3d"inputEl" type\x3d"checkbox" value\x3d"{value}" {inputAttrTpl}', ' size\x3d"1"', '\x3ctpl if\x3d"name"\x3e name\x3d"{name}"\x3c/tpl\x3e', 
'\x3ctpl if\x3d"placeholder"\x3e placeholder\x3d"{placeholder}"\x3c/tpl\x3e', '{%if (values.maxLength !\x3d\x3d undefined){%} maxlength\x3d"{maxLength}"{%}%}', '\x3ctpl if\x3d"readOnly"\x3e readonly\x3d"readonly"\x3c/tpl\x3e', '\x3ctpl if\x3d"disabled"\x3e disabled\x3d"disabled"\x3c/tpl\x3e', '\x3ctpl if\x3d"tabIdx !\x3d null"\x3e tabindex\x3d"{tabIdx}"\x3c/tpl\x3e', '\x3ctpl if\x3d"fieldStyle"\x3e style\x3d"{fieldStyle}"\x3c/tpl\x3e', '\x3ctpl if\x3d"ariaEl \x3d\x3d \'inputEl\'"\x3e', '\x3ctpl foreach\x3d"ariaElAttributes"\x3e {$}\x3d"{.}"\x3c/tpl\x3e', 
'\x3c/tpl\x3e', '\x3ctpl foreach\x3d"inputElAriaAttributes"\x3e {$}\x3d"{.}"\x3c/tpl\x3e', ' class\x3d"x-form-cb-input" autocomplete\x3d"off"/\x3e', '\x3c/span\x3e', {disableFormats:true}], initEvents:function() {
  this.callParent();
  this.inputEl.on('mousedown', this.toggleValue, this);
}, toggleValue:function() {
  this.setValue(!this.getValue());
}, getValue:function() {
  var value = this.callParent();
  if (value === 'true' || value === 'on') {
    value = true;
  }
  return value === true;
}, setValue:function(value) {
  var newValue = false;
  if (value === 'true' || value === 'on' || value === true) {
    newValue = true;
  }
  if (newValue) {
    this.addCls('x-form-cb-checked');
  } else {
    this.removeCls('x-form-cb-checked');
  }
  this.callParent([newValue]);
}, initComponent:function() {
  this.callParent();
  if (this.getValue()) {
    this.addCls('x-form-cb-checked');
  }
}});
Ext.define('monitor.component.grid.filter.check.Check', {extend:'monitor.component.grid.filter.SingleFilter', alias:['grid.filter.check'], operator:'\x3d', emptyValue:false, trueValues:[true, 'true', 1, '1', 'ja', 'yes', 'on'], falseValues:['!', false, 'false', '*0', '*nein', 'no', 'off'], menuOptionFieldDefaults:[], columHeaderFieldDefault:{xtype:'monitorFilterCheckbox', listeners:{scope:'this', change:'submitColumHeaderField'}, margin:0}, getFilterConfig:function(config) {
  config = this.callParent(arguments);
  config.source = this.viewModel.get('source') || this.source;
  config.operator = this.viewModel.get('operator') || this.operator;
  return config;
}, getValue:function(field) {
  return field ? field.getValue() : this.filter.getValue();
}, setValue:function(value, silent) {
  silent = silent === true ? silent : false;
  if (Ext.isArray(value)) {
    value = Ext.Array.contains(value, true);
  }
  if (this.columHeaderField.getValue() !== value) {
    this.columHeaderField.setValue(value);
  }
  if (this.inputItem && this.inputItem.getValue() !== value) {
    this.inputItem.setValue(value);
  }
  if (value === true) {
    value = this.trueValues;
  } else {
    value = this.falseValues;
  }
  if (this.filter && this.filter.getValue() !== value) {
    this.filter.setValue(value);
  }
  if (silent === false) {
    if (Ext.isArray(value) && value.length === 0) {
      value = '';
    }
    if (value && this.active) {
      this.updateStoreFilter();
    } else {
      this.setActive(!!value);
    }
  }
}});
Ext.define('monitor.component.grid.filter.number.NumberField', {extend:'Ext.form.field.Number', alias:'widget.monitorFilterNumberField', operatorStore:undefined, config:{operator:'\x3d'}, publishes:['value', 'rawValue', 'dirty', 'operator'], setTriggerPickerHidden:function(hidden) {
  this.getTrigger('picker').setVisible(!hidden);
}, getTriggerPickerHidden:function() {
  return !this.getTrigger('picker').isVisible();
}, setTriggerResetHidden:function(hidden) {
  this.getTrigger('reset').setVisible(!hidden);
}, getTriggerResetHidden:function() {
  return !this.getTrigger('reset').isVisible();
}, bind:{triggerPickerHidden:'{!dirty}', triggerResetHidden:'{!mouseOver}'}, enableKeyEvents:true, triggers:{reset:{cls:BpcCommon.Icons.FILTER_RESET, extraCls:'resetTrigger', weight:-2, scope:'this', handler:'deactivate'}, picker:{cls:BpcCommon.Icons.OK, weight:-1, scope:'this', handler:'submitColumHeaderField'}, spinner:{hidden:true}}, childEls:['operatorEl', 'triggerWrap', 'inputWrap', 'placeholderLabel'], preSubTpl:['\x3cdiv id\x3d"{cmpId}-triggerWrap" data-ref\x3d"triggerWrap"', '\x3ctpl if\x3d"ariaEl \x3d\x3d \'triggerWrap\'"\x3e', 
'\x3ctpl foreach\x3d"ariaElAttributes"\x3e {$}\x3d"{.}"\x3c/tpl\x3e', '\x3ctpl else\x3e', ' role\x3d"presentation"', '\x3c/tpl\x3e', ' class\x3d"{triggerWrapCls} {triggerWrapCls}-{ui}"\x3e', '\x3cdiv id\x3d"{cmpId}-operatorEl" data-ref\x3d"operatorEl" class\x3d"x-unselectable" role\x3d"presentation" style\x3d"width: 25px;', 'display: table-cell; font-style: normal; text-align: center;"\x3e{operator}\x3c/div\x3e', '\x3cdiv id\x3d"{cmpId}-inputWrap" data-ref\x3d"inputWrap"', ' role\x3d"presentation" class\x3d"{inputWrapCls} {inputWrapCls}-{ui}"\x3e'], 
initComponent:function() {
  this.callParent();
  this.operatorStore = Ext.StoreManager.lookup('numericOperatorStore');
}, getSubTplData:function() {
  var data = this.callParent(arguments);
  data.operator = this.getOperator();
  return data;
}, setOperator:function(operator) {
  this.callParent(arguments);
  if (this.rendered && this.operatorEl) {
    var operatorRecord = this.operatorStore.getRange().find(function(record) {
      return Object.values(record.getData()).includes(operator);
    }) || null;
    var operatorText = '\x3d';
    if (operatorRecord) {
      operatorText = operatorRecord.get('text') || operatorText;
    }
    this.operatorEl.update(operatorText);
  }
}, afterRender:function() {
  var me = this;
  var currentOperator = me.getOperator();
  me.callParent();
  var items = [];
  me.operatorStore.each(function(operatorRecord) {
    items.push({xtype:'menucheckitem', group:'operator', text:operatorRecord.get('text'), operator:operatorRecord.get('id'), checked:operatorRecord.get('id') === currentOperator, checkHandler:function(menuItem) {
      me.setOperator(menuItem.operator);
    }});
  });
  me.operatorMenu = Ext.create('Ext.menu.Menu', {items:items});
  me.operatorEl.on('click', me.operatorMenu.showBy.bind(me.operatorMenu, me, undefined, undefined));
}});
Ext.define('monitor.component.grid.filter.number.Number', {extend:'monitor.component.grid.filter.Text', alias:['grid.filter.bpcMonitorNumber', 'grid.filter.bpcmonitornumber'], operator:'\x3d', menuOptionFieldDefaults:[{xtype:'combobox', fieldLabel:'Operator', itemId:'operator', bind:'{operator}', valueField:'id', store:'numericOperatorStore', forceSelection:true}], columHeaderFieldDefault:{xtype:'monitorFilterNumberField', bind:{operator:'{operator}'}, decimalPrecision:100, margin:0}, constructor:function(config) {
  var me = this;
  me.callParent(arguments);
  this.columHeaderField.on('keyup', this.onKeyUp.bind(this));
  this.viewModel.bind('{operator}', this.submitColumHeaderField.bind(this));
}, onKeyUp:function(src, e) {
  if (e.getKey() === Ext.event.Event.ENTER) {
    this.submitColumHeaderField();
  }
}, isValueFilterable:function(value) {
  return Ext.isNumber(value);
}});
Ext.define('monitor.component.grid.plugin.BpcRowExpander', {extend:'Ext.grid.plugin.RowExpander', alias:'plugin.bpcrowexpander', init:function(grid) {
  this.callParent(arguments);
  this.grid = grid;
  grid.on('headerclick', this.onHeaderClick, this);
  grid.on('filterchange', this.onFilterChange, this);
}, onHeaderClick:function(ct, column) {
  if (column.bpcRowExpander === true) {
    this.grid.getController().filterCurrentViewOrSelect();
  }
}, onFilterChange:function(store, filters) {
  try {
    var filterBtn = this.grid.down('[bpcType\x3dclearFilterBtn]');
    if (!filterBtn) {
      return;
    }
    if (filters.length > 0) {
      filterBtn.addCls('bpc-monitor-filterActive');
    } else {
      filterBtn.removeCls('bpc-monitor-filterActive');
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onFilterChange', dump:e, stack:true, level:'error'});
  }
}, getHeaderConfig:function() {
  var me = this;
  var lockable = me.grid.ownerLockable;
  var grid = me.grid;
  var moduleId = grid.config.moduleId;
  var showFilterFields = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.guiShowFilterBar);
  var returnExpander = {width:me.headerWidth, bpcRowExpander:true, ignoreExport:true, lockable:false, autoLock:true, sortable:false, resizable:false, draggable:false, hideable:false, menuDisabled:true, tdCls:Ext.baseCSSPrefix + 'grid-cell-special', innerCls:Ext.baseCSSPrefix + 'grid-cell-inner-row-expander', style:'cursor: pointer;', tooltip:BpcCommon.Api.getTranslation('MONITOR_FILTER_CURRENT_VIEW_OR_SELECTION'), isLocked:function() {
    return lockable && (lockable.lockedGrid.isVisible() || this.locked);
  }, editRenderer:function() {
    return '\x26#160;';
  }, getColumnState:function() {
    return undefined;
  }, text:'\x3ci style\x3d"cursor: pointer;" class\x3d"fa fa-crosshairs"\x3e\x3c/i\x3e', iconCls:'x-fal fa-filter', layout:{type:'vbox', align:'stretch', pack:'start'}, items:[{xtype:'box', cls:'x-fas fa-times-circle fa-fw', itemId:'clearFilterBox', bpcType:'clearFilterBtn', style:'cursor: pointer; padding-top: 10px;', width:me.headerWidth, hidden:!showFilterFields, listeners:{click:{element:'el', fn:'removeAllFilter'}, render:function(c) {
    Ext.create('Ext.tip.ToolTip', {target:c.getEl(), html:BpcCommon.Api.getTranslation('MONITOR_REMOVE_FILTER')});
  }}}], triggerFilterFields:true};
  var showExpander = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.detailViewInlineEnabled);
  if (showExpander) {
    returnExpander = Ext.apply(returnExpander, {renderer:function() {
      return '\x3cdiv class\x3d"' + Ext.baseCSSPrefix + 'grid-row-expander" role\x3d"presentation" tabIndex\x3d"0"\x3e\x3c/div\x3e';
    }, processEvent:function(type, view, cell, rowIndex, cellIndex, e, record) {
      if (type === 'click' && e.getTarget('.' + Ext.baseCSSPrefix + 'grid-row-expander') || type === 'keydown' && e.getKey() === e.SPACE) {
        me.toggleRow(rowIndex, record);
        return me.selectRowOnExpand;
      }
    }});
  }
  return returnExpander;
}});
Ext.define('monitor.component.grid.plugin.CustomRenderer', {alias:'plugin.monitorCustomRenderer', config:{rendererConfigMap:{}, defaultColumnRendererConfigMap:{}, hostCmp:undefined}, DEFAULT_RENDERER_NAME:'defaultRenderer', NO_RENDERER_NAME:'disabledCustomRenderer_', constructor:function(config) {
  var me = this;
  config = config || {};
  me.initConfig(config);
}, init:function(hostCmp) {
  var me = this;
  if (!hostCmp) {
    return;
  }
  me.setHostCmp(hostCmp);
  me.setRendererConfigMap(BpcCommon.Api.getSetting(hostCmp.moduleId, monitor.assets.ConfigKeys.columnCustomRenderer));
  me.setDefaultColumnRendererConfigMap(BpcCommon.Api.getSetting(hostCmp.moduleId, monitor.assets.ConfigKeys.columnConfig));
  hostCmp.on('afterrender', me.updateRendererInDefaultColumnRendererConfigMap, me);
}, applyRendererConfigMap:function(rendererConfigMap) {
  var preparedConfigMap = {};
  if (Ext.isObject(rendererConfigMap)) {
    Ext.Object.each(rendererConfigMap, function(rendererId, rendererConfig) {
      rendererConfig = rendererConfig || {};
      var valueMap = {};
      Ext.Object.each(rendererConfig.valueMap, function(key, value) {
        value = value || {};
        valueMap[String(key).toLowerCase()] = {color:value.color || 'black', iconCls:value.iconCls || ''};
        if (Ext.isDefined(value.label)) {
          valueMap[String(key).toLowerCase()].label = BpcCommon.Api.getTranslation(value.label);
        }
      });
      if (!valueMap['!']) {
        valueMap['!'] = {color:'black', label:'', iconCls:''};
      }
      if (!valueMap['*']) {
        valueMap['*'] = {color:'black', iconCls:'', fallback:true};
      }
      preparedConfigMap[String(rendererId).toLowerCase()] = {iconOnly:rendererConfig.iconOnly === true, valueMap:valueMap};
    });
  }
  return preparedConfigMap;
}, applyDefaultColumnRendererConfigMap:function(columnConfig) {
  var me = this;
  var preparedConfigMap = {};
  if (Ext.isArray(columnConfig)) {
    Ext.Array.each(columnConfig, function(column) {
      column = column || {};
      if (!column.customRenderer) {
        var id = column.id || column.dataIndex;
        var config = Ext.clone(column);
        config.xtype = config.xtype || 'gridcolumn';
        try {
          var columnCmp = Ext.create(config);
          if (columnCmp.renderer) {
            preparedConfigMap[String(id).toLowerCase()] = {renderer:columnCmp.renderer.bind(columnCmp), columnConfig:column};
          }
          if (columnCmp instanceof Ext.grid.column.Widget) {
            columnCmp.ownerGrid = {destroyManagedWidgets:Ext.emptyFn};
          }
          columnCmp.destroy();
        } catch (e) {
          Ext.log({msg:me.self.getName() + ': ERROR ', dump:e, stack:true, level:'error'});
        }
      }
    });
  }
  return preparedConfigMap;
}, updateRendererInDefaultColumnRendererConfigMap:function() {
  var me = this;
  Ext.Object.each(me.getDefaultColumnRendererConfigMap(), function(key, defaultColumnRendererConfig) {
    var colId = defaultColumnRendererConfig.columnConfig.stateId || defaultColumnRendererConfig.columnConfig.id || defaultColumnRendererConfig.columnConfig.columnId || defaultColumnRendererConfig.columnConfig.dataIndex;
    var column = me.getHostCmp().down('gridcolumn[columnId\x3d' + colId + ']');
    if (column && column.renderer) {
      defaultColumnRendererConfig.renderer = column.renderer.bind(column);
    }
  });
}, render:function(rendererId, value, icon, label, suppressDefaultRenderer, record) {
  var me = this;
  var currentRecord = record || Ext.data.Model.create();
  var result = '';
  if (!rendererId || rendererId === me.NO_RENDERER_NAME) {
    return value;
  }
  var rendererConfig = me.getRendererConfigMap()[String(rendererId).toLowerCase()];
  if (!rendererConfig) {
    if (rendererId.startsWith(me.NO_RENDERER_NAME) || suppressDefaultRenderer === true) {
      var columnRendererConfig = me.getDefaultColumnRendererConfigMap()[String(rendererId).replace(me.NO_RENDERER_NAME, '').toLowerCase()];
      if (columnRendererConfig && columnRendererConfig.renderer) {
        return columnRendererConfig.renderer(value, {}, currentRecord);
      }
      return value;
    }
    rendererConfig = me.getRendererConfigMap()[String(me.DEFAULT_RENDERER_NAME).toLowerCase()];
    rendererId = me.DEFAULT_RENDERER_NAME;
    if (!rendererConfig) {
      return value;
    }
  }
  var valueConfig = me.getValueConfig(rendererId, value);
  if (!Ext.isBoolean(icon)) {
    icon = true;
  }
  if (!Ext.isBoolean(label)) {
    label = rendererConfig.iconOnly !== true;
  }
  if (icon && valueConfig.iconCls) {
    result = result + "\x3ci class\x3d' fa-lg " + valueConfig.iconCls + "' style\x3d'color:" + valueConfig.color + "'\x3e\x26nbsp;\x3c/i\x3e ";
  }
  if (label) {
    var labelString = value || '';
    if (Ext.isDefined(valueConfig.label)) {
      labelString = valueConfig.label;
    }
    result = '' + result + Ext.util.Format.htmlEncode2(labelString);
  }
  return result;
}, getColumnRenderer:function(rendererId, icon, label, cfg) {
  cfg = cfg === void 0 ? {} : cfg;
  var me = this;
  var renderHelpArr = String(rendererId).split('!_!');
  if (renderHelpArr.length === 2) {
    try {
      var rendererClass = Ext.ClassManager.get(renderHelpArr[0] + '.Module');
      if (rendererClass) {
        return rendererClass[renderHelpArr[1]](cfg);
      }
      return null;
    } catch (e) {
      Ext.log({msg:this.self.getName() + ': ERROR getColumnRenderer', dump:e, stack:true, level:'error'});
    }
  }
  return function(value, meta, record) {
    if (rendererId.startsWith(me.NO_RENDERER_NAME)) {
      var columnId = rendererId.replace(me.NO_RENDERER_NAME, '');
      var defaultColumnRenderer = me.getDefaultColumnRendererConfigMap()[columnId.toLowerCase()];
      if (defaultColumnRenderer) {
        return defaultColumnRenderer.renderer.apply(defaultColumnRenderer.renderer, arguments);
      }
    }
    if (meta) {
      var tooltip = me.render(rendererId, value, false, true, undefined, record) || value;
      meta.tdAttr = 'data-qtip\x3d"' + tooltip + '"';
    }
    return me.render(rendererId, value, icon, label, undefined, record);
  };
}, getValueConfig:function(rendererId, value) {
  var me = this;
  var rendererConfig = me.getRendererConfigMap()[String(rendererId).toLowerCase()];
  if (!rendererConfig && !rendererId.startsWith(me.NO_RENDERER_NAME)) {
    rendererConfig = me.getRendererConfigMap()[String(me.DEFAULT_RENDERER_NAME).toLowerCase()];
  }
  if (!rendererConfig) {
    return {iconCls:'', color:'', label:value};
  }
  var mapValue = value;
  if (value === undefined || value === null) {
    mapValue = '!';
  }
  return rendererConfig.valueMap[String(mapValue).toLowerCase().trim()] || rendererConfig.valueMap['*'];
}, getColumnRendererByColumnConfig:function(columnConfig, icon, label) {
  var me = this;
  if (!Ext.isObject(columnConfig)) {
    return;
  }
  return me.getColumnRenderer(me.getRendererIdByColumnConfig(columnConfig), icon, label, columnConfig);
}, getRendererIdByColumnConfig:function(columnConfig) {
  var me = this;
  if (!Ext.isObject(columnConfig)) {
    return;
  }
  var rendererId = columnConfig.id || columnConfig.dataIndex;
  if (Ext.isString(columnConfig.customRenderer) && columnConfig.customRenderer !== '') {
    rendererId = columnConfig.customRenderer;
  } else if (columnConfig.customRenderer === true) {
  } else {
    rendererId = '' + me.NO_RENDERER_NAME + rendererId;
  }
  return rendererId;
}});
Ext.define('monitor.model.AbstractAction', {extend:'Ext.data.Model', fields:[{name:'name'}, {name:'label'}, {name:'id'}, {name:'iconCls'}, {name:'tooltip'}, {name:'sortValue', defaultValue:1}, {name:'handler'}, {name:'type'}]});
Ext.define('monitor.model.Action', {extend:'Ext.data.Model', fields:[{name:'name'}, {name:'label'}, {name:'id'}, {name:'iconCls'}, {name:'tooltip'}, {name:'sortValue', defaultValue:1}, {name:'params'}, {name:'column'}, {name:'selection'}, {name:'target'}]});
Ext.define('monitor.model.Filter', {extend:'Ext.data.Model', alias:'model.monitorFilterModel', fields:[{name:'property', type:'string'}, {name:'value'}, {name:'operator', type:'string'}, {name:'id'}]});
Ext.define('monitor.model.Grid', {extend:'Ext.data.Model', alias:'model.monitorGridModel', idProperty:'PROCESSID'});
Ext.define('monitor.model.StorageRecord', {extend:'Ext.data.Model', alias:'model.bpcStorageRecord', defaultReadRestrictions:{roles:['bpcadmin']}, defaultWriteRestrictions:{roles:['bpcadmin']}, fields:['id', 'moduleId', 'moduleInstanceId', 'name', 'value', {name:'favorite', type:'boolean'}, {name:'readUser', mapping:'readRestriction.user'}, {name:'readOrganisations', mapping:'readRestriction.organisations', defaultValue:[]}, {name:'readRoles', mapping:'readRestriction.roles', defaultValue:[]}, {name:'readRights', 
mapping:'readRestriction.rights', defaultValue:[]}, {name:'writeUser', mapping:'writeRestriction.user'}, {name:'writeOrganisations', mapping:'writeRestriction.organisations', defaultValue:[]}, {name:'writeRoles', mapping:'writeRestriction.roles', defaultValue:[]}, {name:'writeRights', mapping:'writeRestriction.rights', defaultValue:[]}, 'customFields'], proxy:{type:'bpcTemplateUrlRestProxy', url:'cxf/bpc-core/storage/store/{bpcStorageStoreId}', reader:{type:'json', rootProperty:'items'}, writer:{type:'json', 
nameProperty:'mapping', expandData:true, writeAllFields:true, transform:function(data, request) {
  if ((!data.readRestriction.organisations || data.readRestriction.organisations.length === 0) && (!data.readRestriction.roles || data.readRestriction.roles.length === 0) && (!data.readRestriction.rights || data.readRestriction.rights.length === 0) && !data.readRestriction.user) {
    Ext.apply(data.readRestriction, request.getRecords()[0].defaultReadRestrictions);
  }
  if ((!data.writeRestriction.organisations || data.writeRestriction.organisations.length === 0) && (!data.writeRestriction.roles || data.writeRestriction.roles.length === 0) && (!data.writeRestriction.rights || data.writeRestriction.rights.length === 0) && !data.writeRestriction.user) {
    Ext.apply(data.writeRestriction, request.getRecords()[0].defaultWriteRestrictions);
  }
  return data;
}}, actionMethods:{create:'POST', read:'GET', update:'POST', destroy:'DELETE'}}});
Ext.define('monitor.model.MonitorViewState', {extend:'monitor.model.StorageRecord', alias:'model.monitorViewState', defaultReadRestrictions:{roles:['bpcuser']}, defaultWriteRestrictions:{roles:['bpcuser']}, fields:[{name:'searchQuery', mapping:'value.searchQuery'}, {name:'name', mapping:'value.name'}, {name:'tooltip', mapping:'value.tooltip'}, {name:'iconCls', mapping:'value.iconCls'}, {name:'favorite', mapping:'value.favorite'}, {name:'views', mapping:'value.views', isEqual:function(value1, value2) {
  if (Ext.isObject(value1) && Ext.isObject(value2)) {
    return BpcCommon.Util.compareObject(value1, value2);
  }
  return this.compare(value1, value2) === 0;
}}, {name:'activeView', mapping:'value.activeView'}, {name:'priority', mapping:'customFields.priority', defaultValue:0}], getPriorityFactor:function() {
  var me = this;
  if (me.get('readRoles') && me.get('readRoles').includes('bpcuser')) {
    return 50;
  }
  if (me.get('readOrganisations') && me.get('readOrganisations').length > 0) {
    return 25;
  }
  if (me.get('readRoles') && me.get('readRoles').length > 0) {
    return 20;
  }
  if (me.get('readRights') && me.get('readRights').length > 0) {
    return 10;
  }
  if (me.get('readUser') && me.get('readUser').length > 0) {
    return 5;
  }
  return 100;
}});
Ext.define('monitor.model.Sorter', {extend:'Ext.data.Model', alias:'model.monitorSorterModel', fields:[{name:'property', type:'string'}, {name:'direction', type:'string'}, {name:'id'}]});
Ext.define('monitor.model.Starter', {extend:'Ext.data.Model', idProperty:'key', fields:['label', 'key', 'parameters', 'parametersEndpoint', 'description']});
Ext.define('monitor.model.State', {extend:'Ext.data.Model', alias:'model.monitorStateModel', fields:[{name:'name', type:'string'}, {name:'id', type:'int'}, {name:'favorite', type:'boolean'}, {name:'state', type:'auto', defaultValue:{}, convert:null}]});
Ext.define('monitor.moduleConfig.components.MainReference', {extend:'BpcCommon.component.moduleUI.MainReference', isModuleIdReadOnly:true, addToNavigationDefaultValue:false, baseModuleId:'monitor', initialFormRenderingStyle:'complete', searchStringList:[], isSearchActive:false, initComponent:function() {
  var me = this;
  me.callParent();
}, refreshUIComponents:function() {
  var me = this;
  me.removeAll();
  me.add([{xtype:'bpcMonitorConfigurationForm', isUIHelpTextContainerHidden:me.getHelpTextHiddenState(), meinHeight:1200, hideHeader:true, mainReference:me, listeners:{monitorConfigTabChange:function(tabIndex) {
    me.fireEvent('monitorConfigTabChange', tabIndex);
  }}}]);
}, registerEvents:Ext.emptyFn});
Ext.define('monitor.moduleConfig.components.ConfigurationForm', {extend:'BpcCommon.component.moduleUI.ConfigurationForm', alias:'widget.bpcMonitorConfigurationForm', localized:{title:'CORE_LOG_SERVICE_CONFIGURATION'}, isModuleIdReadOnly:true, initComponent:function() {
  var me = this;
  me.on('afterrender', me.lateRenderFormItems, me);
  me.callParent();
}, lateRenderFormItems:function() {
  var me = this;
  me.removeAll();
  me.add(me.getFormItems());
}, getFormItems:function() {
  var me = this;
  var isSearchActive = me.mainReference.getIsSearchActive();
  if (isSearchActive === true) {
    return me.getCompletePanelItems();
  }
  var renderingStyle = me.mainReference.getConfig('formRenderingStyle');
  var formItems;
  switch(renderingStyle) {
    case 'tabbed':
      formItems = me.getTabItems();
      break;
    case 'complete':
      formItems = me.getCompletePanelItems();
      break;
    case 'minimal':
      formItems = me.getMinimalPanelItems();
      break;
    default:
      formItems = me.getCompletePanelItems();
  }
  return formItems;
}, getCompletePanelItems:function() {
  var me = this;
  return [{xtype:'bpcModuleUIFormPanel', mainReference:me.mainReference, formComponents:me.mainReference.getFormComponents()}];
}, getMinimalPanelItems:function() {
  var me = this;
  return [{xtype:'bpcModuleUIFormPanel', mainReference:me.mainReference, formComponents:me.mainReference.getMinimalFormComponents()}];
}, getTabItems:function() {
  var me = this;
  var categoryList = [{id:'module', title:BpcCommon.Api.getTranslation('MONITOR_MODULE_CONFIG_TAB'), icon:'x-fal fa-tools'}, {id:'toolbar', title:BpcCommon.Api.getTranslation('MONITOR_TOOLBAR_CONFIG_TAB'), icon:'x-fal fa-window'}, {id:'data', title:BpcCommon.Api.getTranslation('MONITOR_DATA_CONFIG_TAB'), icon:'x-fal fa-database'}, {id:'sort', title:BpcCommon.Api.getTranslation('MONITOR_SORT_CONFIG_TAB'), icon:'x-fal fa-sort-alpha-up'}, {id:'dataRendering', title:BpcCommon.Api.getTranslation('MONITOR_DATA_RENDERING_CONFIG_TAB'), 
  icon:'x-fal fa-desktop'}, {id:'grid', title:BpcCommon.Api.getTranslation('MONITOR_GRID_CONFIG_TAB'), icon:'x-fal fa-table'}, {id:'columns', title:BpcCommon.Api.getTranslation('MONITOR_COLUMNS_CONFIG_TAB'), icon:'x-fal fa-columns'}, {id:'inubit', title:BpcCommon.Api.getTranslation('MONITOR_INUBIT_CONFIG_TAB'), icon:'x-fal fa-server'}, {id:'process', title:BpcCommon.Api.getTranslation('MONITOR_PROCESS_CONFIG_TAB'), icon:'x-fal fa-lightbulb-exclamation'}, {id:'detail', title:BpcCommon.Api.getTranslation('MONITOR_DETAIL_CONFIG_TAB'), 
  icon:'x-fal fa-info-circle'}, {id:'export', title:BpcCommon.Api.getTranslation('MONITOR_EXPORT_CONFIG_TAB'), icon:'x-fal fa-file-export'}];
  var tabPanelItems = [];
  Ext.Array.each(categoryList, function(category) {
    var filteredItems = me.mainReference.getFormComponents().filter(function(el) {
      return el.category === category.id;
    });
    tabPanelItems.push({xtype:'bpcModuleUIFormPanel', scrollable:true, layout:{type:'vbox', align:'stretch'}, mainReference:me.mainReference, title:category.title, iconCls:category.icon, formComponents:filteredItems});
  }, me);
  return {xtype:'tabpanel', isMonitorConfigTabPanel:true, layout:'fit', items:tabPanelItems, activeTab:me.mainReference.getSelectedTab(), listeners:{tabchange:me.onTabChange.bind(me)}};
}, onTabChange:function(tabPanel) {
  var me = this;
  var activeTab = tabPanel.getActiveTab();
  var activeTabIndex = tabPanel.items.indexOf(activeTab);
  me.fireEvent('monitorConfigTabChange', activeTabIndex);
  me.mainReference.setSelectedTab(activeTabIndex);
}});
Ext.define('monitor.moduleConfig.components.customPicker.dataFilterOperatorPicker', {alias:'widget.bpcMonitorDataFilterOperatorPicker', extend:'BpcCommon.component.form.field.ComboBox', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  return '' + recordData.label;
}})}, initComponent:function() {
  var me = this;
  me.store = [{label:BpcCommon.lang.Current.MONITOR_SETTING_LABEL_DATA_FILTER_OPERATOR_OR, value:'or'}, {label:BpcCommon.lang.Current.MONITOR_SETTING_LABEL_DATA_FILTER_OPERATOR_AND, value:'and'}];
  me.callParent();
}});
Ext.define('monitor.moduleConfig.components.customPicker.formRenderingStylePicker', {alias:'widget.bpcMonitorFormRenderingStylePicker', extend:'BpcCommon.component.form.field.ComboBox', changeHandler:Ext.emptyFn, value:'minimal', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var color = '#00000';
  var iconCls = recordData.icon;
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
  return icon + ' ' + recordData.label;
}})}, initComponent:function() {
  var me = this;
  me.store = [{label:BpcCommon.lang.Current.MONITOR_TABBED_FORM_RENDERING, value:'tabbed', icon:'x-fal fa-window'}, {label:BpcCommon.lang.Current.MONITOR_COMPLETE_FORM_RENDERING, value:'complete', icon:'x-fal fa-list'}, {label:BpcCommon.lang.Current.MONITOR_MINIMAL_FORM_RENDERING, value:'minimal', icon:'x-fal fa-compress-arrows-alt'}];
  me.on('change', me.onComboValueChanged, me);
  me.callParent();
}, onComboValueChanged:function(el, val) {
  var me = this;
  me.changeHandler(val);
}});
Ext.define('monitor.moduleConfig.components.customPicker.sortDirectionPicker', {alias:'widget.bpcMonitorSortDirectionPicker', extend:'BpcCommon.component.form.field.ComboBox', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var color = 'black';
  var iconCls = recordData.icon;
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
  return icon + ' ' + recordData.label;
}})}, initComponent:function() {
  this.store = [{label:BpcCommon.lang.Current.MONITOR_ASCENDING, value:'ASC', icon:'x-fal fa-sort-alpha-up'}, {label:BpcCommon.lang.Current.MONITOR_DESCENDING, value:'DESC', icon:'x-fal fa-sort-alpha-down'}];
  this.callParent();
}});
Ext.define('monitor.moduleConfig.components.customPicker.toolBarContentPicker', {alias:'widget.bpcMonitorToolBarContentPicker', extend:'Ext.form.field.Tag', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var iconCls = recordData.icon;
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d" font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
  return icon + ' ' + recordData.label;
}})}, initComponent:function() {
  this.store = [{label:'patzhalter', value:'resetuserstate', icon:BpcCommon.Icons.USER_REMOVE}];
  this.callParent();
}});
Ext.define('monitor.moduleConfig.mixins.formComponents', {jsonFieldSmallHeight:75, defineFormComponents:function() {
  var me = this;
  var isModuleIdReadOnly = me.isModuleIdReadOnly;
  var defaultAttributes = {mainReference:me, isUIHelpTextContainerHidden:me.getHelpTextHiddenState()};
  var fieldSetList = [{xtype:'bpcDataBindFieldSet', category:'module', title:BpcCommon.Api.getTranslation('CORE_MODULE_CONFIGURATION'), fieldComponents:[{xtype:'moduleIdField', property:'id', isMinimalConfig:true, localized:{fieldLabel:'CORE_MODULE_INSTANCE_ID', tooltip:'CORE_MODULE_INSTANCE_ID_TOOLTIP'}, disabled:isModuleIdReadOnly, allowBlank:false, hideRestoreToDefaultBtn:isModuleIdReadOnly}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.moduleName, isMinimalConfig:true, localized:{fieldLabel:'CORE_MODULE_NAME', 
  tooltip:'CORE_MODULE_NAME_TOOLTIP'}, allowBlank:false}, {xtype:'textarea', property:monitor.assets.ConfigKeys.moduleDescription, localized:{fieldLabel:'CORE_MODULE_DESCRIPTION', tooltip:'CORE_MODULE_DESCRIPTION_TOOLTIP'}}, {xtype:'bpcIconPicker', property:monitor.assets.ConfigKeys.moduleIconCls, localized:{fieldLabel:'CORE_MODULE_ICON', tooltip:'CORE_MODULE_ICON_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.moduleHeaderEnabled, localized:{fieldLabel:'CORE_MODULE_HEADER_ENABLED', 
  tooltip:'CORE_MODULE_HEADER_ENABLED_TOOLTIP'}}, {xtype:'checkbox', property:'sanitizeHtml', localized:{fieldLabel:'CORE_SANITIZE_HTML', tooltip:'CORE_SANITIZE_HTML_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.resourceSavingMode, localized:{fieldLabel:'MONITOR_RESOURCE_SAVING_MODE', tooltip:'MONITOR_RESOURCE_SAVING_MODE_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.guiStateBrowsing, localized:{fieldLabel:'MONITOR_GUI_STATE_BROWSING', tooltip:'MONITOR_GUI_STATE_BROWSING_TOOLTIP'}}]}, 
  {xtype:'bpcDataBindFieldSet', category:'toolbar', title:BpcCommon.Api.getTranslation('MONITOR_TOOLBAR_CONFIGURATION'), fieldComponents:[{xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.moduleHeaderContent, localized:{fieldLabel:'MONITOR_MODULE_HEADER_CONTENT', tooltip:'MONITOR_MODULE_HEADER_CONTENT_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.appToolbarLeftContent, localized:{fieldLabel:'MONITOR_APP_TOOLBAR_LEFT_CONTENT', tooltip:'MONITOR_APP_TOOLBAR_LEFT_CONTENT_TOOLTIP'}}, 
  {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.appToolbarRightContent, localized:{fieldLabel:'MONITOR_APP_TOOLBAR_RIGHT_CONTENT', tooltip:'MONITOR_APP_TOOLBAR_RIGHT_CONTENT_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.customViewsEnabled, localized:{fieldLabel:'MONITOR_SHOW_SAVE_VIEW_MENU', tooltip:'MONITOR_SHOW_SAVE_VIEW_MENU_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'data', title:BpcCommon.Api.getTranslation('MONITOR_DATA_CONFIGURATION'), fieldComponents:[{xtype:'checkbox', 
  property:monitor.assets.ConfigKeys.autoUpdate, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_AUTOMATIC_REFRESH', tooltip:'MONITOR_AUTOMATIC_REFRESH_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.dateFormat, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_DATE_FORMAT', tooltip:'MONITOR_DATE_FORMAT_TOOLTIP'}}, {xtype:'bpcIndexSelector', property:monitor.assets.ConfigKeys.dataIndex, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_DATA_INDEX', tooltip:'MONITOR_DATA_INDEX_TOOLTIP'}}, 
  {xtype:'bpcIndexSelector', property:monitor.assets.ConfigKeys.dataHistoryIndex, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_DATA_HISTORY_INDEX', tooltip:'MONITOR_DATA_HISTORY_INDEX_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.dataFilter, localized:{fieldLabel:'MONITOR_DATA_FILTER', tooltip:'MONITOR_DATA_FILTER_TOOLTIP'}}, {xtype:'bpcMonitorDataFilterOperatorPicker', property:monitor.assets.ConfigKeys.dataFilterOperator, localized:{fieldLabel:'MONITOR_DATA_FILTER_OPERATOR', 
  tooltip:'MONITOR_DATA_FILTER_OPERATOR_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.dataExcludedFields, height:me.jsonFieldSmallHeight, localized:{fieldLabel:'MONITOR_EXCLUDED_FIELDS', tooltip:'MONITOR_EXCLUDED_FIELDS_TOOLTIP'}}, {xtype:'bpcNumberField', property:monitor.assets.ConfigKeys.dataRefreshBufferInSeconds, localized:{fieldLabel:'MONITOR_DATA_REFRESH_BUFFER', tooltip:'MONITOR_DATA_REFRESH_BUFFER_TOOLTIP'}}, {xtype:'bpcNumberField', property:monitor.assets.ConfigKeys.instanceDataCountLimit, 
  localized:{fieldLabel:'MONITOR_DATA_COUNT_LIMIT', tooltip:'MONITOR_DATA_COUNT_LIMIT_TOOLTIP'}}, {xtype:'bpcNumberField', property:monitor.assets.ConfigKeys.instanceDataViewLimit, localized:{fieldLabel:'MONITOR_DATA_VIEW_LIMIT', tooltip:'MONITOR_DATA_VIEW_LIMIT_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnDefaultMimetype, localized:{fieldLabel:'MONITOR_COLUMN_DEFAULT_MIME_TYPE', tooltip:'MONITOR_COLUMN_DEFAULT_MIME_TYPE_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.filterRespectOtherColumns, 
  localized:{fieldLabel:'MONITOR_FILTER_RESPECT_OTHER_COLUMNS', tooltip:'MONITOR_FILTER_RESPECT_OTHER_COLUMNS_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'columns', title:BpcCommon.Api.getTranslation('MONITOR_ID_COLUMN_CONFIGURATION'), fieldComponents:[{xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnId, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_PROCESS_ID', tooltip:'MONITOR_PROCESS_ID_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnHistoryId, 
  isMinimalConfig:true, localized:{fieldLabel:'MONITOR_HISTORY_ID', tooltip:'MONITOR_HISTORY_ID_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnHistorySubId, localized:{fieldLabel:'MONITOR_HISTORY_SUB_ID', tooltip:'MONITOR_HISTORY_SUB_ID_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnTimeseries, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_TIMESTAMP_COLUMN', tooltip:'MONITOR_TIMESTAMP_COLUMN_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', 
  category:'dataRendering', title:BpcCommon.Api.getTranslation('MONITOR_DATA_RENDERING_CONFIGURATION'), fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.rowStyleEnabled, localized:{fieldLabel:'MONITOR_ROW_STYLE_ENABLED', tooltip:'MONITOR_ROW_STYLE_ENABLED_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.rowStyle, localized:{fieldLabel:'MONITOR_ROW_STYLE_CONFIG', tooltip:'MONITOR_ROW_STYLE_CONFIG_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.columnCustomRenderer, 
  localized:{fieldLabel:'MONITOR_COLUMN_CUSTOM_RENDERER_CONFIG', tooltip:'MONITOR_COLUMN_CUSTOM_RENDERER_CONFIG_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'sort', title:BpcCommon.Api.getTranslation('MONITOR_SORT_CONFIGURATION'), fieldComponents:[{xtype:'bpcTextField', property:monitor.assets.ConfigKeys.gridSortingColumn, localized:{fieldLabel:'MONITOR_GRID_MAIN_SORT_COLUMN', tooltip:'MONITOR_GRID_MAIN_SORT_COLUMN_TOOLTIP'}, allowBlank:false}, {xtype:'bpcMonitorSortDirectionPicker', property:monitor.assets.ConfigKeys.gridMainSortDirection, 
  localized:{fieldLabel:'MONITOR_GRID_MAIN_SORT_DIRECTION', tooltip:'MONITOR_GRID_MAIN_SORT_DIRECTION_TOOLTIP'}, allowBlank:false}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.sortMulti, localized:{fieldLabel:'MONITOR_SORT_MULTI', tooltip:'MONITOR_SORT_MULTI_TOOLTIP'}, allowBlank:false}, {xtype:'bpcNumberField', property:monitor.assets.ConfigKeys.sortMultiColumnCount, minValue:1, localized:{fieldLabel:'MONITOR_SORT_MULTI_COLUMN_COUNT', tooltip:'MONITOR_SORT_MULTI_COLUMN_COUNT_TOOLTIP'}, 
  allowBlank:false}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.sortMultiReversedOrder, localized:{fieldLabel:'MONITOR_SORT_MULTI_REVERSED_ORDER', tooltip:'MONITOR_SORT_MULTI_REVERSED_ORDER_TOOLTIP'}, allowBlank:false}]}, {xtype:'bpcDataBindFieldSet', category:'grid', title:BpcCommon.Api.getTranslation('MONITOR_GRID_CONFIGURATION'), fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.multipleGrids, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_GRID_MULTIPLE_VIEWS', 
  tooltip:'MONITOR_GRID_MULTIPLE_VIEWS_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.multipleGridsConfig, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_GRID_MULTIPLE_VIEWS_CONFIG', tooltip:'MONITOR_GRID_MULTIPLE_VIEWS_CONFIG_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.guiShowFilterBar, localized:{fieldLabel:'MONITOR_GRID_SHOW_FILTER_BAR', tooltip:'MONITOR_GRID_SHOW_FILTER_BAR_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.showGridLoadMask, 
  localized:{fieldLabel:'MONITOR_GRID_SHOW_LOAD_MASK', tooltip:'MONITOR_GRID_SHOW_LOAD_MASK_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.countColumns, localized:{fieldLabel:'MONITOR_GRID_COUNT_FILTERED_COLUMNS', tooltip:'MONITOR_GRID_COUNT_FILTERED_COLUMNS_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.hideDefaultTableHeaderTooltip, localized:{fieldLabel:'MONITOR_HIDE_DEFAULT_TABLE_HEADER_TOOLTIP', tooltip:'MONITOR_HIDE_DEFAULT_TABLE_HEADER_TOOLTIP_TOOLTIP'}}, 
  {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.gridEditable, localized:{fieldLabel:'MONITOR_GRID_TEXT_SELECTION', tooltip:'MONITOR_GRID_TEXT_SELECTION_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'columns', title:BpcCommon.Api.getTranslation('MONITOR_COLUMNS_CONFIGURATION'), fieldComponents:[{xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.columnConfig, isMinimalConfig:true, localized:{fieldLabel:'MONITOR_COlUMN_CONFIG', tooltip:'MONITOR_COlUMN_CONFIG_TOOLTIP'}}, 
  {xtype:'bpcExpandableJsonField', height:me.jsonFieldSmallHeight, property:monitor.assets.ConfigKeys.columnHistoryConfig, localized:{fieldLabel:'MONITOR_COlUMN_HISTORY_CONFIG', tooltip:'MONITOR_COlUMN_HISTORY_CONFIG_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnGroupField, localized:{fieldLabel:'MONITOR_INITIAL_GROUP_COLUMN', tooltip:'MONITOR_INITIAL_GROUP_COLUMN_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnTaskView, localized:{fieldLabel:'MONITOR_INTERACTION_COLUMN', 
  tooltip:'MONITOR_INTERACTION_COLUMN_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnTaskViewValue, localized:{fieldLabel:'MONITOR_INTERACTION_COLUMN_VALUE', tooltip:'MONITOR_INTERACTION_COLUMN_VALUE_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnStatus, localized:{fieldLabel:'MONITOR_STATUS_COLUMN', tooltip:'MONITOR_STATUS_COLUMN_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnMimetype, localized:{fieldLabel:'MONITOR_MIME_TYPE_COLUMN', 
  tooltip:'MONITOR_MIME_TYPE_COLUMN_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnFile, localized:{fieldLabel:'MONITOR_SETTING_LABEL_COLUMN_FILE', tooltip:'MONITOR_SETTING_LABEL_COLUMN_FILE_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'inubit', title:BpcCommon.Api.getTranslation('MONITOR_INUBIT_CONFIGURATION'), fieldComponents:[{xtype:'bpcTextField', property:monitor.assets.ConfigKeys.bpmReferenceEndpoint, localized:{fieldLabel:'MONITOR_INUBIT_FILE_DOWNLOAD_ENDPOINT', 
  tooltip:'MONITOR_INUBIT_FILE_DOWNLOAD_ENDPOINT_TOOLTIP'}}, {xtype:'bpcModuleInstanceSelector', baseModuleName:'backendconnection', instanceType:'http_proxy', property:monitor.assets.ConfigKeys.bpmProxyId, localized:{fieldLabel:'MONITOR_INUBIT_PROXY_ID', tooltip:'MONITOR_INUBIT_PROXY_ID_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.bpmBasePath, localized:{fieldLabel:'MONITOR_INUBIT_BASE_URL', tooltip:'MONITOR_INUBIT_BASE_URL_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.legacyPmGridId, 
  localized:{fieldLabel:'MONITOR_INUBIT_DB_GRID', tooltip:'MONITOR_INUBIT_DB_GRID_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.legacyPmTablePrefix, localized:{fieldLabel:'MONITOR_INUBIT_DB_TABLE_PREFIX', tooltip:'MONITOR_INUBIT_DB_TABLE_PREFIX_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.legacyPmMandant, localized:{fieldLabel:'MONITOR_INUBIT_PM_MANDANT', tooltip:'MONITOR_INUBIT_PM_MANDANT_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'process', 
  title:BpcCommon.Api.getTranslation('MONITOR_PROCESS_STARTER_CONFIGURATION'), collapsed:false, fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.processStart, localized:{fieldLabel:'MONITOR_PROCESS_STARTER_ACTIVE', tooltip:'MONITOR_PROCESS_STARTER_ACTIVE_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.processStartEndpoint, localized:{fieldLabel:'MONITOR_PROCESS_STARTER_ENDPOINT', tooltip:'MONITOR_PROCESS_STARTER_ENDPOINT_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', 
  property:monitor.assets.ConfigKeys.processStartConfig, localized:{fieldLabel:'MONITOR_PROCESS_STARTER_CONFIG', tooltip:'MONITOR_PROCESS_STARTER_CONFIG_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'process', title:BpcCommon.Api.getTranslation('MONITOR_PROCESS_ACTIONS_CONFIGURATION'), collapsed:false, fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.processActions, localized:{fieldLabel:'MONITOR_PROCESS_ACTIONS_ACTIVE', tooltip:'MONITOR_PROCESS_ACTIONS_TOOLTIP'}}, {xtype:'bpcTextField', 
  property:monitor.assets.ConfigKeys.processActionsEndpoint, localized:{fieldLabel:'MONITOR_PROCESS_ACTIONS_ENDPOINT', tooltip:'MONITOR_PROCESS_ACTIONS_ENDPOINT_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.parallelProcessActions, localized:{fieldLabel:'MONITOR_PROCESS_ACTIONS_ALLOW_PARALLEL', tooltip:'MONITOR_PROCESS_ACTIONS_ALLOW_PARALLEL_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.columnActionPrefix, localized:{fieldLabel:'MONITOR_PROCESS_ACTIONS_PREFIX', 
  tooltip:'MONITOR_PROCESS_ACTIONS_PREFIX_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.actionSeperator, localized:{fieldLabel:'MONITOR_PROCESS_ACTIONS_SEPARATOR', tooltip:'MONITOR_PROCESS_ACTIONS_SEPARATOR_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'process', title:BpcCommon.Api.getTranslation('MONITOR_CHANGE_STATE_CONFIGURATION'), collapsed:false, fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.changeState, localized:{fieldLabel:'MONITOR_CHANGE_STATE_ACTIVE', 
  tooltip:'MONITOR_CHANGE_STATE_ACTIVE_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.changeStateEndpoint, localized:{fieldLabel:'MONITOR_INUBIT_CHANGE_STATE_ENDPOINT', tooltip:'MONITOR_INUBIT_CHANGE_STATE_ENDPOINT_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.changeStateConfig, localized:{fieldLabel:'MONITOR_CHANGE_STATE_CONFIG', tooltip:'MONITOR_CHANGE_STATE_CONFIG_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'detail', title:BpcCommon.Api.getTranslation('MONITOR_DETAIL_VIEW_CONFIGURATION'), 
  collapsed:false, fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.detailViewPopupEnabled, localized:{fieldLabel:'MONITOR_DETAIL_VIEW_POP_UP_ACTIVE', tooltip:'MONITOR_DETAIL_VIEW_POP_UP_ACTIVE_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.detailViewPopupConfig, localized:{fieldLabel:'MONITOR_DETAIL_VIEW_POP_UP_CONFIG', tooltip:'MONITOR_DETAIL_VIEW_POP_UP_CONFIG_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.detailViewInlineEnabled, 
  localized:{fieldLabel:'MONITOR_DETAIL_VIEW_INLINE_ACTIVE', tooltip:'MONITOR_DETAIL_VIEW_INLINE_ACTIVE_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.detailViewInlineConfig, localized:{fieldLabel:'MONITOR_DETAIL_VIEW_INLINE_CONFIG', tooltip:'MONITOR_DETAIL_VIEW_INLINE_CONFIG_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.detailViewEastEnabled, localized:{fieldLabel:'MONITOR_DETAIL_VIEW_EAST_ACTIVE', tooltip:'MONITOR_DETAIL_VIEW_EAST_ACTIVE_TOOLTIP'}}, 
  {xtype:'bpcExpandableJsonField', property:monitor.assets.ConfigKeys.detailViewEastConfig, localized:{fieldLabel:'MONITOR_DETAIL_VIEW_EAST_CONFIG', tooltip:'MONITOR_DETAIL_VIEW_EAST_CONFIG_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.detailViewEastInitialShow, localized:{fieldLabel:'MONITOR_DETAIL_VIEW_EASTINITIALSHOW_ACTIVE', tooltip:'MONITOR_DETAIL_VIEW_EASTINITIALSHOW_ACTIVE_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'export', title:BpcCommon.Api.getTranslation('MONITOR_GENERAL_EXPORT_CONFIGURATION'), 
  collapsed:false, fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.exportEnabled, localized:{fieldLabel:'MONITOR_EXPORT_ENABLE', tooltip:'MONITOR_EXPORT_ENABLE_TOOLTIP'}}, {xtype:'bpcExpandableJsonField', height:me.jsonFieldSmallHeight, property:monitor.assets.ConfigKeys.exportDialogDefaults, localized:{fieldLabel:'MONITOR_EXPORT_DIALOG_DEFAULTS', tooltip:'MONITOR_EXPORT_DIALOG_DEFAULTS_TOOLTIP'}}, {xtype:'bpcNumberField', property:monitor.assets.ConfigKeys.exportLimit, minValue:0, 
  localized:{fieldLabel:'MONITOR_MAX_EXPORT_LIMIT', tooltip:'MONITOR_MAX_EXPORT_LIMIT_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'export', title:BpcCommon.Api.getTranslation('MONITOR_CSV_EXPORT_CONFIGURATION'), collapsed:false, fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.exportCsv, localized:{fieldLabel:'MONITOR_EXPORT_CSV', tooltip:'MONITOR_EXPORT_CSV_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.exportCsvUseCustomColumnHeaders, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_CSV_USE_CUSTOM_COLUMN_HEADERS', 
  tooltip:'MONITOR_EXPORT_CONFIG_CSV_USE_CUSTOM_COLUMN_HEADERS_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.exportCsvInclHidden, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_CSV_INCL_HIDDEN', tooltip:'MONITOR_EXPORT_CONFIG_CSV_INCL_HIDDEN_TOOLTIP'}}, {xtype:'bpcTextField', property:monitor.assets.ConfigKeys.exportCsvSeparator, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_CSV_SEPARATOR', tooltip:'MONITOR_EXPORT_CONFIG_CSV_SEPARATOR_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'export', 
  title:BpcCommon.Api.getTranslation('MONITOR_JSON_EXPORT_CONFIGURATION'), collapsed:false, fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.exportJson, localized:{fieldLabel:'MONITOR_EXPORT_JSON', tooltip:'MONITOR_EXPORT_JSON_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.exportJsonUseCustomColumnHeaders, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_JSON_USE_CUSTOM_COLUMN_HEADERS', tooltip:'MONITOR_EXPORT_CONFIG_JSON_USE_CUSTOM_COLUMN_HEADERS_TOOLTIP'}}, 
  {xtype:'checkbox', property:monitor.assets.ConfigKeys.exportJsonInclHidden, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_JSON_INCL_HIDDEN', tooltip:'MONITOR_EXPORT_CONFIG_JSON_INCL_HIDDEN_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.exportJsonUseUTCDateFormat, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_JSON_USE_UTC_DATE_FORMAT', tooltip:'MONITOR_EXPORT_CONFIG_JSON_USE_UTC_DATE_FORMAT_TOOLTIP'}}]}, {xtype:'bpcDataBindFieldSet', category:'export', title:BpcCommon.Api.getTranslation('MONITOR_XLSX_EXPORT_CONFIGURATION'), 
  collapsed:false, fieldComponents:[{xtype:'checkbox', property:monitor.assets.ConfigKeys.exportXlsx, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_XLSX', tooltip:'MONITOR_EXPORT_CONFIG_XLSX_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.exportXlsxAutoResizeColumns, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_XLSX_AUTO_RESIZE_COLUMNS', tooltip:'MONITOR_EXPORT_CONFIG_XLSX_AUTO_RESIZE_COLUMNS_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.exportXlsxInclHidden, 
  localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_XLSX_INCL_HIDDEN', tooltip:'MONITOR_EXPORT_CONFIG_XLSX_INCL_HIDDEN_TOOLTIP'}}, {xtype:'checkbox', property:monitor.assets.ConfigKeys.exportXlsxUseCustomColumnHeaders, localized:{fieldLabel:'MONITOR_EXPORT_CONFIG_XLSX_USE_CUSTOM_COLUMN_HEADERS', tooltip:'MONITOR_EXPORT_CONFIG_XLSX_USE_CUSTOM_COLUMN_HEADERS_TOOLTIP'}}]}];
  me.formComponents = fieldSetList.map(function(el) {
    return Ext.apply(el, defaultAttributes);
  });
  me.defineMinimalFormFields();
  me.activeFormComponents = me.formComponents;
}, getFormComponents:function() {
  var me = this;
  return me.activeFormComponents;
}, getAllFields:function() {
  var me = this;
  var fieldsList = [];
  me.formComponents.map(function(el) {
    fieldsList = fieldsList.concat(el.fieldComponents);
    return {};
  });
  fieldsList = fieldsList.map(function(el) {
    return {property:el.property, fieldLabel:el.localized.fieldLabel, tooltip:el.localized.tooltip, isMinimalConfig:el.isMinimalConfig};
  });
  return fieldsList;
}, defineMinimalFormFields:function() {
  var me = this;
  var minimalFormFieldProperties = me.getMinimalFieldProperties();
  me.minimalFormComponents = me.createFormFieldsFromPropertyList(minimalFormFieldProperties);
}, getMinimalFormComponents:function() {
  var me = this;
  return me.minimalFormComponents;
}, getMinimalFieldProperties:function() {
  var me = this;
  var minimalFormComponents = me.getAllFields().filter(function(el) {
    return el.isMinimalConfig === true;
  });
  return minimalFormComponents.map(function(el) {
    return el.property;
  });
}, findSearchListEntries:function(searchString) {
  var me = this;
  var lowerCaseSearchString = searchString.toLowerCase();
  var minLengthForPartialSearch = 3;
  var searchResults;
  if (searchString.length >= minLengthForPartialSearch) {
    searchResults = me.findPartialWordsSearchListEntries(lowerCaseSearchString);
  } else {
    searchResults = me.findWholeWordsSearchListEntries(lowerCaseSearchString);
  }
  return searchResults;
}, findPartialWordsSearchListEntries:function(searchString) {
  var me = this;
  var foundFields = me.searchStringList.filter(function(listElement) {
    var isSearchResult = false;
    Ext.Array.each(listElement.searchString, function(entry) {
      var regExResult = me.regExTest(searchString, entry);
      if (regExResult === true) {
        isSearchResult = true;
      }
    });
    return isSearchResult;
  });
  return me.extractPropertiesFromSearchResult(foundFields);
}, regExTest:function(patternString, testString) {
  var me = this;
  var cleanPatternString = me.clearStringForRegex(patternString);
  var cleanTestString = me.clearStringForRegex(testString);
  var result = false;
  if (Ext.isEmpty(cleanTestString) || Ext.isEmpty(cleanPatternString)) {
    return result;
  }
  try {
    result = (new RegExp(cleanPatternString)).test(cleanTestString);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR regex expression', dump:e, stack:true, level:'error'});
  }
  return result;
}, clearStringForRegex:function(searchString) {
  var blackListChar = ['*', '/'];
  var clearString = searchString;
  Ext.Array.each(blackListChar, function(el) {
    clearString = clearString.replaceAll(el, '');
  });
  return clearString;
}, findWholeWordsSearchListEntries:function(searchString) {
  var me = this;
  var foundFields = me.searchStringList.filter(function(el) {
    return el.searchString.includes(searchString);
  });
  return me.extractPropertiesFromSearchResult(foundFields);
}, extractPropertiesFromSearchResult:function(searchResults) {
  var propertyList = searchResults.map(function(el) {
    return el.property;
  });
  return Ext.Array.unique(propertyList);
}, createFormFieldsFromPropertyList:function(propertyList) {
  var me = this;
  var clonedFormComponents = Ext.clone(me.formComponents);
  Ext.Array.each(clonedFormComponents, function(fieldSet) {
    Ext.Array.each(fieldSet.fieldComponents, function(uiEl, fieldElementIdx, fieldElementsArray) {
      var isFoundUiEl = propertyList.includes(uiEl.property);
      if (!isFoundUiEl) {
        fieldElementsArray[fieldElementIdx] = undefined;
      }
    }, me);
    fieldSet.fieldComponents = fieldSet.fieldComponents.filter(function(el) {
      return el !== undefined;
    });
  }, me);
  return clonedFormComponents.filter(function(el) {
    return !Ext.isEmpty(el.fieldComponents);
  });
}});
Ext.define('monitor.moduleConfig.mixins.localStorageStateManager', {readUIStateFromLocalStorage:function() {
  var me = this;
  var localStorageEntry = Ext.state.Manager.get(me.localStorageMonitorConfigurationUIStateKey);
  if (!Ext.isString(localStorageEntry)) {
    return;
  }
  try {
    var localStorageObj = JSON.parse(localStorageEntry);
    me.applyMonitorUIState(localStorageObj);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': error parsing monitor configuration UI State from local storage', dump:e, stack:true, level:'error'});
  }
}, checkIfRenderingStyleIsValid:function(renderingStyle) {
  var me = this;
  var renderingStylePicker = me.getMonitorRenderingStylePicker();
  var store = renderingStylePicker.getStore();
  var validRenderingStyleList = store.getRange().map(function(el) {
    return el.get('value');
  });
  return validRenderingStyleList.some(function(el) {
    return el === renderingStyle;
  });
}, applyMonitorUIState:function(stateObj) {
  var me = this;
  var renderingStylePicker = me.getMonitorRenderingStylePicker();
  var renderingStyle = stateObj['renderingStyle'];
  if (me.checkIfRenderingStyleIsValid(renderingStyle)) {
    renderingStylePicker.setValue(renderingStyle);
  }
  var searchStringField = me.getMonitorSearchField();
  searchStringField.setValue(stateObj['searchString']);
  var tabPanel = me.getMonitorConfigTabPanel();
  var tabIndex = stateObj['tabIndex'];
  if (Ext.isNumber(tabIndex)) {
    tabPanel.setActiveTab(tabIndex);
  }
}, writeUIStateToLocalStorage:function() {
  var me = this;
  var renderingStyle = me.getMonitorRenderingStyle();
  var searchString = me.getMonitorSearchString();
  var tabIndex = me.getMonitorConfigActiveTabIndex();
  var localStorageItem = {renderingStyle:renderingStyle, searchString:searchString};
  if (me.shouldApplyTabIndex(localStorageItem)) {
    Ext.apply(localStorageItem, {tabIndex:tabIndex});
  }
  Ext.state.Manager.set(me.localStorageMonitorConfigurationUIStateKey, JSON.stringify(localStorageItem));
}, shouldApplyTabIndex:function(stateObj) {
  var renderingStyle = stateObj['renderingStyle'];
  var searchString = stateObj['searchString'];
  return renderingStyle === 'tabbed' && Ext.isEmpty(searchString);
}, getMonitorConfigActiveTabIndex:function() {
  var me = this;
  var failureValue = 0;
  var tabPanel = me.getMonitorConfigTabPanel();
  var activeTab = tabPanel.getActiveTab();
  if (!Ext.isObject(activeTab) || !Ext.isObject(tabPanel.items) || !Ext.isFunction(tabPanel.items.indexOf)) {
    return failureValue;
  }
  return tabPanel.items.indexOf(activeTab);
}, getMonitorConfigTabPanel:function() {
  var me = this;
  var failureValue = {getActiveTab:Ext.emptyFn, setActiveTab:Ext.emptyFn};
  var tabPanel = me.down('[isMonitorConfigTabPanel]');
  return Ext.isObject(tabPanel) ? tabPanel : failureValue;
}, getMonitorRenderingStyle:function() {
  var me = this;
  var renderingStylePicker = me.getMonitorRenderingStylePicker();
  return renderingStylePicker.getValue();
}, getMonitorRenderingStylePicker:function() {
  var me = this;
  var failureValue = {getValue:function() {
    return 'minimal';
  }, setValue:Ext.emptyFn, getStore:function() {
    return Ext.create('Ext.data.Store');
  }};
  var renderingStylePicker = me.down('[isMonitorFormRenderingStylePicker]');
  return Ext.isObject(renderingStylePicker) ? renderingStylePicker : failureValue;
}, getMonitorSearchString:function() {
  var me = this;
  var searchField = me.getMonitorSearchField();
  return searchField.getValue();
}, getMonitorSearchField:function() {
  var me = this;
  var failureValue = {getValue:function() {
    return '';
  }, setValue:Ext.emptyFn};
  var searchField = me.down('[isMonitorFormSearchField]');
  return Ext.isObject(searchField) ? searchField : failureValue;
}});
Ext.define('monitor.moduleConfig.configurationMain', {extend:'monitor.moduleConfig.components.MainReference', alias:'widget.bpcMonitorConfigurationMain', mixins:['BpcCommon.component.moduleUI.StoreManager', 'BpcCommon.component.moduleUI.ConfigurationMixin', 'monitor.moduleConfig.mixins.formComponents', 'BpcCommon.component.moduleUI.helpText.HelpTextManager', 'monitor.moduleConfig.mixins.localStorageStateManager'], localStorageHelpTextContainerHiddenKey:'monitorConfigurationIsUIHelpTextHidden', localStorageMonitorConfigurationUIStateKey:'monitorConfigurationUIState', 
defaultConfigs:[], config:{formRenderingStyle:'minimal', selectedTab:0}, configList:[], defaultConfigList:[], searchBuffer:250, localStorageReadBuffer:50, hasAddNavigationButton:false, initComponent:function() {
  var me = this;
  me.callParent();
  me.on('afterrender', me.onSubAfterRenderHook, me);
  me.on('monitorConfigTabChange', me.onMonitorConfigTabChange, me);
}, readUIState:function() {
  var me = this;
  if (Ext.isFunction(me.readUIStateFromLocalStorage)) {
    me.readUIStateFromLocalStorage.call(me);
  }
}, writeUIState:function() {
  var me = this;
  if (Ext.isFunction(me.writeUIStateToLocalStorage)) {
    me.writeUIStateToLocalStorage();
  }
}, onSubAfterRenderHook:function() {
  var me = this;
  setTimeout(function() {
    me.readUIState();
  }, me.localStorageReadBuffer);
}, onMonitorConfigTabChange:function() {
  var me = this;
  me.writeUIState();
}, getIsSearchActive:function() {
  var me = this;
  return me.isSearchActive;
}, setIsSearchActive:function(value) {
  var me = this;
  me.isSearchActive = value;
}, onChangeFormStyleRendering:function(val) {
  var me = this;
  me.setConfig('formRenderingStyle', val);
  me.refreshUIComponents();
  me.writeUIState();
}, onChangeSearchFieldValue:function(cmp) {
  var me = this;
  me.enableLoadingAnimation();
  var searchString = cmp.getValue();
  var isSearchActive = !Ext.isEmpty(searchString);
  me.setIsSearchActive(isSearchActive);
  if (!isSearchActive) {
    me.activateAllResults();
  } else {
    me.activateSearchResults(searchString);
  }
  me.disableLoadingAnimation();
  me.writeUIState();
}, activateAllResults:function() {
  var me = this;
  me.activeFormComponents = me.formComponents;
  me.refreshUIComponents();
}, activateSearchResults:function(searchString) {
  var me = this;
  var foundFieldsProperties = me.findSearchListEntries(searchString);
  me.activeFormComponents = me.createFormFieldsFromPropertyList(foundFieldsProperties);
  if (Ext.isEmpty(foundFieldsProperties)) {
    me.showNoSearchResultMessage(searchString);
  } else {
    me.refreshUIComponents();
  }
}, showNoSearchResultMessage:function(searchString) {
  var me = this;
  me.removeAll();
  me.add({xtype:'bpcCenterMessageCmp', message:{key:'MONITOR_NO_SEARCH_RESULTS', data:{searchString:searchString || ''}}, margin:'250 0 0 0'});
}, clearSearchField:function(textField) {
  textField.setValue('');
}, createSearchFieldList:function() {
  var me = this;
  if (!Ext.isEmpty(me.searchStringList)) {
    return;
  }
  var list = [];
  var uiItems = me.getAllFields();
  Ext.each(uiItems, function(field) {
    var fieldProperty = field.property;
    var seperatedFieldProperty = fieldProperty.replaceAll('_', ' ');
    var toolTip = BpcCommon.Api.getTranslation(field.tooltip);
    var fieldLabel = BpcCommon.Api.getTranslation(field.fieldLabel);
    var valueString = me.createStringOfValue(fieldProperty);
    var searchString = fieldProperty + ' ' + seperatedFieldProperty + ' ' + toolTip + ' ' + fieldLabel + ' ' + valueString;
    searchString = searchString.toLowerCase();
    searchString.replaceAll('.', ' ');
    searchString.replaceAll(',', ' ');
    var stringList = searchString.split(' ');
    stringList = Ext.Array.unique(stringList);
    var newListItem = {searchString:stringList, property:fieldProperty};
    list.push(newListItem);
  }, me);
  me.searchStringList = list;
}, createStringOfValue:function(property) {
  var me = this;
  var value = me.getConfigValue(property);
  var retString = '';
  var valueType = typeof value;
  if (valueType === 'string' || valueType === 'number') {
    retString = value;
  }
  return retString;
}, enableLoadingAnimation:function() {
  var me = this;
  var message = BpcCommon.Api.getTranslation('CORE_LOADING');
  me.mask(message);
}, disableLoadingAnimation:function() {
  var me = this;
  me.unmask();
}, renderBottomBar:function() {
  var me = this;
  var formStylePicker = {xtype:'bpcMonitorFormRenderingStylePicker', isMonitorFormRenderingStylePicker:true, changeHandler:me.onChangeFormStyleRendering.bind(me)};
  var filterField = {xtype:'bpcTextField', emptyText:BpcCommon.Api.getTranslation('CORE_FILTER'), isMonitorFormSearchField:true, width:120, listeners:{change:{fn:me.onChangeSearchFieldValue.bind(me), buffer:me.searchBuffer, delay:me.searchBuffer}}, triggers:{clear:{cls:BpcCommon.Icons.CANCEL, handler:me.clearSearchField.bind(me)}}};
  var standardBottomBarItems = BpcCommon.component.moduleUI.Helpers.getModuleConfigurationBottomToolbar(me);
  standardBottomBarItems[0].items.splice(4, 0, filterField);
  standardBottomBarItems[0].items.splice(4, 0, formStylePicker);
  me.dockedItems = standardBottomBarItems;
}, renderUIComponents:function() {
  var me = this;
  me.defineFormComponents();
  me.refreshUIComponents();
  me.createSearchFieldList();
}});
Ext.define('monitor.moduleConfig.creationMain', {extend:'monitor.moduleConfig.components.MainReference', alias:'widget.bpcMonitorCreationMain', mixins:['BpcCommon.component.moduleUI.StoreManager', 'BpcCommon.component.moduleUI.CreationMixin', 'monitor.moduleConfig.mixins.formComponents', 'BpcCommon.component.moduleUI.helpText.HelpTextManager'], localStorageHelpTextContainerHiddenKey:'monitorCreationIsUIHelpTextHidden', config:{formRenderingStyle:'minimal', selectedTab:0}, defaultConfigs:[], configList:[], 
defaultConfigList:[], isModuleIdReadOnly:false, hideAddToNavigationCheckBox:false, hasAddNavigationButton:true, addToNavigationDefaultValue:true, renderUIComponents:function() {
  var me = this;
  me.defineFormComponents();
  me.refreshUIComponents();
}, getIsSearchActive:function() {
  return false;
}});
Ext.define('monitor.plugin.ColumnHook', {extend:'Ext.grid.column.Column', alias:'widget.monitorColumnHook', context:{moduleId:undefined}, hidden:true, hideable:false, dataIndex:'monitorColumnHook', pluginPromiseArray:[], columnDefaults:{items:[{xtype:'textfield', editable:false, fieldStyle:'cursor: not-allowed;', margin:0, ui:'columnHeaderField'}], stateId:'plugin_column', columnId:'plugin_column', minWidth:32}, initComponent:function() {
  var me = this;
  me.context.baseModuleId = 'monitor';
  var activeInstance = me.context.moduleId;
  me.stateId = me.columnId;
  me.callParent();
  if (!activeInstance) {
    return;
  }
  me.pluginPromiseArray = BpcCommon.Api.getPluginComponentsAsync('monitor_column', me.context);
  me.pluginPromiseArray.forEach(function(pluginPromise) {
    pluginPromise.then(me.onPluginResolve.bind(me, pluginPromise), me.onPluginReject.bind(me, pluginPromise));
  });
}, onPluginResolve:function(pluginPromise, pluginCmp) {
  if (!BpcCommon.Api.isUserAllowedToSee(pluginCmp.pluginConfiguration)) {
    return;
  }
  var me = this;
  var columnConfig = pluginCmp;
  if (pluginCmp.pluginConfiguration && pluginCmp.pluginConfiguration.configuration && pluginCmp.pluginConfiguration.configuration.wrap === true) {
    columnConfig = me.wrapCmpWithColumn(pluginCmp);
  }
  if (pluginCmp.pluginConfiguration && pluginCmp.pluginConfiguration.configuration && pluginCmp.pluginConfiguration.configuration.columnConfig) {
    columnConfig = Ext.apply(columnConfig, pluginCmp.pluginConfiguration.configuration.columnConfig);
  }
  columnConfig = Ext.apply({xtype:'gridcolumn', stateId:pluginCmp.pluginConfiguration.id, columnId:pluginCmp.pluginConfiguration.id, moduleId:me.context.moduleId}, columnConfig, me.columnDefaults);
  var pluginCmpInstance = columnConfig.isComponent ? columnConfig : Ext.create(columnConfig);
  if (!(pluginCmpInstance instanceof Ext.grid.column.Column)) {
    Ext.log({msg:this.self.getName() + ': onPluginResolve - failed to add plugin ' + pluginCmpInstance.id + ' as column, please use configuration wrap:true', level:'warn'});
    pluginCmp = me.wrapCmpWithColumn(pluginCmp);
    pluginCmpInstance = me.ownerCt.add(pluginCmp);
  } else {
    me.ownerCt.add(pluginCmpInstance);
  }
  me.ownerCt.moveBefore(pluginCmpInstance, me);
  if (!pluginCmpInstance.text || pluginCmpInstance.text === ' ') {
    pluginCmpInstance.setText(BpcCommon.Api.getTranslation('MONITOR_PLUGIN_COLUMN'));
  }
  me.removePluginPromise(pluginPromise);
}, onPluginReject:function(pluginPromise, errorMsg) {
  var me = this;
  Ext.log({msg:me.self.getName() + ': onPluginReject: Plugin loading failed:\n' + errorMsg, level:'error'});
  me.removePluginPromise(pluginPromise);
}, removePluginPromise:function(pluginPromise) {
  var me = this;
  Ext.Array.remove(me.pluginPromiseArray, pluginPromise);
  if (me.pluginPromiseArray.length === 0 && me.items.getCount() === 0) {
    me.hide();
  }
}, wrapCmpWithColumn:function(cmp) {
  return Ext.apply({xtype:'widgetcolumn', widget:cmp, text:this.text}, this.columnDefaults);
}});
Ext.define('monitor.plugin.ColumnHookInterface', {extend:'BpcCommon.plugin.HookContainer', inheritableStatics:{NAME:'Column', HOOK_ID:'monitor_column', MODULE:'monitor'}});
Ext.define('monitor.plugin.DetailHook', {extend:'BpcCommon.plugin.HookContainer', alias:'widget.monitorDetailHook', title:'Plugins', iconCls:BpcCommon.Icons.PLUGIN, inheritableStatics:{NAME:'Detail View', HOOK_ID:'monitor_detail_view', MODULE:'monitor'}, context:{baseModuleId:'monitor'}, autoInjectPlugins:true, pluginTarget:'self', hidden:false, onRecordUpdate:function(newRecord) {
  this.refresh({record:newRecord});
}});
Ext.define('monitor.plugin.Instance', {extend:'BpcCommon.plugin.HookContainer', alias:'widget.monitorInstanceHook', inheritableStatics:{NAME:'Monitor Instance', HOOK_ID:'monitor_root', MODULE:'monitor'}});
Ext.define('monitor.plugin.wrapper.PluginInterface', {extend:'BpcCommon.plugin.Interface', inheritableStatics:{NAME:'Monitor Wrapper', COMPONENT_XTYPE:'monitorWrapperPlugin', PLUGIN_ID:'monitor_wrapper', DEFAULT_CONFIGURATION:{hideMonitorLinkBtn:true, wrapperConfiguration:{title:'Ein Monitortitel', iconCls:'x-fa fa-fire'}, monitorConfiguration:{moduleCfg:{state:{storeState:{filters:[{property:'MONITOR_SPALTE', source:'raw', value:['FILTER_WERT']}]}}}}}}});
Ext.define('monitor.plugin.wrapper.Wrapper', {extend:'Ext.Container', alias:'widget.monitorWrapperPlugin', layout:'card', monitorLinkBtn:undefined, monitorLinkBtnPosition:'tr-tr', viewModel:{parent:BpcCommon.Api.getGlobalViewModel()}, initComponent:function() {
  var me = this;
  me.items = [];
  if (!(me.pluginConfiguration && me.pluginConfiguration.configuration && (me.pluginConfiguration.configuration.monitorId || me.pluginConfiguration.configuration.moduleId))) {
    me.items.push({xtype:'bpcCenterMessageCmp', message:'Missing moduleId config'});
    me.callParent();
    return;
  } else if (!BpcCommon.Util.isStringEqual(BpcCommon.Api.getBaseModuleIdById(me.pluginConfiguration.configuration.moduleId || me.pluginConfiguration.configuration.monitorId), 'monitor')) {
    me.items.push({xtype:'bpcCenterMessageCmp', message:"can't get monitor instance for configured moduleId"});
    me.callParent();
    return;
  }
  Ext.applyIf(me, me.pluginConfiguration.configuration.wrapperConfiguration);
  if (me.pluginConfiguration.configuration.monitorConfiguration && me.pluginConfiguration.context.record) {
    var stringConfig = JSON.stringify(me.pluginConfiguration.configuration.monitorConfiguration);
    Ext.Object.each(me.pluginConfiguration.context.record.getData(), function(key, value) {
      stringConfig = stringConfig.replaceAll('record.' + key, value);
    });
    me.pluginConfiguration.configuration.monitorConfiguration = JSON.parse(stringConfig);
  }
  me.items.push({xtype:'bpcCenterMessageCmp', message:'CORE_LOADING', messageIconCls:'x-fal fa-spinner fa-spin'});
  me.callParent();
  me.on('afterrender', function() {
    if (me.isVisible()) {
      me.loadMonitor();
    } else {
      me.on('show', me.loadMonitor, me, {single:true});
    }
  }, me, {single:true});
  me.on('show', function() {
    me.query('monitorGrid').forEach(function(monitorGrid) {
      monitorGrid.getController().onViewChange();
    });
  });
}, loadMonitor:function() {
  var me = this;
  Ext.defer(function() {
    var stateIdPrefix = me.pluginConfiguration.configuration.stateIdPrefix;
    stateIdPrefix = stateIdPrefix ? stateIdPrefix : 'defaultStateIdPrefix';
    me.monitorCmp = me.add(Ext.applyIf({xtype:'bpcMonitor', moduleId:me.pluginConfiguration.configuration.moduleId || me.pluginConfiguration.configuration.monitorId, stateIdPrefix:stateIdPrefix}, me.pluginConfiguration.configuration.monitorConfiguration));
    me.getLayout().setActiveItem(me.monitorCmp);
  }, 100);
  if (me.pluginConfiguration.configuration.hideMonitorLinkBtn !== true) {
    me.monitorLinkBtn = me.add({xtype:'button', floating:true, iconCls:BpcCommon.Icons.LINK, handler:BpcCommon.Api.showModule.bind(BpcCommon.Api, me.pluginConfiguration.configuration.moduleId || me.pluginConfiguration.configuration.monitorId, false, {}, [], false)});
    if (me.pluginConfiguration.configuration.monitorLinkBtnPosition) {
      me.monitorLinkBtnPosition = me.pluginConfiguration.configuration.monitorLinkBtnPosition;
    }
    me.monitorLinkBtn.showBy(me, me.monitorLinkBtnPosition);
  }
}, onResize:function() {
  var me = this;
  if (me.monitorLinkBtn) {
    me.monitorLinkBtn.showBy(me, me.monitorLinkBtnPosition);
  }
}});
Ext.define('monitor.store.BpcStorage', {extend:'Ext.data.Store', model:'monitor.model.StorageRecord', config:{bpcStorageStoreId:undefined, baseModuleId:undefined, moduleId:undefined}, listeners:{add:'onAdd', beforeload:'setProxyParameters'}, setProxyParameters:function() {
  var me = this;
  me.getProxy().setExtraParam('bpcStorageStoreId', me.getBpcStorageStoreId());
  me.getProxy().setExtraParam('filter', '[\n          {"property":"moduleId","operator":"\x3d","value":"' + me.getBaseModuleId() + '","source":"raw","invert":false},\n          {"property":"moduleInstanceId","operator":"\x3d","value":"' + me.getModuleId() + '","source":"raw","invert":false}\n          ]');
}, onAdd:function(cmp, records) {
  records.forEach(function(r) {
    r.set('moduleId', cmp.getBaseModuleId());
    r.set('moduleInstanceId', cmp.getModuleId());
  });
}});
Ext.define('monitor.store.MonitorViewStates', {extend:'monitor.store.BpcStorage', model:'monitor.model.MonitorViewState', baseModuleId:'monitor', bpcStorageStoreId:'monitor-views', sorters:'priority', insert:function(index, records) {
  var me = this;
  var previousRecord = me.getAt(index - 1);
  var followingRecord = me.getAt(index);
  var priorityFactorSum = 0;
  records = Ext.isArray(records) ? records : [records];
  if (!previousRecord) {
    previousRecord = monitor.model.MonitorViewState.create({priority:0, readRoles:['bpcuser']});
  }
  if (!followingRecord) {
    var lastRecord = me.last() || records[0];
    followingRecord = monitor.model.MonitorViewState.create({priority:lastRecord.get('priority') + 100, readRoles:['bpcuser']});
  }
  var lowestPriority = previousRecord.get('priority');
  var highestPriority = followingRecord.get('priority');
  var firstPriorityFactor = previousRecord.getPriorityFactor();
  priorityFactorSum = priorityFactorSum + firstPriorityFactor;
  var lastPriorityFactor = followingRecord.getPriorityFactor();
  priorityFactorSum = priorityFactorSum + lastPriorityFactor;
  records.forEach(function(record) {
    priorityFactorSum = priorityFactorSum + record.getPriorityFactor();
  });
  var priorityUpperLimit = me.getPriorityUpperLimit(firstPriorityFactor, lastPriorityFactor, lowestPriority, highestPriority, priorityFactorSum);
  records.forEach(function(record) {
    var priority = me.getPriority(previousRecord, record, priorityUpperLimit, priorityFactorSum);
    record.set('priority', priority);
    previousRecord = record;
  });
  me.callParent(arguments);
}, add:function() {
  var me = this;
  me.callParent(arguments);
}, getPriorityUpperLimit:function(firstPriorityFactor, lastPriorityFactor, lowestPriority, highestPriority, priorityFactorSum) {
  return (highestPriority - lowestPriority) / (1 - lastPriorityFactor / priorityFactorSum / 2 - firstPriorityFactor / priorityFactorSum / 2);
}, getHalfDistance:function(record, priorityUpperLimit, priorityFactorSum) {
  return record.getPriorityFactor() / priorityFactorSum * priorityUpperLimit / 2;
}, getPriority:function(previousRecord, record, priorityUpperLimit, priorityFactorSum) {
  var me = this;
  return previousRecord.get('priority') + me.getHalfDistance(previousRecord, priorityUpperLimit, priorityFactorSum) + me.getHalfDistance(record, priorityUpperLimit, priorityFactorSum);
}});
Ext.define('monitor.store.State', {extend:'Ext.data.Store', alias:'store.monitorState', requires:['monitor.model.State'], model:'monitor.model.State', stateModel:undefined, setStateModel:function(stateModel) {
  this.stateModel = stateModel;
  this.setData(stateModel.get('value'));
}, getStateModel:function() {
  return this.stateModel;
}});
Ext.define('monitor.store.Store', {extend:'Ext.data.BufferedStore', alias:'store.process', leadingBufferZone:50, trailingBufferZone:50, pageSize:300, autoLoad:false, proxy:{type:'ajax', url:undefined, reader:{type:'json', rootProperty:'data', totalProperty:'totalCount', keepRawData:true}, extraParams:{query:''}, actionMethods:{create:'POST', read:'POST', update:'POST', destroy:'POST'}}, constructor:function(config) {
  var me = this;
  me.callParent([config]);
  me.on('beforeprefetch', me.updateStoreParams, me);
  me.on('beforeload', me.fixReloadParams, me);
}, updateStoreParams:function(store, operation) {
  var me = this;
  var params = operation.getParams() || {};
  params.queryId = me.queryId + '_page:' + operation.getPage();
  operation.setParams(params);
}, fixReloadParams:function(store, options) {
  var me = this;
  if (!me.lastRequestStart || !me.preserveScrollOnReload) {
    options.count = options.count || me.viewSize;
  }
}, sort:function(field, direction, mode) {
  var me = this;
  var bpcSortMode = this.multiSortMode;
  var sorters = me.getSorters();
  if (sorters.getByKey(field)) {
    var oldDirection = sorters.getByKey(field).getDirection();
    if (oldDirection === 'ASC') {
      direction = 'DESC';
    } else {
      direction = 'ASC';
    }
  }
  if (arguments.length === 0) {
    if (me.getRemoteSort()) {
      me.load();
    } else {
      me.forceLocalSort();
    }
  } else {
    mode = bpcSortMode ? 'append' : mode;
    if (mode === 'append') {
      var multiSortLimit = sorters.getMultiSortLimit();
      var nrOfCurrentSorters = sorters.length;
      if (nrOfCurrentSorters >= multiSortLimit && !sorters.containsKey(field)) {
        sorters.removeAt(0);
      }
    }
    sorters.addSort(field, direction, mode);
  }
}, getState:function() {
  try {
    var result = this.callParent();
    if (result && result.grouper) {
      delete result.grouper;
    }
    return result;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR getState', dump:e, stack:true, level:'error'});
  }
}, getData:function() {
  var me = this;
  var dataObject = me.callParent(arguments);
  Ext.apply(dataObject, {getGroups:Ext.emptyFn});
  return dataObject;
}});
Ext.define('monitor.store.StoreChained', {extend:'Ext.data.ChainedStore'});
Ext.define('monitor.view.Main', {extend:'Ext.panel.Panel', alias:'widget.bpcMonitor', layout:'border', config:{moduleCfg:undefined, iconCls:monitor.assets.Icons.MONITOR, activeGrid:undefined}, controller:'monitorController', viewModel:{type:'monitorMainViewModel'}, bind:{title:'{monitorTitle:translate} - {totalItems} {monitorData:translate} - {activeFilter} {monitorFilter:translate}' + ' - {activeSorter} {monitorSorter:translate} {activeSortOrder}'}, stateful:true, stateId:undefined, moduleCfg:undefined, 
moduleRoute:undefined, moduleId:undefined, stateIdPrefix:undefined, moduleActionsMap:{}, tools:[{iconCls:BpcCommon.Icons.SETTING, bind:{hidden:'{!global.role.bpcadmin}'}, handler:function() {
  var activeModuleId = BpcCommon.Api.getActiveModuleId();
  BpcCommon.Api.showModule('_core', false, undefined, [BpcCommon.Api.getBaseModuleIdById(activeModuleId), activeModuleId]);
}}], plugins:{monitorCustomRenderer:true}, initComponent:function() {
  var me = this;
  var vm = me.getViewModel();
  var errorState = me.verifyMonitorConfig();
  me.itemId = me.xtype + '_' + me.moduleId;
  BpcCommon.Performance.mark('monitor_main_init_start');
  try {
    me.initModuleActionsMap();
    var moduleHeaderContent = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.moduleHeaderContent);
    if (!Ext.isArray(moduleHeaderContent)) {
      moduleHeaderContent = [{id:'metadata', config:{showFilter:false, showSorter:false}}, 'favorites', 'search', 'changestate', 'processaction', 'export', 'views', 'refresh'];
    }
    var appToolbarLeftContent = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.appToolbarLeftContent);
    if (!Ext.isArray(appToolbarLeftContent)) {
      appToolbarLeftContent = [{id:'refresh', config:{switchAutoUpdate:false}}];
    }
    var appToolbarRightContent = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.appToolbarRightContent);
    if (!Ext.isArray(appToolbarRightContent)) {
      appToolbarRightContent = [];
    }
    me.applicationToolbarLeft = {xtype:'toolbar', itemId:'monitorToolbarLeft_' + me.moduleId, defaults:{ui:'apptoolbardefault'}, items:me.getModuleActions(appToolbarLeftContent, 'monitorToolbarLeft')};
    me.applicationToolbarRight = {xtype:'toolbar', itemId:'monitorToolbarRight_' + me.moduleId, defaults:{ui:'apptoolbardefault'}, items:me.getModuleActions(appToolbarRightContent, 'monitorToolbarRight')};
    me.bpcModuleHeader = {xtype:'toolbar', itemId:'monitorModuleHeaderToolbar', layout:{pack:'end'}, defaults:{ui:'apptoolbardefault'}, overflowHandler:'scroller', items:me.getModuleActions(moduleHeaderContent, 'monitorModuleHeaderToolbar')};
    vm.set('moduleId', me.moduleId);
    vm.set('moduleInstance', me);
    me.stateId = me.stateIdPrefix ? me.stateIdPrefix + '_' + me.moduleId : 'monitorMainView_' + me.moduleId;
    me.header = !me.getSetting(monitor.assets.ConfigKeys.moduleHeaderEnabled);
    me.instanceStoreUrl = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + me.moduleId;
    vm.set('monitorTitle', me.getSetting('module_name'));
    var hasUserRight = BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.editMonitorViews) || BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.editMonitorViews + '_' + me.moduleId);
    vm.set(monitor.assets.ModuleRights.editMonitorViews, hasUserRight);
    if (errorState.configError === false) {
      me.items = me.createGrids();
    } else {
      me.items = [{xtype:'bpcCenterMessageCmp', message:errorState.msg, messageIconCls:errorState.iconCls}];
    }
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR initComponent', dump:e, stack:true, level:'error'});
  }
  me.callParent();
  if (!me.getActiveGrid()) {
    me.setActiveGrid(me.down('monitorGrid'));
  }
  vm.set('autoRefreshDisabled', !BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.autoUpdate));
  me.updateInstance(me.moduleCfg, me.moduleRoute);
  BpcCommon.Performance.markAndMeasure('monitor_main_init', 'monitor_main_init_start', 'monitor_main_init_end');
}, verifyMonitorConfig:function() {
  var me = this;
  var configDataIndex = me.getSetting(monitor.assets.ConfigKeys.dataIndex);
  var errorState = {configError:true, msg:BpcCommon.Api.getTranslation('MONITOR_MISSING_CONFIG'), iconCls:''};
  if (configDataIndex === undefined || configDataIndex === '') {
    errorState.msg += BpcCommon.Api.getTranslation('MONITOR_NO_INDEX_SET');
    errorState.iconCls = 'fas fa-exclamation-triangle';
  } else {
    errorState.configError = false;
  }
  return errorState;
}, createGrids:function() {
  try {
    var items = [];
    var me = this;
    var mainView;
    var mainGridObj = {id:'mainGrid'};
    var multipleGrids = this.getSetting(monitor.assets.ConfigKeys.multipleGrids);
    if (multipleGrids) {
      this.isMultiGrid = true;
      var gridMultipleViewsConfig = this.getSetting(monitor.assets.ConfigKeys.multipleGridsConfig);
      var grids = [];
      var activeGrid = 0;
      Ext.Array.each(gridMultipleViewsConfig.views, function(view) {
        if (BpcCommon.Api.isUserAllowedToSee(view)) {
          var grid = me.gridFactory(view);
          grids.push(grid);
        }
      });
      if (activeGrid === 0 && Ext.isObject(gridMultipleViewsConfig.defaultView)) {
        var initialActiveGrid = gridMultipleViewsConfig.defaultView;
        var checkFunction = function(object, checkFn) {
          var result = null;
          if (object && Ext.isObject(object)) {
            var keyArray = Ext.Object.getKeys(object);
            Ext.Array.each(keyArray, function(key) {
              if (checkFn(key)) {
                result = object[key];
                return false;
              }
            });
          }
          return result;
        };
        var viewId = checkFunction(initialActiveGrid.organisation, BpcCommon.Api.hasUserOrganisation.bind(BpcCommon.Api));
        if (!viewId) {
          viewId = checkFunction(initialActiveGrid.role, BpcCommon.Api.hasUserRole.bind(BpcCommon.Api));
        }
        if (!viewId) {
          viewId = checkFunction(initialActiveGrid.right, BpcCommon.Api.hasUserRight.bind(BpcCommon.Api));
        }
        activeGrid = viewId || 0;
      }
      mainView = {xtype:'monitorTabPanel', tabPosition:gridMultipleViewsConfig.tabPosition, tabRotation:gridMultipleViewsConfig.tabRotation, items:grids, activeTab:activeGrid};
    } else {
      mainView = this.gridFactory(mainGridObj);
      mainView.region = 'center';
      mainView.title = 'mainViewGrid';
    }
    var panelEast = {region:'east', reference:'detailPanelEast', itemId:'detailPanelEast', stateful:true, stateId:'detailPanelEast' + this.moduleId, xtype:'panel', margin:'5 0 0 5', width:document.body.clientWidth * 0.3, split:true, hidden:true, collapsed:true, collapsible:true, layout:'fit', tools:[{type:'close', localized:{tooltip:'MONITOR_MAIN_CLOSE_EAST_PANEL'}, callback:function(panel) {
      panel.hide();
    }}]};
    items.push(mainView);
    items.push(panelEast);
    return items;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createGrids', dump:e, stack:true, level:'error'});
  }
}, gridFactory:function(viewObject) {
  var me = this;
  try {
    var header = false;
    var iconCls = viewObject.iconCls ? viewObject.iconCls : null;
    var margin = viewObject.margin ? viewObject.margin : '0 0 0 0';
    var textAlign = viewObject.textAlign ? viewObject.textAlign : 'left';
    var hideColumnHeaders = viewObject.hideColumnHeaders ? viewObject.hideColumnHeaders : false;
    var routePath = undefined;
    if (me.getSetting(monitor.assets.ConfigKeys.guiStateBrowsing) !== true && me.getSetting(monitor.assets.ConfigKeys.multipleGrids) === true) {
      routePath = viewObject.id;
    }
    return {xtype:'monitorGrid', flex:1, moduleId:me.moduleId, localized:{title:viewObject.title}, bpcGridViewName:viewObject.id, routePath:routePath, tabConfig:{iconCls:iconCls, localized:{tooltip:viewObject.title}, margin:margin, textAlign:textAlign}, config:{storeUrl:me.instanceStoreUrl, moduleId:me.moduleId, moduleCfg:me.moduleCfg, bpcGridViewId:viewObject.id, gridStyles:viewObject}, header:header, hideHeaders:hideColumnHeaders};
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR gridFactory', dump:e, stack:true, level:'error'});
  }
}, updateInstance:function(moduleCfg, routeArray) {
  var me = this;
  try {
    if (moduleCfg && Ext.Object.getAllKeys(moduleCfg).length > 0) {
      me.on('beforestaterestore', function() {
        return false;
      }, me, {single:true});
      if (moduleCfg.stateId) {
        monitor.Module.monitorViewStatesController.getMonitorViewState(me.moduleId, moduleCfg.stateId).then(function(state) {
          me.applyState(state.getData());
        });
        return;
      }
      if (moduleCfg.state) {
        me.applyState(moduleCfg.state);
        return;
      }
      var grid = this.getActiveGrid();
      var store = grid.store;
      if (moduleCfg.viewId) {
        grid = this.down('#' + this.moduleId + '_' + moduleCfg.viewId);
        var monitorTabPanel = this.down('monitorTabPanel');
        if (monitorTabPanel && grid) {
          store = grid.store;
          monitorTabPanel.setActiveTab(grid);
        }
      }
      var loadStore = false;
      if (moduleCfg.storeState && moduleCfg.storeState.filters) {
        var filterArr = moduleCfg.storeState.filters;
        store.clearFilter();
        if (filterArr.length !== 0) {
          if (grid) {
            grid.clearFilterFields();
            Ext.each(filterArr, function(filter) {
              store.addFilter(filter);
              loadStore = true;
            });
          }
        }
      }
      var groupField = this.getViewModel().get('groupField');
      if (groupField) {
        store.setGroupField(groupField);
      }
      var searchQuery = moduleCfg.searchQuery || '';
      this.getViewModel().set('globalFilterValue', searchQuery);
      store.proxy.extraParams.query = searchQuery;
      if (moduleCfg.storeState && moduleCfg.storeState.sorters) {
        var sorterArr = moduleCfg.storeState.sorters;
        store.sorters.removeAll();
        if (sorterArr.length !== 0) {
          if (grid) {
            store.sorters.add(sorterArr);
            loadStore = true;
          }
        }
      }
      if (loadStore && store) {
        store.load();
      }
    }
    if (routeArray && Ext.isArray(routeArray)) {
      this.getController().onRouteChange(routeArray);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateInstance', dump:e, stack:true, level:'error'});
  }
}, getState:function() {
  var me = this;
  var state = me.callParent() || {};
  state.views = {};
  state.searchQuery = me.lookupViewModel().get('globalFilterValue');
  me.query('monitorGrid').forEach(function(monitorGrid) {
    state.views[monitorGrid.stateId] = monitorGrid.getState();
  });
  var activeGrid = me.getActiveGrid();
  if (activeGrid) {
    state.activeView = activeGrid.stateId;
  }
  state.id = me.getViewModel().get('activeState.id');
  state.local = true;
  return state;
}, applyState:function(state) {
  var me = this;
  if (state.id) {
    monitor.Module.monitorViewStatesController.getMonitorViewState(me.moduleId, state.id).then(function(state) {
      me.getViewModel().set('activeState', state);
    });
  }
  if (state.stateReset === true) {
    me.resetState();
    return;
  }
  if (state.state) {
    state = Ext.apply(state, state.state);
    delete state.state;
  }
  var legacySearchQuery = '';
  if (state && Ext.isObject(state.views)) {
    Ext.Object.each(state.views, function(stateId, stateObj) {
      var target = me.down('monitorGrid[stateId\x3d' + stateId + ']');
      if (target && stateObj) {
        target.applyState(stateObj);
      }
      legacySearchQuery = legacySearchQuery || stateObj.searchQuery || '';
    });
  } else if (state) {
    me.getActiveGrid().applyState(state);
  }
  me.lookupViewModel().set('globalFilterValue', state.searchQuery || legacySearchQuery || '');
  if (state.activeView) {
    this.setActiveGrid('[stateId\x3d' + state.activeView + ']');
  }
}, resetState:function() {
  var clearState = {columns:undefined, storeState:{sorters:[], filters:[]}, groupingEnabled:false};
  var state = {id:Date.now(), views:{}};
  this.query('monitorGrid').forEach(function(grid) {
    var gridState = Ext.clone(clearState);
    gridState.storeState.sorters = grid.createInitialSorter();
    state.views[grid.stateId] = gridState;
  });
  this.applyState(state);
}, getGrid:function() {
  return this.down('monitorGrid');
}, getGridList:function() {
  return this.query('monitorGrid');
}, getStore:function() {
  var grid = this.getGrid();
  grid.getStore();
  return grid.getStore();
}, getModuleHeader:function() {
  return BpcCommon.Api.getModuleHeader();
}, updateSorterViewModel:function(sorters) {
  var me = this;
  var sortOrderArray = [];
  var viewModel = me.getViewModel();
  viewModel.set('activeSorter', sorters.length);
  this.updateSorterItems(sorters);
  sorters.each(function(sorter) {
    var textA = me.getColumnAttributeMatch('dataIndex', sorter.getProperty(), 'text');
    sortOrderArray.push(textA);
  });
  if (sortOrderArray.length > 0) {
    viewModel.set('activeSortOrder', '(' + sortOrderArray.toString() + ')');
  } else {
    viewModel.set('activeSortOrder', '');
  }
}, getLocalizedStringModuleHeader:function(sorterOrFilter) {
  var me = this;
  var text = '';
  var property = sorterOrFilter.getProperty();
  if (me.getColumnAttributeMatch('dataIndex', property, 'text').length) {
    text = me.getColumnAttributeMatch('dataIndex', property, 'text');
  } else if (me.getColumnAttributeMatch('dataIndex', property, 'localized')) {
    var localizedObj = me.getColumnAttributeMatch('dataIndex', property, 'localized');
    text = BpcCommon.Api.getTranslation(localizedObj.text);
    if (!me.columnLocalizedStrings) {
      me.columnLocalizedStrings = [];
      me.columnLocalizedStrings.push({property:property, text:text});
    } else if (me.columnLocalizedStrings.length) {
      var contains = false;
      me.columnLocalizedStrings.forEach(function(obj) {
        if (obj.property === property) {
          contains = true;
        }
      });
      if (!contains) {
        me.columnLocalizedStrings.push({property:property, text:text});
      }
    }
  } else {
    text = property || '';
  }
  return text;
}, setSorterStringIfFilterisSet:function(sorter) {
  var me = this;
  var text = '';
  var sortProperty = sorter.getProperty();
  me.columnLocalizedStrings.forEach(function(obj) {
    if (obj.property === sortProperty) {
      text = obj.text;
    }
  });
  return text.length ? text : false;
}, getColumnAttributeMatch:function(attribute, value, returnAttribute) {
  var val = '';
  var colConfig = this.getSetting(monitor.assets.ConfigKeys.columnConfig);
  Ext.Array.each(colConfig, function(col) {
    if (col[attribute] === value) {
      if (col.localized && col.localized[returnAttribute]) {
        val = BpcCommon.Api.getTranslation(col.localized[returnAttribute]);
      } else if (col[returnAttribute]) {
        val = col[returnAttribute];
      } else {
        val = col[attribute];
      }
      return false;
    }
  });
  return val;
}, getSetting:function(settingName) {
  return BpcCommon.Api.getSetting(this.moduleId, settingName);
}, applyActiveGrid:function(cmpQueryStringOrCmp) {
  var me = this;
  if (!cmpQueryStringOrCmp) {
    return null;
  }
  var activeGrid = cmpQueryStringOrCmp;
  if (Ext.isString(cmpQueryStringOrCmp)) {
    activeGrid = me.down(cmpQueryStringOrCmp);
  }
  if (activeGrid instanceof Ext.grid.Panel) {
    var tabpanel = this.down('monitorTabPanel');
    if (tabpanel) {
      tabpanel.setActiveTab(activeGrid);
    }
    me.getViewModel().set('activeGridSelection', activeGrid.getSelection());
  }
  return activeGrid;
}, updateDataItems:function(dataLength) {
  var me = this;
  this.fireEvent('dataLengthUpdated', dataLength);
  me.getViewModel().set('itemInt', dataLength);
}, updateFilterItems:function(filters) {
  var me = this;
  var count = filters.length;
  if (me.getViewModel().get('globalFilterValue')) {
    count++;
  }
  me.getViewModel().set('filterInt', count);
}, updateSorterItems:function(sorters) {
  var me = this;
  me.getViewModel().set('sortInt', sorters.length);
}, downloadFile:function(record) {
  var proxyId = BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.bpmProxyId);
  if (!proxyId || proxyId === '') {
    BpcCommon.Api.showNotification({text:'Fehler beim Datei herunterladen. Konfiguration ' + " 'inubit_proxyId' fehlerhaft.", popUp:true, type:'ERROR'});
    return;
  }
  var url = this.createFileDownloadUrl(record);
  if (!Ext.isEmpty(url)) {
    BpcCommon.Util.downloadFile(url);
  }
}, createFileDownloadUrl:function(record, suppressContentDisposition) {
  suppressContentDisposition = suppressContentDisposition === void 0 ? false : suppressContentDisposition;
  var url = null;
  var moduleId = this.moduleId;
  var childIDKey = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnHistorySubId);
  var childId = encodeURIComponent(record.get(childIDKey));
  var processIDkey;
  if (!childId) {
    processIDkey = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnId);
  } else {
    processIDkey = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnHistoryId);
  }
  if (processIDkey.includes('.raw')) {
    processIDkey = processIDkey.substring(0, processIDkey.lastIndexOf('.'));
  }
  if (processIDkey.includes('.lowercase')) {
    processIDkey = processIDkey.substring(0, processIDkey.lastIndexOf('.'));
  }
  var processId = encodeURIComponent(record.get(processIDkey));
  if (!childId && !processId) {
    BpcCommon.Api.showNotification({text:BpcCommon.Api.getTranslation('MONITOR_INCORRECT_ID_COLUMN'), toast:false, popUp:true, type:'ERROR'});
    return null;
  }
  var prefix = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/';
  var blobField = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnFile);
  if (!Ext.isEmpty(blobField)) {
    url = prefix + moduleId + '/data/' + processId + '/' + childId + '/' + blobField;
  } else {
    BpcCommon.Api.showNotification({text:BpcCommon.Api.getTranslation('MONITOR_INCORRECT_FILE_COLUMN'), popUp:true, type:'ERROR'});
    return null;
  }
  if (suppressContentDisposition === true) {
    url += '?noContentDisposition\x3dtrue';
  }
  return url;
}, initModuleActionsMap:function() {
  var me = this;
  me.moduleActionsMap = {metadata:{xtype:'bpcMonitorMeta'}, favorites:{flex:1, xtype:'monitorFavoriteToolbar'}, search:{xtype:'monitorSearchField'}, processaction:{xtype:'monitorProcessActionBtn'}, 'export':{xtype:'monitorExportBtn'}, views:{xtype:'monitorStateToolbarItem'}, refresh:{xtype:'monitorRefreshBtn'}, plugins:{targetContainer:{monitorModuleHeaderToolbar:{xtype:'bpcMonitorModuleHeaderToolbarHook'}, monitorToolbarLeft:{xtype:'bpcMonitorLeftToolbarHook'}, monitorToolbarRight:{xtype:'bpcMonitorRightToolbarHook'}}, 
  itemId:'monitorToolbarPluginHook', context:{moduleId:me.moduleId}}, '-\x3e':{xtype:'tbfill'}, '-':{xtype:'tbseparator'}, ' ':{xtype:'tbspacer'}};
}, getModuleActions:function(contentArray, targetContainerName) {
  var me = this;
  if (!me.getModuleActionConfigurationFromContentSetting(contentArray, 'plugins')) {
    if (targetContainerName === 'monitorToolbarRight') {
      contentArray.unshift('plugins');
    } else {
      contentArray.push('plugins');
    }
  }
  return contentArray.map(function(key) {
    var actionItem = {moduleId:me.moduleId, moduleInstance:me, cls:'monitorToolbarItem', viewModel:{parent:me.getViewModel()}};
    if (Ext.isObject(key)) {
      actionItem = Ext.apply(actionItem, me.moduleActionsMap[String(key.id).toLowerCase()], key.config);
    } else {
      actionItem = Ext.apply(actionItem, me.moduleActionsMap[String(key).toLowerCase()]);
    }
    if (actionItem.targetContainer && actionItem.targetContainer[targetContainerName]) {
      actionItem = Ext.apply(actionItem, actionItem.targetContainer[targetContainerName]);
    }
    return actionItem;
  });
}, getModuleActionConfigurationFromContentSetting:function(contentSettingValue, actionId) {
  if (!Ext.isArray(contentSettingValue)) {
    return;
  }
  return contentSettingValue.find(function(actionConfig) {
    return actionConfig === actionId || actionConfig.id && actionConfig.id === actionId;
  });
}, getCurrentModuleState:function() {
  var currentState = {};
  currentState.moduleCfg = {state:this.getState()};
  return currentState;
}, updateTooltip:function(grid) {
  grid = grid || this.getActiveGrid();
  this.fireEvent('tooltipChange', grid);
}});
Ext.define('monitor.view.contextmenu.ContextMenuController', {extend:'Ext.app.ViewController', alias:'controller.contextMenuController', requires:[], grid:undefined, gridController:undefined, init:function() {
  this.grid = this.getView().grid;
  this.gridController = this.grid.getController();
  this.monitorController = this.grid.up('bpcMonitor').getController();
}, openSplitView:function() {
  var mainVm = this.grid.lookupViewModel();
  var gridRefs = this.monitorController.getReferences();
  var detailPanelEast = gridRefs.detailPanelEast;
  var moduleId = this.getView().moduleId;
  var selectionModel = this.grid.getSelectionModel();
  var selection = selectionModel.getSelection();
  var record = selection[0];
  var conf = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.detailViewEastConfig);
  if (detailPanelEast.hidden) {
    try {
      var detailedContainer = this.gridController.createDetailContainer(conf, record, mainVm, this.grid, 'detailViewEast');
      detailPanelEast.removeAll();
      detailPanelEast.add(detailedContainer);
      detailPanelEast.show().expand(true);
    } catch (e) {
      Ext.log({msg:'Error in split view. ' + e, level:'error'});
      BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation('MONITOR_ERROR_IN_SPLIT_VIEW') + ': ' + e);
    }
  }
}, copyToClipboard:function(item) {
  var copyPasteValue = this.getView().selectedCellValue;
  var dataIndex = item.dataIndex;
  var fname = this.grid.down('[dataIndex\x3d' + dataIndex + ']');
  if (fname.format) {
    var dateFormat = fname.format;
    var dt = new Date(copyPasteValue);
    copyPasteValue = Ext.Date.format(dt, dateFormat);
  }
  BpcCommon.Util.copyToClipboard(copyPasteValue);
}, filterCurrentViewOrSelect:function() {
  this.gridController.filterCurrentViewOrSelect();
}, setMonitorFilter:function() {
  this.monitorController.setMonitorFilter(false, this.getView().selectedCellValue);
}, setMonitorFilterOnColumn:function() {
  this.monitorController.setMonitorFilter(false, this.getView().column.dataIndex + ':' + this.getView().selectedCellValue);
}, removeSingleSorter:function(item) {
  this.gridController.removeSorter(item.dataIndex);
}, removeColumnFilter:function() {
  var filter = this.getView().column.filter;
  filter.setValue(filter.emptyValue);
}, setColumnFilter:function() {
  var column = this.getView().column;
  var colType = column.xtype;
  if (colType !== 'datecolumn') {
    var filterObj = column.filter || {};
    this.gridController.addFilterToStore({property:column.dataIndex, value:[this.getView().selectedCellValue], source:filterObj.source});
  } else {
    var field = column.down('field');
    if (field && field.getXType() !== 'simpleDate') {
      var start = new Date(this.getView().selectedCellValue);
      var end = Ext.Date.add(start, Ext.Date.DAY, 1);
      start = start.setHours(0, 0, 0);
      end = end.setHours(0, 0, 0);
      var startFilter = this.createFilterObj(this.getView().column, '_start', 'gte', start);
      var endFilter = this.createFilterObj(this.getView().column, '_end', 'lt', end);
      this.gridController.addFilterToStore([startFilter, endFilter]);
    } else {
      field.activateFilter(this.getView().selectedCellValue);
    }
  }
}, setExclusiveColumnFilter:function() {
  this.gridController.removeAllFilter();
  this.setColumnFilter();
}, createFilterObj:function(col, idSuffix, operator, date) {
  var filter = new Ext.util.Filter({id:col + idSuffix, property:col, operator:operator, value:date});
  return filter;
}, shareProcess:function() {
  var moduleId = this.getView().moduleId;
  var idDataIndex = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnId);
  if (!idDataIndex) {
    BpcCommon.Api.showNotification({type:'ERROR', popUp:true, text:'MONITOR_ID_COLUMN_MISSING'});
    return;
  }
  var currentMonitor = this.grid.up('bpcMonitor');
  var currentState = currentMonitor.getState();
  var activeView = currentState.activeView;
  var column = this.getView().column;
  var filterObj = column.filter || {};
  var activeViewState = currentState.views[activeView];
  var filterState = activeViewState.storeState.filters || [];
  var selectionModel = this.grid.getSelectionModel();
  var selections = selectionModel.getSelection();
  var recordIds = selections.map(function(record) {
    return record.get(idDataIndex);
  });
  filterState.push({property:idDataIndex, value:recordIds, source:filterObj.source});
  activeViewState.storeState.filters = filterState;
  var currentModuleConfig = {state:currentState};
  var currentStateString = JSON.stringify(currentModuleConfig);
  var base64EncodedState = btoa(currentStateString);
  var urlBase64Encoded = encodeURIComponent(base64EncodedState);
  var url = BpcCommon.Api.getCurrentViewUrl() + '//' + urlBase64Encoded;
  Ext.create({xtype:'bpcShareWindow', title:'MONITOR_SHARE_PROCESS', text:'MONITOR_SHARE_PROCESS_TEXT', content:url}).show();
}, enableGrouping:function(item) {
  this.setGrouping(item.dataIndex, 'ASC');
  var groupingFeature = this.grid.getView().getFeature('groupingFeature');
  if (groupingFeature) {
    groupingFeature.enable();
  }
}, sortGroups:function(item) {
  var store = this.grid.getStore();
  var groupField = store.grouper.getProperty();
  this.setGrouping(groupField, item.direction);
}, setGrouping:function(property, direction) {
  var store = this.grid.getStore();
  store.setGrouper({property:property, direction:direction, root:'data'});
}, removeGrouping:function() {
  var store = this.grid.getStore();
  store.clearGrouping();
}});
Ext.define('monitor.view.contextmenu.ContextMenu', {extend:'Ext.menu.Menu', alias:['widget.monitorContextMenu'], requires:['monitor.view.contextmenu.ContextMenuController'], controller:'contextMenuController', hidden:true, copyCell:undefined, column:undefined, grid:undefined, moduleId:undefined, selectedCellValue:undefined, viewModel:{data:{moduleId:undefined, collabEnabled:false}}, collabEnabled:false, initComponent:function() {
  var me = this;
  me.getViewModel().set('moduleId', me.moduleId);
  if (me.collaborationConfig) {
    me.getViewModel().set('collabEnabled', me.collaborationConfig.collabEnabled);
    me.collabEnabled = me.collaborationConfig.collabEnabled;
  }
  var splitViewEnabled = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.detailViewEastEnabled);
  var selectedRecords = me.grid.getSelection();
  var store = me.grid.getStore();
  var noSelection = selectedRecords.length === 0;
  var selection = !noSelection;
  var multiRowSelection = selectedRecords.length > 1;
  var groupingEnabled = store.grouper;
  var groupedByText = store.grouper && store.grouper.getProperty() ? BpcCommon.Api.getTranslation('MONITOR_GROUPING_GROUPED_BY') + ' ' + store.grouper.getProperty() : BpcCommon.Api.getTranslation('MONITOR_GROUPING_NOT_GROUPED');
  var filterEnabled = true, filterMenuTooltip;
  if (me.column.filter === false || !me.column.filter || me.column.filter.type === false) {
    filterEnabled = false;
    filterMenuTooltip = BpcCommon.Api.getTranslation('MONITOR_COLUMN_FILTER_DISABLED');
  } else if (multiRowSelection) {
    filterEnabled = false;
    filterMenuTooltip = BpcCommon.Api.getTranslation('MONITOR_SINGLE_ROW_FUNCTION');
  }
  me.items = [{localized:{text:{tpl:'\x3cb\x3e\x3cu\x3e{MONITOR_NO_SELECTION_WARNING}\x3c/u\x3e\x3c/b\x3e', keys:['MONITOR_NO_SELECTION_WARNING']}}, hidden:!noSelection, iconCls:BpcCommon.Icons.WARNING + ' warningColor', handler:function() {
    me.grid.setSelection(me.grid.getStore().getAt(0));
  }}, {iconCls:'x-fal fa-outdent', text:BpcCommon.Api.getTranslation('MONITOR_MENU_SPLIT_VIEW'), handler:'openSplitView', hidden:!splitViewEnabled || multiRowSelection, itemId:'menu-item-splitview'}, {xtype:'menuseparator', hidden:!splitViewEnabled}, {iconCls:'x-fal fa-users', text:BpcCommon.Api.getTranslation('MONITOR_COLLABORATION'), itemId:'collabMenuHolder', hidden:!me.collaborationConfig.collabEnabled}, {xtype:'menuseparator', hidden:!me.collaborationConfig.collabEnabled}, {xtype:'monitorProcessActionMenu', 
  moduleId:me.moduleId, reference:'monitorProcessActionMenu', selection:selectedRecords, publishes:['hidden']}, {xtype:'menuseparator', hidden:true, bind:{hidden:'{monitorProcessActionMenu.hidden}'}}, {xtype:'monitorModuleJumpMenu', moduleId:me.moduleId, reference:'monitorModuleJumpMenu', selection:selectedRecords, publishes:['hidden']}, {xtype:'menuseparator', hidden:true, bind:{hidden:'{monitorModuleJumpMenu.hidden}'}}, {iconCls:'x-fa fa-crosshairs', localized:{text:selection ? 'MONITOR_FILTER_CURRENT_SELECTION' : 
  'MONITOR_FILTER_CURRENT_VIEW'}, handler:'filterCurrentViewOrSelect'}, {iconCls:'x-fal fa-share-square', localized:{text:'MONITOR_SHARE_PROCESS', tooltip:'MONITOR_SHARE_PROCESS'}, handler:'shareProcess', itemId:'monitorShareProcessMenuItem'}, {iconCls:'x-fal fa-filter', text:BpcCommon.Api.getTranslation('MONITOR_FILTER'), hidden:!filterEnabled, tooltip:filterMenuTooltip, itemId:'menu-item-filter', menu:[{iconCls:'x-fa fa-filter', text:BpcCommon.Api.getTranslation('MONITOR_SET_COLUMN_FILTER'), handler:'setColumnFilter', 
  hidden:noSelection}, {iconCls:'x-fal fa-filter', text:BpcCommon.Api.getTranslation('MONITOR_SET_THIS_FILTER_ONLY'), handler:'setExclusiveColumnFilter', hidden:noSelection}, {iconCls:BpcCommon.Icons.DELETE, text:BpcCommon.Api.getTranslation('MONITOR_REMOVE_COLUMN_FILTER'), handler:'removeColumnFilter'}, {iconCls:'x-fal fa-filter', text:BpcCommon.Api.getTranslation('MONITOR_SET_VOLLTEXTSUCHE'), handler:'setMonitorFilter', itemId:'menu-item-filter-full-text-search', hidden:noSelection}, {iconCls:'x-fal fa-filter', 
  text:BpcCommon.Api.getTranslation('MONITOR_SET_VOLLTEXTSUCHE_COLUMN'), handler:'setMonitorFilterOnColumn', itemId:'menu-item-filter-full-text-search-column', hidden:noSelection}]}, {iconCls:BpcCommon.Icons.SORT, text:BpcCommon.Api.getTranslation('MONITOR_SORTING'), menu:[{iconCls:BpcCommon.Icons.DELETE, text:BpcCommon.Api.getTranslation('MONITOR_REMOVE_SORTER'), dataIndex:me.column.dataIndex, handler:'removeSingleSorter'}]}, {iconCls:Ext.baseCSSPrefix + 'group-by-icon', text:BpcCommon.Api.getTranslation('MONITOR_GROUPING'), 
  menu:[{plain:'true', itemId:'menu-plain-grouping', text:groupedByText, style:'text-align: center; text-decoration: underline;'}, {xtype:'menuseparator'}, {iconCls:Ext.baseCSSPrefix + 'group-by-icon', text:BpcCommon.Api.getTranslation('MONITOR_GROUPING_BY_THIS_FIELD'), itemId:'menu-item-grouping-enable', dataIndex:me.column.dataIndex, handler:'enableGrouping'}, {iconCls:'x-fal fa-sort-alpha-up', text:BpcCommon.Api.getTranslation('MONITOR_GROUPING_SORT_ASC'), itemId:'menu-item-grouping-sort-asc', 
  direction:'ASC', hidden:!groupingEnabled, handler:'sortGroups'}, {iconCls:'x-fal fa-sort-alpha-down', text:BpcCommon.Api.getTranslation('MONITOR_GROUPING_SORT_DESC'), itemId:'menu-item-grouping-sort-desc', direction:'DESC', hidden:!groupingEnabled, handler:'sortGroups'}, {iconCls:BpcCommon.Icons.DELETE, text:BpcCommon.Api.getTranslation('MONITOR_GROUPING_REMOVE'), itemId:'menu-item-grouping-remove', hidden:!groupingEnabled, handler:'removeGrouping'}]}, {iconCls:'x-fal fa-copy', text:BpcCommon.Api.getTranslation('MONITOR_COPY_CELL_CONTENT'), 
  dataIndex:me.column.dataIndex, hidden:!me.copyCellEnabled, handler:'copyToClipboard'}, {xtype:'menuseparator'}, {iconCls:BpcCommon.Icons.RESET, text:BpcCommon.Api.getTranslation('MONITOR_RESET_VIEW'), handler:function() {
    BpcCommon.Api.getModuleInstanceComponent(me.moduleId).resetState();
  }}, {xtype:'bpcMonitorContextMenuViewHook', context:{moduleId:BpcCommon.Api.getActiveModuleId(), selection:selectedRecords}}];
  me.callParent(arguments);
  me.cmpCreated();
  me.on('hide', me.destroy, me, {delay:100});
}, cmpCreated:function() {
  Ext.GlobalEvents.fireEvent('monitor_contextMenu_created', this);
}});
Ext.define('monitor.view.MainController', {extend:'Ext.app.ViewController', alias:'controller.monitorController', requires:['monitor.view.contextmenu.ContextMenu'], control:{'#':{beforedestroy:'onBeforeDestroy', afterrender:'updateCurrentStateInformation'}, monitorTabPanel:{tabchange:'onTabChange'}, monitorGrid:{show:'checkActiveGrid', activate:'checkActiveGrid', selectionchange:'onSelectionChange', statesave:'saveViewState', load:'onMonitorLoad'}}, currentRoute:[], onTabChange:function() {
  this.saveViewState();
  var view = this.getView();
  view.updateTooltip();
}, onBeforeDestroy:function() {
  try {
    var gridArray = this.getView().query('monitorGrid');
    Ext.Array.each(gridArray, function(grid) {
      Ext.TaskManager.stop(grid.getController().autoRefreshTaskHolder);
    });
    this.getView().applicationToolbarRight.destroy();
    this.getView().applicationToolbarLeft.destroy();
    this.getView().bpcModuleHeader.destroy();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onBeforeDestroy', dump:e, stack:true, level:'error'});
  }
}, grid:undefined, refs:undefined, vm:undefined, lastCellIndex:undefined, lastRowIndex:undefined, lastValue:undefined, lastClickedColumn:undefined, lastRecord:undefined, actionColumns:undefined, actualSelection:undefined, bpcActions:undefined, moduleId:undefined, moduleModel:undefined, monitorViewStatesStore:undefined, ready:false, init:function() {
  var me = this;
  var vm = me.getViewModel();
  me.moduleId = me.getView().moduleId;
  me.moduleModel = BpcCommon.Api.getModuleModel(me.moduleId);
  monitor.Module.monitorViewStatesController.initMonitorViewStateStore(me.moduleId);
  BpcCommon.Api.getSettings(me.moduleId).each(function(setting) {
    vm.set('setting.' + setting.get('name'), Ext.clone(setting.get('value')));
  });
  me.updateCurrentStateInformation = Ext.Function.createBuffered(me.updateCurrentStateInformation, 100, me);
  me.ready = true;
  this.setAutoUpdateState();
}, initViewModel:function(vm) {
  for (var $jscomp$iter$1 = $jscomp.makeIterator(Object.entries(Object.getPrototypeOf(monitor.assets.ModuleRights))), $jscomp$key$$jscomp$destructuring$var4 = $jscomp$iter$1.next(); !$jscomp$key$$jscomp$destructuring$var4.done; $jscomp$key$$jscomp$destructuring$var4 = $jscomp$iter$1.next()) {
    var $jscomp$destructuring$var4 = $jscomp$key$$jscomp$destructuring$var4.value;
    {
      var $jscomp$destructuring$var5 = $jscomp.makeIterator($jscomp$destructuring$var4);
      var key = $jscomp$destructuring$var5.next().value;
      var value = $jscomp$destructuring$var5.next().value;
      {
        if (Ext.isString(value) && value.startsWith('bpcmonitor_')) {
          vm.set('right.' + value, BpcCommon.Api.hasUserRight(value));
        }
      }
    }
  }
}, isSplitViewActive:function() {
  var isOpen = false;
  try {
    var mainCtrlRefs = this.getReferences();
    var detailPanelEast = mainCtrlRefs.detailPanelEast;
    isOpen = detailPanelEast.isVisible(true);
  } catch (e) {
    Ext.log('error' + this.self.getName());
    isOpen = false;
  }
  return isOpen;
}, updateSplitViewWithCmp:function(cmp) {
  try {
    var mainCtrlRefs = this.getReferences();
    var detailPanelEast = mainCtrlRefs.detailPanelEast;
    detailPanelEast.removeAll();
    if (detailPanelEast.isVisible(true)) {
      try {
        detailPanelEast.add(cmp);
      } catch (e) {
        Ext.log({msg:'Error in split view. ' + e, level:'error'});
        BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation('MONITOR_ERROR_EAST_UPDATE') + ': ' + e);
      }
    }
  } catch (e$4) {
    Ext.log('error' + this.self.getName());
  }
}, setMonitorFilter:function(kill, value) {
  try {
    this.getViewModel().set('global.monitor.showSearchField', true);
    var oldValue = this.getViewModel().get('globalFilterValue');
    if (kill) {
      this.getViewModel().set('globalFilterValue', '');
    } else {
      if (oldValue !== value) {
        this.getViewModel().set('globalFilterValue', this.getViewModel().get('globalFilterValue') + ' ' + value);
      }
    }
  } catch (e) {
    BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation('MONITOR_ERROR_SETTING_FILTER') + e);
  }
  this.getView().getActiveGrid().getController().updateActiveFilter();
}, onRouteChange:function(routeArray) {
  try {
    if (Ext.Array.equals(this.currentRoute, routeArray)) {
      return;
    }
    if (routeArray[0] === 'detailPopup') {
      this.showDetailViewForId(routeArray[1]);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onRouteChange', dump:e, stack:true, level:'error'});
  }
}, showDetailViewForId:function(id) {
  try {
    var idKey = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnId);
    var dataObj = {};
    dataObj[idKey] = id;
    var record = Ext.data.Model.create(dataObj);
    var detailedWindow = Ext.createByAlias('widget.monitorDetailWindow', {moduleId:this.getView().moduleId, selectedRecord:record, hackMode:true, grid:this.getView().getActiveGrid(), width:document.body.clientWidth * 0.8, height:document.body.clientHeight * 0.8, minWidth:document.body.clientWidth * 0.4, minHeight:document.body.clientHeight * 0.5, maxWidth:document.body.clientWidth, maxHeight:document.body.clientHeight});
    detailedWindow.show();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR showDetailViewForId', dump:e, stack:true, level:'error'});
  }
}, updateCurrentStateInformation:function() {
  var me = this;
  var vm = me.getViewModel();
  if (!me.ready) {
    return;
  }
  try {
    if (vm.get('activeState')) {
      monitor.Module.monitorViewStatesController.updateMonitorViewState(me.moduleId, vm.get('activeState.id'), monitor.Module.monitorViewStatesController.getCurrentMonitorViewState(me.moduleId), false);
    }
    var monitorViewTitle = '';
    var tabPanel = this.getView().down('monitorTabPanel');
    if (tabPanel && tabPanel.getActiveTab()) {
      monitorViewTitle = tabPanel.getActiveTab().getTitle();
    }
    BpcCommon.Api.setModuleInformation(this.moduleId, {moduleBrowserTitle:monitorViewTitle});
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR updateCurrentStateInformation', dump:e, stack:true, level:'error'});
  }
}, saveViewState:function() {
  var me = this;
  me.getView().saveState();
  if (BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.guiStateBrowsing) === true) {
    BpcCommon.Api.updateRouteWithChangeCheck('/module', ['monitor', me.moduleId, null, BpcCommon.Util.stringifyJsonWithCircularReferences({state:me.getView().getState()})], undefined, true);
  }
  me.updateCurrentStateInformation();
}, checkActiveGrid:function(eventSource) {
  var me = this;
  me.getView().setActiveGrid(eventSource);
}, onSelectionChange:function(selModel, selection) {
  var me = this;
  if (selModel && selModel.view && selModel.view.ownerGrid === me.getView().getActiveGrid()) {
    me.getViewModel().set('activeGridSelection', selection);
  }
}, onMonitorLoad:function(bpcMonitor) {
  var me = this;
  if (bpcMonitor === me.getView().getActiveGrid()) {
    me.getViewModel().set('activeGridSelection', bpcMonitor.getSelection());
  }
}, setAutoUpdateState:function() {
  var vm = this.getViewModel();
  var autoUpdateStateId = this.moduleId + '_autoUpdate';
  var autoUpdateState = Ext.state.Manager.get(autoUpdateStateId);
  var autoUpdate = BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.autoUpdate);
  if (autoUpdateState && Ext.isBoolean(autoUpdateState.checked)) {
    autoUpdate = autoUpdateState.checked;
  }
  if (!Ext.isBoolean(autoUpdate)) {
    autoUpdate = true;
  }
  vm.set('setting.' + monitor.assets.ConfigKeys.autoUpdate, autoUpdate);
}});
Ext.define('monitor.view.MainModel', {extend:'Ext.app.ViewModel', alias:'viewmodel.monitorMainViewModel', data:{moduleId:undefined, totalCount:0, activeFilter:0, activeSorter:0, activeSortOrder:'', activeGrid:undefined, activeGridSelection:[], instanceStoreUrl:undefined, detailViewInline:undefined, detailViewPopup:undefined, detailViewEast:undefined, processAction:true, globalFilterValue:'', groupField:undefined, actions:undefined, monitorTitle:'Virtimo BPC - Monitor', autoRefreshDisabled:false, 
showAutoFilter:false, hideStartProcess:true, hideAlternativeMessage:true, hideExportPartnerManagement:true, stateModel:undefined, processActionsEnabled:undefined, processActionsColumns:undefined, processActionsSeparator:undefined, monitorData:'MONITOR_DATASETS', monitorFilter:'MONITOR_FILTER_ACTIVE', monitorSorter:'MONITOR_SORTER_ACTIVE', activeState:undefined, activeStateDirty:false, setting:{}, right:{}, itemInt:0, filterInt:0, sortInt:0, sorters:'', filters:'', selectionCount:0}, formulas:{totalItems:{get:function(get) {
  var total = get('totalCount');
  var totalString = Ext.util.Format.number(total, '0,000');
  var countLimit = BpcCommon.Api.getSetting('monitor', monitor.assets.ConfigKeys.dataCountLimit);
  if (total >= countLimit) {
    totalString = '\x3e' + String(totalString);
  }
  return totalString;
}}, globalFilterActive:{get:function(get) {
  var query = get('globalFilterValue');
  return query && query.length > 0;
}}}});
Ext.define('monitor.view.admin.MonitorViewStates', {extend:'Ext.Container', alias:'widget.monitorAdminMonitorViewStates', layout:'fit', iconCls:monitor.assets.Icons.VIEW, localized:{title:'MONITOR_MANAGE_VIEWS', tooltip:'MONITOR_MANAGE_VIEWS'}, initComponent:function() {
  var me = this;
  me.items = [{xtype:'monitorStateConfigurationPanel', moduleId:me.moduleInstance.id}];
  me.callParent();
}});
Ext.define('monitor.view.contextmenu.PluginHook', {extend:'BpcCommon.plugin.HookContainer', alias:'widget.bpcMonitorContextMenuViewHook', inheritableStatics:{NAME:'Monitor Kontextmenu', HOOK_ID:'monitor_context_menu'}, autoInjectPlugins:true, pluginTarget:'^', hidden:true});
Ext.define('monitor.view.detail.ActionPanel', {extend:'Ext.panel.Panel', alias:'widget.monitorActionPanel', localized:{title:'MONITOR_ACTIONS'}, controller:'monitorActionPanelController', record:undefined, layout:{type:'vbox', align:'stretch'}, stateful:true, stateId:undefined, closeOnAction:false, initComponent:function() {
  this.stateId = (this.stateId ? this.stateId : '') + 'detailWindowActionPanel' + this.moduleId;
  this.items = this.createItems(this.record);
  this.callParent(arguments);
}, createItems:function(record) {
  var baseConfig = {vertical:undefined, layout:{type:'vbox', align:'stretch'}, moduleId:this.moduleId, padding:'5 0 0 0', defaults:{ui:'default', margin:'2 0 2 0'}};
  return [Ext.apply({}, baseConfig, {xtype:'monitorProcessActionToolbar', selectedProcesses:[record]})];
}});
Ext.define('monitor.view.detail.ActionPanelController', {extend:'Ext.app.ViewController', alias:'controller.monitorActionPanelController', listen:{global:{monitorProcessAction:'onMonitorProcessAction', monitorStartProcess:'onMonitorProcessAction'}}, onMonitorProcessAction:function() {
  var me = this;
  var view = me.getView();
  if (view.closeOnAction === true && view.up('window')) {
    view.up('window').close();
  }
}});
Ext.define('monitor.view.detail.ChildLogGrid', {extend:'Ext.grid.Panel', localized:{title:'MONITOR_PROCESSING_HISTORY'}, alias:'widget.monitorChildLogGrid', controller:'monitorChildLogGridController', loadMask:true, viewConfig:{localized:{emptyText:'MONITOR_NO_DATA_FOUND', loadingText:'MONITOR_LOADING_DATA'}}, configuration:undefined, record:undefined, grid:undefined, columns:undefined, setActionColumn:undefined, fileDataIndex:undefined, sorters:undefined, textSelectable:undefined, listeners:{rowdblclick:'onRowDbClick'}, 
stateful:true, stateId:'', stateEvents:['selectionchange'], queryId:undefined, initComponent:function() {
  if (this.record) {
    this.stateId = this.record.getId() + this.stateId;
  }
  this.stateId = (this.stateId ? this.stateId : '') + 'detailWindowChildLogGrid' + this.moduleId + this.bpcViewId + (this.record ? this.record.getId() : '');
  this.setActionColumn = typeof this.setActionColumn !== 'undefined' ? this.setActionColumn : true;
  this.fileDataIndex = BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.columnFile);
  this.dateFormat = BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.dateFormat);
  this.viewConfig.enableTextSelection = BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.gridEditable);
  this.columns = this.getController().createItems(Ext.clone(this.columns), this.grid, this.setActionColumn, this.fileDataIndex);
  this.store = this.getController().createStore(this.columns);
  this.callParent(arguments);
  this.getView().on('scrollend', this.saveState.bind(this));
}, getState:function() {
  var state = this.callParent();
  try {
    var selection = this.getSelection();
    if (selection && selection[0] && selection[0].getId) {
      state.selectedRecord = this.getSelection()[0].getId();
    }
    state.scrollX = this.getView().getScrollX();
    state.scrollY = this.getView().getScrollY();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR getState', dump:e, stack:true, level:'error'});
  }
  return state;
}, applyState:function(state) {
  var me = this;
  if (!me.destroyed) {
    me.callParent(arguments);
    me.applyStateLazy(state);
  }
}, applyStateLazy:function(state) {
  var me = this;
  if (!me.destroyed && !me.store.isLoaded()) {
    me.store.on('load', me.applyStateLazy.bind(me, state));
    return;
  }
  if (!me.rendered === true) {
    me.on('afterrender', me.applyStateLazy.bind(me, state));
    return;
  }
  var record = me.store.getById(state.selectedRecord);
  me.setSelection(record);
  me.getView().setScrollX(state.scrollX);
  me.getView().setScrollY(state.scrollY);
}});
Ext.define('monitor.view.detail.ChildLogGridController', {extend:'Ext.app.ViewController', alias:'controller.monitorChildLogGridController', init:function() {
  this.callParent(arguments);
}, onRowDbClick:function(grid, record, tr, rowIndex) {
  this.callShowMessageInNewTab(grid, rowIndex, this.getView().fileDataIndex);
}, createItems:function(columns, monitorGrid, setActionColumn, fileDataIndex) {
  try {
    var view = this.getView();
    if (!monitor.Util.isEmpty(columns)) {
      var preparedColumns = this.addRenderer(columns, monitorGrid);
      if (setActionColumn) {
        preparedColumns.unshift(this.setActionForFileColumns(fileDataIndex));
      }
      return preparedColumns;
    }
    var defaultColumns = BpcCommon.Api.getSetting(view.moduleId, monitor.assets.ConfigKeys.columnHistoryConfig) || [];
    var generatedColumns = this.addRenderer(defaultColumns, monitorGrid);
    if (setActionColumn) {
      generatedColumns.unshift(this.setActionForFileColumns(fileDataIndex));
    }
    return generatedColumns;
  } catch (e) {
    Ext.log({msg:'Error in create items in child log grid.' + e, level:'error'});
  }
}, addRenderer:function(columns, monitorGrid) {
  if (!Ext.isArray(columns)) {
    return [];
  }
  try {
    return columns.map(function(column) {
      return monitor.Util.getStandardColumnConfig(column, monitorGrid);
    });
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR  adding renderer', dump:e, stack:true, level:'error'});
  }
}, getColumnByDataName:function(grid, name) {
  var columns = grid.getColumns();
  var searchedColumn = null;
  Ext.each(columns, function(column) {
    if (column.dataIndex === name) {
      searchedColumn = column;
      return;
    }
  });
  return searchedColumn;
}, callShowMessageInNewTab:function(grid, rowIndex, fileDataIndex) {
  var record = grid.getStore().getAt(rowIndex);
  var childGridView = this.getView();
  var detailWindowController = grid.up('monitorDetailContainer').getController();
  if (detailWindowController) {
    detailWindowController.showMessageInNewTab(record, fileDataIndex, childGridView);
  }
}, setActionForFileColumns:function(fileDataIndex) {
  var items = [];
  var me = this;
  var moduleId = this.getView().moduleId;
  var mimetypeDataIndex = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnMimetype);
  var defaultMimetype = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnDefaultMimetype);
  if (this.existsFileViewer()) {
    items.push({tooltip:'Nachricht anzeigen', scope:this, handler:function(grid, rowIndex) {
      me.callShowMessageInNewTab(grid, rowIndex, fileDataIndex);
    }, isDisabled:function(view, rowIndex, colIndex, item, record) {
      return !(record.get(fileDataIndex) || record.get(fileDataIndex.toUpperCase()));
    }, getClass:function(value, metadata, record) {
      var testCls = 'file-preview-actioncolumn';
      var iconCls = me.getIconForMimeType(record.get(mimetypeDataIndex)) || defaultMimetype;
      return testCls + ' ' + iconCls;
    }});
  }
  items.push({iconCls:BpcCommon.Icons.DOWNLOAD, tooltip:BpcCommon.Api.getTranslation('MONITOR_DOWNLOAD_MESSAGE'), scope:this, handler:function(grid, rowIndex) {
    var record = grid.getStore().getAt(rowIndex);
    var detailWindowController = grid.up('monitorDetailContainer');
    if (detailWindowController) {
      this.downloadFile(record, fileDataIndex);
    }
  }, isDisabled:function(view, rowIndex, colIndex, item, record) {
    return !(record.get(fileDataIndex) || record.get(fileDataIndex.toUpperCase()));
  }});
  return {xtype:'actioncolumn', width:items.length === 1 ? 30 : 60, items:items};
}, downloadFile:function(record, fileDataIndex) {
  var moduleId = this.getView().moduleId;
  var monitorInstance = BpcCommon.Api.getModuleInstanceComponent(moduleId);
  return monitorInstance.downloadFile(record, fileDataIndex);
}, existsFileViewer:function() {
  try {
    if (this.existsFileViewerState) {
      return this.existsFileViewerState;
    }
    var configuration = this.getView().configuration;
    var tabPanel = null;
    for (var key in configuration) {
      if (configuration[key].type && configuration[key].type === 'tabPanel') {
        tabPanel = configuration[key].config;
      }
    }
    var configAsString = JSON.stringify(tabPanel);
    this.existsFileViewerState = configAsString.indexOf('fileViewer') > -1;
    return this.existsFileViewerState;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR existsFileViewer', dump:e, stack:true, level:'error'});
  }
}, getParentQueryId:function() {
  var idString = null;
  var detailContainer = this.getView().up('monitorDetailContainer');
  if (detailContainer && detailContainer.isComponent) {
    if (detailContainer.queryId) {
      idString = detailContainer.queryId;
    }
  } else {
    var view = this.getView();
    if (view && view.isComponent && view.queryId && Ext.isDefined(view.queryId)) {
      idString = view.queryId;
    }
  }
  return idString;
}, createStore:function(columns) {
  try {
    var queryId = this.getParentQueryId();
    var childId = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnHistorySubId);
    var newModel = Ext.define(null, {extend:'Ext.data.Model', idProperty:childId, fields:columns});
    var store = Ext.create('Ext.data.Store', {sorters:[], model:newModel, pageSize:5000, proxy:{type:'ajax', url:this.getView().storeUrl, reader:{type:'json', rootProperty:'data', totalProperty:'totalCount'}, extraParams:{query:'', queryId:queryId}}});
    store.load();
    this.createFilterForStore(store);
    this.createSortersForStore(store);
    return store;
  } catch (e) {
    Ext.log({msg:'Error in create store in child log grid.' + e, level:'error'});
  }
}, getStatusRendererFunction:function() {
  var monitorGrid = this.getView().grid;
  var statusColumn = this.getColumnByDataName(monitorGrid, 'status') || {};
  return statusColumn.renderer;
}, createFilterForStore:function(store) {
  var filter = this.getView().filter;
  if (filter) {
    store.filterBy(function(record) {
      var result;
      try {
        result = eval(filter);
      } catch (e) {
        result = true;
      }
      return result;
    }, this);
  }
}, createSortersForStore:function(store) {
  if (!monitor.Util.isEmpty(this.getView().sorters)) {
    this.createSorters(this.getView().sorters, store);
  } else {
    var childId = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnHistorySubId);
    if (monitor.Util.isEmpty(childId)) {
      childId = 'childId';
    }
    var sorters = [{'dataIndex':childId, 'direction':'ASC'}];
    this.createSorters(sorters, store);
  }
}, createSorters:function(sorters, store) {
  store.sorters.clear();
  for (var i = 0; i < sorters.length; i++) {
    var item = sorters[i];
    var directory = item.direction ? item.direction : 'ASC';
    store.sorters.add(new Ext.util.Sorter({property:item.dataIndex, direction:directory}));
  }
  store.sort();
}, loadProcessData:function(data) {
  var store = this.getView().getStore();
  var columns = this.getView().columns;
  if (columns[0] && columns[0].xtype === 'actioncolumn' && columns.length - 1 === 0 || columns.length === 0) {
    this.addNotConfiguredColumns(data);
    this.getView().forceFit = true;
  }
  if (typeof data === 'undefined') {
    data = [];
  }
  store.loadData(data);
  var filter = this.getView().filter;
  if (filter) {
    store.filterBy(function(record) {
      var result;
      try {
        result = eval(filter);
      } catch (e) {
        result = true;
      }
      return result;
    }, this);
  }
  if (!monitor.Util.isEmpty(this.getView().sorters)) {
    this.createSorters(this.getView().sorters, store);
  } else {
    var childId = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnHistorySubId);
    if (monitor.Util.isEmpty(childId)) {
      childId = 'childId';
    }
    var sorters = [{'dataIndex':childId, 'direction':'ASC'}];
    this.createSorters(sorters, store);
  }
}, addNotConfiguredColumns:function(data) {
  var columns = this.getView().columns;
  var missingColumns = [];
  if (data) {
    for (var i = 0; i < data.length; i++) {
      var dataItem = data[i];
      for (var key in dataItem) {
        if (!this.existsColumn(columns, key) && missingColumns.indexOf(key) === -1) {
          missingColumns.push(key);
        }
      }
    }
    Ext.each(missingColumns, function(missingColumnKey) {
      var column = Ext.create('Ext.grid.column.Column', {text:missingColumnKey, dataIndex:missingColumnKey});
      if (missingColumnKey === 'timestamp') {
        column.renderer = Ext.util.Format.dateRenderer(this.getView().dateFormat);
      } else if (missingColumnKey === 'status') {
        column.renderer = this.getStatusRendererFunction();
      }
      this.getView().headerCt.insert(columns.length, column);
    }, this);
  }
}, existsColumn:function(columns, searchColumnName) {
  var columnExists = false;
  Ext.each(columns, function(column) {
    if (column.dataIndex === searchColumnName) {
      columnExists = true;
      return;
    }
  });
  return columnExists;
}, getIconForMimeType:function(mimeType) {
  var iconMappings = [{mimeType:'application/pdf', icon:'fa-file-pdf'}, {mimeType:'text/plain', icon:'fa-file-alt'}, {mimeType:'application/vnd.openxmlformats-officedocument.wordprocessingml.document', icon:'fa-file-word'}, {mimeType:'application/xml', icon:'fa-file-code'}, {mimeType:'text/xml', icon:'fa-file-code'}, {mimeType:'image/', icon:'fa-file-image'}, {mimeType:'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', icon:'fa-file-excel'}];
  var icon = 'fa-file';
  if (Ext.isString(mimeType)) {
    var matchingIconMap = iconMappings.find(function(iconMap) {
      return iconMap.mimeType === mimeType;
    });
    if (!matchingIconMap) {
      matchingIconMap = iconMappings.find(function(iconMap) {
        return mimeType.indexOf(iconMap.mimeType) >= 0;
      });
    }
    if (matchingIconMap) {
      icon = matchingIconMap.icon;
    }
  }
  return 'x-fal ' + icon;
}});
Ext.define('monitor.view.detail.DetailContainer', {extend:'Ext.container.Container', alias:'widget.monitorDetailContainer', controller:'monitorDetailContainerController', layout:'border', selectedRecord:undefined, grid:undefined, instanceConfiguration:undefined, configuration:undefined, loadedChildren:undefined, settings:undefined, componentName:undefined, items:[], stateful:true, stateId:undefined, padding:'5px 10px 10px 5px', queryId:undefined, initComponent:function() {
  this.width = Ext.isDefined(this.configuration.width) ? this.configuration.width : '100%';
  this.height = this.configuration.height ? this.configuration.height : 300;
  this.settings = this.getController().setConfigurationParams(this.moduleId);
  this.stateId = (this.stateId ? this.stateId : '') + 'detailWindowDetailContainer' + this.moduleId;
  if (Ext.isDefined(this.queryId)) {
    this.defaults = {queryId:this.queryId};
  }
  Ext.apply(this, {items:this.getController().createComponents(this.configuration)});
  this.callParent(arguments);
  var pluginHook = this.down('monitorDetailHook');
  if (!pluginHook) {
    var target = this.down('tabpanel') || this.down('container');
    if (target) {
      target.add({xtype:'monitorDetailHook', context:{moduleId:this.moduleId, record:this.selectedRecord}, hidden:true, pluginTarget:'^'});
    }
  }
}});
Ext.define('monitor.view.detail.DetailContainerController', {extend:'Ext.app.ViewController', alias:'controller.monitorDetailContainerController', requires:['monitor.Util', 'monitor.assets.ConfigKeys'], init:function(view) {
  var me = this;
  me.callParent(arguments);
  view.mon(view.grid.getStore(), 'load', me.onDataReload, me);
}, showMessageInNewTab:function(record, fileDataIndex, childLogGridView) {
  try {
    if (this.fileViewContainer) {
      this.fileViewContainer.getController().onShowMessage(record, fileDataIndex, this.fileViewContainerConfig, childLogGridView);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR showMessageInNewTab', dump:e, stack:true, level:'error'});
  }
}, setConfigurationParams:function() {
  try {
    var baseUrl = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.bpmBasePath);
    var processId = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnId);
    var childId = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnHistorySubId);
    var tablePrefix = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.legacyPmTablePrefix);
    var referenceEndPoint = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.bpmReferenceEndpoint);
    var actionEndPoint = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.processActionsEndpoint);
    var actionColumn = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnActionPrefix);
    if (monitor.Util.isEmpty(processId)) {
      processId = 'processId';
    }
    if (monitor.Util.isEmpty(childId)) {
      childId = 'childId';
    }
    if (monitor.Util.isEmpty(tablePrefix)) {
      tablePrefix = '';
    }
    if (monitor.Util.isEmpty(baseUrl) || typeof tablePrefix === 'undefined' || monitor.Util.isEmpty(referenceEndPoint) || monitor.Util.isEmpty(actionEndPoint)) {
      Ext.log({msg:"Problem in detail container: Couldn't find the configuration 'inubit_baseUrl', '" + monitor.assets.ConfigKeys.legacyPmTablePrefix + "', 'inubit_actionEndpoint' or " + "'inubit_referenceEndpoint'. Please check the configuration of monitor.", level:'warn'});
    }
    return {baseUrl:baseUrl, processId:processId, childId:childId, tablePrefix:tablePrefix, referenceEndpoint:referenceEndPoint, actionEndpoint:actionEndPoint, actionColumn:actionColumn};
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR setConfigurationParams', dump:e, stack:true, level:'error'});
  }
}, createComponents:function(configuration) {
  try {
    if (this.isValidConfiguration(configuration)) {
      var defaultHeight = configuration.height ? configuration.height / 3 : 150;
      var defaultWidth = configuration.width ? configuration.width / 3 : 150;
      var collapsible = configuration.collapsible || true;
      var autoScroll = configuration.autoScroll || true;
      var split = configuration.split || true;
      var nonCenterRegionDefault = {autoScroll:autoScroll, split:split, collapsible:collapsible};
      if (configuration.paddingOfElements) {
        nonCenterRegionDefault.padding = configuration.paddingOfElements;
      }
      var items = [];
      var item, defaultConfig = null;
      if (typeof configuration.left !== 'undefined') {
        defaultConfig = Ext.apply({}, nonCenterRegionDefault, {region:'west', width:defaultWidth});
        configuration.left.config = Ext.apply({}, configuration.left.config, defaultConfig);
        item = this.parseLayoutObject(configuration.left, Ext.apply({}, configuration.left.config, defaultConfig));
        if (item) {
          items.push(item);
        }
      }
      if (typeof configuration.right !== 'undefined') {
        defaultConfig = Ext.apply({}, nonCenterRegionDefault, {region:'east', width:defaultWidth});
        configuration.right.config = Ext.apply({}, configuration.right.config, defaultConfig);
        item = this.parseLayoutObject(configuration.right, Ext.apply({}, configuration.right.config, defaultConfig));
        if (item) {
          items.push(item);
        }
      }
      if (typeof configuration.top !== 'undefined') {
        defaultConfig = Ext.apply({}, nonCenterRegionDefault, {region:'north', height:defaultHeight});
        configuration.top.config = Ext.apply({}, configuration.top.config, defaultConfig);
        item = this.parseLayoutObject(configuration.top, Ext.apply({}, configuration.top.config, defaultConfig));
        if (item) {
          items.push(item);
        }
      }
      if (typeof configuration.bottom !== 'undefined') {
        defaultConfig = Ext.apply({}, nonCenterRegionDefault, {region:'south', height:defaultHeight});
        configuration.bottom.config = Ext.apply({}, configuration.bottom.config, defaultConfig);
        item = this.parseLayoutObject(configuration.bottom, Ext.apply({}, configuration.bottom.config, defaultConfig));
        if (item) {
          items.push(item);
        }
      }
      var centerConfig = {region:'center', minWidth:defaultWidth};
      if (configuration.paddingOfElements) {
        centerConfig.padding = configuration.paddingOfElements;
      }
      configuration.center.config = Ext.apply({}, configuration.center.config, centerConfig);
      items.push(this.parseLayoutObject(configuration.center, Ext.apply({}, centerConfig, configuration.center.config)));
      return items;
    }
    return this.createErrorComponents();
  } catch (e) {
    Ext.log({msg:'Error in create components of DetailController: ' + e, level:'error'});
  }
}, isValidConfiguration:function(configuration) {
  if (monitor.Util.isEmpty(configuration) || monitor.Util.isEmpty(configuration.center)) {
    return false;
  }
  if (!monitor.Util.isEmpty(configuration.center)) {
    var center = configuration.center;
    if (center.type) {
      return true;
    }
  }
}, createErrorComponents:function() {
  return Ext.create('Ext.container.Container', {layout:{type:'hbox', align:'stretch'}, padding:10, items:[{xtype:'box', html:'\x3cp\x3e\x3c/p\x3e\x3ci class\x3d"fa fa-exclamation-triangle" style\x3d"color: #D49A6A; ' + 'font-size: 18px; margin-right: 3px;"\x3e\x3c/i\x3e\x3c/p\x3e'}, {xtype:'box', html:BpcCommon.Api.getTranslation({tpl:'\x3cp\x3e{MONITOR_DETAIL_CONTAINER_ERROR1}' + '{MONITOR_DETAIL_CONTAINER_ERROR2}\x3c/p\x3e', keys:['MONITOR_DETAIL_CONTAINER_ERROR1', 'MONITOR_DETAIL_CONTAINER_ERROR2']}), 
  style:{marginLeft:'10px'}}]});
}, parseLayoutObject:function(layoutObject, params) {
  try {
    params = params || {};
    layoutObject.config = Ext.applyIf(layoutObject.config, params.extContainerSettings);
    params = Ext.apply({}, params, {configuration:this.getView().configuration, record:this.getView().selectedRecord, grid:this.getView().grid, moduleId:this.getView().moduleId, settings:this.getView().settings, bpcViewId:this.getView().bpcViewId});
    if (params.title) {
      params.localized = Ext.apply({}, {title:params.title}, params.localized);
    }
    if (layoutObject.type === 'processPanel') {
      return this.createProcessPanel(params);
    } else if (layoutObject.type === 'childGrid') {
      return this.createChildLogGrid(params);
    } else if (layoutObject.type === 'jumpPanel') {
      return this.createJumpPanel(params);
    } else if (layoutObject.type === 'tabPanel') {
      return this.createTabPanel(params);
    } else if (layoutObject.type === 'fileViewer') {
      this.fileViewContainerConfig = params;
      return 'fileViewContainer';
    } else if (layoutObject.type === 'actionPanel') {
      return this.createActionPanel(params);
    } else if (layoutObject.type === 'taskPanel') {
      return this.createTaskPanel(layoutObject.config, params);
    } else if (layoutObject.type === 'widgetPanel') {
      return this.createWidgetPanel(layoutObject.config, params);
    } else if (layoutObject.type === 'interactionPanel') {
      return this.createInteractionPanel(layoutObject.config, params);
    } else if (layoutObject.type === 'forms') {
      return this.createForms(layoutObject.config, params);
    } else if (BpcCommon.Util.isStringEqual(layoutObject.type, 'plugin')) {
      return Ext.applyIf({xtype:'monitorDetailHook', context:Ext.applyIf({moduleId:this.getView().moduleId, record:this.getView().selectedRecord}, layoutObject.config.context), hidden:layoutObject.config.pluginTarget && layoutObject.config.pluginTarget !== 'self', pluginTarget:layoutObject.config.pluginTarget || 'self'}, layoutObject.config);
    }
    return null;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR parseLayoutObject', dump:e, stack:true, level:'error'});
  }
}, createJumpPanel:function(config) {
  return Ext.create(Ext.apply({xtype:'monitorJumpPanel'}, config));
}, createProcessPanel:function(config) {
  return Ext.createByAlias('widget.monitorProcessRecordView', Ext.apply({}, config, {}));
}, createChildLogGrid:function(config) {
  var processIDKey = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnId);
  var url = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + this.getView().moduleId + '/details/' + encodeURIComponent(this.getView().selectedRecord.get(processIDKey));
  var view = this.getView();
  return Ext.createByAlias('widget.monitorChildLogGrid', Ext.apply({}, config, {storeUrl:url, queryId:view.queryId, grid:view.grid}));
}, createTabPanel:function(config) {
  var tabs = [];
  var setFileViewContainer = false;
  var setTabPanel = false;
  Ext.each(config.subItems, function(item) {
    if (item.type === 'taskPanel') {
      var field = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnTaskView);
      var fieldValue = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.columnTaskViewValue);
      if (monitor.Util.isEmpty(field) || monitor.Util.isEmpty(fieldValue)) {
        Ext.log({msg:"Problem in detail container: Couldn't find the configuration '" + monitor.assets.ConfigKeys.columnTaskView + "' or '" + monitor.assets.ConfigKeys.columnTaskViewValue + "'. Please check the configuration of monitor.", level:'warn'});
      } else {
        var fieldValueArray = fieldValue.split(',');
        var currentValue = config.record.get(field.toUpperCase());
        Ext.Array.each(fieldValueArray, function(value) {
          if (currentValue && currentValue.toUpperCase() === Ext.String.trim(value).toUpperCase()) {
            var taskTab = this.parseLayoutObject(item, item.config);
            tabs.push(taskTab);
            return false;
          }
        }, this);
      }
    } else {
      var parsedItem = this.parseLayoutObject(item, item.config);
      if (parsedItem) {
        if (parsedItem === 'fileViewContainer') {
          setFileViewContainer = true;
        } else {
          tabs.push(parsedItem);
        }
      }
    }
  }, this);
  var tabPanel = Ext.createByAlias('widget.monitorDetailTabPanel', Ext.apply({}, config, {tabs:tabs}));
  if (setFileViewContainer) {
    this.fileViewContainer = tabPanel;
  }
  if (setTabPanel) {
    this.tabPanel = tabPanel;
  }
  return tabPanel;
}, createActionPanel:function(params) {
  return Ext.createByAlias('widget.monitorActionPanel', params);
}, createTaskPanel:function(objectConfig, params) {
  return Ext.createByAlias('widget.monitorTaskPanel', Ext.apply({}, objectConfig, {configuration:objectConfig, moduleId:params.moduleId, record:params.record, title:'Aufgabe'}));
}, createInteractionPanel:function(objectConfig, params) {
  return Ext.createByAlias(objectConfig.formId, Ext.apply({}, objectConfig, {configuration:objectConfig, moduleId:params.moduleId, record:params.record}));
}, createForms:function(objectConfig, params) {
  return {xtype:'monitorDetailForms', configuration:objectConfig, detailViewConfiguration:params};
}, createWidgetPanel:function(objectConfig) {
  return Ext.createByAlias('widget.monitorWidgetPanel', Ext.apply({}, objectConfig, {widgetId:objectConfig.widgetId, title:'Widget'}));
}, onDataReload:function() {
  var me = this;
  var oldRecord = me.getView().selectedRecord;
  var store = me.getView().grid.getStore();
  if (!store || store.loading) {
    return;
  }
  var newRecord = store.getById(oldRecord.id);
  if (newRecord) {
    me.getView().selectedRecord = newRecord;
    me.getView().fireEvent('recordUpdate', me.getView(), newRecord);
    me.getView().query('[onRecordUpdate]').forEach(function(cmp) {
      cmp.onRecordUpdate(newRecord);
    });
  }
}});
Ext.define('monitor.view.detail.DetailWindow', {extend:'BpcCommon.component.window.Window', alias:'widget.monitorDetailWindow', layout:'fit', scrollable:false, localized:{title:'MONITOR_PROCESS_DETAIL_WINDOW'}, iconCls:'x-fal fa-cubes', constrain:true, stateful:true, resizable:true, grid:undefined, selectedRecord:undefined, vm:undefined, hackMode:false, initComponent:function() {
  this.stateId = (this.stateId ? this.stateId : '') + 'detailWindow' + this.moduleId;
  var conf = BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.detailViewPopupConfig);
  if (conf.width) {
    this.width = conf.width;
  }
  if (conf.height) {
    this.height = conf.height;
  }
  if (this.hackMode === true) {
    this.items = this.createDetailContainer(conf, this.selectedRecord, this.vm, this.grid, true);
  } else {
    this.items = this.createDetailContainer(conf, this.selectedRecord, this.vm, this.grid);
  }
  if (conf.title) {
    this.localized = Ext.clone(this.localized);
    this.localized.title = this.createTitle(conf.title, this.selectedRecord);
  }
  this.listeners = {close:function() {
    try {
      this.query('monitorFileViewer').forEach(function(element) {
        return element.editor.destroy();
      });
    } catch (err) {
    }
  }};
  this.callParent(arguments);
}, createDetailContainer:function(conf, record, instanceConfig, grid, hackMode) {
  try {
    var instanceConfiguration;
    if (Ext.isDefined(hackMode) && hackMode === true) {
      instanceConfiguration = {instanceStoreUrl:BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + this.moduleId};
    } else {
      instanceConfiguration = instanceConfig.getData();
    }
    return Ext.create('monitor.view.detail.DetailContainer', Ext.apply({}, conf, {configuration:conf, selectedRecord:record, instanceConfiguration:instanceConfiguration, grid:grid, moduleId:this.moduleId, resizable:true, resizeHandles:'s', componentName:'detailViewPopup', bpcViewId:this.bpcViewId, queryId:grid.getQueryId()}));
  } catch (e) {
    Ext.log({msg:'Error while creating the detail container: ' + e, level:'error'});
  }
}, createTitle:function(title, record) {
  try {
    var tpl = new Ext.XTemplate(title);
    return tpl.apply(record.data);
  } catch (e) {
    Ext.log({msg:'Error while setting the title in DetailContainer: ' + e, level:'error'});
  }
}});
Ext.define('monitor.view.detail.FileViewer', {extend:'Ext.panel.Panel', alias:'widget.monitorFileViewer', controller:'monitorFileViewerController', receivedData:undefined, record:undefined, viewModel:{data:{hideSelectAll:false, hideXmlViewBtn:true}}, stateful:true, stateId:undefined, editorTheme:'chrome', layout:'fit', initComponent:function() {
  var me = this;
  var items = [];
  this.stateId = (this.stateId ? this.stateId : '') + 'detailWindowFileViewer' + this.moduleId;
  var mimeType = monitor.Util.getMimeType(me.record, me.moduleId);
  this.tbar = [{localized:{text:'MONITOR_DOWNLOAD'}, itemId:'downloadFileBnt', iconCls:BpcCommon.Icons.DOWNLOAD}, {localized:{text:'MONITOR_SELECT_ALL'}, itemId:'selectFileBnt', iconCls:'x-fal fa-i-cursor', bind:{disabled:'{hideSelectAll}'}}, {xtype:'tbspacer', width:20}, {localized:{text:'MONITOR_FORMATTED_DATA'}, toggleGroup:'changeFileView', iconCls:'x-fal fa-file-alt', handler:'showFormattedView', pressed:true, toggleHandler:function() {
    var me = this;
    if (!(me.pressed || Ext.ButtonToggleManager.getPressed(me.toggleGroup))) {
      me.toggle(true, true);
    }
  }}, {localized:{text:'MONITOR_TREE_VIEW'}, bind:{hidden:'{hideXmlViewBtn}'}, toggleGroup:'changeFileView', iconCls:'x-fal fa-sitemap', handler:'showXmlTreeView', toggleHandler:function() {
    var me = this;
    if (!(me.pressed || Ext.ButtonToggleManager.getPressed(me.toggleGroup))) {
      me.toggle(true, true);
    }
  }}, {localized:{text:'MONITOR_RAW_DATA'}, toggleGroup:'changeFileView', iconCls:'x-fal fa-file', handler:'showRawDataView', toggleHandler:function() {
    var me = this;
    if (!(me.pressed || Ext.ButtonToggleManager.getPressed(me.toggleGroup))) {
      me.toggle(true, true);
    }
  }}];
  this.callParent(arguments);
  lazyLoad.Vkbeautify().then(function() {
    if (mimeType === 'application/edifact') {
      me.mimeType = 'edifact';
      var beautifiedData = me.getController().getPrettifiedEDIData();
      items.push({xtype:'panel', layout:'card', reference:'ediData', defaults:{scrollable:true}, items:[me.getController().getCodeEditor(beautifiedData), {items:[me.getController().getFileView()]}], listeners:{afterrender:function(panel) {
        panel.getLayout().setActiveItem(0);
      }}});
    } else if (me.getController().isXML(me.record, me.receivedData)) {
      me.getViewModel().set('hideXmlViewBtn', false);
      me.mimeType = 'xml';
      var beautifiedData$5 = me.getController().getPrettifiedXMLData();
      items.push({xtype:'panel', layout:'card', reference:'xmlData', defaults:{scrollable:true}, items:[me.getController().getCodeEditor(beautifiedData$5), {layout:'fit', itemId:'xmlTreeView'}, {items:[me.getController().getFileView()]}], listeners:{afterrender:function(panel) {
        panel.getLayout().setActiveItem(0);
      }}});
    } else if (me.getController().isPdf(me.record, me.receivedData)) {
      var objectUrl = monitor.Util.binaryStringToObjectUrl(me.record, me.receivedData, mimeType);
      items.push({xtype:'box', itemId:'fileViewerPdf', autoEl:{tag:'iframe', loadMask:'Opening File...please wait!', width:'100%', height:'100%', src:objectUrl, onLoad:function() {
        window.URL.revokeObjectURL(objectUrl);
      }}});
    } else if (me.getController().isImage(me.record, me.receivedData)) {
      var objectUrl$6 = monitor.Util.binaryStringToObjectUrl(me.record, me.receivedData, mimeType);
      items.push({xtype:'image', itemId:'fileViewerImage', src:objectUrl$6, listeners:{load:{element:'el', fn:function() {
        window.URL.revokeObjectURL(objectUrl$6);
      }}}});
    } else {
      items.push({xtype:'box', reference:'fileViewerTextFile', autoEl:{tag:'pre', style:'overflow: auto; margin-left: 5px;', children:[{tag:'code', style:'display: inline;', html:Ext.util.Format.htmlEncode(me.receivedData)}]}});
    }
    me.add(items);
  });
}});
Ext.define('monitor.view.detail.FileViewerController', {extend:'Ext.app.ViewController', alias:'controller.monitorFileViewerController', control:{'#xmlToggleBnt':{click:'toggleSourceView'}, '#downloadFileBnt':{click:'downloadFile'}, '#selectFileBnt':{click:'selectFile'}, '#':{close:'onClose', resize:'onResize'}}, init:function() {
  this.callParent(arguments);
}, isXML:function(record, data) {
  var me = this;
  var mimeType = monitor.Util.getMimeType(record, me.getView().moduleId);
  if (!(typeof data === 'string')) {
    return false;
  }
  if (mimeType === 'text/xml' && (data.indexOf('\x3c') === -1 || data.indexOf('\x3e') === -1)) {
    return false;
  }
  if (mimeType === 'text/xml' || data.substr(0, 5) === '\x3c?xml') {
    return true;
  }
  if (data.substr(0, 1) !== '\x3c' || data.substr(data.length - 1, 1) !== '\x3e' || data.lastIndexOf('\x3c/') === -1) {
    return false;
  }
  var firstTagEndPosition = data.indexOf('\x3e');
  if (data.indexOf(' ') < firstTagEndPosition) {
    firstTagEndPosition = data.indexOf(' ');
  }
  var firstTag = data.substring(1, firstTagEndPosition);
  var lastTag = data.substr(data.lastIndexOf('\x3c/') + 2, firstTag.length);
  return firstTag === lastTag;
}, isImage:function(record) {
  var me = this;
  return monitor.Util.getMimeType(record, me.getView().moduleId).startsWith('image/');
}, isPdf:function(record) {
  var me = this;
  return 'application/pdf' === monitor.Util.getMimeType(record, me.getView().moduleId);
}, toggleSourceView:function(button) {
  var me = this;
  var moduleId = me.getView().moduleId;
  var mimetypeDataIndex = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnMimetype);
  var defaultMimetype = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnDefaultMimetype);
  this.getView().removeAll();
  if (button.pressed) {
    this.getView().add(this.getFileView());
  } else {
    var record = this.getView().record;
    var mimeType = record.get(mimetypeDataIndex);
    if (!Ext.isDefined(mimeType) || Ext.isEmpty(mimeType)) {
      mimeType = defaultMimetype;
    }
    if (mimeType === 'application/edifact') {
      Ext.util.CSS.swapStyleSheet('prettifyCss', 'resources/code-prettify/prettifyEdi.css');
      this.getView().add(this.getPrettifiedEDICmp());
    } else {
      Ext.util.CSS.swapStyleSheet('prettifyCss', 'resources/code-prettify/prettify.css');
      this.getView().add(this.getPrettifiedXMLCmp());
    }
  }
}, showFormattedView:function(btn) {
  var me = this;
  var xmlDataView = this.lookupReference('xmlData');
  var ediDataView = this.lookupReference('ediData');
  if (btn.pressed) {
    me.getViewModel().set('hideSelectAll', false);
    if (ediDataView && ediDataView.isVisible()) {
      ediDataView.getLayout().setActiveItem(0);
    } else if (xmlDataView && xmlDataView.isVisible()) {
      xmlDataView.getLayout().setActiveItem(0);
    }
  }
}, showXmlTreeView:function(btn) {
  var me = this;
  var xmlDataView = this.lookupReference('xmlData');
  if (btn.pressed) {
    me.getViewModel().set('hideSelectAll', true);
    if (xmlDataView && xmlDataView.isVisible()) {
      if (!xmlDataView.treeViewCreated) {
        xmlDataView.setLoading(true);
        xmlDataView.down('#xmlTreeView').add(me.getXMLTreePanel());
        xmlDataView.treeViewCreated = true;
        xmlDataView.setLoading(false);
      }
      xmlDataView.getLayout().setActiveItem(1);
    }
  }
}, showRawDataView:function(btn) {
  var me = this;
  var xmlDataView = this.lookupReference('xmlData');
  var ediDataView = this.lookupReference('ediData');
  if (btn.pressed) {
    me.getViewModel().set('hideSelectAll', false);
    if (ediDataView && ediDataView.isVisible()) {
      ediDataView.getLayout().setActiveItem(1);
    } else if (xmlDataView && xmlDataView.isVisible()) {
      xmlDataView.getLayout().setActiveItem(2);
    }
  }
}, getEDIMultilinePanel:function() {
  return this.createMultiLinePanel(this.getView().receivedData);
}, getXMLTreePanel:function() {
  var xmlData = monitor.Util.parseXML(this.getView().receivedData.replace(/<([a-zA-Z0-9]+)xmlns=".*"(.*)>/g, '\x3c$1$2\x3e').replace(/^\s/, ''));
  var childNode = CSS.escape(xmlData.children[0].nodeName);
  var treeData = Ext.DomQuery.selectNode(childNode, xmlData);
  return this.createXmlTree(treeData);
}, getFileView:function() {
  return {xtype:'box', reference:'fileViewerTextFile', scrollable:'true', autoEl:{tag:'pre', style:'overflow: auto; margin-left: 5px;', children:[{tag:'code', style:'display: inline;', html:Ext.util.Format.htmlEncode(this.getView().receivedData)}]}};
}, selectFile:function() {
  var me = this;
  var ediDataView = me.lookupReference('ediData');
  var xmlDataView = me.lookupReference('xmlData');
  var textView = me.lookupReference('fileViewerTextFile');
  if (ediDataView && ediDataView.isVisible()) {
    me.selectEditorContent(ediDataView);
  } else if (xmlDataView && xmlDataView.isVisible()) {
    me.selectEditorContent(xmlDataView);
  } else if (textView && textView.getEl && textView.isVisible()) {
    me.selectPlainTextContent(textView);
  }
}, selectEditorContent:function(container) {
  var me = this;
  try {
    var editorCmp = container.down('[editor]');
    editorCmp.editor.selection.selectAll();
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR selecting Code Content', dump:e, stack:true, level:'error'});
  }
}, selectPlainTextContent:function(textBox) {
  var me = this;
  try {
    var code = textBox.getEl().dom.querySelector('code');
    var range = document.createRange();
    range.selectNodeContents(code);
    var sel = window.getSelection();
    sel.removeAllRanges();
    sel.addRange(range);
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR selecting PlainText Content', dump:e, stack:true, level:'error'});
  }
}, createXmlTree:function(data) {
  var children = monitor.Util.nodeToJsonTree(data);
  var store = Ext.create('Ext.data.TreeStore', {root:{expanded:true, children:children}});
  var me = this;
  return Ext.create('Ext.tree.Panel', {store:store, reference:'fileViewerTreePanel', rootVisible:false, scrollable:true, listeners:{itemexpand:{fn:function() {
    var me = this;
    var columns = me.getColumns()[0];
    columns.autoSize();
  }}, afterrender:function(tree) {
    var root = tree.getRootNode();
    root.getChildAt(0).expand();
  }, itemcontextmenu:function(view, record, item, index, event) {
    event.stopEvent();
    if (record.hasChildNodes()) {
      Ext.create('Ext.menu.Menu', {margin:'0 0 10 0', items:[{localized:{text:'MONITOR_EXPAND'}, iconCls:'fal fa-plus', menu:{items:[{localized:{text:'MONITOR_EXPAND_NODE'}, iconCls:'fal fa-chevron-down', listeners:{click:function() {
        record.expand();
      }}}, {localized:{text:'MONITOR_EXPAND_CHILDREN'}, iconCls:'fal fa-chevron-double-down', listeners:{click:function() {
        me.expandChildNodes(record);
      }}}]}}, {localized:{text:'MONITOR_COLLAPSE'}, iconCls:'fal fa-minus', menu:{items:[{localized:{text:'MONITOR_COLLAPSE_NODE'}, iconCls:'fal fa-chevron-up', listeners:{click:function() {
        record.collapse();
      }}}, {localized:{text:'MONITOR_COLLAPSE_CHILDREN'}, iconCls:'fal fa-chevron-double-up', listeners:{click:function() {
        me.collapseChildNodes(record);
      }}}]}}]}).showAt(event.getXY());
    }
  }}});
}, expandChildNodes:function(node) {
  var me = this;
  node.expand();
  if (node.hasChildNodes()) {
    node.eachChild(function(child) {
      me.expandChildNodes(child);
    });
  }
}, collapseChildNodes:function(node) {
  var me = this;
  node.collapse();
  if (node.hasChildNodes()) {
    node.eachChild(function(child) {
      me.collapseChildNodes(child);
    });
  }
}, downloadFile:function() {
  var moduleId = this.getView().moduleId;
  var monitorInstance = BpcCommon.Api.getModuleInstanceComponent(moduleId);
  return monitorInstance.downloadFile(this.getView().record);
}, createMultiLinePanel:function(data) {
  var value = data.toString();
  var newLineRegExp = new RegExp('["\']', 'g');
  return Ext.create('Ext.Component', {reference:'ediMultiLineView', autoEl:{tag:'pre', style:'overflow: auto; margin-left: 5px;', children:[{tag:'code', style:'display: inline;', html:value.replace(newLineRegExp, "'\x3cbr/\x3e")}]}});
}, getPrettifiedEDIData:function() {
  var ediData = this.getView().receivedData;
  var newLineRegExp = new RegExp('["\']', 'g');
  return ediData.toString().replace(newLineRegExp, '\n');
}, getPrettifiedXMLData:function() {
  var xmlData = this.getView().receivedData;
  return vkbeautify.xml(xmlData, 3);
}, getCodeEditor:function(beautifiedData) {
  var mimeType = this.getView().mimeType;
  return {xtype:'bpcEditorField', value:beautifiedData, editorLanguage:mimeType, editorConfigs:{readOnly:true}};
}, onClose:function() {
  var me = this;
  try {
    this.editor.destroy();
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR closing the editor', dump:e, stack:true, level:'error'});
  }
}, onResize:function() {
  var me = this;
  try {
    this.editor.resize();
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR resizing the editor', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.detail.Forms', {extend:'Ext.container.Container', alias:'widget.monitorDetailForms', layout:'fit', title:'Forms', initComponent:function() {
  var me = this;
  if (me.configuration.title) {
    me.title = BpcCommon.Api.getTranslation(me.configuration.title);
  }
  if (me.configuration.iconCls) {
    me.iconCls = me.configuration.iconCls;
  }
  me.currentRecord = me.detailViewConfiguration.record.copy();
  me.callParent();
  BpcCommon.Api.loadModule('forms').then(function() {
    var formsMainViewClassName = BpcCommon.Api.getViewClassForType('forms');
    if (!formsMainViewClassName) {
      return me.showLoadingError();
    }
    var formsMainViewClass = Ext.ClassManager.get(formsMainViewClassName);
    if (!formsMainViewClass || !formsMainViewClass.xtype) {
      return me.showLoadingError();
    }
    me.formsCmp = me.add(Ext.applyIf({xtype:formsMainViewClass.xtype, flex:1, initialData:{records:Object.assign({}, [me.currentRecord.getData()])}}, me.configuration));
  }, me.showLoadingError);
}, onRecordUpdate:function(newRecord) {
  var me = this;
  if (me.configuration.ignoreUpdates === true || !newRecord || !me.hasRecordUpdates(newRecord)) {
    return;
  }
  me.currentRecord = newRecord;
  me.formsCmp.setFormData({records:Object.assign({}, [newRecord.getData()])});
}, showLoadingError:function() {
  var me = this;
  me.add({xtype:'bpcCenterMessageCmp', flex:1, message:'Failed to load BPC Forms module'});
}, hasRecordUpdates:function(newRecord) {
  var me = this;
  var hasUpdate = false;
  me.currentRecord.set(newRecord.getData());
  hasUpdate = me.currentRecord.dirty;
  me.currentRecord.reject();
  return hasUpdate;
}});
Ext.define('monitor.view.detail.InteractionPanel', {extend:'Ext.container.Container', alias:'widget.monitorInteractionPanel', scrollable:true, configuration:undefined, record:undefined, moduleId:undefined, widgetType:undefined, stateful:true, stateId:undefined, layout:'fit', initComponent:function() {
  this.stateId = (this.stateId ? this.stateId : '') + 'interactionPanel' + this.moduleId;
  this.items = [{xtype:this.configuration.formId.replace('widget.', ''), record:this.record, configuration:this.configuration}];
  this.callParent(arguments);
}});
Ext.define('monitor.view.detail.JumpPanel', {extend:'Ext.panel.Panel', alias:'widget.monitorJumpPanel', configuration:undefined, record:undefined, moduleId:undefined, stateful:true, stateId:undefined, layout:{'align':'stretch', 'type':'vbox'}, initComponent:function() {
  this.checkForTitle();
  this.stateId = (this.stateId ? this.stateId : '') + 'jumpPanel' + this.moduleId;
  this.createButtons();
  this.callParent(arguments);
}, checkForTitle:function() {
  if (this.tabTitle !== undefined && this.tabTitle !== null && this.tabTitle !== '') {
    this.setTitle(this.tabTitle);
  } else {
    this.setTitle(BpcCommon.Api.getTranslation('MONITOR_JUMP_TO'));
  }
}, createButtons:function() {
  var moduleJumpController = Ext.getApplication().getController('monitor.view.interaction.moduleJump.Controller');
  var jumpColumns = moduleJumpController.getJumpColumnsForMonitor(this.config.moduleId);
  var selectedRecord = this.config.record;
  if (jumpColumns.length === 0 || !selectedRecord) {
    this.hidden = true;
    return;
  }
  this.items = jumpColumns.map(function(column, index) {
    var value = selectedRecord.get(column.dataIndex);
    return {xtype:'button', margin:10, iconCls:'x-fal fa-external-link', text:value, hidden:value === undefined || value === '', itemId:'jumpToMenuItem-' + index, handler:moduleJumpController.doModuleJump.bind(moduleJumpController, column, selectedRecord, value, column.jumpParamFn)};
  });
}});
Ext.define('monitor.view.detail.ProcessRecordView', {extend:'Ext.panel.Panel', alias:'widget.monitorProcessRecordView', localized:{title:'MONITOR_PROCESS_DATA'}, layout:{type:'hbox', align:'stretch'}, configuration:undefined, record:undefined, grid:undefined, stateful:true, stateId:undefined, autoScroll:true, initComponent:function() {
  try {
    this.stateId = (this.stateId ? this.stateId : '') + 'detailWindowProcessRecordView' + this.moduleId;
    this.items = this.createItems(this.group, this.item, this.record);
    this.callParent(arguments);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent', dump:e, stack:true, level:'error'});
  }
}, createItems:function(group, item, record) {
  try {
    var items = [];
    if (!group && item) {
      items = [this.addItemsToContainer(item, record, this.config)];
    } else if (group) {
      items = [];
      Ext.each(group, function(groupElem, index) {
        var config = {fieldWidth:groupElem.fieldWidth ? groupElem.fieldWidth : 70, labelWidth:groupElem.labelWidth ? groupElem.labelWidth : 70};
        var minWidth = config.fieldWidth + config.labelWidth + 20;
        var fieldSet = {xtype:'fieldset', style:{marginRight:'10px'}, width:groupElem.width ? groupElem.width : '100%', minWidth:groupElem.minWidth ? groupElem.minWidth : minWidth, autoScroll:!monitor.Util.isEmpty(groupElem.autoScroll) ? groupElem.autoScroll : false, scrollable:'y', localized:{title:groupElem.title}, collapsible:groupElem.collapsible ? groupElem.collapsible : false, items:[this.addGroupsToContainer(groupElem.item, record, config)]};
        if (index === 0 || this.config.layout.type === 'vbox') {
          fieldSet.style.marginLeft = '10px';
        }
        items.push(Ext.apply(fieldSet, groupElem));
      }, this);
    }
    return items;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createItems', dump:e, stack:true, level:'error'});
  }
}, addItemsToContainer:function(items, record, config) {
  try {
    var container = {xtype:'box', autoEl:{tag:'ul', style:'padding-left: 10px;margin: 0; list-style: none; -ms-box-orient: horizontal;' + ' -webkit-flex-wrap: wrap;flex-wrap: wrap;', children:[]}, autoScroll:config.autoScroll ? config.autoScroll : false};
    Ext.each(items, function(item) {
      var labelWidth = config.labelWidth || 150;
      container.autoEl.children.push({tag:'li', style:'position: relative; padding: 5px 0 0 ' + labelWidth + 'px; height: 20px; margin-top: 10px', html:this.renderItem(item, record, config)});
    }, this);
    return container;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR addItemsToContainer', dump:e, stack:true, level:'error'});
  }
}, renderItem:function(item, record, config) {
  var me = this;
  try {
    var name = item.name;
    var labelWidth = config.labelWidth || 150;
    var fieldWidth = config.fieldWidth || 300;
    var html = '\x3cspan valign\x3d"top" style\x3d"position: absolute; left: 0px; width: ' + labelWidth + 'px;"\x3e\x3cb\x3e' + (item.title || item.name) + ':\x3c/b\x3e \x3c/span\x3e';
    var renderedValue = me.renderWithCustomRenderer(name, record.get(name), record) || record.get(name);
    renderedValue = '\x3cdiv style\x3d"width: ' + fieldWidth + 'px;"\x3e' + renderedValue + '\x3c/div\x3e';
    html = html + renderedValue;
    return html;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR renderItem', dump:e, stack:true, level:'error'});
  }
}, renderWithCustomRenderer:function(name, value, record) {
  var me = this;
  var monitorMain = BpcCommon.Api.getModuleInstanceComponent(me.moduleId);
  var monitorColumnConfig = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.columnConfig);
  var columnConfig = Ext.Array.findBy(monitorColumnConfig, function(colConfig) {
    return colConfig.id === name;
  });
  if (!columnConfig) {
    columnConfig = Ext.Array.findBy(monitorColumnConfig, function(colConfig) {
      return colConfig.dataIndex === name;
    });
  }
  var rendererId = monitorMain.findPlugin('monitorCustomRenderer').getRendererIdByColumnConfig(columnConfig) || name;
  return monitorMain.findPlugin('monitorCustomRenderer').render(rendererId, value, undefined, undefined, true, record);
}, addGroupsToContainer:function(items, record, config) {
  var me = this;
  try {
    var container = Ext.create('Ext.container.Container', {tpl:['\x3ctpl if\x3d"data.length \x26gt; 0"\x3e', '\x3ctable style\x3d"width: 100%; height: 100%"\x3e', '\x3ctpl for\x3d"data"\x3e', "  \x3ctr data-qtip\x3d'{renderedHtml:htmlEncode2}'\x3e", '    \x3ctd valign\x3d"top" style\x3d"width:' + config.labelWidth + 'px; max-width:' + config.labelWidth + '"\x3e\x3cb\x3e{title}:\x3c/b\x3e\x3c/td\x3e', '    \x3ctd valign\x3d"top" \n                        style\x3d"width:' + config.fieldWidth + 'px; \n                        max-width:' + 
    config.fieldWidth + 'px;\n                        padding-right: 10px;"\x3e\n                        {renderedHtml}\n                     \x3c/td\x3e', '  \x3c/tr\x3e', '\x3c/tpl\x3e', '\x3c/table\x3e', '\x3c/tpl\x3e']});
    var data = [];
    var hideEmptyValues;
    var detailPopupConfig = BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.detailViewPopupConfig);
    if (Ext.isObject(detailPopupConfig) && detailPopupConfig.hideEmptyValues) {
      hideEmptyValues = detailPopupConfig.hideEmptyValues;
    } else {
      hideEmptyValues = false;
    }
    Ext.each(items, function(item) {
      var name = item.name;
      var value = record.get(name);
      var title = item.title ? item.title : item.name;
      var elem = null;
      var html = me.renderWithCustomRenderer(name, value, record) || value;
      elem = {title:BpcCommon.Api.getTranslation(title), renderedHtml:html};
      if (hideEmptyValues) {
        if (elem.renderedHtml && elem.renderedHtml !== '' && elem.renderedHtml !== null) {
          data.push(elem);
        }
      } else {
        data.push(elem);
      }
    }, this);
    container.update({data:data});
    return container;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR addGroupsToContainer', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.detail.TabPanel', {extend:'Ext.tab.Panel', alias:'widget.monitorDetailTabPanel', controller:'monitorDetailTabPanelController', activeTab:0, width:700, tabs:undefined, configuration:undefined, record:undefined, grid:undefined, stateful:true, stateId:undefined, listeners:{tabchange:function(tabPanel) {
  tabPanel.scrollTo('Top', 0, true);
}, afterrender:'onAfterRender'}, initComponent:function() {
  this.stateId = (this.stateId ? this.stateId : '') + 'detailWindowTabPanel' + this.moduleId;
  Ext.apply(this, {items:this.tabs});
  this.callParent(arguments);
}});
Ext.define('monitor.view.detail.TabPanelController', {extend:'Ext.app.ViewController', alias:'controller.monitorDetailTabPanelController', init:function() {
  this.callParent(arguments);
}, onAfterRender:function() {
  if (this.getView().items.items.length === 0) {
    this.getView().getEl().mask(BpcCommon.Api.getTranslation('MONITOR_LOADING_DATA'));
  }
}, unMask:function() {
  var el = this.getView().getEl();
  if (typeof el !== 'undefined') {
    el.unmask();
    this.getView().setActiveTab(0);
  }
}, onShowMessage:function(record, fileDataIndex, config, childLogGridView) {
  var data = record.get(fileDataIndex);
  var me = this;
  var settings = this.getView().settings;
  var baseUrl = settings.baseUrl;
  var referenceEndpoint = settings.referenceEndpoint;
  var moduleId = this.getView().moduleId;
  if (BpcCommon.Util.isBase64(data)) {
    me.showFileInTab(atob(data), record, fileDataIndex, config);
    return;
  }
  if (!monitor.Util.isEmpty(baseUrl) && !monitor.Util.isEmpty(referenceEndpoint)) {
    var monitorInstance = BpcCommon.Api.getModuleInstanceComponent(moduleId);
    var url = monitorInstance.createFileDownloadUrl(record);
    var isBinary = monitor.Util.isBinaryMimeType(monitor.Util.getMimeType(record, moduleId));
    me.getView().setLoading(true);
    Ext.Ajax.request({url:url, method:'GET', binary:isBinary, callback:function(options, success, response) {
      if (success) {
        var responseData = isBinary ? response.responseBytes : response.responseText;
        var previewMaxAllowedSize = BpcCommon.Api.getSetting(moduleId, 'function_filePreviewSizeLimit') || 300000;
        if (responseData.length > previewMaxAllowedSize) {
          BpcCommon.Msg.show({title:BpcCommon.Api.getTranslation('MONITOR_INFORMATION'), msg:BpcCommon.Api.getTranslation({key:'MONITOR_PREVIEW_UNABLE_MSG', data:{fileSize:Ext.util.Format.fileSize((new Blob([responseData])).size).replace('bytes', 'Bytes'), maxSizeForPreview:Ext.util.Format.fileSize(previewMaxAllowedSize).replace('bytes', 'Bytes')}}), fn:function(btn) {
            if (btn === 'yes') {
              me.triggerDownloadFile(record, fileDataIndex, childLogGridView);
            } else if (btn === 'no') {
              me.showFileInTab(responseData, record, fileDataIndex, config);
            }
          }, icon:BpcCommon.Msg['WARNING'], buttons:BpcCommon.Msg.YESNOCANCEL, buttonIcons:{yes:'x-fal fa-download', no:'x-fal fa-desktop', cancel:BpcCommon.Icons.CANCEL}, buttonText:{yes:BpcCommon.Api.getTranslation('MONITOR_DOWNLOAD'), no:BpcCommon.Api.getTranslation('MONITOR_PREVIEW'), cancel:BpcCommon.Api.getTranslation('MONITOR_CANCEL')}});
        } else {
          me.showFileInTab(responseData, record, fileDataIndex, config);
        }
      } else {
        BpcCommon.Msg.alert(BpcCommon.Api.getTranslation('MONITOR_ERROR'), BpcCommon.Api.getTranslation('MONITOR_TAB_PANEL_ERROR_MSG'));
      }
      me.getView().setLoading(false);
    }});
  } else {
    BpcCommon.Msg.alert(BpcCommon.Api.getTranslation('MONITOR_ERROR_DOWNLOADING_DATA'), BpcCommon.Api.getTranslation('MONITOR_ERROR_DOWNLOADING_DATA_MSG'));
    Ext.log({msg:"Problem in tab panel: Couldn't find the configuration '" + monitor.assets.ConfigKeys.bpmBasePath + "', '" + monitor.assets.ConfigKeys.legacyPmTablePrefix + "', or '" + monitor.assets.ConfigKeys.bpmReferenceEndpoint + "'. Please check the configuration of monitor.", level:'warn'});
  }
}, showFileInTab:function(recordFileData, record, fileDataIndex, config) {
  var view = this.getView();
  var childIDKey = view.settings.childId;
  var childId = record.get(childIDKey);
  var itemId = ('tab-' + childId).replace(/[^a-zA-Z0-9]/g, '');
  var title = '';
  if (config.tabTitle) {
    var tpl = new Ext.XTemplate(config.tabTitle);
    title = tpl.apply(record.data);
  } else {
    title = childId !== '' ? BpcCommon.Api.getTranslation('MONITOR_FILE_CONTENT') + childId : BpcCommon.Api.getTranslation('MONITOR_FILE_CONTENT');
  }
  var newTab = Ext.createByAlias('widget.monitorFileViewer', Ext.apply({}, {title:title, itemId:itemId, closable:true, receivedData:recordFileData, record:record, sizeLimits:config.sizeLimits || {}, fileDataIndex:fileDataIndex, settings:this.getView().settings, moduleId:this.getView().moduleId}));
  if (record.fileViewerTab) {
    var fileViewerTab = record.fileViewerTab;
    var index = view.items.indexOf(record.fileViewerTab);
    view.remove(fileViewerTab);
    view.insert(index, newTab);
  } else {
    this.getView().add(newTab);
  }
  record.fileViewerTab = newTab;
  this.getView().setActiveTab(newTab);
}, getTabIndex:function(itemId) {
  var index = -1;
  this.getView().items.each(function(tab, idx) {
    if (tab.itemId && tab.itemId === itemId) {
      index = idx;
      return;
    }
  });
  return index;
}, triggerDownloadFile:function(record, fileDataIndex, childLogGridView) {
  childLogGridView.getController().downloadFile(record, fileDataIndex);
}});
Ext.define('monitor.view.detail.TaskPanel', {extend:'Ext.panel.Panel', alias:'widget.monitorTaskPanel', autoScroll:true, configuration:undefined, record:undefined, moduleId:undefined, widgetType:undefined, stateful:true, stateId:undefined, initComponent:function() {
  this.stateId = (this.stateId ? this.stateId : '') + 'detailWindowTaskPanel' + this.moduleId;
  var items = [];
  items.push(this.addComponent(this.configuration));
  this.items = items;
  this.callParent(arguments);
}, addComponent:function(configuration) {
  var type = configuration.widgetType;
  try {
    if (Ext.ClassManager.getByAlias('widget.' + type)) {
      return Ext.createByAlias('widget.' + type, {configuration:configuration, record:this.record, moduleId:this.moduleId});
    }
    Ext.log({msg:'Unknown type: ' + type, level:'warn'});
    return this.showErrorMessage(BpcCommon.Api.getTranslation('MONITOR_TYPE_IS_UNKNOWN') + ': ' + type + '.');
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR', dump:e, stack:true, level:'error'});
    return this.showErrorMessage('');
  }
}, showErrorMessage:function(message) {
  return Ext.create('Ext.container.Container', {layout:{type:'hbox', align:'stretch'}, padding:10, items:[{xtype:'box', html:'\x3cp\x3e\x3c/p\x3e\x3ci class\x3d"fa fa-exclamation-triangle" style\x3d"color: #D49A6A; font-size: 18px; margin-right: 3px;"\x3e\x3c/i\x3e\x3c/p\x3e'}, {xtype:'box', html:'\x3cp\x3e' + BpcCommon.Api.getTranslation('MONITOR_ERROR_LOADING_COMPONENT') + message + '\x3c/p\x3e', style:{marginLeft:'10px'}}]});
}});
Ext.define('monitor.view.detail.WidgetPanel', {extend:'Ext.panel.Panel', alias:'widget.monitorWidgetPanel', controller:'monitorWidgetPanelController', autoScroll:true, layout:'fit', configuration:undefined, widgetId:undefined, stateful:true, stateId:undefined, initComponent:function() {
  this.stateId = (this.stateId ? this.stateId : '') + 'detailWindowWidgetPanel' + this.moduleId;
  var items = [];
  items.push(this.getController().addComponent(this.widgetId));
  Ext.apply(this, {items:items});
  this.callParent(arguments);
}});
Ext.define('monitor.view.detail.WidgetPanelController', {extend:'Ext.app.ViewController', alias:'controller.monitorWidgetPanelController', addComponent:function(widgetId) {
  var widget = BpcCommon.Api.getWidgetComponent(widgetId);
  if (!monitor.Util.isEmpty(widget)) {
    var title = widget.config.configuration.title;
    this.getView().title = title;
    return widget;
  }
  return this.showErrorMessage(BpcCommon.Api.getTranslation('MONITOR_WIDGET_PANEL_ERROR1') + widgetId + BpcCommon.Api.getTranslation('MONITOR_WIDGET_PANEL_ERROR2'));
}, showErrorMessage:function(message) {
  return Ext.create('Ext.container.Container', {layout:{type:'hbox', align:'stretch'}, padding:10, items:[{xtype:'box', html:'\x3cp\x3e\x3c/p\x3e\x3ci class\x3d"fa fa-exclamation-triangle" style\x3d"color: #D49A6A; font-size: 18px; margin-right: 3px;"\x3e\x3c/i\x3e\x3c/p\x3e'}, {xtype:'box', html:'\x3cp\x3eFehler beim Laden der Komponente. ' + BpcCommon.Api.getTranslation('MONITOR_ERROR_LOADING_COMPONENT') + message + '\x3c/p\x3e', style:{marginLeft:'10px'}}]});
}});
Ext.define('monitor.view.grid.GridController', {extend:'Ext.app.ViewController', alias:'controller.monitorGridController', control:{'monitorGrid \x3e gridview':{expandbody:'onExpandBody', collapsebody:'onCollapseBody', refresh:'onRefresh', itemremove:'onViewItemRemove', itemadd:'onViewItemAdd', scroll:{element:'body', buffer:200, fn:'removeFocus'}}, 'monitorGrid':{selectionchange:'onSelectionChange', cellcontextmenu:'onShowContextMenu', rowdblclick:'onDetailWindow', sortchange:{fn:'onSortChange', 
buffer:100}, filterchange:'onFilterChange', show:'onViewChange'}}, listen:{global:{dataLoaded:'onDataLoaded', moduleShow:'onViewChange'}}, lastCellIndex:undefined, lastRowIndex:undefined, lastValue:undefined, lastClickedColumn:undefined, lastRecord:undefined, actualSelection:undefined, expandMap:undefined, storeItems:0, init:function() {
  this.moduleId = this.getView().config.moduleId;
  this.handleGlobalDateRange = BpcCommon.Util.getGlobalDateRangeHandler(this.getView().getStore(), BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.columnTimeseries));
  var vm = this.getViewModel();
  BpcCommon.Api.getGlobalViewModel().bind({bindTo:'{global.dateRange}', deep:true}, Ext.Function.createBuffered(this.handleGlobalDateRange, 100, this));
  this.handleGlobalDateRange(true);
  vm.bind('{globalFilterValue}', Ext.Function.createBuffered(this.onGlobalSearch, 500, this));
  if (!this.grid) {
    this.grid = this.getView();
    this.grid.store.on('load', this.onStoreLoad.bind(this));
    this.grid.store.on('beforeload', this.onBeforeLoad.bind(this));
    var proxy = this.grid.store.getProxy();
    proxy.on('exception', this.onException.bind(this));
    this.proxy = proxy;
    this.grid.up('bpcMonitor').on('show', this.onViewChange.bind(this));
  }
  this.expandMap = [];
  var bufferSetting = BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.dataRefreshBufferInSeconds, true);
  bufferSetting.on('changed', this.updateRefreshDataBufferedFn, this);
  this.updateRefreshDataBufferedFn(bufferSetting);
  this.autoRefreshTaskHolder = Ext.TaskManager.start({run:this.autoRefreshTask.bind(this), interval:1000 * 30});
  this.bufferdNewDataNotification = Ext.Function.createThrottled(this.showNewDataNotification, 15000, this);
}, showNewDataNotification:function() {
  var me = this;
  if (!me.getView().isVisible(true)) {
    this.showNotificationOnView = true;
    return;
  }
  BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation({key:'MONITOR_NEW_DATA_AVAILABLE', data:{monitorId:me.moduleId}}));
}, updateRefreshDataBufferedFn:function(bufferSettingModel) {
  if (bufferSettingModel instanceof Ext.data.Model) {
    var bufferInSeconds = bufferSettingModel.get('value');
    this.refreshDataBuffered = Ext.Function.createThrottled(this.refreshData, bufferInSeconds * 1000);
  }
}, refreshDataBuffered:function() {
  this.refreshData();
}, autoRefreshTask:function() {
  try {
    var autoRefresh = !this.getMainViewModel().get('autoRefreshDisabled');
    if (autoRefresh && !BpcCommon.Api.hasWebsocketConnection()) {
      this.onRefresh();
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR autoRefreshTask', dump:e, stack:true, level:'error'});
  }
}, onDataLoaded:function(dataObj) {
  try {
    var queryId = this.getView().getQueryId();
    var store = this.getView().getStore();
    if (dataObj.properties.componentId && (dataObj.properties.componentId === 'monitor' || dataObj.properties.componentId === 'monitor_details') && dataObj.properties.customId && dataObj.properties.customId.startsWith(queryId)) {
      if (dataObj.properties.dataCount === 0) {
        var needRefresh = false;
        if (Ext.isArray(dataObj.properties.obsoleteIDs)) {
          Ext.Array.each(dataObj.properties.obsoleteIDs, function(oId) {
            if (store.getById(oId) || store.getById(parseFloat(oId))) {
              needRefresh = true;
              return false;
            }
          });
        }
        if (!needRefresh && Ext.isArray(dataObj.properties.deletedIDs)) {
          Ext.Array.each(dataObj.properties.deletedIDs, function(dId) {
            if (store.getById(dId) || store.getById(parseFloat(dId))) {
              needRefresh = true;
              return false;
            }
          });
        }
        if (!needRefresh) {
          return;
        }
      }
      if (this.getViewModel().get('setting.' + monitor.assets.ConfigKeys.autoUpdate)) {
        this.refreshDataBuffered();
      } else {
        this.bufferdNewDataNotification();
      }
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onDataLoaded', dump:e, stack:true, level:'error'});
  }
}, onViewItemRemove:function(records) {
  try {
    Ext.Array.each(records, function(record) {
      if (record && record.detailViewInlineCmp) {
        record.detailViewInlineCmp.destroy();
        delete record.detailViewInlineCmp;
      }
    });
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onViewItemRemove', dump:e, stack:true, level:'error'});
  }
}, onViewItemAdd:function(records) {
  try {
    var expandMap = this.getView().getPlugin('bpcrowexpander').recordsExpanded;
    Ext.Array.each(records, function(record) {
      if (record && expandMap[record.internalId] === true) {
        this.showInlineDetailView(record);
      }
    }, this);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onViewItemAdd', dump:e, stack:true, level:'error'});
  }
}, onException:function(proxy, response) {
  try {
    var me = this;
    if (response.aborted === true) {
      return;
    }
    var msg = '';
    if (response.status === 401) {
      BpcCommon.Api.handleSessionLost();
      return;
    }
    var errorObj = BpcCommon.Util.getErrorFromFailureResponse(response);
    msg = BpcCommon.Api.getTranslation('MONITOR_DATA_LOADING_ERROR') + ' - ' + errorObj.message;
    if (me.isFullTextSearchActive()) {
      msg = BpcCommon.Api.getTranslation('MONITOR_DATA_LOADING_ERROR_FULL_TEXT_SEARCH');
    }
    BpcCommon.Api.showNotification({type:'ERROR', text:msg, toast:true, toastTarget:me.getView()});
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onException', dump:e, stack:true, level:'error'});
  }
}, isFullTextSearchActive:function() {
  var me = this;
  var vm = me.getViewModel();
  var isSearchFieldActive = vm.get('globalFilterActive');
  var searchFieldString = vm.get('globalFilterValue');
  return !!isSearchFieldActive && !Ext.isEmpty(searchFieldString);
}, onRefresh:function() {
  try {
    var grid = this.getView();
    var expander = grid.getPlugin('bpcrowexpander');
    var moduleId = this.getView().config.moduleId;
    var viewId = this.getView().bpcGridViewName;
    var orphanCmps = Ext.ComponentQuery.query('detailContainer[moduleId\x3d' + moduleId + '][bpcViewId\x3d' + viewId + '][componentName\x3ddetailViewInline]');
    Ext.Array.each(orphanCmps, function(cmp) {
      cmp.destroy();
    });
    this.updateColumnText();
    var store = grid.getStore();
    if (this.expandMap) {
      Ext.Array.each(this.expandMap, function(id) {
        try {
          var extModel = store.getById(id);
          var row = store.indexOf(extModel);
          if (row >= 0) {
            expander.toggleRow(row, extModel);
          }
        } catch (e) {
          Ext.log('Fehler beim Herstellen der detailInlineView' + e);
        }
      });
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onRefresh', dump:e, stack:true, level:'error'});
  }
}, onSortChange:function() {
  var me = this;
  var view = me.getView();
  var sorters = view.getStore().sorters;
  var mainView = me.getMainView();
  me.updateColumnText();
  mainView.updateSorterItems(sorters);
  mainView.updateTooltip(view);
  me.expandMap = [];
}, updateColumnText:function() {
  try {
    var columnManager = this.getView().getColumnManager();
    var columnArray = columnManager.getColumns();
    var sorters = this.grid.getStore().getSorters();
    Ext.Array.each(columnArray, function(column) {
      if (column.textEl && column.textEl.setHtml) {
        var headerIconEl = column.headerIconCls ? '\x3cspan role\x3d"presentation" data-ref\x3d"iconEl" class\x3d"' + column.headerIconCls + ' ' + Ext.baseCSSPrefix + 'column-header-iconEl"\x3e\x26nbsp\x3c/span\x3e' : '';
        var filterIcon = '\x3cspan role\x3d"presentation" class\x3d"' + Ext.baseCSSPrefix + 'bpc-filter-icon"\x3e\x3c/span\x3e';
        var filterInverted = false;
        if (column.filter && column.filter.filter && column.filter.active) {
          filterInverted = column.filter.filter.getConfig('invert');
        }
        if (filterInverted === true) {
          filterIcon = '\x3cspan class\x3d"fa-stack fa-1x ' + Ext.baseCSSPrefix + 'bpc-filter-invert-icon  ""\x3e\n                                    \x3ci class\x3d"fas fa-circle fa-stack-2x"\x3e\x3c/i\x3e\n                                    \x3ci class\x3d"fas fa-filter fa-stack-1x fa-inverse"\x3e\x3c/i\x3e\n                                \x3c/span\x3e';
        }
        var headerTextEl = '\x3cspan role\x3d"presentation" data-ref\x3d"textInnerEl" class\x3d"' + Ext.baseCSSPrefix + 'column-header-text-inner"\x3e' + column.text + '\x3c/span\x3e';
        var sortRank = '';
        var sortCls = '';
        if (sorters.get(column.dataIndex)) {
          sortRank = sorters.indexOfKey(column.dataIndex) + 1;
          sortCls = 'bpc-monitor-filterActive';
        }
        var headerRankEl = '\x3cspan role\x3d"presentation" class\x3d"' + Ext.baseCSSPrefix + 'bpc-rank ' + sortCls + '"\x3e' + sortRank + '\x3c/span\x3e';
        column.textEl.setHtml(headerIconEl + headerTextEl + filterIcon + headerRankEl);
      }
    });
  } catch (e) {
    Ext.log('Fehler beim Resetten des Spaltentextes' + e);
  }
}, updateModuleHeaderInfos:function() {
  try {
    var mainView = this.getMainView();
    var gridView = this.getView();
    var store = gridView.getStore();
    var filters = store.getFilters();
    var sorters = store.getSorters();
    mainView.updateFilterItems(filters);
    mainView.updateTooltip(gridView);
    mainView.updateSorterItems(sorters);
    mainView.updateDataItems(this.storeItems);
    mainView.getViewModel().set('totalCount', this.storeItems);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateModuleHeaderInfos', dump:e, stack:true, level:'error'});
  }
}, onFilterChange:function(store, filters) {
  try {
    var vm = this.getMainViewModel();
    vm.set('activeFilter', filters.length);
    var mainView = this.getMainView();
    var gridView = this.getView();
    mainView.updateFilterItems(filters);
    this.updateActiveFilter(filters);
    mainView.updateTooltip(gridView);
    this.expandMap = [];
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR', dump:e, stack:true, level:'error'});
  }
}, updateActiveFilter:function(filters) {
  try {
    var vm = this.getMainViewModel();
    if (!filters) {
      var store = this.getView().getStore();
      filters = store.getFilters().items;
    }
    var filtersLength = filters.length;
    var countColumns = BpcCommon.Api.getSetting(this.getView().config.moduleId, monitor.assets.ConfigKeys.countColumns);
    if (countColumns) {
      var filterProperties = [];
      if (countColumns) {
        if (filters.length > 0) {
          Ext.Array.each(filters, function(filter) {
            if (filter) {
              filterProperties.push(filter.getConfig('property'));
            }
          });
        }
        filterProperties = Ext.Array.unique(filterProperties);
        filtersLength = filterProperties.length;
      }
    }
    if (vm.get('globalFilterValue')) {
      filtersLength++;
    }
    vm.set('activeFilter', filtersLength);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateActiveFilter', dump:e, stack:true, level:'error'});
  }
}, onViewChange:function() {
  try {
    var view = this.getView();
    if (view.isVisible(true)) {
      if (this.refreshDataOnView) {
        this.refreshDataOnView = false;
        this.refreshData(true);
      }
      if (this.showNotificationOnView) {
        this.showNotificationOnView = false;
        this.bufferdNewDataNotification();
      }
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onViewChange', dump:e, stack:true, level:'error'});
  }
}, onDetailWindow:function(view, record) {
  var vm = this.getMainViewModel();
  var showWindow = BpcCommon.Api.getSetting(vm.get('moduleId'), monitor.assets.ConfigKeys.detailViewPopupEnabled) && BpcCommon.Api.getLicenseInformationForKey('MODULE_MONITOR_DETAILVIEW_POPUP') !== false;
  if (showWindow) {
    try {
      var detailedWindow = Ext.createByAlias('widget.monitorDetailWindow', {moduleId:vm.get('moduleId'), selectedRecord:record, vm:vm, grid:this.getView(), width:document.body.clientWidth * 0.8, height:document.body.clientHeight * 0.8, minWidth:document.body.clientWidth * 0.4, minHeight:document.body.clientHeight * 0.5, maxWidth:document.body.clientWidth, maxHeight:document.body.clientHeight, bpcViewId:this.getView().bpcGridViewName, viewModel:{parent:vm}});
      detailedWindow.show();
    } catch (e) {
      BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation('MONITOR_DETAIL_WINDOW_ERROR') + e);
    }
  }
}, refreshData:function(force) {
  var me = this;
  var forceRefresh = force || false;
  var view = this.getView();
  if (!forceRefresh && !view.isVisible(true)) {
    this.refreshDataOnView = true;
    return;
  }
  var selectionModel = this.getView().getSelectionModel();
  this.actualSelection = selectionModel.getSelection();
  me.reloadStore(function() {
    me.updateActiveFilter();
    me.updateModuleHeaderInfos();
    me.updateChildLogGrids();
    me.updateDetailViews();
    view.fireEvent('selectionchange', selectionModel, view.getSelection());
  });
}, reloadStore:function(callback) {
  this.getView().store.reload({callback:callback});
}, onSelectionChange:function(selModel, selection) {
  var me = this;
  try {
    me.updateModuleHeaderInfos();
    selection = [].concat(selection);
    me.getViewModel().set('selectionCount', selection.length);
    me.getView().setSelectedRecords(selection);
    this.actualSelection = selection;
    if (selection.length > 0) {
      this.updateSplitView(selection[0]);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onSelectionChange', dump:e, stack:true, level:'error'});
  }
}, getMainView:function() {
  try {
    return this.getView().up('bpcMonitor');
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR getMainView', dump:e, stack:true, level:'error'});
  }
}, getMainViewModel:function() {
  try {
    return this.getView().up('bpcMonitor').getViewModel();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR getMainViewModel', dump:e, stack:true, level:'error'});
  }
}, getMainViewController:function() {
  try {
    return this.getView().up('bpcMonitor').getController();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR getMainViewController', dump:e, stack:true, level:'error'});
  }
}, fireStoreLoadEventOnSuccess:function(moduleId) {
  Ext.GlobalEvents.fireEvent('monitor_store_load', {baseModuleId:'monitor', moduleId:moduleId});
}, onStoreLoad:function(store, records, success) {
  var me = this;
  var gridView = this.getView();
  if (me.destroyed) {
    return;
  }
  if (success === true) {
    this.fireStoreLoadEventOnSuccess(gridView.moduleId);
    var total = store.getTotalCount();
    var vm = this.getView().up('bpcMonitor').getViewModel();
    vm.set('totalCount', total);
    if (this.getView() === this.getMainView().getActiveGrid()) {
      me.getMainView().updateDataItems(total);
    }
    me.storeItems = total;
    if (gridView.multiColumnSort && !isNaN(gridView.multiSortLimit) && gridView.multiSortLimit > 0) {
      store.sorters.getOptions().setMultiSortLimit(gridView.multiSortLimit);
    } else {
      store.sorters.getOptions().setMultiSortLimit(1);
    }
    me.getMainView().updateTooltip(gridView);
    me.getView().fireEvent('load', me.getView());
  } else {
    Ext.log({level:'info', msg:'MONITOR_UNEXPECTED_ERROR_DATA'});
  }
  gridView.saveState();
}, getElementIdForRecord:function(record) {
  return this.getView().rowExpandDivPrefix + record.get('bpcCleanedId');
}, createDetailContainer:function(config, record, instanceConfig, grid, resizable, componentName) {
  var targetId = this.getElementIdForRecord(record);
  if (componentName === 'detailViewInline') {
    config = BpcCommon.Api.getSetting(this.getView().config.moduleId, monitor.assets.ConfigKeys.detailViewInlineConfig);
    if (!config.width) {
      var visibleWidth = this.getView().getView().getWidth() - 64;
      var parentWidth = Ext.get(targetId).getWidth();
      config.width = parentWidth < visibleWidth ? parentWidth : visibleWidth;
    }
    config = Ext.applyIf(config, {resizable:{handles:'s'}, padding:0});
  }
  return Ext.create('monitor.view.detail.DetailContainer', Ext.apply({}, config, {configuration:config, selectedRecord:record, bpcViewId:this.getView().bpcGridViewName, instanceConfiguration:instanceConfig.getData(), grid:grid, renderTo:targetId, moduleId:this.getView().config.moduleId, componentName:componentName, queryId:this.getView().getQueryId()}));
}, onExpandBody:function(rowNode, record) {
  this.showInlineDetailView(record);
}, showInlineDetailView:function(record) {
  try {
    var conf = false;
    var detailedContainer = this.createDetailContainer(conf, record, this.getMainViewModel(), this.getView(), true, 'detailViewInline');
    record.detailViewInlineCmp = detailedContainer;
    detailedContainer.show();
    var enthalten = Ext.Array.contains(this.expandMap, record.id);
    if (!enthalten) {
      this.expandMap.push(record.id);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR showInlineDetailView', dump:e, stack:true, level:'error'});
  }
}, onCollapseBody:function(rowNode, record) {
  Ext.Array.remove(this.expandMap, record.id);
  if (record.detailViewInlineCmp) {
    record.detailViewInlineCmp.destroy();
    delete record.detailViewInlineCmp;
  }
  var elem = Ext.get(this.getElementIdForRecord(record));
  elem.update('');
}, removeAllFilter:function() {
  this.getView().getStore().clearFilter();
  this.getMainViewController().setMonitorFilter(true);
}, removeSorter:function(dataIndex) {
  var sorters = this.grid.getStore().sorters;
  if (dataIndex) {
    sorters.removeAtKey(dataIndex);
  } else {
    sorters.clear();
    this.updateColumnText();
  }
  this.refreshData();
}, onShowContextMenu:function(view, td, cellIndex, record, tr, rowIndex, e) {
  e.preventDefault();
  var grid = this.getView();
  var moduleId = grid.config.moduleId;
  var copyCellEnabled = true;
  var column = view.panel.headerCt.getHeaderAtIndex(cellIndex);
  var clickedDataIndex = column.dataIndex;
  var cellValue = record.get(clickedDataIndex);
  this.lastValue = cellValue;
  if (cellValue) {
    if (cellValue === '') {
      copyCellEnabled = false;
    }
  } else {
    copyCellEnabled = false;
  }
  this.lastCellIndex = cellIndex;
  this.lastRowIndex = rowIndex;
  this.lastClickedColumn = clickedDataIndex;
  this.lastRecord = record;
  if (Ext.isArray(grid.getSelectedRecords()) && grid.getSelectedRecords().length < 2) {
    grid.getSelectionModel().select(rowIndex);
  }
  var contextMenu = Ext.create({xtype:'monitorContextMenu', column:column, copyCellEnabled:copyCellEnabled, selectedCellValue:cellValue, grid:grid, record:record, moduleId:moduleId, viewModel:{parent:this.getViewModel()}, collaborationConfig:{moduleId:moduleId, collabEnabled:grid.collabEnabled, processKey:BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnId)}});
  contextMenu.showAt(e.getXY());
}, updateSplitView:function(record) {
  if (this.getMainViewController().isSplitViewActive()) {
    var conf = this.getMainView().getSetting(monitor.assets.ConfigKeys.detailViewEastConfig);
    var detailedContainer = this.createDetailContainer(conf, record, this.getMainViewModel(), this.getView(), 'detailViewEast');
    this.getMainViewController().updateSplitViewWithCmp(detailedContainer);
  }
}, addFilterToStore:function(filterObj) {
  this.getView().store.addFilter(filterObj);
}, setMonitorFilter:function(kill) {
  try {
    if (kill) {
      this.getViewModel().set('globalFilterValue', '');
    } else {
      this.getViewModel().set('globalFilterValue', this.getViewModel().get('globalFilterValue') + ' ' + this.lastValue);
    }
  } catch (e) {
    BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation('MONITOR_ERROR_SETTING_FILTER') + e);
  }
  this.updateActiveFilter();
  this.updateModuleHeaderInfos();
}, removeFocus:function() {
  try {
    Ext.get(document.activeElement).blur();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR removeFocus', dump:e, stack:true, level:'error'});
  }
}, filterCurrentViewOrSelect:function() {
  var recordsToFilter = this.getView().getSelection();
  var store = this.getView().getStore();
  if (!recordsToFilter || recordsToFilter.length === 0) {
    recordsToFilter = this.getVisibleRecords();
  }
  recordsToFilter = Ext.Array.map(recordsToFilter, function(record) {
    return record.getId();
  });
  var filter = new Ext.util.Filter({operator:'\x3d', value:recordsToFilter, source:'raw', property:BpcCommon.Api.getSetting(this.moduleId, monitor.assets.ConfigKeys.columnId)});
  store.setFilters([filter]);
}, getVisibleRecords:function() {
  var gridView = this.getView().getView();
  var firstIndex = gridView.getFirstVisibleRowIndex();
  var lastIndex = gridView.getLastVisibleRowIndex() + 1;
  var store = this.getView().getStore();
  var records = [];
  for (var i = firstIndex; i <= lastIndex; i++) {
    records.push(store.getAt(i));
  }
  return records;
}, onBeforeLoad:function() {
  var me = this;
  var view = this.getView();
  if (me.destroyed || !view.isVisible(true)) {
    this.refreshDataOnView = true;
    return false;
  }
}, onGlobalSearch:function(searchQuery) {
  var me = this;
  var value = String(searchQuery).replace(/UND/g, 'AND').replace(/ODER/g, 'OR');
  var storeProxy = me.getView().getStore().getProxy();
  if (storeProxy.getExtraParams().query === value) {
    return;
  }
  storeProxy.setExtraParam('query', value);
  me.refreshData();
}, updateChildLogGrids:function() {
  var childLogGrids = Ext.ComponentQuery.query('monitorChildLogGrid:visible(true)');
  childLogGrids.forEach(function(grid) {
    if (grid.up('monitorDetailContainer').componentName !== 'detailViewInline') {
      grid.store.reload();
    }
  });
}, updateDetailViews:function() {
  var detailComponents = Ext.ComponentQuery.query('monitorDetailContainer');
  var updateDetailComponents = function(detailComponent, reloadedRecord) {
    var processRecordViews = detailComponent.query('monitorProcessRecordView');
    if (processRecordViews && processRecordViews.length !== 0) {
      processRecordViews.forEach(function(processRecordView) {
        processRecordView.removeAll();
        processRecordView.add(processRecordView.createItems(processRecordView.group, processRecordView.item, reloadedRecord));
      });
    }
    var actionPanels = detailComponent.query('monitorActionPanel');
    if (actionPanels && actionPanels.length !== 0) {
      actionPanels.forEach(function(actionPanel) {
        actionPanel.removeAll();
        actionPanel.add(actionPanel.createItems(reloadedRecord));
      });
    }
  };
  if (detailComponents && detailComponents.length !== 0) {
    var firstDetailView = detailComponents[0];
    var currentRecord = firstDetailView.selectedRecord;
    var currentRecordId = currentRecord.id;
    var store = this.getView().store;
    var reloadedRecord = store.getById(currentRecordId);
    detailComponents.forEach(function(detailComponent) {
      return updateDetailComponents(detailComponent, reloadedRecord);
    });
  }
}});
Ext.define('monitor.view.grid.Grid', {extend:'Ext.grid.Panel', alias:'widget.monitorGrid', cls:'monitorGrid', requires:['monitor.component.grid.plugin.BpcRowExpander', 'monitor.view.grid.GridController'], config:{moduleId:undefined, storeUrl:undefined, selectedRecords:[]}, publishes:['selectedRecords'], store:undefined, stateId:undefined, stateful:true, viewModel:{data:{gridTitle:'', monitorData:'MONITOR_DATASETS', monitorFilter:'MONITOR_FILTER_ACTIVE', monitorSorter:'MONITOR_SORTER_ACTIVE', initialState:undefined}}, 
listeners:{beforerender:function(me) {
  me.setFilterBarVisibility();
}, afterrender:function(me) {
  me.down('tableview').rowSelector = 'tr.x-grid-row.monitorGrid-row';
  me.lookupViewModel().set('activeGrid', me);
}, focus:function(me) {
  me.lookupViewModel().set('activeGrid', me);
}, show:function(me) {
  me.lookupViewModel().set('activeGrid', me);
}, filterchange:function() {
  if (this.up('bpcMonitor') && Ext.isFunction(this.up('bpcMonitor').updateTooltip)) {
    this.up('bpcMonitor').updateTooltip(this);
  }
}, sortchange:function() {
  if (this.up('bpcMonitor') && Ext.isFunction(this.up('bpcMonitor').updateTooltip)) {
    this.up('bpcMonitor').updateTooltip(this);
  }
}, columnschanged:function() {
  this.throttledFilterBarVisibility();
}}, features:[{ftype:'grouping', id:'groupingFeature'}], bind:{title:'{monitorTitle} {gridTitle} - {totalItems} {monitorData:translate} - ' + '{activeFilter} {monitorFilter:translate} - {activeSorter} {monitorSorter:translate} {activeSortOrder}'}, controller:'monitorGridController', selModel:{mode:'MULTI', pruneRemoved:false}, autoScroll:true, multiColumnSort:undefined, multiSortLimit:undefined, reserveScrollbar:true, rowExpandDivPrefix:undefined, minHeight:100, collabEnabled:false, initComponent:function() {
  var me = this;
  me.stateId = me.moduleId + '_' + (me.bpcGridViewName || 'mainGrid');
  me.itemId = me.stateId;
  BpcCommon.Performance.mark('monitor_grid_init_start_' + me.stateId);
  me.viewConfig = {loadMask:BpcCommon.Api.getSetting(me.config.moduleId, monitor.assets.ConfigKeys.showGridLoadMask), enableTextSelection:BpcCommon.Api.getSetting(me.config.moduleId, monitor.assets.ConfigKeys.gridEditable), preserveScrollOnRefresh:true, preserveScrollOnReload:true, deferEmptyText:false, localized:{emptyText:'MONITOR_NO_DATA_AVAILABLE'}, getRowClass:monitor.Module.rowStyleController.getRowClass.bind(this, me.config.moduleId, me.bpcGridViewName), onItemMouseLeave:function(record) {
    if (!record.detailViewInlineCmp) {
      if (me.trackOver) {
        me.clearHighlight();
      }
    }
  }};
  me.creationTimeStamp = (new Date()).getTime();
  this.collabEnabled = me.isCollabEnabled();
  me.store = me.createStore();
  me.store.on('load', function(store) {
    var splitViewEnabled = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.detailViewEastEnabled);
    if (splitViewEnabled) {
      var conf = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.detailViewEastConfig);
      var initialShow = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.detailViewEastInitialShow);
      if (initialShow) {
        var record = store.getAt(0);
        var mainVm = me.lookupViewModel();
        var gridRefs = me.up('bpcMonitor').getController().getReferences();
        var detailPanelEast = gridRefs.detailPanelEast;
        if (record) {
          var detailedContainer = me.getController().createDetailContainer(conf, record, mainVm, me, true, 'detailViewEast');
          detailPanelEast.removeAll();
          detailPanelEast.add(detailedContainer);
          detailPanelEast.show().expand(true);
        }
      }
    }
  });
  me.store.load();
  var rowExpandDivPrefix = BpcCommon.Util.cleanUpId(me.bpcGridViewName + '-' + me.config.moduleId + '-');
  me.rowExpandDivPrefix = rowExpandDivPrefix;
  me.columnId = BpcCommon.Api.getSetting(me.config.moduleId, monitor.assets.ConfigKeys.columnId);
  me.multiColumnSort = BpcCommon.Api.getSetting(me.config.moduleId, monitor.assets.ConfigKeys.sortMulti);
  me.multiSortLimit = BpcCommon.Api.getSetting(me.config.moduleId, monitor.assets.ConfigKeys.sortMultiColumnCount);
  me.multiSortMode = BpcCommon.Api.getSetting(me.config.moduleId, monitor.assets.ConfigKeys.sortMultiReversedOrder);
  me.plugins = [{ptype:'bpcrowexpander', pluginId:'bpcrowexpander', selectRowOnExpand:true, expandOnDblClick:false, rowBodyTpl:new Ext.XTemplate('\x3cdiv id\x3d"' + rowExpandDivPrefix + '{bpcCleanedId}"\x3e\x3c/div\x3e')}, {ptype:'gridfilters'}];
  me.columns = {items:me.createGridColumnConfig(), initFocusableContainerKeyNav:function(el) {
    var me = this;
    if (!me.focusableKeyNav) {
      me.focusableKeyNav = new Ext.util.KeyNav({target:el, scope:me, down:me.showHeaderMenu, home:me.onHomeKey, end:me.onEndKey, space:me.onHeaderActivate, enter:me.onHeaderActivate});
    }
  }};
  me.callParent(arguments);
  this.throttledFilterBarVisibility = Ext.Function.createThrottled(this.setFilterBarVisibility, 10, this);
  BpcCommon.Performance.markAndMeasure('monitor_grid_init_' + me.stateId, 'monitor_grid_init_start_' + me.stateId, 'monitor_grid_init_end_' + me.stateId);
}, isCollabEnabled:function() {
  var $jscomp$this = this;
  try {
    if (!window.collaboration) {
      return false;
    }
    var pluginConfigurations = BpcCommon.Api.getSetting('monitor', 'plugin_configuration') || [];
    var collabColumnPluginsForModuleId = pluginConfigurations.filter(function(conf) {
      return Ext.isDefined(conf.targetInstance) && Ext.isArray(conf.targetInstance) && conf.targetInstance.indexOf($jscomp$this.config.moduleId) !== -1;
    }).filter(function(conf) {
      return conf.hook === 'monitor_root' && conf.plugin === 'collabArtefactGridColumn';
    });
    return collabColumnPluginsForModuleId.length > 0;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR isCollabEnabled', dump:e, stack:true, level:'error'});
    return false;
  }
}, getQueryId:function() {
  return this.bpcGridViewName + ':::' + this.creationTimeStamp;
}, createStore:function() {
  var me = this;
  var newModel = Ext.define(null, {extend:'Ext.data.Model', idProperty:BpcCommon.Api.getSetting(this.config.moduleId, monitor.assets.ConfigKeys.columnId), fields:[{name:'bpcCleanedId', convert:function(value, record) {
    var cleanId = value;
    try {
      var idColumn = BpcCommon.Api.getSetting(me.config.moduleId, monitor.assets.ConfigKeys.columnId);
      if (Ext.isDefined(idColumn)) {
        cleanId = BpcCommon.Util.cleanUpId(record.get(idColumn));
      }
    } catch (e) {
      Ext.log({msg:this.self.getName() + ': ERROR convert', dump:e, stack:true, level:'error'});
    }
    return cleanId;
  }}]});
  var params = {view:this.bpcGridViewName, timezoneOffset:'+0' + (new Date()).getTimezoneOffset() / 60 * -1 + ':00', timezoneName:BpcCommon.Util.getTimezoneName()};
  if (this.collabEnabled === true) {
    params.collaboration = 'true';
  }
  var id = this.config.moduleId + '_' + this.config.bpcGridViewId;
  return Ext.create('monitor.store.Store', {model:newModel, moduleId:this.config.moduleId, bpcGridViewId:this.config.bpcGridViewId, storeId:this.bpcGridViewName, sorters:this.createInitialSorter(), groupField:BpcCommon.Api.getSetting(this.config.moduleId, monitor.assets.ConfigKeys.columnGroupField), queryId:this.getQueryId(), proxy:{url:this.config.storeUrl, extraParams:params}, statefulFilters:true, multiSortMode:BpcCommon.Api.getSetting(this.config.moduleId, monitor.assets.ConfigKeys.sortMultiReversedOrder), 
  listeners:{beforeload:BpcCommon.Performance.mark.bind(BpcCommon.Performance, 'monitor_store_load_start_' + id), load:BpcCommon.Performance.markAndMeasure.bind(BpcCommon.Performance, 'monitor_store_load_' + id, 'monitor_store_load_start_' + id, 'monitor_store_load_end_' + id)}});
}, createInitialSorter:function() {
  var sorterList = BpcCommon.Api.getSetting(this.config.moduleId, monitor.assets.ConfigKeys.gridSortingColumn) || '';
  var sorterArrHelper = BpcCommon.Util.trimArray(sorterList.split(','));
  var sorterArr = [];
  var sortDirection = BpcCommon.Api.getSetting(this.config.moduleId, monitor.assets.ConfigKeys.gridMainSortDirection) || 'DESC';
  Ext.Array.each(sorterArrHelper, function(sorterCol) {
    sorterArr.push({property:sorterCol, direction:sortDirection});
  });
  return sorterArr;
}, clearFilterFields:function() {
  var columns = this.getColumns();
  Ext.each(columns, function(column) {
    var trigger;
    column.removeCls('bpc-monitor-filterActive');
    var field = column.down('field');
    if (field && field instanceof Ext.form.field.Base) {
      field.suspendEvents();
      field.setValue('');
      if (field.getTrigger) {
        trigger = field.getTrigger('resetField');
        if (trigger) {
          trigger.hide();
        }
      }
      field.resumeEvents(true);
      field.saveState();
    }
  });
}, getState:function() {
  var me = this;
  var state = me.callParent();
  if (this.getStore().isGrouped()) {
    state.groupingEnabled = true;
    state.grouper = {property:me.getStore().grouper.getProperty(), direction:me.getStore().grouper.getDirection(), root:me.getStore().grouper.getRoot()};
  } else {
    state.groupingEnabled = false;
  }
  return state;
}, applyState:function(state) {
  var me = this;
  if (!state) {
    return;
  }
  state = Ext.clone(state);
  state.storeState = state.storeState || {};
  Ext.applyIf(state.storeState, {filters:[], sorters:[]});
  if (!state.columns || Ext.isArray(state.columns) && state.columns.length === 0) {
    state.columns = me.createGridColumnConfig();
  }
  delete state.id;
  if (me.rendered) {
    me.stateful = false;
    var newColumns = [];
    var oldColumnsMap = {};
    var configuredColumns = me.createGridColumnConfig();
    configuredColumns.forEach(function(column) {
      oldColumnsMap[column.columnId] = column;
    });
    state.columns.forEach(function(columnState) {
      var column = oldColumnsMap[columnState.id] || oldColumnsMap[columnState.columnId];
      if (column) {
        delete columnState.id;
        newColumns.push(Ext.apply(column, columnState));
        configuredColumns.splice(configuredColumns.indexOf(column), 1);
      }
    });
    me.reconfigure(newColumns.concat(configuredColumns));
    me.stateful = true;
    delete state.columns;
  }
  me.callParent([state]);
  me.applyGroupersFromState(state);
}, applyGroupersFromState:function(state) {
  var me = this;
  var monitorCmp = me.up('bpcMonitor');
  if (!Ext.isObject(monitorCmp) || !Ext.isObject(state)) {
    return;
  }
  var stateIsGrouped = function() {
    return state.grouper && state.groupingEnabled === true;
  };
  if (stateIsGrouped()) {
    monitorCmp.getViewModel().set('groupField', state.grouper.property);
    me.store.setGrouper(state.grouper);
    me.store.setGroupField(state.grouper.property);
  } else {
    me.store.setGroupField('');
  }
}, createGridColumnConfig:function() {
  var me = this;
  var columns = [];
  try {
    var columnObj = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.columnConfig);
    var colConfig = Ext.clone(columnObj);
    var pluginHookColumns = colConfig.filter(function(col) {
      return col.xtype === 'monitorColumnHook';
    });
    if (pluginHookColumns.length > 0) {
      pluginHookColumns.forEach(function(colConfig, index) {
        colConfig.columnId = colConfig.columnId || 'monitorColumnHook-' + index + '-' + me.bpcGridViewName;
        colConfig.context = {moduleId:me.moduleId};
      });
    } else {
      colConfig.push({xtype:'monitorColumnHook', columnId:'monitorColumnHook-0-' + me.bpcGridViewName, context:{moduleId:me.moduleId}});
    }
    if (this.isCollabEnabled()) {
      colConfig = Ext.Array.insert(colConfig, 0, [collaboration.Module.getCollabColumn(me.moduleId)]);
    }
    var columnStateIdMap = {};
    Ext.each(colConfig, function(col) {
      if (!BpcCommon.Api.isUserAllowedToSee(col)) {
        return;
      }
      if (!col.dataIndex) {
        Ext.log({msg:this.self.getName() + ': dataIndex missing or empty', level:'warn'});
        col.dataIndex = 'dummy';
      } else if (!col.tooltip && BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.hideDefaultTableHeaderTooltip) !== true) {
        col.tooltip = col.dataIndex;
      }
      if (col.id) {
        col.id = col.id + '-' + me.bpcGridViewName;
      }
      var stateId = col.id || col.dataIndex;
      if (columnStateIdMap[stateId]) {
        var counter = columnStateIdMap[stateId];
        var baseStateId = stateId;
        stateId = baseStateId + '_' + counter;
        columnStateIdMap[baseStateId] = counter + 1;
      } else {
        columnStateIdMap[stateId] = 1;
      }
      var defaultColumnConfig = {xtype:'gridcolumn', stateId:stateId, columnId:stateId, testId:stateId, layout:{type:'vbox', align:'stretch', pack:'start'}, minWidth:64, items:[], moduleId:me.moduleId, getColumnState:function() {
        var me = this;
        var items = me.items.items;
        var state = {id:me.getStateId(), hidden:me.hidden, flex:me.flex};
        if (me.width) {
          state.width = me.width;
        }
        me.savePropsToState(['sortable', 'locked'], state);
        if (me.isGroupHeader && items && items.length) {
          state.columns = me.getColumnsState();
        }
        if ('width' in state) {
          delete state.flex;
        }
        return state;
      }};
      if (col.filter && col.filter.type) {
        if (BpcCommon.Util.isStringEqual(col.filter.type, 'date') || BpcCommon.Util.isStringEqual(col.filter.type, 'simpledate')) {
          col.filter.type = 'daterange';
        }
        if (BpcCommon.Util.isStringEqual(col.filter.type, 'number')) {
          col.filter.type = 'bpcMonitorNumber';
        }
      }
      if (col.filter && col.filter.type && Ext.isString(col.filter.type)) {
        col.filter.type = col.filter.type.toLowerCase();
        if (!Ext.ClassManager.getByAlias('grid.filter.' + col.filter.type)) {
          Ext.log({msg:this.self.getName() + ': unknown filter type ' + col.filter.type, level:'warn'});
          col.filter.type = false;
        }
      } else {
        defaultColumnConfig.items.push({xtype:'textfield', editable:false, fieldStyle:'cursor: not-allowed;', margin:0, ui:'columnHeaderField'});
      }
      col = monitor.Util.getStandardColumnConfig(col, me);
      columns.push(Ext.applyIf(col, defaultColumnConfig));
    }, this);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createColumns', dump:e, stack:true, level:'error'});
  }
  return columns;
}, setFilterBarVisibility:function() {
  var me = this;
  var showFilterFields = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.guiShowFilterBar);
  if (!showFilterFields) {
    var cm = me.getColumnManager();
    var columnArr = cm.getColumns();
    Ext.each(columnArr, function(col) {
      var field = col.down('field');
      if (field) {
        field.setVisible(showFilterFields);
      }
    });
  }
}});
Ext.define('monitor.view.grid.columns.AttachmentColumn', {extend:'Ext.grid.column.Column', alias:['widget.attachmentColumn', 'widget.attachmentcolumn', 'widget.monitorAttachmentColumn'], stopSelection:true, actionIdRe:new RegExp(Ext.baseCSSPrefix + 'action-col-(\\d+)'), altText:'', localized:{menuText:{tpl:'\x3ci\x3e{MONITOR_ACTIONS}\x3c/i\x3e', keys:['MONITOR_ACTIONS']}}, ignoreExport:true, sortable:false, innerCls:Ext.baseCSSPrefix + 'grid-cell-inner-action-col', actionIconCls:Ext.baseCSSPrefix + 
'action-col-icon', constructor:function(config) {
  var me = this;
  var cfg = Ext.apply({}, config);
  var iconItems = cfg.iconItems || me.iconItems || [me];
  var hasGetClass, i, len;
  me.origRenderer = cfg.renderer || me.renderer;
  me.origScope = cfg.scope || me.scope;
  me.renderer = me.scope = cfg.renderer = cfg.scope = null;
  cfg.iconItems = null;
  me.callParent([cfg]);
  me.iconItems = iconItems;
  for (i = 0, len = iconItems.length; i < len; ++i) {
    if (iconItems[i].getClass) {
      hasGetClass = true;
      break;
    }
  }
  if (me.origRenderer || hasGetClass) {
    me.hasCustomRenderer = true;
  }
}, initComponent:function() {
  var me = this;
  me.callParent();
  if (me.sortable && !me.dataIndex) {
    me.sortable = false;
  }
}, defaultRenderer:function(v, cellValues, record, rowIdx, colIdx, store, view) {
  var me = this;
  var scope = me.origScope || me;
  var iconItems = me.iconItems || [];
  var len = iconItems.length;
  var i, item, ret, disabled, tooltip, altText, icon;
  ret = Ext.isFunction(me.origRenderer) ? me.origRenderer.apply(scope, arguments) || '' : '';
  cellValues.tdCls += ' ' + Ext.baseCSSPrefix + 'action-col-cell';
  for (i = 0; i < len; i++) {
    item = iconItems[i];
    icon = item.icon;
    disabled = item.disabled || (item.isDisabled ? item.isDisabled.call(item.scope || scope, view, rowIdx, colIdx, item, record) : false);
    tooltip = disabled ? null : item.tooltip || (item.getTip ? item.getTip.apply(item.scope || scope, arguments) : null);
    altText = item.getAltText ? item.getAltText.apply(item.scope || scope, arguments) : item.altText || me.altText;
    if (!item.hasActionConfiguration) {
      item.stopSelection = me.stopSelection;
      item.disable = Ext.Function.bind(me.disableAction, me, [i], 0);
      item.enable = Ext.Function.bind(me.enableAction, me, [i], 0);
      item.hasActionConfiguration = true;
    }
    ret += '\x3c' + (icon ? 'img' : 'div') + ' tabIndex\x3d"0" role\x3d"button"' + (icon ? ' alt\x3d"' + altText + '" src\x3d"' + item.icon + '"' : '') + ' class\x3d"' + me.actionIconCls + ' ' + Ext.baseCSSPrefix + 'action-col-' + String(i) + ' ' + (disabled ? me.disabledCls + ' ' : ' ') + (Ext.isFunction(item.getClass) ? item.getClass.apply(item.scope || scope, arguments) : item.iconCls || me.iconCls || '') + '"' + (tooltip ? ' data-qtip\x3d"' + tooltip + '"' : '') + (icon ? '/\x3e' : '\x3e\x3c/div\x3e');
  }
  return ret;
}, updater:function(cell, value, record, view, dataSource) {
  var cellValues = {};
  Ext.fly(cell).addCls(cellValues.tdCls).down(this.getView().innerSelector, true).innerHTML = this.defaultRenderer(value, cellValues, record, null, null, dataSource, view);
}, enableAction:function(index, silent) {
  var me = this;
  if (!index) {
    index = 0;
  } else if (!Ext.isNumber(index)) {
    index = Ext.Array.indexOf(me.iconItems, index);
  }
  me.iconItems[index].disabled = false;
  me.up('tablepanel').el.select('.' + Ext.baseCSSPrefix + 'action-col-' + index).removeCls(me.disabledCls);
  if (!silent) {
    me.fireEvent('enable', me);
  }
}, disableAction:function(index, silent) {
  var me = this;
  if (!index) {
    index = 0;
  } else if (!Ext.isNumber(index)) {
    index = Ext.Array.indexOf(me.iconItems, index);
  }
  me.iconItems[index].disabled = true;
  me.up('tablepanel').el.select('.' + Ext.baseCSSPrefix + 'action-col-' + index).addCls(me.disabledCls);
  if (!silent) {
    me.fireEvent('disable', me);
  }
}, beforeDestroy:function() {
  this.renderer = this.iconItems = null;
  return this.callParent(arguments);
}, processEvent:function(type, view, cell, recordIndex, cellIndex, e, record, row) {
  var me = this;
  var key = type === 'keydown' && e.getKey();
  var cellFly = Ext.fly(cell);
  var target = e.getTarget(), match, item, disabled;
  e.stopSelection = !key && me.stopSelection;
  if (key && (target === cell || !cellFly.contains(target))) {
    target = cellFly.query('.' + me.actionIconCls, true);
    if (target.length === 1) {
      target = target[0];
    } else {
      return;
    }
  }
  if (target && (match = target.className.match(me.actionIdRe))) {
    item = me.iconItems[parseInt(match[1], 10)];
    disabled = item.disabled || (item.isDisabled ? item.isDisabled.call(item.scope || me.origScope || me, view, recordIndex, cellIndex, item, record) : false);
    if (item && !disabled) {
      if (type === 'mousedown' && !me.getView().actionableMode) {
        e.preventDefault();
      } else if (type === 'click' || (key === e.ENTER || key === e.SPACE)) {
        Ext.callback(item.handler || me.handler, item.scope || me.origScope, [view, recordIndex, cellIndex, item, e, record, row], undefined, me);
        if (!view.el.contains(Ext.Element.getActiveElement())) {
          return false;
        }
      }
    }
  }
  return me.callParent(arguments);
}, cascade:function(fn, scope) {
  fn.call(scope || this, this);
}, iconItems:[{iconCls:BpcCommon.Icons.DOWNLOAD, localized:{tooltip:'MONITOR_DOWNLOAD_MESSAGE'}, scope:this, handler:function(tableView, rowIndex, colIndex, item, event, record) {
  try {
    if (tableView.up('bpcMonitor')) {
      tableView.up('bpcMonitor').downloadFile(record);
    }
  } catch (e) {
    Ext.log({msg:': ERROR file download handler', dump:e, stack:true, level:'error'});
  }
}, isDisabled:function(view, rowIndex, colIndex, item, record) {
  try {
    var column = 'REFERENCE';
    if (view.up('bpcMonitor')) {
      column = BpcCommon.Api.getSetting(view.up('bpcMonitor').moduleId, monitor.assets.ConfigKeys.columnFile);
    }
    return !record.get(column);
  } catch (e) {
    Ext.log({msg:': ERROR file download isDisabled', dump:e, stack:true, level:'error'});
  }
}}]});
Ext.define('monitor.view.grid.tabbed.TabModeController', {extend:'Ext.app.ViewController', alias:'controller.monitorTabPanelController', control:{'#':{'tabchange':'onTabChange'}}, onTabChange:function(tabPanel, newCard, oldCard) {
  newCard.getController().updateModuleHeaderInfos();
  var oldFilters = oldCard.getStore().getFilters().getRange();
  var oldSorters = oldCard.getStore().getSorters().getRange();
  var oldQuery = oldCard.getStore().proxy.extraParams.query;
  var newFilters = newCard.getStore().getFilters().getRange();
  var newSorters = newCard.getStore().getSorters().getRange();
  var newQuery = newCard.getStore().proxy.extraParams.query;
  if (oldQuery !== newQuery) {
    newCard.getStore().proxy.extraParams.query = oldQuery;
    this.getView().up('bpcMonitor').getActiveGrid().getController().refreshData();
  }
  if (!Ext.Array.equals(oldFilters, newFilters) || !Ext.Array.equals(oldSorters, newSorters)) {
    newCard.getStore().sorters.add(newSorters);
    newCard.getStore().addFilter(newFilters);
  }
}});
Ext.define('monitor.view.grid.tabbed.Tabbed', {extend:'Ext.tab.Panel', alias:'widget.monitorTabPanel', controller:'monitorTabPanelController', region:'center', items:[]});
Ext.define('monitor.view.interaction.SearchField', {extend:'Ext.toolbar.Toolbar', alias:'widget.monitorSearchField', items:[{xtype:'button', iconCls:BpcCommon.Icons.SEARCH, stateful:{pressed:true}, stateId:'monitorSearchFieldButton', itemId:'fullTextSearchButton', stateEvents:['toggle'], enableToggle:true, bind:{pressed:'{global.monitor.showSearchField}'}}, {xtype:'bpcTextField', htmlEncode:false, flex:1, minWidth:250, maxWidth:350, name:'searchField', itemId:'fullTextSearch', localized:{emptyText:'MONITOR_FULL_TEXT_SEARCH'}, 
hidden:true, triggers:{resetField:{cls:BpcCommon.Icons.FILTER_RESET, handler:function() {
  this.setValue('');
}}}, setTriggerResetHidden:function(hidden) {
  this.getTrigger('resetField').setVisible(!hidden);
}, getTriggerResetHidden:function() {
  return !this.getTrigger('resetField').isVisible();
}, bind:{value:'{globalFilterValue}', hidden:'{!global.monitor.showSearchField}', triggerResetHidden:'{!globalFilterValue}'}, listeners:{afterrender:function(sf) {
  Ext.create('Ext.tip.ToolTip', {target:sf.id, dismissDelay:0, anchor:true, defaultAlign:'t-b', style:{borderStyle:'none', padding:'0px'}, bodyStyle:{borderStyle:'none', padding:'0px', backgroundColor:'white'}, items:[{xtype:'monitorSearchFieldExplanationGrid'}]});
}, beforehide:function() {
  this.oldValue = this.getValue();
  this.setValue('');
}, show:function(searchField) {
  var currentValue = searchField.lookupViewModel().get('globalFilterValue');
  if (this.oldValue && currentValue === '') {
    this.setValue(this.oldValue);
    delete this.oldValue;
  } else {
    this.setValue(currentValue);
    this.oldValue = searchField.lookupViewModel().get('globalFilterValue');
  }
  searchField.focus();
}}}]});
Ext.define('monitor.view.interaction.SearchFieldExplanationGrid', {extend:'Ext.grid.Panel', alias:'widget.monitorSearchFieldExplanationGrid', width:500, initComponent:function() {
  var me = this;
  me.columns = [{text:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_SEARCH_HEADER_OPERATOR'), dataIndex:'operator', flex:1, renderer:function(value) {
    return '\x3cdiv style\x3d"white-space:normal"\x3e\x3cb\x3e' + value + '\x3c/b\x3e\x3c/div\x3e';
  }}, {text:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_SEARCH_HEADER_DESCRIPTION'), dataIndex:'description', flex:2, renderer:function(value) {
    return '\x3cdiv style\x3d"white-space:normal"\x3e' + value + '\x3c/div\x3e';
  }}, {text:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_SEARCH_HEADER_EXAMPLE'), dataIndex:'example', flex:1, renderer:function(value) {
    return '\x3cdiv style\x3d"white-space:normal"\x3e' + value + '\x3c/div\x3e';
  }}];
  me.bbar = {xtype:'container', style:{padding:'10px', borderTop:'1px solid #ddd', color:'black'}, html:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_NOTE')};
  me.store = Ext.create('Ext.data.Store', {data:[{operator:'AND', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_AND_OPERATOR_DESCRIPTION'), example:'auto AND blau'}, {operator:'OR', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_OR_OPERATOR_DESCRIPTION'), example:'auto OR flugzeug'}, {operator:'""', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_SLASH_OPERATOR_DESCRIPTION'), example:'"blaues auto"'}, {operator:'-', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_MINUS_OPERATOR_DESCRIPTION'), 
  example:'auto -blau'}, {operator:'SPALTE:', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_SPALTE_OPERATOR_DESCRIPTION'), example:'FARBE:blau'}, {operator:'_exists_', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_EXISTS_OPERATOR_DESCRIPTION'), example:'_exists_:FARBE'}, {operator:'_missing_', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_MISSING_OPERATOR_DESCRIPTION'), example:'_missing_:FARBE'}, {operator:'*', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_ANY_OPERATOR_DESCRIPTION'), 
  example:'FARBE:b*u'}, {operator:'?', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_QUESTION_MARK_OPERATOR_DESCRIPTION'), example:'FARBE:b??u'}, {operator:'~', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_SIMILAR_OPERATOR_DESCRIPTION'), example:'blau~'}, {operator:'[min TO max] \x3cbr\x3e {min TO max}', description:BpcCommon.Api.getTranslation('MONITOR_FULL_TEXT_MIN_MAX_OPERATOR_DESCRIPTION'), example:' DATUM:[2012-01-01 TO 2012-12-31]\x3cbr\x3e ANZAHL:[2 TO 22]'}]});
  me.callParent(arguments);
}});
Ext.define('monitor.view.interaction.export.ExportButton', {extend:'Ext.Button', alias:'widget.monitorExportBtn', iconCls:monitor.assets.Icons.DOWNLOAD, localized:{tooltip:'MONITOR_EXPORT_DATA'}, hidden:true, bind:{hidden:'{!setting.' + monitor.assets.ConfigKeys.exportEnabled + '}'}, handler:function() {
  var me = this;
  Ext.create({xtype:'monitorExportWindow', moduleId:me.moduleId, viewModel:{parent:me.lookupViewModel()}});
}});
Ext.define('monitor.view.interaction.export.ExportWindow', {extend:'BpcCommon.component.window.Window', alias:'widget.monitorExportWindow', controller:'monitorExportWindowController', localized:{title:'MONITOR_PROCESS_EXPORT'}, width:700, height:430, bodyPadding:'0 5 0 5', layout:'fit', iconCls:monitor.assets.Icons.DOWNLOAD, grid:undefined, moduleId:undefined, layout:{type:'fit'}, resizable:false, viewModel:{data:{isFormatXlsx:false, hideWarning:true, field:{}}}, initComponent:function() {
  var me = this;
  try {
    me.backendUrl = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + me.moduleId;
    me.grid = me.lookupViewModel().get('activeGrid');
    var defaults = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.exportDialogDefaults);
    if (!defaults.exportChildrenDisable) {
      defaults.exportChildrenDisable = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.dataHistoryIndex) ? false : true;
    }
    if (defaults.exportChildrenDisable) {
      defaults.exportChildren = false;
    }
    me.getViewModel().set('field', defaults);
    var currentLanguage = BpcCommon.Api.getSetting('_core', 'language');
    if (defaults && defaults.title && defaults.title[currentLanguage]) {
      me.title = defaults.title[currentLanguage];
    }
    var maxExportLimit = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.exportLimit) || 10000;
    me.dockedItems = [{xtype:'toolbar', dock:'bottom', ui:'footer', overflowHandler:'scroller', height:50, items:[{xtype:'bpcToolbarMsg', flex:3, iconCls:BpcCommon.Icons.WARNING, iconColorCls:'warningColor', text:'MONITOR_FILE_DOWNLOAD_CUT_DATA', hidden:true, bind:{hidden:'{hideWarning}'}}, '-\x3e', {localized:{text:'MONITOR_CANCEL'}, iconCls:BpcCommon.Icons.CANCEL, ui:'secondary', itemId:'formCancel'}, {localized:{text:'MONITOR_DOWNLOAD'}, iconCls:monitor.assets.Icons.DOWNLOAD, ui:'primary', itemId:'formSubmit'}]}];
    me.items = [{xtype:'form', standardSubmit:true, method:'GET', url:me.backendUrl, items:[{xtype:'fieldset', layout:{type:'vbox', align:'stretch'}, defaults:{labelWidth:300}, width:688, height:350, title:BpcCommon.Api.getTranslation('CORE_CONFIGURATION'), items:[{xtype:'monitorExportWindowFormatPicker', itemId:'outputFormat', bind:'{field.format}', moduleId:me.moduleId, reference:'format'}, {xtype:'monitorExportWindowViewPicker', localized:{fieldLabel:'MONITOR_VIEW'}, grid:me.grid, moduleId:me.moduleId, 
    bind:'{field.view}', reference:'viewSelection', disabled:!BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.multipleGrids)}, {xtype:'numberfield', localized:{fieldLabel:'MONITOR_FILE_DOWNLOAD_LIMIT'}, bind:{disabled:'{!isFormatXlsx}', value:'{field.limit}'}, itemId:'Limit', allowBlank:true, reference:'limit', value:1000, maxValue:maxExportLimit, minValue:0}, {xtype:'checkbox', reference:'exportChildren', bind:{value:'{field.exportChildren}', disabled:'{field.exportChildrenDisable}'}, 
    localized:{fieldLabel:'MONITOR_EXPORT_CHILD_DATA', onText:'MONITOR_YES', offText:'MONITOR_NO'}}, {xtype:'radiogroup', localized:{fieldLabel:'MONITOR_CONSIDER_CURRENT_VIEW'}, itemId:'considerCurrentView', columns:1, allowBlank:false, bind:'{field.currentView}', reference:'currentView', simpleValue:true, items:[{name:'currentView', inputValue:true, itemId:'currentViewYes', localized:{boxLabel:'MONITOR_EXPORT_ONLY_DISPLAYED_COLUMNS'}}, {name:'currentView', inputValue:false, itemId:'currentViewNo', 
    localized:{boxLabel:'MONITOR_EXPORT_ALL_COLUMNS'}}]}]}]}];
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent', dump:e, stack:true, level:'error'});
  }
  me.callParent();
}});
Ext.define('monitor.view.interaction.export.ExportWindowController', {extend:'Ext.app.ViewController', alias:'controller.monitorExportWindowController', control:{'#formSubmit':{click:'onFormSubmit'}, '#formCancel':{click:'destroyWindow'}, '#Prozesse':{change:'onSelectionChange'}, '#Limit':{change:'showCutUpNotification'}, '#outputFormat':{change:'onOutputFormatChanged'}}, onSelectionChange:function(cmp, newVal) {
  try {
    var refs = this.getReferences();
    if (newVal.selection === 'all') {
      refs.limit.show();
      this.showCutUpNotification();
    } else {
      refs.limit.hide();
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onSelectionChange', dump:e, stack:true, level:'error'});
  }
}, onFormSubmit:function() {
  try {
    var refs = this.getReferences();
    var form = this.getView().down('form').getForm();
    if (form.isValid()) {
      var format = refs.format.getValue();
      var limit;
      if (this.getViewModel().get('isFormatXlsx') === true) {
        limit = refs.limit.getValue();
      }
      this.downloadFile(format, limit);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onFormSubmit', dump:e, stack:true, level:'error'});
  }
}, onOutputFormatChanged:function(radioGroup, newVal) {
  if (newVal === 'xlsx') {
    this.getViewModel().set('isFormatXlsx', true);
    this.showCutUpNotification();
  } else {
    this.getViewModel().set('isFormatXlsx', false);
    this.getViewModel().set('hideWarning', true);
  }
}, createUrl:function(format, limit) {
  try {
    var url = this.getView().backendUrl;
    var filterArray = this.getView().grid.store.getFilters().items;
    var sorterArray = this.getView().grid.store.getSorters().items;
    var fullTextQuery = this.getView().lookupViewModel().get('globalFilterValue');
    var timezoneOffset = '+0' + (new Date()).getTimezoneOffset() / 60 * -1 + ':00';
    var view = this.getSelectedView();
    var timeZoneName = BpcCommon.Util.getTimezoneName();
    var refs = this.getReferences();
    var exportChildren = refs.exportChildren.getValue();
    var bpcLanguage = BpcCommon.Api.getSetting('_core', 'language');
    var currentView = refs.currentView.getValue();
    var columns = [];
    if (currentView === true && this.getView().grid.getState() && this.getView().grid.getState().columns) {
      Ext.each(this.getView().grid.down('headercontainer').getVisibleGridColumns(), function(column) {
        if (column.dataIndex && column.dataIndex !== null) {
          columns.push(column.dataIndex);
        }
      });
    }
    var params = {format:format, timezoneOffset:timezoneOffset, timezoneName:timeZoneName, query:fullTextQuery, exportChildren:exportChildren, bpcLanguage:bpcLanguage};
    if (limit) {
      Ext.apply(params, {limit:limit});
    }
    if (columns.length > 0) {
      Ext.apply(params, {exportColumns:JSON.stringify(columns)});
    }
    if (Ext.isArray(sorterArray) && sorterArray.length > 0) {
      params['sort'] = this.getView().grid.store.proxy.encodeSorters(sorterArray);
    }
    if (Ext.isArray(filterArray) && filterArray.length > 0) {
      params['filter'] = this.getView().grid.store.proxy.encodeFilters(filterArray);
    }
    if (Ext.isDefined(view)) {
      params['view'] = view;
    }
    url += '?' + Ext.Object.toQueryString(params);
    return url;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createUrl', dump:e, stack:true, level:'error'});
  }
}, getSelectedView:function() {
  try {
    var view = 'mainGrid';
    var refs = this.getReferences();
    if (BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.multipleGrids)) {
      view = refs.viewSelection.getValue();
    }
    return view;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR getSelectedView', dump:e, stack:true, level:'error'});
  }
}, showCutUpNotification:function() {
  try {
    var grid = this.getView().grid;
    var store = grid.getStore();
    var totalCount = store.getCount();
    var refs = this.getReferences();
    if (totalCount > refs.limit.getValue()) {
      this.getViewModel().set('hideWarning', false);
    } else {
      this.getViewModel().set('hideWarning', true);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR showCutUpNotification', dump:e, stack:true, level:'error'});
  }
}, showLoading:function(loadingState) {
  var me = this;
  var view = me.getView();
  if (loadingState) {
    view.mask(BpcCommon.Api.getTranslation('CORE_LOADING'));
  } else {
    view.unmask();
  }
}, showError:function(error) {
  BpcCommon.Api.showNotification({text:error.message, popUp:true, type:'error'});
}, downloadFile:function(format) {
  var $jscomp$async$this = this;
  var me;
  var url;
  var response;
  var blob;
  var objectURL;
  var a;
  var e;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    switch($jscomp$generator$context.nextAddress) {
      case 1:
        me = $jscomp$async$this;
        $jscomp$generator$context.setCatchFinallyBlocks(2, 3);
        me.showLoading(true);
        url = me.createUrl(format);
        return $jscomp$generator$context.yield(fetch(url), 5);
      case 5:
        response = $jscomp$generator$context.yieldResult;
        if (!response.ok) {
          throw new Error('HTTP error! Status: ' + response.status);
        }
        return $jscomp$generator$context.yield(response.blob(), 6);
      case 6:
        blob = $jscomp$generator$context.yieldResult;
        objectURL = URL.createObjectURL(blob);
        a = document.createElement('a');
        a.href = objectURL;
        a.download = me.generateFileName(format);
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        setTimeout(function() {
          return URL.revokeObjectURL(objectURL);
        }, 1000);
      case 3:
        $jscomp$generator$context.enterFinallyBlock();
        me.showLoading(false);
        $jscomp$generator$context.leaveFinallyBlock(0);
        break;
      case 2:
        e = $jscomp$generator$context.enterCatchBlock();
        me.showError(e);
        Ext.log({msg:$jscomp$async$this.self.getName() + ': ERROR download monitor export file', dump:e, stack:true, level:'error'});
        $jscomp$generator$context.jumpTo(3);
        break;
    }
  });
}, generateFileName:function(format) {
  var me = this;
  return 'monitor_' + me.getView().moduleId + '.' + (format || 'file');
}, destroyWindow:function() {
  try {
    this.getView().destroy();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR destroyWindow', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.interaction.export.FormatPicker', {alias:'widget.monitorExportWindowFormatPicker', extend:'BpcCommon.component.form.field.ComboBox', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, editable:false, localized:{fieldLabel:'MONITOR_FILE_DOWNLOAD_FORMAT'}, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var color = '#4CAF50';
  var iconCls = recordData.icon;
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
  return icon + ' ' + recordData.label;
}})}, initComponent:function() {
  var me = this;
  var storeData = [];
  var exportOptions = [{label:'CSV', value:'csv', setting:monitor.assets.ConfigKeys.exportCsv}, {label:'XLSX', value:'xlsx', setting:monitor.assets.ConfigKeys.exportXlsx}, {label:'JSON', value:'json', setting:monitor.assets.ConfigKeys.exportJson}, {label:'XML', value:'xml', setting:monitor.assets.ConfigKeys.exportXml}, {label:'PDF', value:'pdf', setting:monitor.assets.ConfigKeys.exportPdf}];
  exportOptions.forEach(function(option) {
    if (BpcCommon.Api.getSetting(me.moduleId, option.setting)) {
      storeData.push({label:option.label, value:option.value, icon:'x-fal fa'});
    }
  });
  me.store = storeData;
  me.callParent();
}});
Ext.define('monitor.view.interaction.export.ViewPicker', {alias:'widget.monitorExportWindowViewPicker', extend:'BpcCommon.component.form.field.ComboRenderedValue', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var color = '#4CAF50';
  var iconCls = recordData.icon;
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
  return icon + ' ' + recordData.label;
}})}, initComponent:function() {
  var me = this;
  var storeData = [];
  var defaultValue = null;
  if (BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.multipleGrids)) {
    var activeTab = null;
    if (me.grid.up('tabpanel')) {
      activeTab = me.grid.up('tabpanel').getActiveTab();
    }
    var views = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.multipleGridsConfig);
    Ext.Array.each(views.views, function(view) {
      if (BpcCommon.Api.isUserAllowedToSee(view)) {
        var item = {label:view.title, value:view.id};
        storeData.push(item);
        if (activeTab && activeTab.bpcGridViewName === view.id) {
          defaultValue = view.id;
        }
      }
    });
  }
  me.store = storeData;
  me.emptyText = BpcCommon.Api.getTranslation('MONITOR_EXPORT_NO_VIEWS_AVAILABLE');
  me.callParent();
  me.setValue(defaultValue);
}});
Ext.define('monitor.view.interaction.moduleJump.Controller', {extend:'Ext.app.Controller', statics:{JUMP_NEW_TAB:'jumpToNewTab', JUMP_CURRENT_TAB:'jumpCurrentTab', JUMP_COPY_CLIPBOARD:'jumpClipboard'}, doModuleJump:function(column, record, value, jumpParamFnXtype, target) {
  if (jumpParamFnXtype) {
    var linkedClass = Ext.ClassManager.getByAlias(jumpParamFnXtype);
    var fn = linkedClass ? linkedClass.jumpParamFn : null;
    if (Ext.isFunction(fn)) {
      this.jumpParamFn = fn;
    } else {
      Ext.log({msg:this.self.getName() + ": doModuleJump - can't find jumpParamFn " + jumpParamFnXtype, level:'warn'});
    }
  }
  var jumpParams = this.jumpParamFn(record, column, value);
  if (column.targetConfig) {
    jumpParams = Ext.applyIf(jumpParams, column.targetConfig);
  }
  var jumpController = monitor.view.interaction.moduleJump.Controller;
  if (target === jumpController.JUMP_NEW_TAB || target === jumpController.JUMP_COPY_CLIPBOARD) {
    var minimalJumpParams = {storeState:jumpParams.storeState, record:{data:record.getData()}, column:column.initialConfig};
    var moduleUrl = BpcCommon.Api.getModuleUrl(column.targetModule, minimalJumpParams);
    if (target === jumpController.JUMP_NEW_TAB) {
      window.open(moduleUrl, '_blank');
    } else if (target === jumpController.JUMP_COPY_CLIPBOARD) {
      BpcCommon.Util.copyToClipboard(moduleUrl);
    }
  } else {
    BpcCommon.Api.showModule(column.targetModule, column.forceNewInstance, jumpParams, column.targetRoute);
  }
}, jumpParamFn:function(record, column, value) {
  var params = {};
  try {
    return {storeState:{filters:[{property:column.targetColumn, operator:column.filterOperation || 'analyzed', value:value, source:column.filterSourceType || 'analyzed', type:column.filterType}]}, record:record, column:column};
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': jumpParamFn + Error While Fetching Jump Params', level:'warn'});
    params = {};
  }
  return params;
}, getJumpColumnsForMonitor:function(monitorInstanceId) {
  var monitorInstance = BpcCommon.Api.getModuleInstanceComponent(monitorInstanceId);
  var columns = monitorInstance.activeGrid.getColumns();
  return columns.filter(function(col) {
    return 'bpcModuleJumpColumn' === col.xtype;
  });
}});
Ext.define('monitor.view.interaction.processAction.ButtonMenuMixin', {moduleId:undefined, iconCls:monitor.assets.Icons.PROCESS_ACTION});
Ext.define('monitor.view.interaction.moduleJump.Menu', {extend:'Ext.menu.Item', alias:'widget.monitorModuleJumpMenu', mixins:['monitor.view.interaction.processAction.ButtonMenuMixin'], iconCls:BpcCommon.Icons.LINK, localized:{text:'MONITOR_JUMP_TO'}, selection:undefined, initComponent:function() {
  var me = this;
  var moduleJumpController = Ext.getApplication().getController('monitor.view.interaction.moduleJump.Controller');
  var jumpColumns = moduleJumpController.getJumpColumnsForMonitor(me.moduleId);
  var selectedRecord = me.selection[0];
  if (jumpColumns.length === 0 || !selectedRecord) {
    me.hidden = true;
  } else {
    var getOptionsMenu = function(column, selectedRecord, value) {
      return Ext.create('Ext.menu.Menu', {margin:'0 0 10 0', items:[{text:BpcCommon.Api.getTranslation('MONITOR_OPEN_NEW_TAB'), itemId:'jumpToNewTab', iconCls:'x-fal fa-window-restore', handler:moduleJumpController.doModuleJump.bind(moduleJumpController, column, selectedRecord, value, column.jumpParamFn, monitor.view.interaction.moduleJump.Controller.JUMP_NEW_TAB)}, {text:BpcCommon.Api.getTranslation('MONITOR_STAY_IN_TAB'), itemId:'jumpStayInTab', iconCls:'x-fal fa-window-maximize', handler:moduleJumpController.doModuleJump.bind(moduleJumpController, 
      column, selectedRecord, value, column.jumpParamFn, monitor.view.interaction.moduleJump.Controller.JUMP_CURRENT_TAB)}, {text:BpcCommon.Api.getTranslation('MONITOR_COPY_URL'), itemId:'jumpCopyUrl', iconCls:'x-fal fa-copy', handler:moduleJumpController.doModuleJump.bind(moduleJumpController, column, selectedRecord, value, column.jumpParamFn, monitor.view.interaction.moduleJump.Controller.JUMP_COPY_CLIPBOARD)}]});
    };
    me.menu = Ext.create('Ext.menu.Menu', {margin:'0 0 10 0', items:jumpColumns.map(function(column, index) {
      var jumpMenuIcon = column.jumpMenuIconCls || 'x-fal fa-external-link';
      var menuTextTemplate = new Ext.XTemplate(column.jumpMenuText);
      var value = selectedRecord.get(column.dataIndex);
      var displayMenuText = menuTextTemplate.apply({value:value, column:column});
      return {text:displayMenuText || value, hidden:value === undefined || value === '', iconCls:jumpMenuIcon, itemId:'jumpToMenuItem-' + index, handler:column.jumpMenuDisableOptions ? moduleJumpController.doModuleJump.bind(moduleJumpController, column, selectedRecord, value, column.jumpParamFn) : null, menu:column.jumpMenuDisableOptions ? null : getOptionsMenu(column, selectedRecord, value)};
    })});
  }
  this.callParent();
}});
Ext.define('monitor.view.interaction.moduleJump.ModuleJump', {extend:'Ext.grid.column.Widget', alias:['widget.bpcCommonModuleJumpColumn', 'widget.bpcModuleJumpColumn'], menuDisabled:false, targetModule:undefined, targetRoute:undefined, targetColumn:undefined, showIcon:true, showValue:true, filterOperation:'\x3d', filterSourceType:'raw', filterType:'text', sourceColumn:undefined, forceNewInstance:false, iconCls:'fal fa-external-link', sortable:true, separator:undefined, displayText:'', renderValue:false, 
initComponent:function() {
  var me = this;
  if (me.targetModule) {
    var monitor$7 = me.up('bpcMonitor');
    var rendererFn;
    if (monitor$7) {
      var customRenderer = monitor$7.findPlugin('monitorCustomRenderer');
      rendererFn = customRenderer.render.bind(customRenderer, customRenderer.getRendererIdByColumnConfig(me));
    }
    this.renderer = rendererFn;
    this.widget = {xtype:'bpcCommonModuleJumpColumnWidget', showIcon:this.showIcon, showValue:this.showValue, iconCls:this.iconCls, separator:this.separator, displayText:this.displayText, renderValue:this.renderValue, rendererFn:Ext.isFunction(rendererFn) ? rendererFn : undefined, bind:{value:'{record.' + me.dataIndex + '}', sourceColumnValue:'{record.' + me.sourceColumn + '}'}};
    if (this.jumpParamFn) {
      var moduleFnArray = String(this.jumpParamFn).split('.');
      if (moduleFnArray.length) {
        BpcCommon.Api.loadModule(moduleFnArray[0]);
        this.widget.jumpParamFnXtype = this.jumpParamFn;
      }
    }
  } else {
    this.dataIndex = undefined;
    this.widget = {xtype:'box', html:'\x3ci class\x3d"fal fa-exclamation-triangle"\x3e\x3c/i\x3e Missing targetModule in column configuration'};
  }
  this.callParent(arguments);
}});
Ext.define('monitor.view.interaction.moduleJump.ModuleJumpWidget', {extend:'Ext.Widget', alias:['widget.bpcCommonModuleJumpColumnWidget'], element:{reference:'element', style:'cursor: pointer;\n' + '    overflow: hidden;\n' + '    text-overflow: ellipsis; ', listeners:{click:'onModuleJump'}}, config:{sourceColumnValue:undefined, value:undefined}, defaultBindProperty:'value', showIcon:true, showValue:true, renderValue:undefined, separator:undefined, rendererFn:undefined, tpl:'', valueArray:undefined, 
constructor:function(config) {
  var $jscomp$destructuring$var6 = config;
  var iconCls = $jscomp$destructuring$var6.iconCls;
  var renderValue = $jscomp$destructuring$var6.renderValue;
  var rendererFn = $jscomp$destructuring$var6.rendererFn;
  var displayText = $jscomp$destructuring$var6.displayText;
  var showIcon = $jscomp$destructuring$var6.showIcon;
  var showValue = $jscomp$destructuring$var6.showValue;
  var separator = $jscomp$destructuring$var6.separator;
  this.iconCls = iconCls || 'fal fa-external-link';
  this.renderValue = renderValue;
  this.rendererFn = rendererFn;
  this.displayText = displayText;
  this.showValue = showValue;
  this.separator = separator;
  this.showIcon = showIcon;
  this.tpl = "\x3ctpl for\x3d\"valueArray\"\x3e\x3ctpl if\x3d'.'\x3e\x3ca data-valueindex\x3d'{jumpValueIndex}' data-value\x3d'{jumpValue:htmlEncode2}'\x3e";
  if (this.renderValue && Ext.isFunction(this.rendererFn)) {
    this.tpl += '{displayValue}';
  } else {
    if (this.showIcon) {
      this.tpl += "\x3ctpl if\x3d'displayValue'\x3e \x3cspan class\x3d\"" + this.iconCls + '" aria-hidden\x3d"true"\x3e \x3c/span\x3e \x3c/tpl\x3e';
    }
    this.tpl += this.showValue ? ' {displayValue:htmlEncode2}' : this.displayText || '';
  }
  this.tpl += '\x3c/a\x3e \x3c/tpl\x3e\x3c/tpl\x3e';
  this.tpl = new Ext.XTemplate(this.tpl);
  this.callParent([config]);
}, setValue:function() {
  this.callParent(arguments);
  this.updateContent();
}, setSourceColumnValue:function() {
  this.callParent(arguments);
  this.updateContent();
}, updateContent:function() {
  var me = this;
  var displayValue = me.getValue();
  var jumpValue = me.getSourceColumnValue() || displayValue;
  if (me.destroyed) {
    return;
  }
  var valueArray;
  if (me.renderValue && Ext.isFunction(me.rendererFn)) {
    var renderedValue = me.rendererFn(displayValue);
    if (renderedValue !== null) {
      valueArray = [{displayValue:renderedValue, jumpValue:jumpValue}];
    }
  }
  if (!valueArray) {
    if (Ext.isArray(displayValue)) {
      valueArray = displayValue.map(function(value, index) {
        return {displayValue:value, jumpValue:value, jumpValueIndex:index};
      });
    } else if (Ext.isString(displayValue)) {
      valueArray = me.separator ? displayValue.split(me.separator).map(function(value, index) {
        return {displayValue:value.trim(), jumpValue:value, jumpValueIndex:index};
      }) : [{displayValue:displayValue, jumpValue:jumpValue, jumpValueIndex:0}];
    } else {
      valueArray = [{displayValue:displayValue, jumpValue:jumpValue, jumpValueIndex:0}];
    }
  }
  me.valueArray = valueArray;
  me.element.setHtml(me.tpl.apply({valueArray:valueArray, jumpValue:jumpValue}));
}, onModuleJump:function(event, domEl) {
  var valueElement = domEl;
  if (!valueElement.hasAttribute('data-valueindex')) {
    valueElement = Ext.get(valueElement).up('[data-valueindex]');
  }
  var record = this.getWidgetRecord();
  var column = this.getWidgetColumn();
  var valueIndex = valueElement ? valueElement.getAttribute('data-valueindex') : null;
  if (!valueIndex) {
    return;
  }
  var value = this.valueArray[valueIndex].jumpValue;
  var target;
  if (column.openNewTab === true) {
    target = monitor.view.interaction.moduleJump.Controller.JUMP_NEW_TAB;
  }
  Ext.getApplication().getController('monitor.view.interaction.moduleJump.Controller').doModuleJump(column, record, value, this.jumpParamFnXtype, target);
}});
Ext.define('monitor.view.interaction.processAction.Button', {extend:'Ext.button.Button', alias:'widget.monitorProcessActionBtn', controller:'monitorProcessActionController', mixins:['monitor.view.interaction.processAction.ButtonMenuMixin'], viewModel:{data:{menuActive:false}}, hidden:true, disabled:true, bind:{disabled:'{!menuActive}'}, initComponent:function() {
  this.hidden = !monitor.view.interaction.processAction.Util.isAbleToRunProcessAction(this.moduleId) && !monitor.view.interaction.processAction.startProcess.Util.isAbleToStartProcess(this.moduleId);
  this.callParent();
}});
Ext.define('monitor.view.interaction.processAction.WidgetActionButton', {extend:'Ext.button.Button', alias:['widget.actionColumnWidgetButton', 'widget.actionColumnWidget'], cls:'widgetActionButton', iconCls:monitor.assets.Icons.PROCESS_ACTION, localized:{text:'MONITOR_ACTIONS'}, tooltip:'!', padding:'2 2 2 2', textAlign:'left', actionData:undefined, instanceId:undefined, moduleId:undefined, referenceId:undefined, instanceType:undefined, showIconOnly:undefined, activeBGColor:undefined, record:undefined, 
initComponent:function() {
  this.callParent(arguments);
}, initActionData:function(actionStore, showIconOnly, hideEmptyButton) {
  var actionTranslation = BpcCommon.Api.getTranslation('MONITOR_ACTIONS');
  var me = this;
  if (actionStore) {
    me.showIconOnly = showIconOnly;
    if (!me.activeBGColor) {
      me.setActiveBGColor();
    }
    if (me.activeBGColor) {
      me.styleComponent();
    }
    if (actionStore.getCount() < 1) {
      me.setButtonText(0, actionTranslation);
      me.setTooltip('0 ' + actionTranslation);
      me.setArrowVisible(false);
      if (hideEmptyButton) {
        me.hide();
      } else {
        me.disable();
      }
      me.disabled = true;
      me.styleComponent('#D3D3D3');
    } else {
      me.setArrowVisible(true);
      me.enable();
      me.show();
      me.disabled = false;
    }
    if (actionStore.getCount() === 1) {
      me.createActionButton(actionStore, showIconOnly);
    } else {
      me.setMenu(me.createActionMenu(actionStore));
      me.setButtonText(actionStore.getCount(), actionTranslation);
      me.setTooltip(actionStore.getCount() + ' ' + actionTranslation);
    }
  }
}, createActionButton:function(actionStore) {
  if (actionStore) {
    var me = this;
    var actionRecord = actionStore.first();
    var actionLabel = BpcCommon.Api.getTranslation(actionRecord.get('label'));
    me.setButtonText('', actionLabel);
    me.setIconCls(actionRecord.get('iconCls') || 'x-fal fa-indent');
    me.setTooltip(actionRecord.get('label') || actionLabel);
    me.setHandler(actionRecord.get('handler'));
    me.setArrowVisible(false);
    me.disabled = false;
  }
}, createActionMenu:function(actionStore) {
  var actionMenu = Ext.create('Ext.menu.Menu', {margin:'0 0 10 0', items:[]});
  if (actionStore) {
    actionStore.each(function(actionRecord) {
      actionMenu.add({text:actionRecord.get('label'), itemId:BpcCommon.Util.cleanUpId(actionRecord.get('id')) || 'actions-item', iconCls:actionRecord.get('iconCls'), handler:actionRecord.get('handler'), tooltip:actionRecord.get('tooltip')});
    });
  }
  return actionMenu;
}, setActiveBGColor:function() {
  var me = this;
  var button = document.getElementById(me.id);
  me.activeBGColor = window.getComputedStyle(button, null).getPropertyValue('background-color') || '#96bebe';
}, setButtonText:function(itemCount, buttonText) {
  var me = this;
  var text = '';
  if (!me.showIconOnly) {
    text = itemCount + ' ' + buttonText;
  } else {
    text = itemCount;
    me.addCls('iconOnly');
  }
  me.setText(text);
}, styleComponent:function() {
  var me = this;
  var elem = me.getEl();
  if (Ext.isDefined(elem)) {
    if ('dom' in elem && 'style' in elem.dom) {
      elem.dom.style.backgroundColor = me.activeBGColor;
    }
  }
}});
Ext.define('monitor.view.interaction.processAction.GridActionColumn', {extend:'Ext.grid.column.Widget', alias:'widget.gridActionColumn', localized:{text:'MONITOR_ACTIONS'}, dataIndex:'BpcActions', widget:{xtype:'actionColumnWidgetButton'}, moduleId:undefined, instanceType:undefined, onWidgetAttach:function(col, widget, rec) {
  if (widget && rec) {
    var me = this;
    var moduleId = me.moduleId || me.up('[moduleId]')['moduleId'];
    var actionStore = monitor.view.interaction.processAction.Util.getAbstractActionStoreForSelection(moduleId, [rec]);
    widget.moduleId = moduleId;
    widget.record = rec;
    widget.initActionData(actionStore, !!this.showIconOnly, !!this.hideEmptyButton);
  }
}});
Ext.define('monitor.view.interaction.processAction.Menu', {extend:'Ext.menu.Item', alias:'widget.monitorProcessActionMenu', controller:'monitorProcessActionController', mixins:['monitor.view.interaction.processAction.ButtonMenuMixin'], hidden:true, text:BpcCommon.Api.getTranslation('MONITOR_ACTIONS'), menu:[], bind:{disabled:'{!menuActive}'}, initComponent:function() {
  var me = this;
  this.hidden = !monitor.view.interaction.processAction.Util.isAbleToRunProcessAction(this.moduleId) && !monitor.view.interaction.processAction.startProcess.Util.isAbleToStartProcess(this.moduleId) && !monitor.view.interaction.processAction.changeState.Util.isAbleToChangeState(this.moduleId);
  me.callParent();
}});
Ext.define('monitor.view.interaction.processAction.Toolbar', {extend:'Ext.toolbar.Toolbar', alias:'widget.monitorProcessActionToolbar', selectedProcesses:undefined, vertical:false, moduleId:undefined, initComponent:function() {
  var me = this;
  var actionStore = monitor.view.interaction.processAction.Util.getAbstractActionStoreForSelection(me.moduleId, me.selectedProcesses);
  me.items = [];
  if (actionStore) {
    actionStore.each(function(actionRecord) {
      me.items.push({text:BpcCommon.Api.getTranslation(actionRecord.get('label')), itemId:BpcCommon.Util.cleanUpId(actionRecord.get('id')) || 'actions-item', iconCls:actionRecord.get('iconCls'), handler:actionRecord.get('handler'), tooltip:BpcCommon.Api.getTranslation(actionRecord.get('tooltip'))});
    });
  }
  me.callParent();
}});
Ext.define('monitor.view.interaction.processAction.Util', {singleton:true, alternateClassName:['monitor.ProcessActionUtil'], instanceCacheMap:{docuDummy:{actionColumns:undefined, processActionsSeparator:',', currentActionsMap:{}}}, initInstanceCache:function(moduleId) {
  var me = this;
  var actionsActive = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.processActions);
  var actionList = [];
  if (me.instanceCacheMap[moduleId] && Ext.isArray(me.instanceCacheMap[moduleId].actionColumns)) {
    return;
  }
  if (actionsActive) {
    me.instanceCacheMap[moduleId] = {currentActionsMap:{}};
    me.instanceCacheMap[moduleId].processActionsSeparator = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.actionSeperator);
    var columnPrefix = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnActionPrefix);
    var columnList = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnList);
    var columnHistoryList = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnHistoryList);
    var getActionsFromList = function(list, prefix) {
      return Ext.Array.filter(Ext.Object.getAllKeys(list), function(key) {
        return Ext.String.startsWith(key, prefix);
      });
    };
    if (columnList) {
      var mainActions = getActionsFromList(columnList, columnPrefix);
      actionList = Ext.Array.union(actionList, mainActions);
    }
    if (columnHistoryList) {
      var historyActions = getActionsFromList(columnHistoryList, columnPrefix);
      actionList = Ext.Array.union(actionList, historyActions);
    }
  }
  me.instanceCacheMap[moduleId].actionColumns = actionList;
}, sendActionToInubit:function(moduleId, preparedParam, multiAction, action, buttonId, showAlert) {
  var deferred = new Ext.Deferred();
  var me = this;
  var unlockFn = Ext.emptyFn;
  try {
    var inubitProxyId = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.bpmProxyId);
    var inubitBaseUrl = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.bpmBasePath);
    var inubitActionEndpoint = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.processActionsEndpoint);
    var dbGridId = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.legacyPmGridId);
    var allowParallelActions = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.parallelProcessActions);
    var extGridId = 'PM-core-' + dbGridId + '-grid';
    var multiUrl = BpcCommon.Api.getBackendUrl(inubitProxyId) + inubitBaseUrl + inubitActionEndpoint + '?gridId\x3d' + dbGridId + '\x26gridExtId\x3d' + extGridId + '\x26multiRecords\x3dtrue' + ('\x26buttonId\x3d' + buttonId + '\x26' + buttonId + '\x3d' + action);
    var singleUrl = BpcCommon.Api.getBackendUrl(inubitProxyId) + inubitBaseUrl + inubitActionEndpoint + '?gridId\x3d' + dbGridId + '\x26gridExtId\x3d' + extGridId + '\x26buttonId\x3d' + buttonId + '\x26' + buttonId + '\x3d' + action;
    var url = singleUrl;
    if (multiAction) {
      url = multiUrl;
    }
    url = url.replace('/bpc-httpproxy/httpProxy/', '/bpc-monitor/monitor/httpProxy/');
    if (!allowParallelActions && me.instanceCacheMap[moduleId].currentActionsMap[action]) {
      BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation('MONITOR_ACTION_ALREADY_STARTED'));
      return;
    }
    me.instanceCacheMap[moduleId].currentActionsMap[action] = true;
    if (!allowParallelActions) {
      unlockFn = BpcCommon.Util.blockComponentForAsyncAction(action, 'action');
    }
    BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation({tpl:'{MONITOR_ACTION_ALREADY_STARTED1}' + action + '{MONITOR_ACTION_ALREADY_STARTED2}', keys:['MONITOR_ACTION_ALREADY_STARTED1', 'MONITOR_ACTION_ALREADY_STARTED2']}));
    Ext.fireEvent('monitorProcessAction', action);
    var connection = BpcCommon.Api.getConnection(inubitProxyId);
    connection.setUrl(encodeURI(url));
    var notificationDisplayMode = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.processNotificationDisplayMode);
    var notificationSilent = notificationDisplayMode === 'silent';
    var notificationPopUp = notificationDisplayMode === 'popup';
    var notification = {};
    if (showAlert) {
      notification.popUp = showAlert;
    } else {
      notification.popUp = notificationPopUp;
      notification.silent = notificationSilent;
    }
    connection.request({params:preparedParam, headers:{ISPortalUser:BpcCommon.Api.getUserName(), 'X-BPC-ModuleId':moduleId}, success:function(response) {
      try {
        var responseJson;
        try {
          responseJson = response.responseJSON || Ext.decode(response.responseText);
        } catch (e) {
          Ext.log({msg:this.self.getName() + ': parse JSON not successfully, response might be XML', level:'warn'});
        }
        if (!responseJson && response.responseXML) {
          responseJson = {msg:Ext.DomQuery.selectValue('msg', response.responseXML, ''), success:Ext.DomQuery.selectValue('success', response.responseXML)};
          var download = Ext.DomQuery.selectNode('download', response.responseXML);
          if (download) {
            responseJson.download = {data:Ext.DomQuery.selectValue('data', download), fileName:Ext.DomQuery.selectValue('fileName', download), contentType:Ext.DomQuery.selectValue('contentType', download)};
          }
        }
        if (responseJson.success === true || responseJson.success === 'true') {
          notification.text = BpcCommon.Api.getTranslation('MONITOR_ACTION_CALL_SUCCESSFUL');
          if (responseJson.msg) {
            notification.text = notification.text + ': ' + responseJson.msg;
          }
          BpcCommon.Api.showNotification(notification);
          if (responseJson.download) {
            try {
              var download$8 = responseJson.download;
              BpcCommon.Util.download(download$8.data, download$8.fileName, download$8.contentType);
            } catch (e$9) {
              BpcCommon.Api.showNotification('Fehler beim Bereitstellen des Dateidownloads.');
              this.lookupReference('startProcessBnt').enable();
              return;
            }
          }
        } else {
          notification.text = BpcCommon.Api.getTranslation('MONITOR_ACTION_CALL_ERROR');
          notification.type = 'ERROR';
          if (responseJson.msg) {
            notification.text = notification.text + ': ' + responseJson.msg;
          }
          BpcCommon.Api.showNotification(notification);
        }
      } catch (e$10) {
        Ext.log({msg:this.self.getName() + ': ERROR success', dump:e$10, stack:true, level:'error'});
      }
      deferred.resolve();
    }, failure:function(response) {
      try {
        var responseJson, msg;
        try {
          responseJson = response.responseJSON || Ext.decode(response.responseText);
        } catch (e) {
          Ext.log({msg:this.self.getName() + ': parse JSON not successfully, response might be XML', dump:e, stack:true, level:'warn'});
        }
        if (responseJson) {
          msg = responseJson.msg;
        } else {
          msg = Ext.DomQuery.selectValue('msg', response.responseXML, BpcCommon.Api.getTranslation('MONITOR_UNKNOWN_ERROR'));
        }
        notification.text = BpcCommon.Api.getTranslation('MONITOR_ACTION_CALL_ERROR');
        notification.type = 'ERROR';
        if (msg) {
          notification.text = notification.text + ': ' + msg;
        }
        BpcCommon.Api.showNotification(notification);
      } catch (e$11) {
        Ext.log({msg:this.self.getName() + ': ERROR failure', dump:e$11, stack:true, level:'error'});
      }
      deferred.reject();
    }, callback:function() {
      delete me.instanceCacheMap[moduleId].currentActionsMap[action];
      unlockFn();
    }, scope:this});
  } catch (e) {
    BpcCommon.Api.showNotification(BpcCommon.Api.getTranslation('MONITOR_ACTION_SEND_ERROR') + e);
    delete me.instanceCacheMap[moduleId].currentActionsMap[action];
    unlockFn();
  }
  return deferred.promise;
}, getAbstractActionsFromProcessAction:function(moduleId, actionString, separator, columnName) {
  var result = [];
  var me = this;
  try {
    var newColumnName = String(columnName).toLowerCase();
    var json = Ext.JSON.decode(actionString, true);
    if (!json) {
      json = [];
      Ext.Array.each(String(actionString).split(separator), function(actionName) {
        try {
          json.push({name:actionName, column:String(columnName).toLowerCase()});
        } catch (e) {
          Ext.log({msg:me.self.getName() + ': ERROR parseAction', dump:e, stack:true, level:'error'});
        }
      });
    }
    if (!Ext.isArray(json)) {
      json = [json];
    }
    Ext.Array.each(json, function(action) {
      if (BpcCommon.Api.isUserAllowedToSee(action)) {
        try {
          action = Ext.applyIf(action, {label:BpcCommon.Api.getTranslation(action.name), id:newColumnName + '_' + action.name, column:newColumnName, type:'processAction'});
          var actionModel = monitor.model.AbstractAction.loadData(action);
          result.push(actionModel);
        } catch (e) {
          Ext.log({msg:me.self.getName() + ': ERROR parseAction', dump:e, stack:true, level:'error'});
        }
      }
    });
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR parseAction', dump:e, stack:true, level:'error'});
  }
  return result;
}, isAbleToRunProcessAction:function(moduleId) {
  var isProcessActionActive = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.processActions);
  var hasUserRight = BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.processAction) || BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.processAction + '_' + moduleId);
  return isProcessActionActive && hasUserRight;
}, getAbstractActionStoreForSelection:function(moduleId, selection) {
  var me = this;
  var actionStore = Ext.create('Ext.data.Store', {model:'monitor.model.AbstractAction', sorters:[{property:'type', direction:'ASC'}, {property:'sortValue', direction:'DESC'}, {property:'name', direction:'ASC'}], data:[]});
  var selectionArray = selection || [];
  if (me.isAbleToRunProcessAction(moduleId)) {
    me.initInstanceCache(moduleId);
    var actionIdArray = [];
    Ext.each(selectionArray, function(record) {
      var selectionActionIdArray = [];
      Ext.each(me.instanceCacheMap[moduleId].actionColumns, function(column) {
        var actionString = record.get(column);
        if (actionString) {
          var actions = me.getAbstractActionsFromProcessAction(moduleId, actionString, me.instanceCacheMap[moduleId].processActionsSeparator, column, selection);
          actionStore.add(actions);
          Ext.Array.each(actions, function(action) {
            selectionActionIdArray.push(action.getId());
          });
        }
      });
      actionIdArray.push(selectionActionIdArray);
    });
    var distinctActionIds = Ext.Array.intersect.apply(undefined, actionIdArray);
    actionStore.each(function(action) {
      if (!Ext.Array.contains(distinctActionIds, action.getId())) {
        actionStore.remove(action);
      } else {
        action.set('handler', me.getProcessActionHandler(moduleId, action.getData(), action.get('column'), selection));
      }
    });
  }
  if (selectionArray.length > 0 && monitor.view.interaction.processAction.startProcess.Util.isAbleToStartProcess(moduleId)) {
    var processes = monitor.view.interaction.processAction.startProcess.Util.getProcessesWithContext(moduleId);
    processes.forEach(function(processConfig) {
      var sortValue = processConfig.startWithContext ? -50 : -100;
      actionStore.add(monitor.model.AbstractAction.loadData({name:processConfig.key, label:processConfig.label, iconCls:processConfig.iconCls, handler:monitor.view.interaction.processAction.startProcess.Util.openStartProcessForm.bind(monitor.view.interaction.processAction.startProcess.Util, moduleId, processConfig), type:'processStart', sortValue:sortValue}));
    });
  }
  return actionStore;
}, getProcessActionHandler:function(moduleId, actionObject, columnName, selection) {
  var $jscomp$this = this;
  var currentSelection = [].concat(selection);
  if (actionObject.target && BpcCommon.Util.isStringEqual(actionObject.target, 'processStarter')) {
    var targetProcess = actionObject.params ? actionObject.params.process : undefined;
    return monitor.view.interaction.processAction.startProcess.Util.openStartProcessForm.bind(monitor.view.interaction.processAction.startProcess.Util, moduleId, {key:targetProcess});
  } else if (actionObject.target && BpcCommon.Util.isStringEqual(actionObject.target, 'forms')) {
    var matchString = 'formsMain';
    if (actionObject.params && actionObject.params.selector) {
      matchString = matchString + '[' + actionObject.params.selector + ']';
    }
    return function() {
      var formsInstance = Ext.first(matchString);
      if (formsInstance) {
        var submitOriginator = {trigger:'monitorAction', actionConfig:actionObject};
        formsInstance.submitData(submitOriginator);
        BpcCommon.Api.showNotification('MONITOR_ACTION_FORM_SUBMITTED');
      } else {
        BpcCommon.Api.showNotification({type:'ERROR', text:'MONITOR_ACTION_FORM_NO_FOUND', popUp:true});
      }
    };
  }
  var me = this;
  var preparedParam = {};
  var action = actionObject.name;
  var buttonId = columnName;
  var proxyId = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.bpmProxyId);
  var legacyPmTablePrefix = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.legacyPmTablePrefix);
  var multi = false;
  if (!proxyId || proxyId === '') {
    return BpcCommon.Api.showNotification.bind(BpcCommon.Api, {text:BpcCommon.Api.getTranslation('MONITOR_BACKEND_CONNECTION_MISSING') + ' ' + BpcCommon.Api.getTranslation('MONITOR_ACTION_CALL_ERROR'), popUp:true, type:'ERROR'});
  }
  if (currentSelection.length === 1) {
    Ext.Object.each(currentSelection[0].getData(), function(key, value) {
      preparedParam[key.toLowerCase()] = value;
    });
    preparedParam.tablePrefix = legacyPmTablePrefix;
  } else if (currentSelection.length > 1) {
    multi = true;
    var prefix = 'index';
    currentSelection.forEach(function(record, i) {
      Ext.Object.each(record.getData(), function(key, value) {
        preparedParam[prefix + i + '_' + key.toLowerCase()] = value;
      });
      preparedParam[prefix + i + '_' + 'tablePrefix'] = legacyPmTablePrefix;
    });
  }
  Ext.Object.each(actionObject, function(key, value) {
    preparedParam['actionConfig_' + key] = value;
  });
  if (actionObject.requireConfirmation || actionObject.requireComment) {
    var defaultText = BpcCommon.Api.getTranslation({key:'MONITOR_EXECUTE_ACTION', data:{name:actionObject.name || ''}});
    var confirmationText = BpcCommon.Api.getTranslation(actionObject.confirmationText) || defaultText;
    var msg = confirmationText + '\x3c/br\x3e' + BpcCommon.Api.getTranslation('MONITOR_COMMENT') + ':';
    return function() {
      Ext.create({xtype:'bpcMessageBoxWindow', resizable:true}).show({title:BpcCommon.Api.getTranslation('MONITOR_PROCESS_ACTIONS') + ': ' + actionObject.name, msg:actionObject.requireComment ? msg : confirmationText, width:300, multiline:actionObject.requireComment || false, scope:$jscomp$this, buttons:!actionObject.requireConfirmation && actionObject.requireComment ? BpcCommon.Msg.OKCANCEL : BpcCommon.Msg.YESNO, fn:function(btn, text) {
        if (btn === 'yes' || btn === 'ok') {
          preparedParam.actionConfig_comment = text;
          me.sendActionToInubit(moduleId, preparedParam, multi, action, buttonId, false);
        }
      }});
    };
  }
  return me.sendActionToInubit.bind(me, moduleId, preparedParam, multi, action, buttonId, false);
}});
Ext.define('monitor.view.interaction.processAction.ViewController', {extend:'Ext.app.ViewController', alias:'controller.monitorProcessActionController', actionColumns:undefined, processActionsSeparator:',', currentActionsMap:{}, actionStore:undefined, init:function() {
  var me = this;
  var vm = me.getViewModel();
  vm.bind('{activeGridSelection}', me.onSelectionChange, me);
}, onSelectionChange:function(selection) {
  var me = this;
  var view = me.getView();
  var menu = Ext.create('Ext.menu.Menu', {margin:'0 0 10 0', items:[]});
  if (monitor.view.interaction.processAction.changeState.Util.isAbleToChangeState(view.moduleId)) {
    menu.add({text:BpcCommon.Api.getTranslation('MONITOR_CHANGE_STATUS'), tooltip:BpcCommon.Api.getTranslation('MONITOR_CHANGE_STATUS'), itemId:'monitorChangeStateMenu', iconCls:'x-fal fa-tasks', handler:monitor.view.interaction.processAction.changeState.Util.changeStatus.bind(this, view.moduleId)});
  }
  var actionStore = monitor.view.interaction.processAction.Util.getAbstractActionStoreForSelection(view.moduleId, selection);
  if (actionStore) {
    actionStore.each(function(actionRecord) {
      if (me.shouldAddActionToMenu(actionRecord)) {
        menu.add({text:BpcCommon.Api.getTranslation(actionRecord.get('label')), itemId:BpcCommon.Util.cleanUpId(actionRecord.get('id')) || 'actions-item', iconCls:actionRecord.get('iconCls'), handler:actionRecord.get('handler'), tooltip:BpcCommon.Api.getTranslation(actionRecord.get('tooltip'))});
      }
    });
  }
  if (monitor.view.interaction.processAction.startProcess.Util.isAbleToStartProcess(view.moduleId) && view.xtype === 'monitorProcessActionBtn') {
    if (actionStore.getCount() > 0) {
      menu.add({xtype:'menuseparator'});
    }
    menu.add({text:BpcCommon.Api.getTranslation('MONITOR_START_PROCESS'), itemId:'startProcessBtn', iconCls:monitor.assets.Icons.PROCESS_START, handler:monitor.view.interaction.processAction.startProcess.Util.openStartProcessForm.bind(monitor.view.interaction.processAction.startProcess.Util, view.moduleId), tooltip:BpcCommon.Api.getTranslation('MONITOR_START_PROCESS')});
  }
  if (menu.items.getCount() < 1) {
    view.setMenu(null, true);
    view.setTooltip(BpcCommon.Api.getTranslation('MONITOR_NO_ACTIONS_AVAILABLE'));
    if (view.xtype === 'monitorProcessActionBtn') {
      view.setTooltip(BpcCommon.Api.getTranslation('MONITOR_NO_ACTIONS_AVAILABLE'));
    }
    me.getViewModel().set('menuActive', false);
  } else {
    view.setMenu(menu, true);
    view.setTooltip(BpcCommon.Api.getTranslation('MONITOR_PROCESS_ACTIONS'));
    if (view.xtype === 'monitorProcessActionBtn') {
      view.setTooltip(BpcCommon.Api.getTranslation('MONITOR_PROCESS_ACTIONS'));
    }
    me.getViewModel().set('menuActive', true);
  }
}, shouldAddActionToMenu:function(actionRecord) {
  var defaultValue = true;
  var actionTarget = actionRecord.get('target');
  return BpcCommon.Util.isStringEqual(actionTarget, 'forms') ? false : defaultValue;
}});
Ext.define('monitor.view.interaction.processAction.changeState.ChangeStateMixin', {moduleId:undefined, iconCls:monitor.assets.Icons.TASKS, localized:{tooltip:'MONITOR_CHANGE_STATUS'}, handler:function() {
  var me = this;
  monitor.view.interaction.processAction.changeState.Util.changeStatus(me.moduleId);
}, isAbleToChangeState:function(moduleId) {
  return monitor.view.interaction.processAction.changeState.Util.isAbleToChangeState(moduleId);
}});
Ext.define('monitor.view.interaction.processAction.changeState.ChangeStateButton', {extend:'Ext.Button', alias:'widget.monitorChangeStateBtn', mixins:['monitor.view.interaction.processAction.changeState.ChangeStateMixin'], hidden:true, disabled:true, bind:{disabled:'{activeGridSelection.length \x3c 1}'}, initComponent:function() {
  this.hidden = !this.isAbleToChangeState(this.moduleId, this);
  this.handler = monitor.view.interaction.processAction.changeState.Util.changeStatus.bind(this, this.moduleId);
  this.callParent();
}});
Ext.define('monitor.view.interaction.processAction.changeState.ChangeStateWindowController', {extend:'Ext.app.ViewController', alias:'controller.changeStateWindowController', control:{'#stateColumnSelector':{change:'onColumnStoreChange'}, '#statesField':{change:'onStateChange'}, 'form':{validitychange:'onValidityChange'}}, onValidityChange:function(_form, formIsValid) {
  var me = this;
  var vm = me.getViewModel();
  vm.set('formIsValid', formIsValid);
}, onFormSubmit:function() {
  var me = this;
  var unlockFn = BpcCommon.Util.blockComponentForAsyncAction('onFormSubmit', 'handler', BpcCommon.Api.getTranslation('MONITOR_IS_BEING_PROCESSED'), '^ monitorChangeStateWindow');
  try {
    var form = me.getView().down('form');
    if (form.isValid()) {
      var postObj = {};
      var stateConfig = me.getView().changeStateConfigMap;
      postObj.tablePrefix = BpcCommon.Api.getSetting(me.getView().moduleId, monitor.assets.ConfigKeys.legacyPmTablePrefix);
      postObj.mandant = BpcCommon.Api.getSetting(me.getView().moduleId, monitor.assets.ConfigKeys.legacyPmMandant);
      postObj.timelineUpdate = BpcCommon.Api.getSetting(me.getView().moduleId, monitor.assets.ConfigKeys.columnTimeseries);
      var statusfield = me.getView().down('#stateColumnSelector').getValue();
      var childStatus = 'Info';
      if (Ext.isDefined(stateConfig[statusfield].childStatus)) {
        childStatus = stateConfig[statusfield].childStatus;
      }
      postObj.childStatus = childStatus;
      postObj.command = 'StatusChange';
      postObj.columnsstring = statusfield.toLowerCase();
      postObj['changeStatusBox_' + statusfield.toLowerCase()] = 'on';
      postObj['newStatusCombo_' + statusfield.toLowerCase()] = me.getView().down('#statesField').getValue();
      postObj.commentfield = me.getView().down('#comment').getValue();
      var processes = [];
      var columnId = BpcCommon.Api.getSetting(me.getView().moduleId, monitor.assets.ConfigKeys.columnId);
      columnId = columnId.split(',').length > 1 ? columnId.split(',') : columnId;
      if (Ext.isArray(columnId)) {
        Ext.Array.each(me.getView().selectedRecords, function(record) {
          var joinedVal = [];
          columnId.forEach(function(val) {
            joinedVal.push(record.get(val));
          });
          processes.push(joinedVal.join(','));
        });
      } else {
        Ext.Array.each(me.getView().selectedRecords, function(record) {
          processes.push(record.id);
        });
      }
      postObj.processes = processes.join(';');
      if (postObj) {
        var moduleId = this.getView().moduleId;
        var inubitUrl = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.bpmBasePath);
        var inubitEndpoint = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.changeStateEndpoint);
        var inubitProxyId = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.bpmProxyId);
        var connectionTimeout = BpcCommon.Api.getSetting(inubitProxyId, 'connectionTimeout') * 1000 || Ext.Ajax.getTimeout();
        var proxyUrl = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/httpProxy/' + (inubitProxyId + inubitUrl + inubitEndpoint);
        var notification = {toast:true, toastTarget:this.getView()};
        Ext.Ajax.request({url:proxyUrl, method:'POST', timeout:connectionTimeout, params:postObj, headers:{ISPortalUser:BpcCommon.Api.getUserSession().data.loginName, 'X-BPC-ModuleId':moduleId}, success:function(response) {
          var obj = Ext.decode(response.responseText, true);
          if (obj && obj.success) {
            notification.text = BpcCommon.Api.getTranslation('MONITOR_STATUS_CHANGE_SUCCESSFUL');
            if (obj.msg) {
              notification.text = notification.text + ': ' + obj.msg;
            }
            BpcCommon.Api.showNotification(notification);
            me.destroyWindow();
          } else {
            var errorMsg = BpcCommon.Util.getErrorFromFailureResponse(response).message;
            notification.text = BpcCommon.Api.getTranslation('MONITOR_STATUS_CHANGE_FAILED') + ': ' + errorMsg;
            notification.type = 'ERROR';
            BpcCommon.Api.showNotification(notification);
          }
        }, failure:function(response) {
          var errorMsg = BpcCommon.Util.getErrorFromFailureResponse(response).message;
          notification.text = BpcCommon.Api.getTranslation('MONITOR_STATUS_CHANGE_FAILED') + ': ' + errorMsg;
          notification.type = 'ERROR';
          BpcCommon.Api.showNotification(notification);
        }, callback:unlockFn});
      }
    } else {
      unlockFn();
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onFormSubmit', dump:e, stack:true, level:'error'});
    unlockFn();
  }
}, onColumnStoreChange:function(combo, newValue) {
  this.modifyStateStore(newValue);
}, onStateChange:function(combo, newValue) {
  try {
    var vm = this.getViewModel();
    vm.set('displayState', newValue);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onStateChange', dump:e, stack:true, level:'error'});
  }
}, modifyStateStore:function(columnName) {
  var me = this;
  var monitorMain = BpcCommon.Api.getModuleInstanceComponent(me.getView().moduleId);
  try {
    var moduleId = this.getView().moduleId;
    var store = Ext.data.StoreManager.lookup('dynStates');
    store.removeAll();
    var viewModel = this.getViewModel();
    viewModel.set('columnName', columnName);
    var changeStateConfigMap = this.getView().changeStateConfigMap;
    var config = changeStateConfigMap[columnName];
    if (config) {
      var monitorCustomRenderer = monitorMain.findPlugin('monitorCustomRenderer');
      var monitorColumnConfig = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.columnConfig);
      var columnConfig = Ext.Array.findBy(monitorColumnConfig, function(colConfig) {
        return colConfig.id === columnName;
      });
      if (!columnConfig) {
        columnConfig = Ext.Array.findBy(monitorColumnConfig, function(colConfig) {
          return colConfig.dataIndex === columnName;
        });
      }
      var customRendererId = monitorCustomRenderer.getRendererIdByColumnConfig(columnConfig);
      if (!config.states) {
        config.states = [];
      }
      var addToStore = function(value) {
        var label = monitorCustomRenderer.render(customRendererId, value, false, true);
        if (!Ext.isDefined(label) || label === null) {
          label = value;
        }
        store.add({valueR:monitorCustomRenderer.render(customRendererId, value), value:value, raw:value, translated:label});
      };
      if (config.useDistinctValues || config.useDistinctValues === undefined) {
        var url = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + this.getView().moduleId + '/distinctFieldValues/' + columnName + '.raw';
        monitor.Util.ajaxGetCall(url, function(success, response) {
          Ext.each(response.values, function(value) {
            addToStore(value.value);
          });
        });
      }
      if (config.useRendererValues) {
        var rendererConfig = monitorCustomRenderer.getRendererConfigMap()[String(customRendererId).toLowerCase()];
        Ext.Object.getAllKeys(rendererConfig.valueMap).forEach(addToStore);
      }
      if (!config.useRendererValues && !(config.useDistinctValues || config.useDistinctValues === undefined)) {
        Ext.each(config.states, function(stateVal) {
          if (stateVal.settable !== false) {
            addToStore(stateVal.state);
          }
        });
      }
    } else {
      Ext.log({msg:this.self.getName() + ': modifyStateStore : no config found for ' + columnName, level:'warning'});
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR modifyStateStore', dump:e, stack:true, level:'error'});
  }
}, destroyWindow:function() {
  this.getView().destroy();
}});
Ext.define('monitor.view.interaction.processAction.changeState.ChangeStateWindow', {extend:'BpcCommon.component.window.Window', alias:['widget.monitorChangeStateWindow'], requires:['monitor.view.interaction.processAction.changeState.ChangeStateWindowController'], controller:'changeStateWindowController', stateful:true, stateId:'changeStateWindow', constrain:true, layout:'fit', bodyPadding:'10', width:700, minWidth:300, minHeight:300, iconCls:monitor.assets.Icons.TASKS, moduleId:undefined, columnStore:undefined, 
defaultFocus:'#changeStateComment', user:undefined, viewModel:{data:{columnName:undefined, displayState:''}}, autoShow:true, initComponent:function() {
  var me = this;
  var monitorMain = BpcCommon.Api.getModuleInstanceComponent(me.moduleId);
  me.changeStateConfig = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.changeStateConfig);
  me.selectedRecords = me.getViewModel().get('activeGrid.selectedRecords');
  if (!me.changeStateConfig || !me.moduleId) {
    me.items = [{xtype:'bpcCenterMessageCmp', message:'MONITOR_MODULE_NO_CONFIG_FOR_STATE_CHANGE'}];
    me.callParent();
    return;
  }
  var proxyId = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.bpmProxyId);
  if (!proxyId || proxyId === '') {
    me.items = [{xtype:'bpcCenterMessageCmp', message:BpcCommon.Api.getTranslation('MONITOR_BACKEND_CONNECTION_MISSING') + ' \x3cbr\x3e ' + BpcCommon.Api.getTranslation('MONITOR_STATUS_CHANGE_ERROR')}];
    me.callParent();
    return;
  }
  if (Ext.isArray(me.changeStateConfig)) {
    me.changeStateConfigMap = Ext.Array.toValueMap(me.changeStateConfig, 'dataIndex');
  } else {
    me.items = [{xtype:'bpcCenterMessageCmp', message:'MONITOR_ERROR_READING_CONFIGURATIONS'}];
    me.callParent();
    return;
  }
  me.user = BpcCommon.Api.getUserName();
  me.columnStore = new Ext.data.ArrayStore({fields:['value']});
  var columnId = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.columnId);
  columnId = columnId.split(',').length > 1 ? columnId.split(',') : columnId;
  var fieldArray = Ext.isArray(columnId) ? columnId : [columnId];
  if (Ext.isArray(me.changeStateConfig)) {
    var additionalColumns = [];
    Ext.each(me.changeStateConfig, function(columnConfig) {
      if (Ext.isArray(columnConfig.additionalColumns)) {
        additionalColumns = additionalColumns.concat(columnConfig.additionalColumns);
      }
    }, me);
    fieldArray = fieldArray.concat(additionalColumns);
  }
  Ext.Array.each(me.changeStateConfig, function(stateColumnConfig) {
    var dataIndex = stateColumnConfig.dataIndex;
    var column = me.getViewModel().get('activeGrid').getColumnManager().getHeaderByDataIndex(dataIndex);
    if (!column) {
      return;
    }
    me.columnStore.add({value:dataIndex, display:column.text, column:column});
    fieldArray.push(dataIndex);
  });
  var records = Ext.Array.map(me.selectedRecords, function(record) {
    return record.clone();
  });
  var monitorColumnConfig = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.columnConfig);
  me.items = [{xtype:'form', layout:{type:'vbox', align:'stretch'}, items:[{xtype:'bpcComboBox', autoSelect:true, localized:{fieldLabel:'MONITOR_COLUMN'}, store:me.columnStore, queryMode:'local', displayField:'display', valueField:'value', allowBlank:false, itemId:'stateColumnSelector', width:150, listeners:{afterrender:function(combo) {
    var recordSelected = combo.getStore().getAt(0);
    if (recordSelected) {
      combo.setValue(recordSelected.get('value'));
    }
  }}}, {xtype:'fieldcontainer', localized:{fieldLabel:'MONITOR_PROCESSES'}, items:[{xtype:'grid', maxHeight:300, store:{data:records, fields:fieldArray.concat([])}, columns:Ext.Array.map(fieldArray, function(field) {
    var columnConfig = Ext.Array.findBy(monitorColumnConfig, function(colConfig) {
      return colConfig.id === field;
    });
    if (!columnConfig) {
      columnConfig = Ext.Array.findBy(monitorColumnConfig, function(colConfig) {
        return colConfig.dataIndex === field;
      });
    }
    if (!columnConfig) {
      columnConfig = {text:field};
    }
    var text = columnConfig.text;
    if (columnConfig.localized && columnConfig.localized.text) {
      text = BpcCommon.Api.getTranslation(columnConfig.localized.text);
    }
    return {dataIndex:field, renderer:monitorMain.findPlugin('monitorCustomRenderer').getColumnRendererByColumnConfig(columnConfig), text:text, flex:1};
  })}]}, {xtype:'bpcComboRenderedValue', width:150, reference:'states', itemId:'statesField', autoSelect:true, localized:{fieldLabel:'MONITOR_STATE'}, bind:{value:'{displayState}'}, store:{fields:['value', 'raw', 'valueR', 'translated'], storeId:'dynStates'}, queryMode:'local', displayField:'valueR', valueField:'value', typeAhead:true, listeners:{beforequery:function(record) {
    record.query = new RegExp(record.query, 'ig');
  }}, forceSelection:true, allowBlank:false}, {xtype:'bpcTextArea', name:'changeStateComment', itemId:'comment', localized:{fieldLabel:'MONITOR_COMMENT', emptyText:'MONITOR_ENTER_COMMENT'}, flex:1, allowBlank:false}]}];
  me.dockedItems = [{xtype:'toolbar', dock:'bottom', ui:'footer', overflowHandler:'scroller', items:['-\x3e', {text:BpcCommon.Api.getTranslation('MONITOR_CANCEL'), iconCls:BpcCommon.Icons.CANCEL, ui:'secondary', handler:'destroyWindow'}, {text:BpcCommon.Api.getTranslation('MONITOR_CHANGE_STATUS'), iconCls:BpcCommon.Icons.OK, ui:'primary', itemId:'submit-change-status-btn', bind:{disabled:'{!formIsValid}'}, handler:'onFormSubmit'}]}];
  me.title = me.generateWindowTitle();
  this.callParent();
}, generateWindowTitle:function() {
  var me = this;
  var amountOfAffectedDataSets = 0;
  if (Ext.isArray(me.selectedRecords)) {
    amountOfAffectedDataSets = me.selectedRecords.length;
  }
  var correctLanguageKey = '';
  if (amountOfAffectedDataSets === 1) {
    correctLanguageKey = 'MONITOR_AFFECTED_DATA_SET';
  } else {
    correctLanguageKey = 'MONITOR_AFFECTED_DATA_SETS';
  }
  return BpcCommon.Api.getTranslation('MONITOR_EDIT_STATE') + ' (' + me.selectedRecords.length + ' ' + BpcCommon.Api.getTranslation(correctLanguageKey) + ')';
}});
Ext.define('monitor.view.interaction.processAction.changeState.Util', {singleton:true, alternateClassName:['monitor.StartProcessUtil'], instanceCacheMap:{docuDummy:{store:undefined}}, initInstanceCache:function(moduleId) {
  var me = this;
  if (me.instanceCacheMap[moduleId]) {
    return;
  }
  me.instanceCacheMap[moduleId] = {store:undefined};
}, isAbleToChangeState:function(moduleId) {
  var isChangeStateActive = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.changeState);
  var hasUserRight = BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.changeProcessState) || BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.changeProcessState + '_' + moduleId);
  var currentMonitorView = Ext.ComponentQuery.query('bpcMonitor').find(function(item) {
    return item.moduleId === moduleId;
  });
  var isItemSelected = currentMonitorView.getViewModel().get('activeGridSelection').length !== 0;
  return isChangeStateActive && hasUserRight && isItemSelected;
}, changeStatus:function(moduleId) {
  Ext.create({xtype:'monitorChangeStateWindow', moduleId:moduleId, viewModel:{parent:Ext.first('bpcMonitor:visible') ? Ext.first('bpcMonitor:visible').getViewModel() : undefined}});
}});
Ext.define('monitor.view.interaction.processAction.startProcess.ButtonPlugin', {extend:'Ext.Container', alias:'widget.monitorStartProcessButtonPlugin', layout:'card', viewModel:{parent:BpcCommon.Api.getGlobalViewModel()}, initComponent:function() {
  var me = this;
  me.items = [];
  if (!(me.pluginConfiguration && me.pluginConfiguration.configuration && (me.pluginConfiguration.configuration.moduleId || me.pluginConfiguration.configuration.monitorId))) {
    me.items.push({xtype:'bpcCenterMessageCmp', message:'Missing moduleId config'});
    me.callParent();
    return;
  } else if (!BpcCommon.Util.isStringEqual(BpcCommon.Api.getBaseModuleIdById(me.pluginConfiguration.configuration.moduleId || me.pluginConfiguration.configuration.monitorId), 'monitor')) {
    me.items.push({xtype:'bpcCenterMessageCmp', message:"can't get monitor instance for configured moduleId"});
    me.callParent();
    return;
  }
  Ext.applyIf(me, me.pluginConfiguration.configuration.wrapperConfiguration);
  if (me.pluginConfiguration.configuration.monitorConfiguration && me.pluginConfiguration.context.record) {
    var stringConfig = JSON.stringify(me.pluginConfiguration.configuration.monitorConfiguration);
    Ext.Object.each(me.pluginConfiguration.context.record.getData(), function(key, value) {
      stringConfig = stringConfig.replaceAll('record.' + key, value);
    });
    me.pluginConfiguration.configuration.monitorConfiguration = JSON.parse(stringConfig);
  }
  me.items.push({xtype:'monitorStartProcessBtn', moduleId:me.pluginConfiguration.configuration.moduleId || me.pluginConfiguration.configuration.monitorId});
  me.callParent();
}});
Ext.define('monitor.view.interaction.processAction.startProcess.ButtonPluginInterface', {extend:'BpcCommon.plugin.Interface', inheritableStatics:{NAME:'Monitor Process Starter - Button', COMPONENT_XTYPE:'monitorStartProcessButtonPlugin', PLUGIN_ID:'monitor_process_starter', DEFAULT_CONFIGURATION:{}}});
Ext.define('monitor.view.interaction.processAction.startProcess.PanelPluginInterface', {extend:'BpcCommon.plugin.Interface', inheritableStatics:{NAME:'Monitor Process Starter - Panel', PLUGIN_ID:'monitor_process_starter_panel', DEFAULT_CONFIGURATION:{moduleId:'moduleId', processKey:'processKey'}, createPlugin:function(pluginParams) {
  pluginParams.configuration = pluginParams.configuration || {};
  if (!(pluginParams && pluginParams.configuration && (pluginParams.configuration.moduleId || pluginParams.configuration.monitorId))) {
    return {xtype:'bpcCenterMessageCmp', message:'Missing moduleId config'};
  } else if (!BpcCommon.Util.isStringEqual(BpcCommon.Api.getBaseModuleIdById(pluginParams.configuration.moduleId || pluginParams.configuration.monitorId), 'monitor')) {
    return {xtype:'bpcCenterMessageCmp', message:"can't get monitor instance for configured moduleId"};
  }
  return Ext.applyIf({xtype:'monitorStartProcessPanel', pluginConfiguration:pluginParams, moduleId:pluginParams.configuration.moduleId || pluginParams.configuration.monitorId, preselectedProcess:{key:pluginParams.configuration.processKey}, header:false}, pluginParams.configuration.cmpConfiguration);
}}});
Ext.define('monitor.view.interaction.processAction.startProcess.StartProcessMixin', {moduleId:undefined, iconCls:monitor.assets.Icons.PROCESS_START, localized:{tooltip:'MONITOR_START_PROCESS'}, handler:function() {
  var me = this;
  monitor.view.interaction.processAction.startProcess.Util.openStartProcessForm(me.moduleId);
}});
Ext.define('monitor.view.interaction.processAction.startProcess.StartProcessButton', {extend:'Ext.Button', alias:'widget.monitorStartProcessBtn', mixins:['monitor.view.interaction.processAction.startProcess.StartProcessMixin'], hidden:true, initComponent:function() {
  this.hidden = !monitor.view.interaction.processAction.startProcess.Util.isAbleToStartProcess(this.moduleId);
  this.callParent();
}});
Ext.define('monitor.view.interaction.processAction.startProcess.StartProcessController', {extend:'Ext.app.ViewController', alias:'controller.startProcessController', TODAY_SYNTAX:'CURRENT_DATE', control:{'#processTypeCombo':{change:'onSelectProcessType'}, '#startProcessBnt':{click:'onCommit'}, '#startProcessItemsContainer field':{blur:'onFieldBlur', change:'onFieldChange'}}, typeMapping:{'text':'bpcTextField', 'textarea':'bpcTextArea', 'date':'bpcDateField', 'dateFromTo':'bpcDateField', 'month':'monthfield', 
'time':'timefield', 'number':'numberfield', 'spinner':'spinnerfield', 'boolean':'checkbox', 'choice':'combo', 'upload':'fileuploadfield', 'table':'bpcEditableGrid'}, monitorInstance:undefined, reloadOnChangeCmps:undefined, selectedProcess:undefined, init:function(view) {
  var me = this;
  this.reloadOnChangeCmps = [];
  me.monitorInstance = BpcCommon.Api.getModuleInstanceComponent(view.moduleId);
}, onSelectProcessType:function(combobox, value, oldValue) {
  var me = this;
  if (value !== oldValue) {
    me.selectProcess(value);
  }
}, loadFormsContent:function(container, processRecord) {
  var me = this;
  var selectedRecords = me.getViewModel().get('activeGrid.selectedRecords');
  var selectedRecordsData;
  if (selectedRecords) {
    selectedRecordsData = selectedRecords.map(function(item) {
      return item.getData();
    });
  }
  BpcCommon.Api.loadModule('forms').then(function() {
    container.add({xtype:'formsMain', flex:1, moduleId:processRecord.get('bpcFormsId'), initialData:{processStarterConfig:processRecord.getData(), records:Object.assign({}, selectedRecordsData)}});
  }, function() {
    container.add({xtype:'bpcCenterMessageCmp', flex:1, message:'Failed to load BPC Forms module'});
  });
}, selectProcess:function(processKey) {
  var me = this;
  var container = me.lookupReference('startProcessItemsContainer');
  var processStore = monitor.view.interaction.processAction.startProcess.Util.getStore(me.getView().moduleId);
  var processRecord = processStore.getById(processKey);
  if (!processRecord) {
    return;
  }
  me.reloadOnChangeCmps = [];
  me.selectedProcess = processRecord;
  var startProcessWindow = me.getStartProcessWindow();
  if (startProcessWindow) {
    startProcessWindow.setMinWidth(processRecord.get('windowWidth') || 400);
    startProcessWindow.setMinHeight(processRecord.get('windowHeight') || 200);
  }
  me.getViewModel().set('processConfig', processRecord);
  if (processRecord.get('bpcFormsId')) {
    this.loadFormsContent(container, processRecord);
  } else {
    me.createElements(processRecord);
    if (processRecord.get('dataEndpoint')) {
      if (processRecord.get('dataLoadInitial') === true) {
        me.loadFormData();
      }
      me.onFieldChangeBuffered = Ext.Function.createBuffered(me.onFieldChangeBufferedImpl, processRecord.get('changeBufferTimeInMs') || 500);
    }
  }
  me.getView().setIconCls(processRecord.get('iconCls') || 'x-fal fa-play');
  return processRecord;
}, loadRemoteParameters:function(parametersEndpoint, processRecord, container) {
  var me = this;
  var unlockFn = BpcCommon.Util.blockComponentForAsyncAction('id', me.getView().id);
  var selectedRecords = me.getViewModel().get('activeGrid.selectedRecords') || [];
  Ext.Ajax.request({url:parametersEndpoint, method:'POST', callback:unlockFn, jsonData:{records:selectedRecords.map(function(record) {
    return record.getData();
  }), process:processRecord.getData()}, headers:{ISPortalUser:BpcCommon.Api.getUserName(), 'X-BPC-ModuleId':me.getView().moduleId}, success:function(response) {
    var data = Ext.JSON.decode(response.responseText, true);
    if (data) {
      me.createElements(processRecord, data);
    } else {
      container.add({xtype:'bpcCenterMessageCmp', messageIconCls:BpcCommon.Icons.ERROR, message:BpcCommon.Api.getTranslation('MONITOR_PROCESS_START_LOAD_PARAMS_FAILED') + '\nno valid JSON'});
    }
  }, failure:function(response) {
    container.add({xtype:'bpcCenterMessageCmp', messageIconCls:BpcCommon.Icons.ERROR, message:BpcCommon.Api.getTranslation('MONITOR_PROCESS_START_LOAD_PARAMS_FAILED') + '\n                  \n' + BpcCommon.Util.getErrorFromFailureResponse(response).message});
  }});
}, createElements:function(processRecord, loadedParams) {
  var me = this;
  var items = [];
  var container = me.lookupReference('startProcessItemsContainer');
  var parameters = processRecord.get('parameters') || loadedParams;
  var parametersEndpoint = processRecord.get('parametersEndpoint');
  container.removeAll();
  if (!parameters && parametersEndpoint) {
    me.loadRemoteParameters(parametersEndpoint, processRecord, container);
    return;
  }
  Ext.each(parameters, function(parameter) {
    if (parameter.type in me.typeMapping && BpcCommon.Api.isUserAllowedToSee(parameter)) {
      var newField = me.getComponent(parameter);
      var iconContainer = {width:0};
      if (parameter.tooltip !== undefined && parameter.hideTooltipIcon !== true) {
        iconContainer = {xtype:'displayfield', tooltip:BpcCommon.Api.getTranslation(parameter.tooltip), value:'\x3ci class\x3d"fa fa-info-circle" ' + 'style\x3d"color: #003333; margin-left: 10px"\x3e\x3c/i\x3e', width:50};
      }
      var wrapContainer = {xtype:'container', layout:{type:'hbox'}, style:{marginTop:'10px'}, items:[newField, iconContainer]};
      items.push(wrapContainer);
    }
  }, this);
  container.add(items);
}, getComponent:function(parameter) {
  var me = this;
  var selectedRecords = me.getViewModel().get('activeGrid.selectedRecords');
  parameter = parameter || {};
  if (parameter.initialValueFrom && selectedRecords && selectedRecords.length > 0) {
    var value = Ext.valueFrom(selectedRecords[0].get(parameter.initialValueFrom), '');
    if (!Ext.isEmpty(value)) {
      if (parameter.type === 'date' || parameter.type === 'time') {
        value = Ext.Date.parse(value, 'c');
      } else if (parameter.type === 'boolean') {
        value = String(value) === 'true';
      }
    }
    parameter.initialValue = value;
  }
  var fieldConfig = {xtype:me.typeMapping[parameter.type], name:parameter.key, itemId:parameter.key, fieldLabel:BpcCommon.Api.getTranslation(parameter.label), width:me.fieldWidth || 400, allowBlank:!!parameter.optional, readOnly:!!parameter.readOnly, disabled:!!parameter.disabled, regex:parameter.regex ? new RegExp(parameter.regex) : null, regexText:parameter.regexText ? BpcCommon.Api.getTranslation(parameter.regexText) : null, dataLoadEventIssuer:parameter.dataLoadEventIssuer !== false, msgTarget:'side', 
  reference:'processFormInputField', cls:'processFormInputField', value:parameter.initialValue, labelWidth:me.getViewModel().get('processConfig.labelWidth') || 150, flex:1, tooltip:BpcCommon.Api.getTranslation(parameter.tooltip)};
  var newField;
  if (parameter.type === 'choice') {
    newField = me.createCombobox(parameter, fieldConfig);
  } else if (parameter.type === 'date') {
    newField = me.createDateField(parameter, fieldConfig);
  } else if (BpcCommon.Util.isStringEqual(parameter.type, 'dateFromTo')) {
    newField = me.createDateFieldFromTo(parameter, fieldConfig);
  } else if (parameter.type === 'month') {
    newField = me.createMonthField(parameter, fieldConfig);
  } else if (BpcCommon.Util.isStringEqual(parameter.type, 'table')) {
    newField = me.createTable(parameter, fieldConfig);
  } else if (parameter.type === 'boolean') {
    newField = me.createBooleanField(parameter, fieldConfig);
  } else if (parameter.type === 'time') {
    newField = me.createTimeField(parameter, fieldConfig);
  } else if (parameter.type === 'number' || parameter.type === 'spinner') {
    newField = me.createNumberField(parameter, fieldConfig);
  } else if (parameter.type === 'upload') {
    fieldConfig.name = 'fileuploadfield-' + fieldConfig.name;
    newField = fieldConfig;
  } else if (parameter.type === 'textarea') {
    fieldConfig.height = parameter.height;
    newField = fieldConfig;
  } else {
    newField = fieldConfig;
  }
  return newField;
}, createTable:function(parameter, fieldConfig) {
  var me = this;
  var columns = [{xtype:'rownumberer'}];
  var fields = [];
  var errorMsg = '';
  if (!parameter.columns || !Ext.isArray(parameter.columns)) {
    errorMsg = errorMsg + 'no columns configured\x3cbr\x3e';
  } else {
    Ext.Array.each(parameter.columns, function(columnConfig) {
      if (!columnConfig.key) {
        errorMsg = errorMsg + 'missing key in column config\x3cbr\x3e';
        return;
      }
      if (!columnConfig.type) {
        errorMsg = errorMsg + 'missing key in column config\x3cbr\x3e';
        return;
      }
      var editorCmp = me.getComponent(columnConfig);
      delete editorCmp.fieldLabel;
      var column = {text:columnConfig.label || columnConfig.key, dataIndex:columnConfig.key, formatter:'htmlEncode2', flex:1, editor:editorCmp};
      if (BpcCommon.Util.isStringEqual(columnConfig.type, 'date')) {
        fields.push({name:columnConfig.key, type:'date', dateFormat:'Y-m-d'});
        column.xtype = 'datecolumn';
        column.format = 'Y-m-d';
      } else {
        fields.push(columnConfig.key);
        if (BpcCommon.Util.isStringEqual(columnConfig.type, 'choice')) {
          column.formatter = null;
          column.renderer = function(value, record) {
            var columnConfig = record.column.getEditor();
            var store = columnConfig.getStore();
            var foundRecord = store.findRecord(columnConfig.valueField, value);
            var displayValue = foundRecord ? foundRecord.get('label') : value;
            return Ext.util.Format.htmlEncode2(displayValue);
          };
        }
      }
      Ext.applyIf(column, columnConfig);
      columns.push(column);
    });
  }
  if (errorMsg !== '') {
    return {xtype:'box', html:'Wrong configuration for table:\x3cbr\x3e' + errorMsg};
  }
  return Ext.apply(fieldConfig, {height:parameter.height || 250, columns:columns, fieldLabel:undefined, title:parameter.label, resizable:{handles:'s'}, store:{fields:fields, data:[]}});
}, createCombobox:function(parameter, fieldConfig) {
  var me = this;
  var referenceName = fieldConfig.name + 'Combo';
  var initialValue = fieldConfig.initialValue || fieldConfig.value;
  var baseUrl = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.bpmBasePath);
  var inubitVpsEndpoint = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.processStartEndpoint);
  var inubitProxyId = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.bpmProxyId);
  var comboStore = new Ext.data.JsonStore({idProperty:'key', fields:['label', 'key']});
  var combobox = Ext.apply(fieldConfig, {store:comboStore, displayField:'label', valueField:'key', triggerAction:'all', forceSelection:true, mode:'local', queryMode:'local', reference:referenceName, value:initialValue, anyMatch:!!parameter.anyMatch});
  var url = '';
  combobox.onFocusLeave = function() {
    Ext.form.field.ComboBox.prototype.onFocusLeave.apply(this, arguments);
    this.validate();
  };
  combobox.reloadRemoteStore = function(me, additionalUrlParam) {
    var processId = me.selectedProcess.getId();
    var requestData = me.buildRequestDataForCombo(processId, fieldConfig.name);
    me.loadRemoteStoreData('' + url + (additionalUrlParam ? additionalUrlParam : ''), requestData, referenceName);
  };
  if (parameter.mode && parameter.mode === 'initialRemote') {
    var processStartConfig = this.getView().processStartConfig;
    if (processStartConfig && processStartConfig.url === void 0) {
      if (!baseUrl) {
        Ext.log({msg:"Problem in start process: Couldn't find the configuration to get the data for the combo box. " + 'Please check the monitor configurations.', level:'warn'});
      } else {
        if (baseUrl && inubitVpsEndpoint) {
          if (processStartConfig.endpoint !== void 0) {
            url = '' + (BpcCommon.Api.getBackendUrl(inubitProxyId) + baseUrl + processStartConfig.endPoint);
          } else {
            url = '' + (BpcCommon.Api.getBackendUrl(inubitProxyId) + baseUrl + inubitVpsEndpoint);
          }
        } else {
          Ext.log({msg:"Problem in start process: Couldn't find the configuration to get the data for the combo box. " + 'Please add a backend url and endpoint.', level:'warn'});
        }
      }
    } else if (processStartConfig && processStartConfig.url.length) {
      url = processStartConfig.url;
    } else {
      url = '' + (BpcCommon.Api.getBackendUrl(inubitProxyId) + baseUrl + inubitVpsEndpoint);
    }
    if (url.length > 0) {
      var me$12 = this;
      combobox.emptyText = BpcCommon.Api.getTranslation('MONITOR_LOADING_LIST');
      combobox.reloadRemoteStore(me$12);
      if (parameter.reloadRemoteData) {
        combobox.reloadOnChange = true;
        me$12.reloadOnChangeCmps.push(combobox);
      }
    }
  } else {
    var translatedComboboxOptions = me.getTranslatedComboBoxOptions(parameter.options);
    comboStore.loadData(translatedComboboxOptions);
  }
  return combobox;
}, getTranslatedComboBoxOptions:function(comboBoxOptions) {
  if (!Ext.isArray(comboBoxOptions)) {
    return [];
  }
  return comboBoxOptions.map(function(option) {
    return Ext.apply(option, {label:BpcCommon.Api.getTranslation(option.label)});
  });
}, onFieldChange:function(field) {
  var me = this;
  me.markFieldForReloadTrigger(field);
  me.onFieldChangeBuffered(field);
}, onFieldChangeBuffered:Ext.emptyFn, onFieldChangeBufferedImpl:function(field) {
  var me = this;
  if (me.selectedProcess.get('dataLoadOnChange') === true && field.dataLoadEventIssuer) {
    me.loadFormData({event:'change', eventSource:field.getName()});
  }
}, markFieldForReloadTrigger:function(field) {
  field.reloadOnBlur = true;
}, onFieldBlur:function(field) {
  var me = this;
  if (field.reloadOnBlur === true) {
    field.reloadOnBlur = false;
    me.reloadRemoteStores();
  }
  if (me.selectedProcess.get('dataLoadOnBlur') === true && field.dataLoadEventIssuer) {
    me.loadFormData({event:'blur', eventSource:field.getName()});
  }
}, reloadRemoteStores:function() {
  var me = this;
  var remoteDataCmps = me.reloadOnChangeCmps;
  Ext.Array.each(remoteDataCmps, function(cmp) {
    cmp.reloadRemoteStore(me, '?reloadOnChange\x3dtrue');
  });
}, loadRemoteStoreData:function(url, requestData, referenceName) {
  var me = this;
  var actualUser = BpcCommon.Api.getUserName();
  var moduleId = this.getView().moduleId;
  Ext.Ajax.request({url:url, method:'POST', params:{request:requestData}, headers:{ISPortalUser:actualUser ? actualUser : '', 'X-BPC-ModuleId':moduleId}, scope:this, callback:function(options, success, response) {
    var responseData = '';
    if (success) {
      try {
        responseData = Ext.JSON.decode(response.responseText);
      } catch (e) {
        Ext.log({msg:'Monitor: Invalid json configured: ' + e, level:'warn'});
      }
    } else {
      responseData = 'serverNotResponse';
    }
    var combobox = me.lookupReference(referenceName);
    if (responseData === 'serverNotResponse') {
      BpcCommon.Msg.alert(BpcCommon.Api.getTranslation('MONITOR_ERROR_RETRIEVING_DATA'), BpcCommon.Api.getTranslation('MONITOR_SERVER_UNREACHABLE'));
    } else {
      if (responseData.success === true || responseData.success === 'true') {
        if (!combobox) {
          return;
        }
        combobox.getStore().loadData(responseData.data);
        combobox.enable();
        combobox.setValueOnData();
        combobox.validate();
      } else {
        var message = monitor.Util.isEmpty(responseData.message) ? 'Keine Antwort vom Server.' : responseData.message;
        BpcCommon.Msg.alert('Fehler beim Datenabruf', message);
      }
    }
    combobox.setEmptyText(' ');
  }});
}, buildRequestDataForCombo:function(processId, comboboxId) {
  var me = this;
  return me.getRequestData({process:processId, key:comboboxId, operation:'choiceList'});
}, loadFormData:function(additionalParams) {
  var me = this;
  var processId = me.selectedProcess.getId();
  var unblockFn = Ext.emptyFn;
  if (me.selectedProcess.get('dataLoadMask') !== false) {
    unblockFn = BpcCommon.Util.blockComponentForAsyncAction(me.getView().getId(), 'id');
  }
  Ext.Ajax.request({url:me.selectedProcess.get('dataEndpoint'), method:'POST', params:{request:me.getRequestData(Ext.applyIf({process:processId, operation:'loadFormData'}, additionalParams))}, headers:{ISPortalUser:BpcCommon.Api.getUserName() ? BpcCommon.Api.getUserName() : '', 'X-BPC-ModuleId':this.getView().moduleId}, scope:this, callback:function(options, success, response) {
    unblockFn();
    if (success) {
      try {
        var responseData = Ext.JSON.decode(response.responseText);
        if (responseData.data) {
          Ext.Object.each(responseData.data, function(key, value) {
            var targetField = me.getView().down('field[name\x3d' + key + ']');
            if (targetField) {
              targetField.suspendEvents();
              targetField.setValue(value);
              targetField.resumeEvents(true);
            }
          });
        }
      } catch (e) {
        Ext.log({msg:'Monitor: Invalid json configured: ' + e, level:'warn'});
      }
    } else {
      BpcCommon.Api.showNotification({type:'ERROR', text:BpcCommon.Api.getTranslation('MONITOR_PROCESS_START_LOAD_DATA_ERROR') + '\n\n' + ('' + BpcCommon.Util.getErrorFromFailureResponse(response).message)});
    }
    var startProcessWindow = me.getStartProcessWindow();
    if (startProcessWindow && Ext.isFunction(startProcessWindow.show)) {
      startProcessWindow.show();
    }
  }});
}, getRequestData:function(additionalParams) {
  var me = this;
  var processConfig = me.getViewModel().get('processConfig');
  var selectedRecords = me.getViewModel().get('activeGrid.selectedRecords');
  if (!Ext.isObject(additionalParams)) {
    additionalParams = {};
  }
  var requestData = monitor.Util.toBeginXML('root', 'atom');
  requestData += monitor.Util.toAtomXML('portletArchiveName', '');
  requestData += monitor.Util.toAtomXML('mandant', this.getTenant());
  requestData += monitor.Util.toAtomXML('gridID', this.getGridId());
  requestData += monitor.Util.toAtomXML('custom', 'true');
  requestData += monitor.Util.toAtomXML('bpcModule', 'monitor');
  requestData += monitor.Util.toAtomXML('bpcModuleInstanceId', this.getView().moduleId);
  Ext.Object.each(additionalParams, function(key, value) {
    requestData += monitor.Util.toAtomXML(key, value);
  });
  if (processConfig.get('startWithContext')) {
    if (selectedRecords) {
      requestData += monitor.Util.toBeginXML('records', 'array');
      Ext.each(selectedRecords, function(r) {
        requestData += monitor.Util.toBeginXML('item', 'tuple');
        Ext.iterate(r.data, function(key, value) {
          requestData += monitor.Util.toAtomXML(key.toLowerCase(), value);
        }, this);
        requestData += monitor.Util.toEndXML('item');
      }, this);
      requestData += monitor.Util.toEndXML('records');
    }
  }
  requestData += monitor.Util.toBeginXML('formData');
  if (this.getView().rendered === true) {
    var fields = this.getView().down('#startProcessItemsContainer').query('field');
    fields.forEach(function(field) {
      requestData += monitor.Util.toAtomXML(field.getFieldIdentifier(), field.getValue());
    });
  }
  me.getView().query('bpcEditableGrid').forEach(function(grid) {
    requestData += BpcCommon.Util.getSerializedStoreData(grid.store, 'xml', {documentRoot:grid.name, writeAllFields:true});
  });
  requestData += monitor.Util.toEndXML('formData');
  requestData += monitor.Util.toEndXML('root');
  return requestData;
}, createDateField:function(parameter, fieldConfig) {
  var initialDate = null;
  var initialValue = parameter.initialValue;
  if (initialValue !== undefined && typeof initialValue === 'string' && this.isValidRelativeDate(initialValue)) {
    initialDate = this.createRelativeDate(initialValue);
  }
  var maxValue, minValue = null;
  var paramMin = parameter.minValue;
  if (paramMin !== undefined && typeof paramMin === 'string' && this.isValidRelativeDate(paramMin)) {
    minValue = this.createRelativeDate(paramMin);
  } else if (parameter.minValue !== undefined && /\d\d\d\d-\d\d-\d\dT/i.test(parameter.minValue)) {
    minValue = new Date(parameter.minValue.split('T')[0].split('-'));
  }
  var paramMax = parameter.maxValue;
  if (paramMax !== undefined && typeof paramMax === 'string' && this.isValidRelativeDate(paramMax)) {
    maxValue = this.createRelativeDate(paramMax);
  } else if (parameter.maxValue !== undefined && /\d\d\d\d-\d\d-\d\dT/i.test(parameter.maxValue)) {
    maxValue = new Date(parameter.maxValue.split('T')[0].split('-'));
  }
  return Ext.apply(fieldConfig, {format:'d.m.Y', maxValue:maxValue, minValue:minValue, value:initialDate !== null ? initialDate : initialValue, submitFormat:'Y-m-d'});
}, createDateFieldFromTo:function(parameter, fieldConfig) {
  delete fieldConfig.fieldLabel;
  var fromConfig = Ext.apply({}, {initialValue:parameter.initialDateFrom || parameter.initialValue, name:parameter.key + '_from', bind:{maxValue:'{valueTo}', value:'{valueFrom}'}}, fieldConfig);
  var fromField = this.createDateField(parameter, fromConfig);
  var toConfig = Ext.apply({}, {initialValue:parameter.initialDateTo || parameter.initialValue, name:parameter.key + '_to', bind:{minValue:'{valueFrom}', value:'{valueTo}'}}, fieldConfig);
  var toField = this.createDateField(parameter, toConfig);
  var fieldContainer = {xtype:'fieldcontainer', layout:'hbox', defaults:{flex:1}, viewModel:{data:{valueFrom:fromField.value, valueTo:toField.value}}, fieldLabel:parameter.label, width:fieldConfig.width, items:[fromField, toField]};
  return fieldContainer;
}, createMonthField:function(parameter, fieldConfig) {
  var initialDate = null;
  var initialValue = parameter.initialValue;
  if (initialValue !== undefined && typeof initialValue === 'string' && this.isValidRelativeDate(initialValue)) {
    initialDate = this.createRelativeDate(initialValue);
  }
  var maxValue, minValue = null;
  var paramMin = parameter.minValue;
  if (paramMin !== undefined && typeof paramMin === 'string' && this.isValidRelativeDate(paramMin)) {
    minValue = this.createRelativeDate(paramMin);
  } else if (parameter.minValue !== undefined && /\d\d\d\d-\d\d-\d\dT/i.test(parameter.minValue)) {
    minValue = new Date(parameter.minValue.split('T')[0].split('-'));
  }
  var paramMax = parameter.maxValue;
  if (paramMax !== undefined && typeof paramMax === 'string' && this.isValidRelativeDate(paramMax)) {
    maxValue = this.createRelativeDate(paramMax);
  } else if (parameter.maxValue !== undefined && /\d\d\d\d-\d\d-\d\dT/i.test(parameter.maxValue)) {
    maxValue = new Date(parameter.maxValue.split('T')[0].split('-'));
  }
  if ('format' in parameter) {
    fieldConfig.format = parameter.format;
  }
  return Ext.apply(fieldConfig, {maxValue:maxValue, minValue:minValue, value:initialDate !== null ? initialDate : initialValue, submitFormat:'Y-m-d'});
}, isValidRelativeDate:function(value) {
  value = value.trim();
  if (value.indexOf(this.TODAY_SYNTAX) === -1) {
    return false;
  }
  var regExp = new RegExp('^' + this.TODAY_SYNTAX + '(\\s*[-+]\\s*[0-9]+){0,1}$');
  var result = value.match(regExp);
  if (result !== null && result.length > 0) {
    return true;
  }
  Ext.log({msg:"Problem in start process: Problem to read the relative value of date. This isn't correct configured: " + value, level:'warn'});
  return false;
}, createRelativeDate:function(valueAsString) {
  var newDate = new Date();
  var lastValueOfString = valueAsString.substring(this.TODAY_SYNTAX.length, valueAsString.length);
  if (lastValueOfString.length > 0) {
    var indexOfSign = lastValueOfString.indexOf('-') !== -1 ? lastValueOfString.indexOf('-') : lastValueOfString.indexOf('+');
    var number = parseInt(lastValueOfString.substr(indexOfSign + 1, lastValueOfString.length));
    if (lastValueOfString.indexOf('-') !== -1) {
      newDate.setDate(newDate.getDate() - number);
    } else {
      newDate.setDate(newDate.getDate() + number);
    }
  }
  return newDate;
}, createBooleanField:function(parameter, fieldConfig) {
  return Ext.apply(fieldConfig, {checked:parameter.initialValue, uncheckedValue:false, inputValue:true});
}, createTimeField:function(parameter, fieldConfig) {
  var timeIncrement = parameter.increment !== undefined ? parseInt(parameter.increment, 10) : 60;
  return Ext.apply(fieldConfig, {increment:timeIncrement, format:'H:i'});
}, createNumberField:function(parameter, fieldConfig) {
  var minValue, maxValue = '';
  if (parameter.minValue !== undefined) {
    minValue = parameter.minValue;
  }
  if (parameter.maxValue !== undefined) {
    maxValue = parameter.maxValue;
  }
  return Ext.apply(fieldConfig, {minValue:minValue, maxValue:maxValue});
}, onCommit:function() {
  var me = this;
  var process = me.selectedProcess.get('label');
  Ext.GlobalEvents.fireEvent('processStarted', {process:process});
  var unlockFn = Ext.emptyFn;
  if (this.getViewModel().get('processConfig.bpcFormsId')) {
    unlockFn = BpcCommon.Util.blockComponentForAsyncAction('startProcessBnt', 'itemId', 'Wird ausgeführt', '^ startProcessWindow');
    this.getView().down('formsMain').submitData().then(unlockFn, unlockFn);
    return;
  }
  try {
    var panel = this.lookupReference('startProcessForm');
    var form = panel.getForm();
    if (form.isValid()) {
      var values = form.getValues();
      var url = '';
      var moduleId = this.getView().moduleId;
      var baseUrl = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.bpmBasePath);
      var inubitProxyId = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.bpmProxyId);
      var inubitVpsEndpoint = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.processStartEndpoint);
      url = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/httpProxy/' + (inubitProxyId + baseUrl + inubitVpsEndpoint);
      unlockFn = BpcCommon.Util.blockComponentForAsyncAction('startProcessBnt', 'itemId', 'Wird ausgeführt', '^ startProcessWindow');
      if (!monitor.Util.isEmpty(url)) {
        if (panel.down('fileuploadfield')) {
          form.submit({url:url, params:{request:this.buildRequestData(values), ISPortalUser:BpcCommon.Api.getUserSession().data.loginName, 'X-BPC-ModuleId':moduleId}, scope:this, success:function(source, action) {
            unlockFn();
            this.handleSubmitSuccess(action.response);
          }, failure:function(source, action) {
            unlockFn();
            this.handleSubmitFailure(action.response);
          }});
        } else {
          Ext.Ajax.request({url:url, method:'POST', params:{request:this.buildRequestData(values)}, headers:{ISPortalUser:BpcCommon.Api.getUserSession().data.loginName, 'X-BPC-ModuleId':moduleId}, callback:unlockFn, scope:this, success:function(response) {
            this.handleSubmitSuccess(response);
          }, failure:function(response) {
            this.handleSubmitFailure(response);
          }});
        }
      }
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onCommit', dump:e, stack:true, level:'error'});
    unlockFn();
  }
}, handleSubmitSuccess:function(response) {
  var me = this;
  var obj = Ext.decode(response.responseText, true);
  if (obj.success && obj.success !== 'false') {
    if (obj.download) {
      try {
        BpcCommon.Util.download(obj.download.data, obj.download.fileName, obj.download.contentType);
      } catch (e) {
        BpcCommon.Api.showNotification('Fehler beim Bereitstellen des Dateidownloads.');
        me.lookupReference('startProcessBnt').enable();
        return;
      }
    }
    var notificationText = BpcCommon.Api.getTranslation('MONITOR_PROCESS_START_SUCCESSFUL');
    if (obj.msg || obj.message) {
      notificationText = notificationText + ': ' + (obj.msg || obj.message);
    }
    me.showNotification(notificationText, 'INFO');
    var checkbox = me.getView().down('#startFurtherProcess');
    if (checkbox && checkbox.getValue() !== true) {
      Ext.Function.defer(function() {
        me.getView().up('window').close();
      }, 200, this);
    }
    me.lookupReference('startProcessBnt').enable();
  } else {
    me.handleSubmitFailure(response);
  }
}, handleSubmitFailure:function(response) {
  var me = this;
  var errorMsg = BpcCommon.Util.getErrorFromFailureResponse(response).message;
  var notificationText = BpcCommon.Api.getTranslation('MONITOR_PROCESS_START_FAILED') + ': ' + errorMsg;
  me.showNotification(notificationText, 'ERROR');
  me.lookupReference('startProcessBnt').enable();
}, showNotification:function(text, type) {
  var me = this;
  var validModes = ['toast', 'silent', 'popup'];
  var selectedMode = me.selectedProcess.get('notificationDisplayMode');
  var notificationDisplayMode = selectedMode && validModes.includes(selectedMode.toLowerCase()) ? selectedMode.toLowerCase() : BpcCommon.Api.getSetting(me.getView().moduleId, monitor.assets.ConfigKeys.processNotificationDisplayMode);
  var notificationSilent = notificationDisplayMode === 'silent';
  var notificationPopUp = notificationDisplayMode === 'popup';
  var notification = {text:text, type:type, silent:notificationSilent, popUp:notificationPopUp};
  BpcCommon.Api.showNotification(notification);
}, buildRequestData:function(values) {
  var me = this;
  var processConfig = me.getViewModel().get('processConfig');
  var selectedRecords = me.getViewModel().get('activeGrid.selectedRecords');
  var requestData = monitor.Util.toBeginXML('root', 'atom');
  requestData += monitor.Util.toAtomXML('portletArchiveName', '');
  requestData += monitor.Util.toAtomXML('operation', 'startProcess');
  requestData += monitor.Util.toAtomXML('mandant', this.getTenant());
  requestData += monitor.Util.toAtomXML('gridID', this.getGridId());
  requestData += monitor.Util.toAtomXML('key', values.processType);
  requestData += monitor.Util.toAtomXML('bpcModule', 'monitor');
  requestData += monitor.Util.toAtomXML('bpcModuleInstanceId', this.getView().moduleId);
  requestData += monitor.Util.toBeginXML('data', 'array');
  requestData += monitor.Util.toBeginXML('item', 'tuple');
  var processParameters = me.selectedProcess.get('parameters');
  Ext.Object.each(values, function(key, value) {
    if (key !== 'processType') {
      if (this.getParameterConfig(processParameters, key) && this.getParameterConfig(processParameters, key).type === 'time') {
        value = value ? value + ':00' : '';
      }
      requestData += monitor.Util.toAtomXML(key, value);
    }
  }, this);
  if (processConfig.get('startWithContext')) {
    if (selectedRecords) {
      requestData += monitor.Util.toBeginXML('records', 'array');
      Ext.each(selectedRecords, function(r) {
        requestData += monitor.Util.toBeginXML('item', 'tuple');
        Ext.iterate(r.data, function(key, value) {
          requestData += monitor.Util.toAtomXML(key.toLowerCase(), value);
        }, this);
        requestData += monitor.Util.toEndXML('item');
      }, this);
      requestData += monitor.Util.toEndXML('records');
    }
  }
  me.getView().query('bpcEditableGrid').forEach(function(grid) {
    requestData += BpcCommon.Util.getSerializedStoreData(grid.store, 'xml', {documentRoot:grid.name, writeAllFields:true});
  });
  requestData += monitor.Util.toEndXML('item');
  requestData += monitor.Util.toEndXML('data');
  requestData += monitor.Util.toEndXML('root');
  return requestData;
}, getTenant:function() {
  var tenant = '';
  var pmMandant = BpcCommon.Api.getSetting(this.getView().moduleId, 'inubit_pmMandant');
  if (!monitor.Util.isEmpty(pmMandant)) {
    tenant = pmMandant;
  }
  return tenant;
}, getGridId:function() {
  var gridId = '';
  var readGrid = BpcCommon.Api.getSetting(this.getView().moduleId, monitor.assets.ConfigKeys.legacyPmGridId);
  if (!monitor.Util.isEmpty(readGrid)) {
    gridId = readGrid;
  } else {
    Ext.log({msg:"Problem in start process: Couldn't read the configuration 'db_gridId'. " + 'Please configure this in your monitor. ', level:'warn'});
  }
  return gridId;
}, getParameterConfig:function(config, key) {
  var requiredConfig = null;
  Ext.each(config, function(parameter) {
    if (parameter.key === key) {
      requiredConfig = parameter;
      return false;
    }
  });
  return requiredConfig;
}, getStartProcessWindow:function() {
  var me = this;
  var view = me.getView();
  if (!Ext.isFunction(view.up)) {
    return null;
  }
  return view.up('bpcWindow');
}});
Ext.define('monitor.view.interaction.processAction.startProcess.StartProcessPanel', {extend:'Ext.Panel', alias:'widget.monitorStartProcessPanel', layout:'fit', controller:'startProcessController', bind:{title:'{MONITOR_START_PROCESS:translate} - {processConfig.label:translate}'}, viewModel:{data:{processConfig:{label:'', key:undefined, hideFormValidator:false}, MONITOR_START_PROCESS:'MONITOR_START_PROCESS', inWindow:false, translate:''}}, iconCls:'x-fal fa-play', bodyPadding:10, preselectedProcess:undefined, 
configuration:undefined, records:undefined, listeners:{added:function(me) {
  me.getViewModel().set('inWindow', !!me.up('window'));
}}, initComponent:function() {
  var me = this;
  me.processStartConfig = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.processStartConfig);
  if (!me.processStartConfig || !me.processStartConfig.processes || me.processStartConfig.processes.length === 0) {
    me.items = [{xtype:'bpcCenterMessageCmp', message:'MONITOR_ERROR_PROCESS_START'}];
    me.callParent(arguments);
    return;
  }
  if (me.preselectedProcess instanceof Ext.data.Model) {
    me.preselectedProcess = me.preselectedProcess.getData();
  }
  var preselectedProcessKey = undefined;
  if (me.preselectedProcess && me.preselectedProcess.key) {
    preselectedProcessKey = me.preselectedProcess.key;
  }
  me.items = [{xtype:'form', reference:'startProcessForm', itemId:'startProcessForm', layout:{align:'stretch', type:'vbox'}, plugins:[{ptype:'datatip'}, {ptype:'bpcFieldValidator', validationButtonSelector:'#formChecker'}], scrollable:true, items:[{xtype:'bpcMonitorStartProcessPicker', moduleId:me.moduleId, hidden:preselectedProcessKey ? true : false, preselectedProcessKey:preselectedProcessKey}, {xtype:'displayfield', hidden:true, localized:{fieldLabel:'CORE_DESCRIPTION'}, bind:{hidden:'{!processConfig.description}', 
  value:'{processConfig.description:translate}'}}, {xtype:'fieldset', flex:1, reference:'startProcessItemsContainer', itemId:'startProcessItemsContainer', layout:{type:'vbox', align:'stretch'}, localized:{title:'MONITOR_PROCESSSTART_PARAMETERS_TITLE'}, bind:{hidden:'{!processConfig.key}'}, scrollable:true, hidden:true, items:[]}], buttons:[{xtype:'displayfield', boxLabel:BpcCommon.Api.getTranslation('MONITOR_START_WITH_CONTEXT'), hidden:true, bind:{hidden:'{!processConfig.startWithContext}', value:"\x3cspan class\x3d'x-fal fa-list-ul'\x3e\x3c/span\x3e {translate:translate('MONITOR_PROCESSES')} : {activeGrid.selectedRecords.length}"}}, 
  {iconCls:BpcCommon.Icons.CHECKLIST, ui:'secondary', itemId:'formChecker', hidden:true, bind:{hidden:'{processConfig.hideFormValidator}'}}, '-\x3e', {xtype:'checkbox', boxLabel:BpcCommon.Api.getTranslation('MONITOR_KEEP_WINDOW_OPENED_AFTER_PROCESS_START'), labelWidth:250, itemId:'startFurtherProcess', submitValue:false, hidden:true, bind:{hidden:'{!inWindow}'}}, {text:BpcCommon.Api.getTranslation('MONITOR_CANCEL'), itemId:'cancelBtn', iconCls:BpcCommon.Icons.CANCEL, ui:'secondary', handler:function(button) {
    button.up('window').close();
  }, style:{marginLeft:'10px'}, hidden:true, bind:{hidden:'{!inWindow}'}}, {text:BpcCommon.Api.getTranslation('MONITOR_START_PROCESS'), iconCls:BpcCommon.Icons.OK, ui:'primary', itemId:'startProcessBnt', reference:'startProcessBnt', formBind:true}]}];
  me.callParent(arguments);
}});
Ext.define('monitor.view.interaction.processAction.startProcess.StartProcessPicker', {alias:'widget.bpcMonitorStartProcessPicker', extend:'BpcCommon.component.form.field.ComboBox', moduleId:undefined, preselectedProcessKey:undefined, itemId:'processTypeCombo', forceSelection:true, fieldLabel:BpcCommon.Api.getTranslation('MONITOR_PROCESS_TYPE') + ':', emptyText:BpcCommon.Api.getTranslation('MONITOR_CHOOSE_PROCESS_TYPE'), editable:false, queryMode:'local', displayField:'label', valueField:'key', allowBlank:false, 
name:'processType', initComponent:function() {
  var me = this;
  var processStarterStore = monitor.view.interaction.processAction.startProcess.Util.getStore(me.moduleId);
  processStarterStore.each(function(el) {
    el.set('label', BpcCommon.Api.getTranslation(el.get('label')));
  });
  me.store = processStarterStore;
  me.on('afterrender', me.onAfterRender, me);
  me.callParent();
}, onAfterRender:function(combo) {
  var me = this;
  if (combo && combo.getStore().getCount() === 1) {
    combo.setValue(combo.getStore().first());
  } else if (me.preselectedProcessKey) {
    combo.setValue(me.preselectedProcessKey);
  }
}});
Ext.define('monitor.view.interaction.processAction.startProcess.Util', {singleton:true, alternateClassName:['monitor.StartProcessUtil'], instanceCacheMap:{docuDummy:{store:undefined}}, initInstanceCache:function(moduleId) {
  var me = this;
  if (me.instanceCacheMap[moduleId]) {
    return;
  }
  me.instanceCacheMap[moduleId] = {store:undefined};
}, getStore:function(moduleId) {
  var me = this;
  me.initInstanceCache(moduleId);
  if (me.instanceCacheMap[moduleId].store) {
    return me.instanceCacheMap[moduleId].store;
  }
  var processStartConfig = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.processStartConfig);
  var processes = [];
  if (processStartConfig && Ext.isArray(processStartConfig.processes)) {
    processes = processStartConfig.processes;
  }
  me.instanceCacheMap[moduleId].store = Ext.create('Ext.data.JsonStore', {model:'monitor.model.Starter', filters:[BpcCommon.Api.isUserAllowedToSee, function(record) {
    return record.get('hidden') !== true;
  }], data:processes});
  return me.instanceCacheMap[moduleId].store;
}, openStartProcessForm:function(moduleId, preselectedProcess) {
  Ext.create({xtype:'bpcWindow', width:document.body.clientWidth * 0.4, minWidth:800, height:document.body.clientHeight * 0.6, stateId:moduleId + '_monitorStartProcessWindow', items:[{xtype:'monitorStartProcessPanel', moduleId:moduleId, preselectedProcess:preselectedProcess, viewModel:{parent:Ext.first('bpcMonitor:visible') ? Ext.first('bpcMonitor:visible').getViewModel() : undefined}}]});
}, isAbleToStartProcess:function(moduleId) {
  var me = this;
  var isStartProcessActive = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.processStart);
  var hasUserRight = BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.startProcess) || BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.startProcess + '_' + moduleId);
  var selectableProcesses = me.getStore(moduleId);
  return isStartProcessActive && hasUserRight && selectableProcesses.getCount() > 0;
}, getProcessesWithContext:function(moduleId) {
  var spConfig = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.processStartConfig);
  if (spConfig && spConfig.processes && Ext.isArray(spConfig.processes)) {
    return spConfig.processes.filter(function(processConfig) {
      return processConfig.startWithContext === true && BpcCommon.Api.isUserAllowedToSee(processConfig);
    });
  }
  return [];
}});
Ext.define('monitor.view.interaction.refresh.Button', {extend:'Ext.button.Split', alias:'widget.monitorRefreshBtn', localized:{tooltip:'MONITOR_REFRESH'}, config:{switchAutoUpdate:true}, publishes:['switchAutoUpdate'], bind:{iconCls:'{setting.' + monitor.assets.ConfigKeys.autoUpdate + " ? '" + monitor.assets.Icons.REFRESH + " bpcPlay' : '" + monitor.assets.Icons.REFRESH + " bpcPause'}", arrowVisible:'{switchAutoUpdate}'}, initComponent:function() {
  var autoUpdate = this.lookupViewModel().get('setting.' + monitor.assets.ConfigKeys.autoUpdate);
  var autoUpdateStateId = this.moduleId + '_autoUpdate';
  this.iconCls = autoUpdate ? monitor.assets.Icons.REFRESH + ' bpcPlay' : monitor.assets.Icons.REFRESH + ' bpcPause';
  this.menu = {hidden:true, bind:{hidden:'{!switchAutoUpdate}'}, items:[{xtype:'menucheckitem', hideOnClick:true, stateful:{checked:true}, stateEvents:['checkchange'], stateId:autoUpdateStateId, clickHideDelay:50, localized:{text:'MONITOR_AUTOMATIC_REFRESH'}, checked:autoUpdate, bind:{checked:'{setting.' + monitor.assets.ConfigKeys.autoUpdate + '}'}}]};
  this.callParent();
}, handler:function() {
  var me = this;
  try {
    var grid = me.lookupViewModel().get('activeGrid');
    grid.getController().refreshData();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR refreshData', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.moduleheader.MonitorInfo', {extend:'Ext.container.Container', alias:'widget.bpcMonitorMeta', layout:{type:'hbox', pack:'end', align:'center'}, controller:'monitorinfocontroller', config:{showCount:true, showFilter:true, showSorter:true, showInfoIcon:false, showSelectionCounter:false}, publishes:['showCount', 'showFilter', 'showSorter', 'showInfoIcon', 'showSelectionCounter'], viewModel:{data:{translate:''}, formulas:{moduleHeaderInfos:{get:function(get) {
  var itemInt = get('itemInt');
  var filterInt = get('filterInt');
  var sortInt = get('sortInt');
  var itemCount = Ext.util.Format.number(itemInt, '0,000');
  var countLimit = BpcCommon.Api.getSetting('monitor', monitor.assets.ConfigKeys.dataCountLimit);
  if (itemInt >= countLimit) {
    itemCount = '\x3e' + itemCount;
  }
  var hideDatasets = false;
  var infos = '' + ("\x3c/div\x3e\x3cdiv class\x3d'metaInfoItem' data-testid\x3d'metaFilterInfo'\x3e" + '\x3cspan class\x3d\'metaInfoLabel\'\x3e\x3ci class\x3d"fal fa-filter monitorInfoIcons"\x3e\x3c/i\x3e') + BpcCommon.Api.getTranslation('MONITOR_FILTER') + ':\x3c/span\x3e' + ("\x3cspan class\x3d'metaInfo'\x3e " + filterInt + '\x3c/span\x3e') + "\x3c/div\x3e\x3cdiv class\x3d'metaInfoItem'\x3e" + ('\x3cspan class\x3d\'metaInfoLabel\'\x3e\x3ci class\x3d"fal fa-sort-amount-down monitorInfoIcons"\x3e\x3c/i\x3e' + 
  BpcCommon.Api.getTranslation('MONITOR_SORTING') + ':\x3c/span\x3e') + ("\x3cspan class\x3d'metaInfo'\x3e " + sortInt + '\x3c/span\x3e') + '\x3c/div\x3e';
  if (hideDatasets === false) {
    infos = '' + ("\x3cdiv class\x3d'metaInfoItem'\x3e" + '\x3cspan class\x3d\'metaInfoLabel\'\x3e\x3ci class\x3d"fal fa-table monitorInfoIcons"\x3e\x3c/i\x3e') + BpcCommon.Api.getTranslation('MONITOR_DATASETS') + ':\x3c/span\x3e' + ("\x3cspan class\x3d'metaInfo'\x3e " + String(itemCount) + infos + '\x3c/span\x3e');
  }
  return infos;
}}}}, initComponent:function() {
  this.itemCounterCmp = Ext.create({xtype:'button', isMonitorInfoButton:true, ui:'noButton', itemId:'dataCounter', iconCls:'x-fal fa-sigma', hidden:true, bind:{hidden:'{!showCount}'}});
  this.items = [this.itemCounterCmp, {xtype:'button', isMonitorInfoButton:true, ui:'noButton', iconCls:'fal fa-ballot-check', hidden:true, bind:{hidden:'{!showSelectionCounter}', text:"{translate:translate('MONITOR_SELECTED_DATA_SETS')}: {selectionCount}"}}, {xtype:'button', isMonitorInfoButton:true, ui:'noButton', iconCls:'x-fal fa-filter', hidden:true, bind:{hidden:'{!showFilter}', text:"{translate:translate('MONITOR_FILTER')}: {filterInt}"}}, {xtype:'button', isMonitorInfoButton:true, ui:'noButton', 
  iconCls:'x-fal fa-sort-amount-down', hidden:true, bind:{hidden:'{!showSorter}', text:"{translate:translate('MONITOR_SORTING')}: {sortInt}"}}, {xtype:'button', ui:'noButton', iconCls:'x-fal fa-info', hidden:true, bind:{hidden:'{!showInfoIcon}', tooltip:'{moduleHeaderInfos}'}}];
  this.moduleInstance.on('dataLengthUpdated', this.updateItemCounterCmp, this);
  this.moduleInstance.on('tooltipChange', this.createTooltip, this);
  this.callParent();
}, updateItemCounterCmp:function(itemCount) {
  if (this.itemCounterCmp) {
    this.itemCounterCmp.setText(BpcCommon.Api.getTranslation('MONITOR_DATASETS') + ': ' + itemCount);
  }
}, createTooltip:function(grid) {
  var me = this;
  var filters = grid.store.filters.items;
  var sorters = grid.store.sorters.items;
  var filterString = this.createFilterString(filters);
  var sorterString = this.createSorterString(sorters);
  var tooltipString = filterString + sorterString;
  var buttonList = Ext.first('bpcMonitorMeta').query('[isMonitorInfoButton\x3dtrue]');
  Ext.each(buttonList, function(button) {
    button.setTooltip(tooltipString);
  }, me);
}, createFilterString:function(filters) {
  var me = this;
  var globalFilterValue = me.moduleInstance.getViewModel().get('globalFilterValue');
  try {
    var filterStr = '';
    var globalFilterStr = '';
    var filterIcon = '\x3ci class\x3d"fal fa-filter monitorInfoIcons"\x3e\x3c/i\x3e';
    if (globalFilterValue && globalFilterValue.length) {
      var filterText = BpcCommon.Api.getTranslation('MONITOR_SET_VOLLTEXTSUCHE');
      var filterValue = ': \x3ci\x3e' + globalFilterValue + '\x3c/i\x3e';
      globalFilterStr = filterIcon + filterText + filterValue + '\x3cbr\x3e';
    }
    if (filters.length > 0) {
      Ext.each(filters, function(filter) {
        var filterText = me.moduleInstance.getColumnAttributeMatch('dataIndex', filter.getProperty(), 'text');
        var filterValue = ': \x3ci\x3e' + filter.getValue() + '\x3c/i\x3e';
        filterStr += filterIcon + filterText + filterValue + '\x3cbr\x3e';
      }, me);
      if (globalFilterStr !== '') {
        filterStr += globalFilterStr;
      }
    } else if (filters.length === 0 && globalFilterValue && globalFilterValue.length) {
      filterStr += globalFilterStr;
    }
    return filters.length + ' ' + BpcCommon.Api.getTranslation('MONITOR_FILTER') + '\x3cbr\x3e' + filterStr;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createFilterString', dump:e, stack:true, level:'error'});
  }
}, createSorterString:function(sorters) {
  try {
    var me = this;
    var sortersStr = '';
    if (sorters.length > 0) {
      Ext.each(sorters, function(sorter) {
        var textA;
        if (me.moduleInstance.columnLocalizedStrings) {
          textA = me.moduleInstance.setSorterStringIfFilterisSet(sorter) || me.moduleInstance.getLocalizedStringModuleHeader(sorter);
        } else {
          textA = me.moduleInstance.getLocalizedStringModuleHeader(sorter);
        }
        var sortDirection = sorter['_direction'] === 'ASC' ? '\x3ci class\x3d"fas fa-long-arrow-alt-up monitorInfoIcons"\x3e\x3c/i\x3e' : sorter['_direction'] === 'DESC' ? '\x3ci class\x3d"fas fa-long-arrow-alt-down monitorInfoIcons"\x3e\x3c/i\x3e' : '';
        sortersStr += sortDirection + textA + '\x3cbr\x3e';
      }, me);
    }
    return sorters.length + ' ' + BpcCommon.Api.getTranslation('MONITOR_SORTING') + '\x3cbr\x3e' + sortersStr;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createSorterString', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.moduleheader.MonitorInfoController', {extend:'Ext.app.ViewController', alias:'controller.monitorinfocontroller', refreshData:function() {
  Ext.log(this.self.getName() + ': refreshData');
  try {
    var grids = this.getView().moduleInstance.getGridList();
    Ext.Array.each(grids, function(grid) {
      grid.getController().refreshData();
    });
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR refreshData', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.state.FavoriteToolbar', {extend:'Ext.toolbar.Toolbar', alias:'widget.monitorFavoriteToolbar', cls:'favourite-toolbar', viewModel:true, overflowHandler:'menu', monitorStateViewStore:undefined, favoriteViewsStore:undefined, moduleId:undefined, defaults:{enableToggle:true, ui:'toolbarWithBorder', margin:'0 5 0  0'}, initComponent:function() {
  var me = this;
  var vm = this.getViewModel();
  me.monitorStateViewStore = monitor.Module.monitorViewStatesController.getMonitorViewStateStore(me.moduleId);
  me.callParent(arguments);
  me.updateComponent();
  me.mon(me.monitorStateViewStore, 'datachanged', me.updateComponent, me);
  vm.bind('{activeState}', me.checkToggleState.bind(me), me, {deep:true});
}, updateComponent:function() {
  var me = this;
  me.removeAll();
  me.monitorStateViewStore.each(function(stateObj) {
    if (stateObj.get('favorite')) {
      me.add({xtype:'button', cls:'favourite-toolbar-item', iconCls:stateObj.get('iconCls'), text:stateObj.get('name'), tooltip:stateObj.get('tooltip'), defaultTooltip:stateObj.get('tooltip'), state:stateObj, moduleId:me.moduleId, handler:function(btn) {
        if (!btn.pressed) {
          if (stateObj.isDirty()) {
            monitor.Module.monitorViewStatesController.resetDirtyChangesFromState(me.moduleId, stateObj.get('id'));
          } else {
            monitor.Module.monitorViewStatesController.resetState(me.moduleId);
          }
        } else {
          monitor.Module.monitorViewStatesController.applyState(me.moduleId, stateObj);
        }
      }});
    }
  });
  me.checkToggleState();
}, checkToggleState:function() {
  var me = this;
  var vm = this.getViewModel();
  var activeState = vm.get('activeState');
  var activeStateId = activeState ? activeState.get('id') : null;
  me.query('button').forEach(function(btn) {
    btn.setPressed(btn.state.get('id') === activeStateId);
    if (btn.state.isDirty()) {
      btn.addCls('activeStateDirty');
      if (btn.defaultTooltip) {
        btn.setTooltip('\x3ci\x3e' + BpcCommon.Api.getTranslation('MONITOR_VIEW_DIRTY') + '\x3c/i\x3e \x3cbr/\x3e ' + btn.defaultTooltip);
      } else {
        btn.setTooltip('\x3ci\x3e' + BpcCommon.Api.getTranslation('MONITOR_VIEW_DIRTY') + '\x3c/i\x3e');
      }
    } else {
      btn.removeCls('activeStateDirty');
      btn.setTooltip(btn.defaultTooltip || '');
    }
  });
}});
Ext.define('monitor.view.state.NewStatePanel', {extend:'Ext.panel.Panel', alias:'widget.monitorNewStatePanel', layout:{type:'vbox', align:'stretch'}, viewModel:{data:{isFormValid:true, personal:false}}, header:false, moduleId:undefined, bodyPadding:10, scrollable:true, defaults:{labelWidth:150}, items:[{xtype:'bpcTextField', itemId:'stateName', localized:{fieldLabel:'CORE_NAME'}, listeners:{change:function() {
  this.lookupViewModel().set('isFormValid', true);
}}}, {xtype:'bpcIconPicker', itemId:'stateIconCls', localized:{fieldLabel:'MONITOR_VIEW_ICONCLS'}, value:'x-fal fa-list-alt', listeners:{change:function() {
  this.lookupViewModel().set('isFormValid', true);
}}}, {xtype:'bpcTextField', itemId:'stateTooltip', localized:{fieldLabel:'MONITOR_VIEW_TOOLTIP'}, listeners:{change:function() {
  this.lookupViewModel().set('isFormValid', true);
}}}, {xtype:'checkbox', itemId:'stateFavorite', value:true, localized:{fieldLabel:'MONITOR_VIEW_FAVORITE'}}, {xtype:'label', html:'\x3cspan class\x3d"' + BpcCommon.Icons.WARNING + ' warningColor"\x3e\x3c/span\x3e ' + BpcCommon.Api.getTranslation('MONITOR_NEW_VIEW_VALIDATION'), bind:{hidden:'{isFormValid}'}}, {xtype:'monitorAccessControlFieldSet'}], buttons:[{iconCls:BpcCommon.Icons.CANCEL, ui:'secondary', localized:{text:'CORE_CANCEL'}, handler:function() {
  this.up('window').close();
}}, {iconCls:BpcCommon.Icons.ADD, ui:'primary', localized:{text:'MONITOR_SAVE_VIEW'}, handler:function() {
  var monitorNewStatePanel = this.up('monitorNewStatePanel');
  var moduleId = monitorNewStatePanel.moduleId;
  var state = monitor.Module.monitorViewStatesController.getCurrentMonitorViewState(moduleId);
  state.id = String(Date.now());
  state.iconCls = monitorNewStatePanel.down('#stateIconCls').getValue();
  state.name = monitorNewStatePanel.down('#stateName').getValue();
  state.favorite = monitorNewStatePanel.down('#stateFavorite').getValue();
  state.tooltip = monitorNewStatePanel.down('#stateTooltip').getValue();
  if (!state.iconCls && !state.name) {
    this.lookupViewModel().set('isFormValid', false);
    return;
  }
  var stateModel = monitor.model.MonitorViewState.create(state);
  if (monitorNewStatePanel.down('#personalView').getValue() === true) {
    stateModel.set('readUser', BpcCommon.Api.getUserName());
    stateModel.set('writeUser', BpcCommon.Api.getUserName());
  } else {
    stateModel.set('readOrganisations', monitorNewStatePanel.down('#readOrganisations').getValue());
    stateModel.set('writeOrganisations', monitorNewStatePanel.down('#writeOrganisations').getValue());
    stateModel.set('readRoles', monitorNewStatePanel.down('#readRoles').getValue());
    stateModel.set('writeRoles', monitorNewStatePanel.down('#writeRoles').getValue());
    stateModel.set('readRights', monitorNewStatePanel.down('#readRights').getValue());
    stateModel.set('writeRights', monitorNewStatePanel.down('#writeRights').getValue());
  }
  var addedState = monitor.Module.monitorViewStatesController.addMonitorViewState(moduleId, stateModel);
  var vm = BpcCommon.Api.getModuleInstanceComponent(moduleId).getViewModel();
  vm.set('activeState', addedState);
  this.up('window').close();
}}]});
Ext.define('monitor.view.state.StateToolbarItem', {extend:'Ext.button.Button', alias:'widget.monitorStateToolbarItem', controller:'monitorStateToolbarItemController', moduleInstance:undefined, hidden:true, bind:{hidden:'{!setting.' + monitor.assets.ConfigKeys.customViewsEnabled + '}'}, viewModel:{data:{translate:''}}, iconCls:monitor.assets.Icons.VIEW, localized:{tooltip:'MONITOR_VIEWS'}, menu:{plain:true, items:[{localized:{text:'MONITOR_REMOVE_FILTER'}, iconCls:'x-fal fa-times-circle', handler:'removeAllFilter'}, 
{localized:{text:'MONITOR_REMOVE_SORTERS'}, iconCls:'x-fal fa-ban', handler:'removeSorter'}, {xtype:'menuseparator'}, {localized:{text:'MONITOR_SAVE_CURRENT_VIEW'}, iconCls:BpcCommon.Icons.ADD, disabled:false, handler:'saveCurrentState', bind:{hidden:'{!' + monitor.assets.ModuleRights.editMonitorViews + '}'}}, {localized:{text:'MONITOR_RESET_VIEW'}, itemId:'resetView', iconCls:BpcCommon.Icons.RESET, handler:'resetState'}, {localized:{text:'MONITOR_MANAGE_VIEWS'}, bind:{hidden:'{!' + monitor.assets.ModuleRights.editMonitorViews + 
'}'}, iconCls:BpcCommon.Icons.EDIT, handler:'showStateManager'}, {xtype:'menuseparator', bind:{hidden:'{!activeState}'}}, {plain:'true', bind:{text:"{translate:translate('MONITOR_ACTIVE_VIEW')}: " + '\x3cspan class\x3d"{activeState.iconCls}"\x3e \x3c/span\x3e {activeState.name:translate}', hidden:'{!activeState}'}, style:'text-align: center; text-decoration: underline;'}, {bind:{text:"{translate:translate('MONITOR_SAVE_VIEW_AS')}", hidden:'{!' + monitor.assets.ModuleRights.editMonitorViews + ' || !activeState.dirty}'}, 
hidden:true, iconCls:BpcCommon.Icons.SAVE, handler:'updateCurrentState'}, {bind:{text:"{translate:translate('MONITOR_DISCARD_CHANGES')}", hidden:'{!activeState.dirty}'}, hidden:true, iconCls:BpcCommon.Icons.RESET, handler:'resetCurrentStateChanges'}, {xtype:'menuseparator'}, {localized:{text:'MONITOR_NO_FILTER_SAVED'}, itemId:'filterListPlaceholder', disabled:true, cls:'bpc-menu-item-text-default'}]}, initComponent:function() {
  var me = this;
  var hasUserRight = BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.editMonitorViews) || BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.editMonitorViews + '_' + me.moduleId);
  me.getViewModel().set(monitor.assets.ModuleRights.editMonitorViews, hasUserRight);
  me.callParent();
}});
Ext.define('monitor.view.state.StateToolbarItemController', {extend:'Ext.app.ViewController', alias:'controller.monitorStateToolbarItemController', requires:[], init:function() {
  try {
    var me = this;
    me.moduleId = me.getView().moduleId;
    me.monitorStateViewStore = monitor.Module.monitorViewStatesController.getMonitorViewStateStore(me.moduleId);
    me.mon(me.monitorStateViewStore, 'datachanged', me.updateViewList, me);
    me.getViewModel().bind('{activeState}', me.updateViewList, me, {deep:true});
    me.updateViewList();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initBind', dump:e, stack:true, level:'error'});
  }
}, updateViewList:function() {
  try {
    var me = this;
    var targetMenu = me.getView().getMenu();
    var activeStateId = me.getViewModel().get('activeState.id');
    var refItem = targetMenu.down('#filterListPlaceholder');
    if (me.monitorStateViewStore.getCount() > 0) {
      me.getView().query('[viewFavorite\x3dtrue]').forEach(function(item) {
        return item.destroy();
      });
      refItem.hide();
      var insertIndex = targetMenu.items.indexOf(refItem) + 1;
      me.monitorStateViewStore.each(function(state) {
        var text = state.get('name') || '\x26nbsp;';
        if (activeStateId === state.get('id')) {
          text = '\x3cb\x3e' + text + '\x3c/b\x3e';
        }
        var stateObj = {text:text, iconCls:state.get('iconCls'), tooltip:'\x3ci\x3e' + BpcCommon.Api.getTranslation('MONITOR_SAVED_VIEW') + '\x3c/i\x3e\x3cbr\x3e\n                                     \x3ci class\x3d"' + state.get('iconCls') + '"\x3e\x3c/i\x3e\x3cspan\x3e ' + state.get('name') + '\x3c/span\x3e', handler:function() {
          monitor.Module.monitorViewStatesController.applyState(me.moduleId, state);
        }, stateId:state.get('id'), viewFavorite:true};
        targetMenu.insert(insertIndex, stateObj);
        insertIndex++;
      }, this);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateViewList', dump:e, stack:true, level:'error'});
  }
}, saveCurrentState:function() {
  Ext.create({width:1068, height:660, xtype:'bpcWindow', iconCls:BpcCommon.Icons.ADD, header:true, modal:true, closable:true, localized:{title:'MONITOR_SAVE_VIEW'}, items:[{xtype:'monitorNewStatePanel', moduleId:this.moduleId}]}).show();
}, showStateManager:function() {
  var me = this;
  Ext.create({xtype:'bpcWindow', header:true, modal:true, closable:true, localized:{title:'MONITOR_MANAGE_VIEWS'}, iconCls:monitor.assets.Icons.VIEW, items:[{xtype:'monitorStateConfigurationPanel', moduleId:me.moduleId, showCloseBtn:true}]}).show();
}, updateCurrentState:function() {
  var me = this;
  var vm = me.getViewModel();
  var state = monitor.Module.monitorViewStatesController.getCurrentMonitorViewState(me.moduleId);
  monitor.Module.monitorViewStatesController.updateMonitorViewState(me.moduleId, vm.get('activeState.id'), state, true);
}, resetCurrentStateChanges:function() {
  var me = this;
  var vm = me.getViewModel();
  monitor.Module.monitorViewStatesController.resetDirtyChangesFromState(me.moduleId, vm.get('activeState.id'));
}, removeAllFilter:function() {
  var me = this;
  me.getViewModel().get('activeGrid').getController().removeAllFilter();
}, removeSorter:function() {
  var me = this;
  me.getViewModel().get('activeGrid').getController().removeSorter();
}, resetState:function() {
  var me = this;
  monitor.Module.monitorViewStatesController.resetState(me.moduleId);
}});
Ext.define('monitor.view.state.manager.ConfigurationController', {extend:'Ext.app.ViewController', alias:'controller.monitorStateConfigurationController', control:{'#stateGrid':{select:'onStateSelect', afterrender:{fn:function(cmp) {
  if (cmp.getSelection().length === 0) {
    this.selectFirstRecord();
  }
  cmp.mon(cmp.store, 'datachanged', this.checkSelection, this);
}, delay:100}}}, init:function() {
  var me = this;
  me.moduleId = me.getView().moduleId;
  me.stateViews = new Ext.util.HashMap();
}, onStateSelect:function(grid, record) {
  try {
    var container = this.getView().down('#stateObjectViewContainer');
    if (this.stateViews.containsKey(record.getId())) {
      container.setActiveItem(this.stateViews.get(record.getId()));
    } else {
      var stateView = {xtype:'monitorStateView', stateObject:record, moduleId:this.getView().moduleId};
      stateView = container.add(stateView);
      this.stateViews.add(record.getId(), stateView);
      container.setActiveItem(stateView);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onStateSelect', dump:e, stack:true, level:'error'});
  }
}, closeWindow:function() {
  var me = this;
  var window = me.getView().up('window');
  if (window) {
    window.close();
  }
}, reset:function() {
  var me = this;
  var view = me.getView();
  var stateGrid = view.down('#stateGrid');
  var store = stateGrid.getStore();
  store.rejectChanges();
  me.setDirty(false);
  stateGrid.setSelection(store.first());
}, 'delete':function(record) {
  var me = this;
  var stateStore = me.getView().down('monitorStateConfigurationGrid').store;
  this.setDirty(true);
  stateStore.remove(record);
}, save:function() {
  var me = this;
  monitor.Module.monitorViewStatesController.saveMonitorViewStates(me.moduleId).then(function() {
    me.setDirty(false);
  });
}, setDirty:function(dirty) {
  var me = this;
  var vm = me.getViewModel();
  vm.set('unsavedChanges', dirty === true);
}, applyState:function() {
  var me = this;
  var stateGrid = me.getView().down('#stateGrid');
  var selection = stateGrid.getSelection();
  monitor.Module.monitorViewStatesController.applyState(me.moduleId, selection[0]);
}, selectFirstRecord:function() {
  var me = this;
  var grid = me.getView().down('monitorStateConfigurationGrid');
  if (grid.getStore().getCount() > 0) {
    grid.getSelectionModel().select(0);
  } else {
    me.getView().down('#stateObjectViewContainer').setActiveItem(0);
  }
}, checkSelection:function() {
  var me = this;
  var grid = me.getView().down('monitorStateConfigurationGrid');
  var selection = grid.getSelection();
  if (!selection || selection.length === 0) {
    me.selectFirstRecord();
  }
}});
Ext.define('monitor.view.state.manager.ConfigurationGrid', {extend:'Ext.grid.Panel', alias:'widget.monitorStateConfigurationGrid', viewModel:true, rowViewModel:{}, viewConfig:{plugins:{ptype:'gridviewdragdrop'}}, columns:[{xtype:'bpcHandleColumn'}, {localized:{text:'MONITOR_VIEW'}, menuDisabled:true, sortable:false, dataIndex:'name', renderer:function(value, metaData, record) {
  return '\x3cspan class\x3d"' + record.get('iconCls') + '" aria-hidden\x3d"true" \x3e\x3c/span\x3e ' + BpcCommon.Api.getTranslation(value);
}, flex:3}, {localized:{text:'MONITOR_FAVORITE'}, menuDisabled:true, sortable:false, dataIndex:'favorite', flex:1, xtype:'checkcolumn', bind:{disabled:'{!global.right.' + monitor.assets.ModuleRights.editMonitorViews + '}'}}, {xtype:'widgetcolumn', menuDisabled:true, sortable:false, hidden:false, hideable:false, width:40, minWidth:40, align:'end', widget:{xtype:'toolbar', itemId:'monitorViewStateActions', overflowHandler:'menu', defaultButtonUI:'widgetcolumn-toolbar', items:['-\x3e', {iconCls:BpcCommon.Icons.DELETE, 
itemId:'deleteBtn', localized:{overflowText:'MONITOR_REMOVE', tooltip:'MONITOR_REMOVE'}, handler:function(btn) {
  btn.lookupController()["delete"](btn.lookupViewModel().get('record'));
}, bind:{hidden:'{!' + monitor.assets.ModuleRights.editMonitorViews + '}'}}]}}]});
Ext.define('monitor.view.state.manager.StateWindow', {extend:'Ext.panel.Panel', alias:'widget.monitorStateConfigurationPanel', controller:'monitorStateConfigurationController', localized:{title:'MONITOR_MANAGE_VIEWS'}, iconCls:monitor.assets.Icons.VIEW, viewContext:undefined, header:false, layout:{type:'hbox', align:'stretch'}, moduleId:undefined, config:{showCloseBtn:false}, publishes:['showCloseBtn'], viewModel:{data:{unsavedChanges:false}}, buttons:['-\x3e', {xtype:'bpcToolbarMsg', iconCls:BpcCommon.Icons.WARNING, 
iconColorCls:'warningColor', text:'MONITOR_UNSAVED_CHANGES', hidden:true, bind:{hidden:'{!unsavedChanges}'}}, {iconCls:BpcCommon.Icons.RESET, ui:'secondary', itemId:'resetBtn', handler:'reset', localized:{text:'CORE_RESET'}}, {iconCls:BpcCommon.Icons.CANCEL, ui:'secondary', handler:'closeWindow', localized:{text:'MONITOR_CLOSE'}, hidden:true, bind:{hidden:'{!showCloseBtn}'}}, {iconCls:BpcCommon.Icons.SAVE, ui:'primary', localized:{text:'MONITOR_SAVE'}, handler:'save', bind:{hidden:'{!' + monitor.assets.ModuleRights.editMonitorViews + 
'}'}}], initComponent:function() {
  var me = this;
  me.monitorViewStatesStore = monitor.Module.monitorViewStatesController.getMonitorViewStateStore(me.moduleId);
  me.mon(me.monitorViewStatesStore, 'datachanged', me.getController().setDirty.bind(me.getController(), true));
  this.items = this.getViewStatesItems();
  this.callParent();
}, getViewStatesItems:function() {
  var me = this;
  return [{xtype:'monitorStateConfigurationGrid', itemId:'stateGrid', flex:1, store:me.monitorViewStatesStore}, {xtype:'bpcSplitter'}, {xtype:'container', flex:3, layout:'card', itemId:'stateObjectViewContainer', items:[{xtype:'bpcCenterMessageCmp', message:'CORE_NO_COMPONENT_AVAILABLE'}]}];
}});
Ext.define('monitor.view.state.manager.StateView', {extend:'Ext.panel.Panel', alias:['widget.monitorStateView'], controller:'monitorStateViewController', layout:{type:'vbox', align:'stretch'}, bodyPadding:5, scrollable:true, viewModel:{data:{stateObject:undefined, moduleInstance:undefined, isGrouped:false}}, initComponent:function() {
  try {
    var me = this;
    me.items = [{xtype:'bpcTextField', fieldLabel:BpcCommon.Api.getTranslation('MONITOR_NAME'), bind:'{stateObject.name}'}, {xtype:'bpcIconPicker', fieldLabel:BpcCommon.Api.getTranslation('MONITOR_VIEW_ICONCLS'), bind:'{stateObject.iconCls}'}, {xtype:'bpcTextField', fieldLabel:BpcCommon.Api.getTranslation('MONITOR_VIEW_TOOLTIP'), bind:'{stateObject.tooltip}'}, {xtype:'checkbox', fieldLabel:BpcCommon.Api.getTranslation('MONITOR_VIEW_FAVORITE'), bind:'{stateObject.favorite}'}, {xtype:'bpcTextField', 
    fieldLabel:BpcCommon.Api.getTranslation('MONITOR_SET_VOLLTEXTSUCHE'), bind:'{stateObject.searchQuery}'}, {xtype:'bpcNumberField', fieldLabel:BpcCommon.Api.getTranslation('MONITOR_PRIORITY'), bind:'{stateObject.priority}'}, {xtype:'bpcTextField', fieldLabel:BpcCommon.Api.getTranslation('MONITOR_ACTIVE_VIEW'), bind:'{stateObject.activeView}'}, {xtype:'monitorAccessControlFieldSet'}];
    this.callParent();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.state.manager.StateViewController', {extend:'Ext.app.ViewController', alias:'controller.monitorStateViewController', requires:[], control:{'#personalView':{change:'togglePersonalView'}}, init:function(view) {
  var me = this;
  me.setStateObject(view.stateObject);
  me.getViewModel().set('moduleInstance', view.moduleInstance);
}, initViewModel:function(vm) {
  var me = this;
  var hasUserRight = BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.editMonitorViews) || BpcCommon.Api.hasUserRight(monitor.assets.ModuleRights.editMonitorViews + '_' + me.moduleId);
  vm.set(monitor.assets.ModuleRights.editMonitorViews, hasUserRight);
}, toggleGrouping:function(cmp) {
  var me = this;
  this.getViewModel().set('isGrouped', cmp.getValue());
  if (cmp.getValue() !== true) {
    me.getView().down('monitorStateViewDetails:visible').down('#groupingCombo').setValue('');
  }
}, togglePersonalView:function() {
  var me = this;
  var isPersonal = me.getView().down('#personalView').getValue();
  var fields = ['writeOrganisations', 'writeRoles', 'writeRights', 'readOrganisations', 'readRoles', 'readRights'];
  if (isPersonal) {
    fields.forEach(function(field) {
      var currentValue = me.getViewModel().get('stateObject.' + field);
      if (!currentValue || !Ext.isArray(currentValue) || currentValue.length > 0) {
        me.getViewModel().set('stateObject.' + field, []);
      }
    });
    me.getViewModel().set('stateObject.writeUser', BpcCommon.Api.getUserName());
    me.getViewModel().set('stateObject.readUser', BpcCommon.Api.getUserName());
  } else {
    var model = me.getViewModel().get('stateObject');
    fields.forEach(function(field) {
      if (model.modified && model.modified[field]) {
        model.set(field, model.modified[field]);
      }
    });
    me.getViewModel().set('stateObject.writeUser', null);
    me.getViewModel().set('stateObject.readUser', null);
  }
}, setStateObject:function(stateObject) {
  var me = this;
  var view = me.getView();
  me.getViewModel().set('stateObject', stateObject);
  me.getViewModel().set('personal', !!stateObject.get('readUser'));
  try {
    if (!stateObject) {
      return;
    }
    if (Ext.isObject(stateObject.get('views'))) {
      var viewCmps = [];
      Ext.Object.each(stateObject.get('views'), function(key, value) {
        viewCmps.push({xtype:'monitorStateViewDetails', moduleId:view.moduleId, viewId:key, viewState:value, title:key});
      });
      view.add({xtype:'tabpanel', items:viewCmps});
    } else {
      view.add({xtype:'monitorStateViewDetails', moduleId:view.moduleId, viewId:'mainViewGrid', viewState:stateObject.getData()});
    }
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR setStateObject', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('monitor.view.state.manager.StateViewDetails', {extend:'Ext.Container', alias:['widget.monitorStateViewDetails'], moduleId:undefined, viewModel:true, initComponent:function() {
  var me = this;
  var columnConfig = BpcCommon.Api.getSetting(me.moduleId, monitor.assets.ConfigKeys.columnConfig);
  var columnDataIndexToTextRenderer = function(value) {
    var renderedValue = value;
    var column = columnConfig.find(function(currentColumn) {
      return currentColumn.dataIndex === value;
    });
    if (column) {
      if (column.text) {
        renderedValue = BpcCommon.Api.getTranslation(column.text);
      } else if (column.localized && column.localized.text) {
        renderedValue = BpcCommon.Api.getTranslation(column.localized.text);
      }
    }
    return renderedValue;
  };
  me.items = [{xtype:'fieldset', title:BpcCommon.Api.getTranslation('MONITOR_FILTER'), anchor:'100%', collapsible:true, layout:'fit', items:[{xtype:'gridpanel', itemId:'filterGrid', emptyText:BpcCommon.Api.getTranslation('MONITOR_NO_FILTER'), store:{model:'monitor.model.Filter', data:[]}, columns:[{text:BpcCommon.Api.getTranslation('MONITOR_COLUMN'), dataIndex:'property', flex:3, renderer:columnDataIndexToTextRenderer}, {text:BpcCommon.Api.getTranslation('MONITOR_OPERATOR'), dataIndex:'operator', 
  flex:1}, {text:BpcCommon.Api.getTranslation('MONITOR_VALUE'), dataIndex:'value', flex:2}, {xtype:'actioncolumn', menuDisabled:true, width:32, bind:{hidden:'{!' + monitor.assets.ModuleRights.editMonitorViews + '}'}, items:[{iconCls:BpcCommon.Icons.DELETE, tooltip:BpcCommon.Api.getTranslation('MONITOR_DELETE'), handler:function(grid, rowIndex) {
    var rec = grid.getStore().getAt(rowIndex);
    grid.getStore().remove(rec);
  }}]}]}]}, {xtype:'fieldset', title:BpcCommon.Api.getTranslation('MONITOR_SORT'), anchor:'100%', collapsible:true, layout:'fit', items:[{xtype:'gridpanel', itemId:'sorterGrid', emptyText:BpcCommon.Api.getTranslation('MONITOR_NO_SORTING'), store:{model:'monitor.model.Sorter', data:[]}, plugins:{ptype:'cellediting', clicksToEdit:1}, columns:[{text:BpcCommon.Api.getTranslation('MONITOR_COLUMN'), dataIndex:'property', flex:3, renderer:columnDataIndexToTextRenderer}, {text:BpcCommon.Api.getTranslation('MONITOR_DIRECTION'), 
  dataIndex:'direction', width:130, editor:{xtype:'combo', queryMode:'local', displayField:'name', valueField:'id', store:{fields:['id', 'name'], data:[{'id':'ASC', 'name':BpcCommon.Api.getTranslation('MONITOR_ASCENDING')}, {'id':'DESC', 'name':BpcCommon.Api.getTranslation('MONITOR_DESCENDING')}]}}, renderer:function(value) {
    var newValue = String(value);
    if (newValue.toLowerCase() === 'asc') {
      newValue = BpcCommon.Api.getTranslation('MONITOR_ASCENDING');
    } else if (newValue.toLowerCase() === 'desc') {
      newValue = BpcCommon.Api.getTranslation('MONITOR_DESCENDING');
    }
    return newValue;
  }}, {xtype:'actioncolumn', menuDisabled:true, width:32, bind:{hidden:'{!' + monitor.assets.ModuleRights.editMonitorViews + '}'}, items:[{iconCls:BpcCommon.Icons.DELETE, tooltip:'delete', handler:function(grid, rowIndex) {
    var rec = grid.getStore().getAt(rowIndex);
    grid.getStore().remove(rec);
  }}]}]}]}, {xtype:'fieldset', title:BpcCommon.Api.getTranslation('MONITOR_GROUPING'), anchor:'100%', collapsible:true, layout:{type:'vbox', align:'stretch'}, itemId:'groupingContainer', items:[{xtype:'checkbox', fieldLabel:BpcCommon.Api.getTranslation('MONITOR_GROUPING_ENABLED'), itemId:'groupingCheck', bind:'{stateObject.groupingEnabled}', handler:'toggleGrouping'}, {xtype:'combo', fieldLabel:BpcCommon.Api.getTranslation('MONITOR_GROUPING'), displayField:'dataIndex', valueField:'dataIndex', store:columnConfig, 
  queryMode:'local', itemId:'groupingCombo', bind:{disabled:'{!isGrouped}', value:'{stateObject.groupField}'}}]}, {xtype:'fieldset', title:BpcCommon.Api.getTranslation('MONITOR_COLUMNS'), anchor:'100%', collapsible:true, layout:'fit', items:[{xtype:'gridpanel', itemId:'columnGrid', emptyText:BpcCommon.Api.getTranslation('MONITOR_NO_COLUMN_CONFIG'), plugins:{ptype:'cellediting', clicksToEdit:1}, store:{}, columns:[{xtype:'rownumberer', text:BpcCommon.Api.getTranslation('MONITOR_POSITION')}, {text:BpcCommon.Api.getTranslation('MONITOR_COLUMN_ID'), 
  dataIndex:'id', flex:1}, {text:BpcCommon.Api.getTranslation('MONITOR_WIDTH'), dataIndex:'width', editor:'numberfield', flex:2}, {text:BpcCommon.Api.getTranslation('MONITOR_HIDDEN'), xtype:'checkcolumn', dataIndex:'hidden', flex:2}, {xtype:'actioncolumn', menuDisabled:true, width:32, bind:{hidden:'{!' + monitor.assets.ModuleRights.editMonitorViews + '}'}, items:[{iconCls:BpcCommon.Icons.DELETE, tooltip:'delete', handler:function(grid, rowIndex) {
    var rec = grid.getStore().getAt(rowIndex);
    grid.getStore().remove(rec);
  }}]}]}]}];
  me.callParent();
  var storeState = me.viewState.storeState;
  if (storeState && storeState.filters) {
    this.down('#filterGrid').store.setData(storeState.filters);
  }
  if (storeState && storeState.sorters) {
    this.down('#sorterGrid').store.setData(storeState.sorters);
  }
  if (me.viewState && me.viewState.columns) {
    this.down('#columnGrid').store.setData(me.viewState.columns);
  }
}});
Ext.define('monitor.view.tableviewconfiguration.ColumnTypeSelector', {extend:'Ext.form.field.ComboBox', alias:['widget.bpcconfig.columnTypeSelector'], editable:true, displayField:'name', valueField:'name', queryMode:'local', viewModel:{data:{setting:null}}, listeners:{change:function(field) {
  var cellEditor = field.up('bpcCellEditor');
  if (cellEditor && cellEditor.isComponent) {
    cellEditor.valueChanged = true;
  }
}}, initComponent:function() {
  var me = this;
  var storeData = [{name:''}];
  var dataSourceFactoryNames = ['gridcolumn', 'datecolumn', 'bpcCheckColumn', 'booleancolumn', 'gridActionColumn', 'monitorAttachmentColumn', 'bpcModuleJumpColumn', 'monitorColumnHook', 'templatecolumn'];
  if (dataSourceFactoryNames) {
    Ext.each(dataSourceFactoryNames, function(value) {
      storeData.push({name:value});
    });
  }
  me.store = {fields:['name'], data:storeData};
  me.callParent();
  me.getViewModel().set('setting', me.setting);
}});
Ext.define('monitor.view.tableviewconfiguration.CustomRendererSelector', {extend:'Ext.form.field.ComboBox', alias:['widget.bpcconfig.customRendererSelector'], editable:true, displayField:'name', valueField:'name', queryMode:'local', setting:undefined, forceSelection:true, listeners:{change:function(field) {
  var cellEditor = field.up('bpcCellEditor');
  if (cellEditor && cellEditor.isComponent) {
    cellEditor.valueChanged = true;
  }
}}, initComponent:function() {
  var moduleId = this.setting.get('moduleId');
  var customRenderers = [{name:true}, {name:false}];
  var customRendererSetting = BpcCommon.Api.getSetting(moduleId, 'column_customRenderer');
  Ext.Object.each(customRendererSetting, function(key) {
    customRenderers.push({name:key});
  });
  this.store = {fields:['name'], data:customRenderers};
  this.callParent();
}});
Ext.define('monitor.view.tableviewconfiguration.TableViewConfigurator', {extend:'Ext.grid.Panel', alias:'widget.tableViewConfigurator', controller:'tableViewConfiguratorController', initColumnConfigs:undefined, viewModel:{data:{unsavedChanges:false}}, bodyMargin:10, localized:{title:'MONITOR_COLUMN_CONFIGURATOR', tooltip:'MONITOR_COLUMN_CONFIGURATOR'}, cls:'column-config-grid', iconCls:'x-fal fa-table', viewConfig:{getRowClass:function(record) {
  var classes = 'column-configurator-grid-row';
  if (record.get('hidden')) {
    classes += ' column-config-hidden-row';
  }
  return classes;
}, plugins:{ptype:'gridviewdragdrop'}}, rowViewModel:{}, selModel:{type:'spreadsheet', columnSelect:true, rowSelect:true, cellSelect:false, dragSelect:false, pruneRemoved:true, extensible:false, mode:'SINGLE'}, initComponent:function() {
  var $jscomp$this = this;
  this.moduleId = this.moduleInstance.get('id');
  var dataIndex = this.moduleInstance.get(monitor.assets.ConfigKeys.dataIndex);
  this.initColumnConfigs = this.getFullColumnConfigs();
  if (!dataIndex) {
    this.items = {xtype:'bpcCenterMessageCmp', message:'MONITOR_NO_INDEX_SET'};
  } else if (!this.initColumnConfigs) {
    this.items = {xtype:'bpcCenterMessageCmp', message:'MONITOR_NO_INDEX_SET'};
  } else {
    this.store = this.createStore(this.initColumnConfigs);
    this.columns = this.createColumns(this.initColumnConfigs);
    this.cellEditing = new Ext.grid.plugin.CellEditing({clicksToEdit:1});
    this.plugins = [this.cellEditing];
    this.listeners = {edit:function(grid, context) {
      context.record.set(context.field, context.value);
    }, drop:function(node, data) {
      $jscomp$this.getViewModel().set('unsavedChanges', true);
    }, afterrender:function(grid) {
      var selModel = grid.getSelectionModel();
      var firstRecord = grid.getStore().getAt(0);
      selModel.selectRows(firstRecord);
    }, selectionchange:function(grid, selection) {
      var removeAttributeBtn = grid.down('#removeAttribute');
      var selectedColumns = selection.selectedColumns;
      if (selectedColumns && selectedColumns.length > 0) {
        removeAttributeBtn.enable();
      } else {
        removeAttributeBtn.disable();
      }
    }};
    this.buttons = [{text:BpcCommon.Api.getTranslation('MONITOR_ADD_ATTRIBUTE'), iconCls:BpcCommon.Icons.ADD, ui:'secondary', handler:'onAddAttribute'}, {text:BpcCommon.Api.getTranslation('MONITOR_REMOVE_ATTRIBUTE'), iconCls:BpcCommon.Icons.DELETE, disabled:true, itemId:'removeAttribute', ui:'secondary', handler:'onRemoveAttribute'}, {text:BpcCommon.Api.getTranslation('MONITOR_ADD_COLUMN'), iconCls:BpcCommon.Icons.COLUMNS, ui:'secondary', handler:'onAddColumn'}, '-\x3e', {xtype:'bpcToolbarMsg', iconCls:BpcCommon.Icons.WARNING, 
    iconColorCls:'warningColor', text:'CORE_UNSAVED_CHANGES', hidden:true, bind:{hidden:'{!unsavedChanges}'}}, {text:BpcCommon.Api.getTranslation('CORE_RESET'), iconCls:BpcCommon.Icons.RESET, ui:'secondary', handler:'onReset'}, {text:BpcCommon.Api.getTranslation('MONITOR_SAVE'), iconCls:BpcCommon.Icons.SAVE, handler:'onSave'}];
  }
  this.callParent();
}, createColumns:function(configs) {
  var baseConfigs = this.getBaseConfigObject(configs);
  var columns = this.getFirstColumns();
  Ext.Object.each(baseConfigs, function(key, value) {
    if (key === 'name' || key === 'role' || key === 'right' || key === 'organisation') {
      return;
    }
    var type = 'text';
    switch(typeof value) {
      case 'object':
        type = 'json';
        break;
      case 'string':
        type = 'text';
        break;
      case 'number':
        type = 'number';
        break;
      case 'boolean':
        type = 'boolean';
        break;
    }
    switch(key) {
      case 'headerIconCls':
        type = 'icon';
        break;
      case 'xtype':
        type = 'columnTypeSelector';
        break;
      case 'customRenderer':
        type = 'customRendererSelector';
        break;
    }
    var column = {xtype:'bpcSettingColumn', text:key, dataIndex:key, type:type, hidden:key === 'localized' ? true : false, flex:1, minWidth:180, sortable:false, lockable:false, codeEditorTitle:key, renderer:function(value, metaData, record) {
      var settingClass = Ext.ClassManager.getByAlias('widget.bpcconfig.' + type);
      if (settingClass && settingClass.prototype.renderer) {
        return settingClass.prototype.renderer.apply(this, arguments);
      }
      record.renderer = false;
      return Ext.util.Format.htmlEncode2(value);
    }};
    columns.push(column);
  });
  return columns;
}, createStore:function(configs) {
  var me = this;
  var vm = me.getViewModel();
  var columnConfigs = Ext.clone(configs);
  var storeData = columnConfigs.map(function(item) {
    var baseConfigs = me.getBaseConfigObject(configs);
    var mergedConfigs = Ext.apply({}, item, baseConfigs);
    var columnName = item.localized && item.localized.text ? BpcCommon.Api.getTranslation(item.localized.text) : item.text;
    if (!item.text && item.localized && item.localized.text) {
      mergedConfigs.text = item.localized.text;
    }
    mergedConfigs.columnName = columnName;
    mergedConfigs.moduleId = me.moduleInstance.get('id');
    return mergedConfigs;
  });
  return Ext.create('Ext.data.Store', {data:storeData, model:'BpcCommon.component.data.ObservableModel', proxy:'memory', listeners:{update:function(store, record, operation, modifiedFieldNames) {
    if (modifiedFieldNames && modifiedFieldNames.includes('text')) {
      record.set('columnName', BpcCommon.Api.getTranslation(record.get('text')), {dirty:false});
    }
    if (modifiedFieldNames && modifiedFieldNames.includes('localized') && record.get('localized').text) {
      record.set('columnName', BpcCommon.Api.getTranslation(record.get('localized').text), {dirty:false});
    }
    if (modifiedFieldNames && modifiedFieldNames.includes('hidden')) {
      var hidden = record.get('hidden');
      var recordIndex = me.store.getData().indexOf(record);
      me.el.dom.querySelectorAll('[data-recordindex\x3d"' + recordIndex + '"] .x-grid-row').forEach(function(item) {
        if (hidden) {
          item.classList.add('column-config-hidden-row');
        } else {
          item.classList.remove('column-config-hidden-row');
        }
      });
    }
    vm.set('unsavedChanges', true);
  }}});
}, getBaseConfigObject:function(configs) {
  var clonedConfigs = Ext.clone(configs);
  var baseColumnConfig = clonedConfigs.reduce(function(accumulator, currentValue) {
    return Ext.apply(accumulator, currentValue);
  });
  Ext.Object.each(baseColumnConfig, function(key, value) {
    if (Ext.isString(value)) {
      baseColumnConfig[key] = '';
    } else if (Ext.isObject(value)) {
      baseColumnConfig[key] = {};
    } else if (Ext.isBoolean(value)) {
      baseColumnConfig[key] = !value;
    } else if (Ext.isNumber(value)) {
      baseColumnConfig[key] = undefined;
    }
  });
  baseColumnConfig.xtype = '';
  baseColumnConfig.minWidth = 64;
  baseColumnConfig.width = 200;
  baseColumnConfig.hidden = true;
  baseColumnConfig.hideable = true;
  baseColumnConfig.role = [];
  baseColumnConfig.right = [];
  baseColumnConfig.organisation = [];
  baseColumnConfig.format = '';
  baseColumnConfig.tpl = '';
  return baseColumnConfig;
}, getFirstColumns:function() {
  return [{xtype:'bpcHandleColumn', draggable:false, locked:true, menuDisabled:true}, {xtype:'templatecolumn', text:BpcCommon.Api.getTranslation('MONITOR_COLUMN_HEADER_PREVIEW') + '\n                  \x26nbsp\x3ci data-qtip\x3d"' + BpcCommon.Api.getTranslation('MONITOR_HEADER_PREVIEW') + '" class\x3d"fal fa-info-circle"\x3e\x3c/i\x3e', dataIndex:'columnName', locked:true, width:200, draggable:false, menuDisabled:true, sortable:false, tpl:Ext.create('Ext.XTemplate', '\x3cspan role\x3d"iconEl" class\x3d"{headerIconCls}"\x3e\x3c/span\x3e', 
  '\x3cspan role\x3d"textEl" \x3e\x26nbsp{columnName}\x3c/span\x3e')}, {xtype:'widgetcolumn', menuDisabled:true, locked:true, width:50, widget:{xtype:'toolbar', itemId:'columnActions', overflowHandler:'menu', defaultButtonUI:'widgetcolumn-toolbar', items:['-\x3e', {iconCls:'x-fal fa-trash', itemId:'deleteColumn', localized:{overflowText:'CORE_DELETE', tooltip:'CORE_DELETE'}, handler:function(btn) {
    btn.lookupController().deleteColumn(btn.lookupViewModel().get('record'));
  }}]}}, {localized:{text:'CORE_ROLES'}, lockable:false, width:150, dataIndex:'role', editor:{completeOnEnter:true, field:{xtype:'bpcRoleSelect', grow:false}}}, {localized:{text:'CORE_RIGHTS'}, lockable:false, width:150, dataIndex:'right', editor:{completeOnEnter:true, field:{xtype:'bpcRightSelect', grow:false}}}, {localized:{text:'CORE_ORGANISATION'}, lockable:false, width:150, dataIndex:'organisation', editor:{completeOnEnter:true, field:{xtype:'bpcOrganisationSelect', grow:false}}}];
}, getFullColumnConfigs:function() {
  var columnConfigs = this.moduleInstance.get('column_config') || [];
  var columnList = this.moduleInstance.get('column_list') || [];
  if (columnList.length === 0) {
    return null;
  }
  Ext.Object.each(columnList, function(key) {
    var isNewColumn = columnConfigs.filter(function(columnConfig) {
      return columnConfig.dataIndex === key;
    }).length === 0;
    if (isNewColumn) {
      var newColumnConfig = {dataIndex:key, hidden:true, text:key, xtype:'gridcolumn'};
      columnConfigs.push(newColumnConfig);
    }
  });
  return columnConfigs;
}});
Ext.define('monitor.view.tableviewconfiguration.TableViewConfiguratorController', {extend:'Ext.app.ViewController', alias:'controller.tableViewConfiguratorController', onAddColumn:function() {
  var grid = this.getView();
  var store = grid.getStore();
  var vm = grid.getViewModel();
  var storeData = store.getData().items.map(function(record) {
    var clonedData = Ext.clone(record.data);
    delete clonedData.id;
    return clonedData;
  });
  store.add(grid.getBaseConfigObject(storeData));
  vm.set('unsavedChanges', true);
}, addNewAttribute:function(key, type) {
  var grid = this.getView();
  var store = grid.getStore();
  var vm = grid.getViewModel();
  var selectionModel = grid.getSelectionModel();
  var value;
  switch(type) {
    case 'json':
      value = {};
      break;
    case 'text':
      value = '';
      break;
    case 'number':
      value = 0;
      break;
    case 'boolean':
      value = false;
      break;
  }
  var columnConfigs = grid.initColumnConfigs.map(function(item) {
    var $jscomp$compprop0 = {};
    return Ext.apply({}, item, ($jscomp$compprop0[key] = value, $jscomp$compprop0));
  });
  store.each(function(record) {
    record.set(key, value);
  });
  selectionModel.deselectAll();
  Ext.suspendLayouts();
  grid.setColumns(grid.createColumns(columnConfigs));
  Ext.resumeLayouts(true);
  vm.set('unsavedChanges', true);
}, onSave:function() {
  var configuratorView = this.getView();
  var store = configuratorView.getStore();
  var vm = configuratorView.getViewModel();
  var moduleId = configuratorView.moduleId;
  if (vm.get('unsavedChanges')) {
    var toSaveConfigs = store.getData().items.map(function(record) {
      var toSaveConfig = Ext.clone(record.data);
      if (toSaveConfig.tpl) {
        toSaveConfig.tpl = Ext.htmlDecode(toSaveConfig.tpl);
      }
      delete toSaveConfig.id;
      delete toSaveConfig.columnName;
      return toSaveConfig;
    });
    var unlockFn = BpcCommon.Util.blockComponentForAsyncAction('onSave', undefined, BpcCommon.Api.getTranslation('CORE_PLEASE_WAIT'), '^ bpcConfigModule \x3e monitorTableViewConfigurationContainer');
    var configSettingModel = BpcCommon.Api.getSetting(configuratorView.moduleInstance.get('id'), 'column_config', true);
    configSettingModel.set('value', toSaveConfigs);
    monitor.Module.monitorViewStatesController.resetState(moduleId);
    store.save();
    configSettingModel.save({callback:function() {
      BpcCommon.Api.showNotification('CORE_SETTINGS_SAVE_SUCCESS');
      unlockFn();
    }});
    vm.set('unsavedChanges', false);
  }
}, onReset:function() {
  var configuratorView = this.getView();
  var vm = configuratorView.getViewModel();
  var selectionModel = configuratorView.getSelectionModel();
  var initConfigs = configuratorView.moduleInstance.get('column_config');
  configuratorView.setStore(configuratorView.createStore(initConfigs));
  var columns = configuratorView.createColumns(initConfigs);
  selectionModel.deselectAll();
  Ext.suspendLayouts();
  configuratorView.setColumns(columns);
  Ext.resumeLayouts(true);
  vm.set('unsavedChanges', false);
}, onAddAttribute:function() {
  var me = this;
  Ext.create('Ext.window.Window', {title:BpcCommon.Api.getTranslation('MONITOR_ADD_ATTRIBUTE'), modal:true, height:200, width:350, padding:10, items:{xtype:'form', items:[{xtype:'textfield', fieldLabel:'Name', name:'key'}, {xtype:'combobox', name:'type', fieldLabel:'Type', queryMode:'local', store:{data:[{name:'json'}, {name:'text'}, {name:'boolean'}, {name:'number'}]}, displayField:'name'}]}, buttons:[{text:BpcCommon.Api.getTranslation('MONITOR_CANCEL'), iconCls:BpcCommon.Icons.CANCEL, ui:'secondary', 
  handler:function() {
    this.up('window').close();
  }}, {text:BpcCommon.Api.getTranslation('MONITOR_OK'), iconCls:BpcCommon.Icons.OK, ui:'primary', handler:function() {
    var formValues = this.up('window').down('form').getValues();
    me.addNewAttribute(formValues.key, formValues.type);
    this.up('window').close();
  }}]}).show();
}, deleteColumn:function(record) {
  var $jscomp$this = this;
  BpcCommon.Msg.show({title:BpcCommon.Api.getTranslation('MONITOR_REMOVE_COLUMN'), message:BpcCommon.Api.getTranslation('MONITOR_REMOVE_ATTRIBUTE_DESCRIPTION'), buttons:BpcCommon.Msg.DELETECANCEL, buttonText:{cancel:BpcCommon.Api.getTranslation('MONITOR_CANCEL'), 'delete':BpcCommon.Api.getTranslation('MONITOR_REMOVE')}, icon:BpcCommon.Msg.QUESTION, fn:function(btn) {
    if (btn === 'delete') {
      var view = $jscomp$this.getView();
      var vm = view.getViewModel();
      var store = view.getStore();
      store.remove(record);
      vm.set('unsavedChanges', true);
    }
  }});
}, onRemoveAttribute:function() {
  var me = this;
  BpcCommon.Msg.show({title:BpcCommon.Api.getTranslation('MONITOR_REMOVE_ATTRIBUTE'), message:BpcCommon.Api.getTranslation('MONITOR_REMOVE_ATTRIBUTE_DESCRIPTION'), buttons:BpcCommon.Msg.DELETECANCEL, buttonText:{cancel:BpcCommon.Api.getTranslation('MONITOR_CANCEL'), 'delete':BpcCommon.Api.getTranslation('MONITOR_REMOVE')}, icon:BpcCommon.Msg.QUESTION, fn:function(btn) {
    if (btn === 'delete') {
      var view = me.getView();
      var vm = view.getViewModel();
      var store = view.getStore();
      var selectionModel = view.getSelectionModel();
      var selectedColumns = selectionModel.getSelected().getColumns();
      if (selectedColumns && selectedColumns.length > 0) {
        Ext.each(selectedColumns, function(column) {
          selectionModel.deselectColumn(column);
          column.hideable = false;
          column.disable();
          column.setVisible(false);
          var dataIndex = column.dataIndex;
          store.each(function(item) {
            delete item.data[dataIndex];
          });
        });
        vm.set('unsavedChanges', true);
      }
    }
  }});
}});
Ext.define('monitor.view.toolbar.PluginHookLeft', {extend:'BpcCommon.plugin.HookContainer', alias:'widget.bpcMonitorLeftToolbarHook', layout:{type:'hbox', align:'middle'}, inheritableStatics:{NAME:'Monitor Toolbar links', HOOK_ID:'monitor_toolbar_left'}});
Ext.define('monitor.view.toolbar.PluginHookModuleHeader', {extend:'BpcCommon.plugin.HookContainer', alias:'widget.bpcMonitorModuleHeaderToolbarHook', layout:{type:'hbox', align:'middle'}, inheritableStatics:{NAME:'Monitor Toolbar Module Header', HOOK_ID:'monitor_toolbar_module_header'}, onAdd:function(component) {
  var me = this;
  if (component.isButton && component.ui === 'default' && !component.hasOwnProperty('ui')) {
    component.setUi('default-toolbar');
  }
  me.callParent();
}});
Ext.define('monitor.view.toolbar.PluginHookRight', {extend:'BpcCommon.plugin.HookContainer', alias:'widget.bpcMonitorRightToolbarHook', layout:{type:'hbox', align:'middle'}, inheritableStatics:{NAME:'Monitor Toolbar rechts', HOOK_ID:'monitor_toolbar_right'}});
Ext.define('monitor.widget.abstract.MonitorWidgetAbstractController', {extend:'Ext.app.ViewController', alias:'controller.monitorWidgetAbstractController', control:{'#':{moduleIdChange:'onModuleIdChange', afterrender:'onAfterrender', add:'onWidgetAdd', successDataLoad:'onSuccessDataLoad', widgetConfigurationChange:'onWidgetConfigurationChange'}, 'monitorWidgetTimeSeriesMain':{moduleIdChange:'timeSeriesOnModuleIdChange', aggregationColumnChange:'timeSeriesOnAggregationColumnChange'}, 'monitorWidgetProcessDistributionMain':{moduleIdChange:'processDistributionOnModuleIdChange', 
columnChange:'processDistributionOnColumnChange'}, 'monitorWidgetProcessPreviewMain':{moduleIdChange:'processPreviewOnModuleIdChange'}}, listen:{global:{bpcActiveModuleChanged:'onBpcActiveModuleChanged'}}, init:function() {
  var me = this;
  var view = me.getView();
  view.initSettingStore();
  view.loadMonitorViewState().then();
}, onWidgetConfigurationChange:function() {
}, processDistributionOnColumnChange:function() {
  var me = this;
  var view = me.getView();
  view.resetLoadedDataFields();
}, timeSeriesOnAggregationColumnChange:function() {
  var me = this;
  var view = me.getView();
  view.resetLoadedDataFields();
}, onSuccessDataLoad:function(newData) {
  if (!Ext.isArray(newData)) {
    return;
  }
  var me = this;
  var view = me.getView();
  var mappedFields = newData.map(function(el) {
    return el.name;
  });
  view.onNewIncomingDataFields(mappedFields);
}, onWidgetAdd:function(cmp, item) {
  if (!Ext.isFunction(item.query)) {
    return;
  }
  if (item.query('tabpanel')) {
    var tabs = item.query('tabpanel');
    for (var $jscomp$iter$2 = $jscomp.makeIterator(tabs), $jscomp$key$tab = $jscomp$iter$2.next(); !$jscomp$key$tab.done; $jscomp$key$tab = $jscomp$iter$2.next()) {
      var tab = $jscomp$key$tab.value;
      {
        tab.on('tabchange', function(subTabs, newSubTab) {
          var index = subTabs.items.items.findIndex(function(tab) {
            return tab === newSubTab;
          });
          tabs.forEach(function(tab) {
            tab.setActiveItem(index);
          });
        });
      }
    }
  }
}, processPreviewOnModuleIdChange:function() {
  var me = this;
  var view = me.getView();
  var settingsList = ['columnList'];
  view.restoreSettingsListToDefault(settingsList);
}, processDistributionOnModuleIdChange:function() {
  var me = this;
  var view = me.getView();
  var settingsList = ['column'];
  view.restoreSettingsListToDefault(settingsList);
}, timeSeriesOnModuleIdChange:function() {
  var me = this;
  var view = me.getView();
  var settingsList = ['aggregationColumn'];
  view.restoreSettingsListToDefault(settingsList);
}, onModuleIdChange:function() {
  var me = this;
  var view = me.getView();
  var settingsList = ['selectedView', 'views', 'initialView', 'timeseriesColumn'];
  view.restoreSettingsListToDefault(settingsList);
  view.loadCustomRenderer();
  view.deleteMonitorViewState();
  view.resetLoadedDataFields();
}, onBpcActiveModuleChanged:function() {
  var me = this;
  var view = me.getView();
  var isVisible = view.isVisible(true);
  if (isVisible && view.pendingUpdate) {
    view.refreshWidget();
  }
}, onAfterrender:function() {
  var me = this;
  var view = me.getView();
  view.enableGlobalDateRangeListeners();
  if (view.generalErrorHandling()) {
    view.renderWidget();
  }
}});
Ext.define('monitor.widget.abstract.controller.WidgetViewController', {extend:'Ext.app.ViewController', listen:{global:{dataLoaded:'onDataLoaded', bpcActiveModuleChanged:'onModuleShow'}}, init:function() {
  var bufferInSeconds = this.getView().widgetReference.getSetting('dataRefreshBuffer');
  this.refreshDataBuffered = Ext.Function.createThrottled(this.refreshData, bufferInSeconds * 1000);
}, refreshDataBuffered:function() {
  this.refreshData();
}, refreshData:function(force) {
  force = force === void 0 ? false : force;
  var me = this;
  var view = me.getView();
  if (!force && !view.isVisible(true)) {
    me.refreshDataOnView = true;
    return;
  }
  if (Ext.isFunction(view.refreshWidget) && (force || me.getAutoUpdateState())) {
    view.refreshWidget();
  }
}, getAutoUpdateState:function() {
  var me = this;
  var view = me.getView();
  try {
    return view.widgetReference.dbViewModel.get('autoUpdate');
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR read dashboard view model', dump:e, stack:true, level:'error'});
    return true;
  }
}, onDataLoaded:function(dataObj) {
  var me = this;
  var queryId = this.getView().queryId;
  var doRefresh = me.getView().widgetReference.shouldRefreshOnGlobalDataLoadedEvent(dataObj, me.endpointName, queryId);
  if (doRefresh) {
    this.refreshDataBuffered();
  }
}, onModuleShow:function() {
  var view = this.getView();
  if (view.isVisible(true)) {
    if (this.refreshDataOnView) {
      this.refreshDataOnView = false;
      this.refreshData(true);
    }
  }
}});
Ext.define('monitor.widget.abstract.mixins.ViewModeRendering', {createChartCmp:Ext.emptyFn, createGridCmp:Ext.emptyFn, renderContent:function() {
  var me = this;
  try {
    var contentItems = [];
    var viewMode = me.getSetting('viewMode') ? me.getSetting('viewMode') : '';
    var chartComponent = me.createChartCmp();
    var gridComponent = me.createGridCmp();
    if (Ext.String.startsWith(viewMode, 'stacked')) {
      contentItems = me.getStackedViewItems(chartComponent, gridComponent);
    } else if (Ext.String.startsWith(viewMode, 'tabbed')) {
      contentItems = me.getTabbedViewItems(chartComponent, gridComponent);
    } else if (viewMode === 'chartOnly') {
      contentItems = me.getChartOnlyViewItems(chartComponent);
    } else if (viewMode === 'tableOnly') {
      contentItems = me.getTableOnlyViewItems(gridComponent);
    }
    me.removeAll();
    me.add(contentItems);
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR initOverview', dump:e, stack:true, level:'error'});
  }
}, getStackedViewItems:function(chartComponent, gridComponent) {
  Ext.apply(chartComponent, {flex:1, resizeable:true});
  Ext.apply(gridComponent, {collapsible:true, collapseDirection:'bottom', flex:1, resizeable:true, header:false});
  return [{xtype:'panel', header:false, layout:{type:'vbox', align:'stretch'}, resizeable:true, items:[chartComponent, {xtype:'bpcSplitter', collapseTarget:'next'}, gridComponent]}];
}, getChartOnlyViewItems:function(chartComponent) {
  return chartComponent;
}, getTableOnlyViewItems:function(gridComponent) {
  return gridComponent;
}, getTabbedViewItems:function(chartComponent, gridComponent) {
  var me = this;
  Ext.apply(chartComponent, {tabConfig:{itemId:'chartTab'}});
  Ext.apply(gridComponent, {tabConfig:{itemId:'dataGrid'}});
  var activeTab = me.getInitialActiveTabIndex();
  return [{xtype:'tabpanel', activeTab:activeTab, items:[chartComponent, gridComponent]}];
}, getInitialActiveTabIndex:function() {
  var me = this;
  var activeTab = 0;
  var initViewStr = me.widgetReference.getSetting('initialView') || 'chart';
  if (Ext.isString(initViewStr)) {
    var initViewArr = initViewStr.split(':');
    if (initViewArr.length === 2) {
      var subView = initViewArr[1];
      if (subView === 'chart') {
        activeTab = 0;
      } else if (subView === 'data') {
        activeTab = 1;
      }
    }
  }
  return activeTab;
}});
Ext.define('monitor.widget.components.form.CellAlignSelector', {alias:'widget.monitorCellAlignSelector', extend:'BpcCommon.component.form.field.ComboBox', store:['right', 'left', 'center'], queryMode:'local', forceSelection:true});
Ext.define('monitor.widget.components.form.ChartTypeSelector', {alias:'widget.monitorChartTypeSelector', extend:'BpcCommon.component.form.field.ComboBox', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var color = '#4CAF50';
  var iconCls = recordData.icon;
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
  return icon + ' ' + recordData.label;
}})}, initComponent:function() {
  this.store = [{label:BpcCommon.lang.Current.MONITOR_LINE_GRAPH, value:'line', icon:'x-fal fa-chart-line'}, {label:BpcCommon.lang.Current.MONITOR_COLUMN_GRAPH, value:'column', icon:'x-fal fa-chart-bar'}];
  this.callParent();
}});
Ext.define('monitor.widget.components.form.ConfigurationPanel', {extend:'Ext.FormPanel', alias:'widget.monitorWidgetFormPanel', plugins:[{ptype:'datatip'}], scrollable:'vertical', formComponents:[], initComponent:function() {
  var me = this;
  me.formComponents = me.formComponents.map(function(el) {
    return Ext.apply(el, {settingStore:me.settingStore, moduleId:me.moduleId});
  });
  me.callParent();
  me.refreshItems();
}, refreshItems:function() {
  var me = this;
  me.removeAll();
  me.add(me.formComponents);
}});
Ext.define('monitor.widget.components.form.multiple.DataFieldTagSelector', {alias:'widget.monitorWidgetDataFieldTagSelector', extend:'Ext.form.field.Tag', queryMode:'local', displayField:'name', valueField:'name', createNewOnEnter:true, createNewOnBlur:true, forceSelection:false, initComponent:function() {
  var me = this;
  me.emptyText = BpcCommon.Api.getTranslation('MONITOR_NO_DATA_FIELDS_SELECTED');
  me.widget = me.up('[widgetReference]').widgetReference;
  me.mon(me.widget, 'loadedDataFieldsChanged', function() {
    me.fillStore();
  });
  me.store = me.widget.getMappedLoadedDataFields();
  this.callParent();
}, fillStore:function() {
  var me = this;
  me.store.loadRawData(me.widget.getMappedLoadedDataFields());
}});
Ext.define('monitor.widget.components.form.DockSummarySelector', {alias:'widget.monitorDockSummarySelector', extend:'BpcCommon.component.form.field.ComboBox', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var color = '#4CAF50';
  var iconCls = recordData.icon;
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
  return icon + ' ' + recordData.label;
}})}, initComponent:function() {
  this.store = [{label:BpcCommon.lang.Current.MONITOR_DOCK_SUMMARY_INLINE, value:'', icon:'x-fal fa'}, {label:BpcCommon.lang.Current.MONITOR_DOCK_SUMMARY_TOP, value:'top', icon:'x-fal fa-level-up'}, {label:BpcCommon.lang.Current.MONITOR_DOCK_SUMMARY_BOTTOM, value:'bottom', icon:'x-fal fa-level-down'}];
  this.callParent();
}});
Ext.define('monitor.widget.components.form.InstanceSelector', {alias:'widget.monitorInstanceSelector', extend:'BpcCommon.component.form.field.ModuleInstanceSelector', baseModuleName:'monitor', forceSelection:true});
Ext.define('monitor.widget.components.form.IntervalSelector', {alias:'widget.monitorIntervalSelector', extend:'BpcCommon.component.form.field.ComboBox', displayField:'label', valueField:'value', queryMode:'local', forceSelection:true, initComponent:function() {
  this.store = [{label:BpcCommon.lang.Current.CORE_YEAR, value:'year'}, {label:BpcCommon.lang.Current.CORE_QUARTER, value:'quarter'}, {label:BpcCommon.lang.Current.CORE_MONTH, value:'month'}, {label:BpcCommon.lang.Current.CORE_WEEK, value:'week'}, {label:BpcCommon.lang.Current.CORE_DAY, value:'day'}, {label:BpcCommon.lang.Current.CORE_HOUR, value:'hour'}, {label:BpcCommon.lang.Current.CORE_MINUTE, value:'minute'}, {label:BpcCommon.lang.Current.CORE_SECOND, value:'second'}];
  this.callParent();
}});
Ext.define('monitor.widget.components.form.SettingsStoreFieldSet', {alias:'widget.monitorWidgetFieldSet', extend:'Ext.form.FieldSet', collapsible:true, defaults:{labelWidth:200, margin:5}, layout:'fit', fieldComponents:[], initComponent:function() {
  var me = this;
  me.widgetReference = me.up('[widgetReference]').widgetReference;
  me.prepareComponents();
  me.callParent(arguments);
  me.initializeFields();
  me.on('afterrender', function() {
    var fieldContainerList = me.query('fieldcontainer');
    Ext.Array.each(fieldContainerList, function(fieldContainer) {
      var fieldContainerEl = fieldContainer.getEl();
      var restoreButton = fieldContainer.down('[fieldContainerRestoreButton]');
      fieldContainerEl.hover(function() {
        restoreButton.getEl().setOpacity(1);
      }, function() {
        restoreButton.getEl().setOpacity(0);
      });
    });
  }, me);
}, prepareComponents:function() {
  var me = this;
  var clonedComponents = Ext.clone(me.fieldComponents);
  me.items = clonedComponents.map(function(el) {
    var localized = Ext.clone(el.localized);
    delete el.localized.fieldLabel;
    el.width = 350;
    el.minWidth = 350;
    return {xtype:'fieldcontainer', defaults:{labelWidth:200, moduleId:me.moduleId}, layout:{type:'hbox', align:'stretch'}, fieldLabel:BpcCommon.Api.getTranslation(localized.fieldLabel), tooltip:BpcCommon.Api.getTranslation(localized.tooltip), items:[{xtype:'button', fieldContainerRestoreButton:true, iconCls:'x-fal fa-undo fa-sm', ui:'transparent', width:50, minWidth:50, hidden:false, handler:me.restoreFieldToDefault.bind(me, el.property), localized:{tooltip:'CORE_RESET'}, style:'opacity: 0'}, el]};
  });
}, restoreFieldToDefault:function(property) {
  var me = this;
  me.widgetReference.restoreSettingToDefault(property);
}, initializeFields:function() {
  var me = this;
  var inputElements = me.query('[property]');
  var directAssociatedProperties = inputElements.map(function(el) {
    return el.property;
  });
  Ext.Array.each(directAssociatedProperties, function(field) {
    me.initializeField(field, undefined);
  }, me);
}, initializeField:function(property, handler) {
  var me = this;
  var selectorCmp = me.down('[property\x3d' + property + ']');
  me.writeValueFromStoreToField(selectorCmp, property);
  var defaultHandler = function(element) {
    me.writeValueToStore(property, element.getValue());
    if (selectorCmp.triggerChangeEvent) {
      me.widgetReference.fireEvent(property + 'Change');
    }
  };
  var appliedHandler = Ext.isFunction(handler) ? handler : defaultHandler;
  selectorCmp.on({change:{fn:appliedHandler, scope:me, buffer:200}});
}, writeValueToStore:function(propertyName, value) {
  var me = this;
  var settingsEntry = me.settingStore.getById(propertyName);
  if (!settingsEntry) {
    return;
  }
  me.settingStore.getById(propertyName).set('value', value);
}, writeValueFromStoreToField:function(cmp, property) {
  var me = this;
  if (!cmp || !me.settingStore) {
    return false;
  }
  var storeRecord = me.settingStore.getById(property);
  if (!storeRecord || !Ext.isFunction(storeRecord.get)) {
    return false;
  }
  var storeValue = storeRecord.get('value');
  if (!Ext.isFunction(cmp.setValue)) {
    return false;
  }
  cmp.setValue(storeValue);
}});
Ext.define('monitor.widget.components.form.ViewModeSelector', {alias:'widget.monitorViewModeSelector', extend:'BpcCommon.component.form.field.ComboBox', store:['tabbed', 'stacked', 'chartOnly', 'tableOnly'], queryMode:'local', forceSelection:true});
Ext.define('monitor.widget.components.form.column.ColumnSelector', {alias:'widget.monitorColumnSelector', extend:'BpcCommon.component.form.field.ComboBox', displayField:'label', valueField:'value', queryMode:'local', initComponent:function() {
  var me = this;
  me.widget = me.up('[widgetReference]').widgetReference;
  me.emptyText = BpcCommon.Api.getTranslation('MONITOR_WIDGET_NO_COLUMNS_SELECTED');
  me.mon(me.widget, 'moduleIdChange', function() {
    me.fillStore();
  });
  me.store = me.widget.getAllMonitorColumns();
  me.callParent(arguments);
}, fillStore:function() {
  var me = this;
  me.store.loadRawData(me.widget.getAllMonitorColumns());
}});
Ext.define('monitor.widget.components.form.column.ColumnTagSelector', {alias:'widget.monitorColumnTagSelector', extend:'Ext.form.field.Tag', displayField:'label', valueField:'value', queryMode:'local', initComponent:function() {
  var me = this;
  me.emptyText = BpcCommon.Api.getTranslation('MONITOR_WIDGET_NO_COLUMNS_SELECTED');
  me.widget = me.up('[widgetReference]').widgetReference;
  me.mon(me.widget, 'moduleIdChange', function() {
    me.fillStore();
  });
  me.store = me.widget.getAllMonitorColumns();
  me.callParent(arguments);
}, fillStore:function() {
  var me = this;
  me.store.loadRawData(me.widget.getAllMonitorColumns());
}});
Ext.define('monitor.widget.components.form.multipleGrid.MultiGridComboSelector', {alias:'widget.monitorWidgetMultiGridComboSelector', extend:'BpcCommon.component.form.field.ComboBox', displayField:'label', valueField:'value', queryMode:'local', listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var resetIcon = "\x3cspan class\x3d'x-fal fa-undo'  style \x3d 'margin-right: 3px;' \x3e\x3c/span\x3e";
  return recordData.value === '' ? '\x3chr\x3e' + resetIcon + ' ' + recordData.label : recordData.value;
}})}, displayTpl:new Ext.XTemplate('\x3ctpl for\x3d"."\x3e{[this.returnRenderer(values)]}\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  return recordData.value === '' ? BpcCommon.Api.getTranslation('MONITOR_NO_INITIAL_GRID_SELECTED') : recordData.value;
}}), initComponent:function() {
  var me = this;
  me.widget = me.up('[widgetReference]').widgetReference;
  this.store = me.getComboStoreData();
  me.mon(me.widget, 'moduleIdChange', function() {
    me.fillStore();
  });
  this.callParent();
}, fillStore:function() {
  var me = this;
  me.store.loadRawData(me.getComboStoreData());
}, getComboStoreData:function() {
  var me = this;
  var monitorViews = me.widget.getMonitorMultipleGridViews();
  var mappedViews = monitorViews.map(function(element) {
    return {label:element.title, value:element.id};
  });
  mappedViews.push({value:'', label:BpcCommon.Api.getTranslation('CORE_RESET')});
  return mappedViews;
}});
Ext.define('monitor.widget.components.form.multipleGrid.InitialGridComboSelector', {alias:'widget.monitorWidgetInitialGridComboSelector', extend:'monitor.widget.components.form.multipleGrid.MultiGridComboSelector', initComponent:function() {
  var me = this;
  me.callParent(arguments);
  me.widget.on('viewsChange', me.fillStore, me);
}, getComboStoreData:function() {
  var me = this;
  var monitorViews = me.widget.getMonitorMultipleGridViews();
  var mappedViews = monitorViews.map(function(element) {
    return {label:element.title, value:element.id};
  });
  var availableViews = me.widget.getSetting('views');
  if (!Ext.isEmpty(availableViews)) {
    mappedViews = mappedViews.filter(function(el) {
      return availableViews.includes(el.value);
    });
  }
  mappedViews.push({value:'', label:BpcCommon.Api.getTranslation('CORE_RESET')});
  return mappedViews;
}});
Ext.define('monitor.widget.components.form.multipleGrid.MultiGridTagSelector', {alias:'widget.monitorWidgetMultiGridTagSelector', extend:'Ext.form.field.Tag', displayField:'id', valueField:'id', queryMode:'local', initComponent:function() {
  var me = this;
  me.emptyText = BpcCommon.Api.getTranslation('MONITOR_WIDGET_NO_GRIDS_SELECTED');
  me.widget = me.up('[widgetReference]').widgetReference;
  this.store = me.widget.getMonitorMultipleGridViews();
  me.mon(me.widget, 'moduleIdChange', function() {
    me.fillStore();
  });
  this.callParent();
}, fillStore:function() {
  var me = this;
  me.store.loadRawData(me.widget.getMonitorMultipleGridViews());
}});
Ext.define('monitor.widget.components.form.stateView.StateViewSelector', {alias:'widget.monitorWidgetStateViewSelector', extend:'BpcCommon.component.form.field.ComboBox', moduleId:undefined, displayField:'name', valueField:'value', queryMode:'local', config:{settings:{}}, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{name:htmlDoubleEncode}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  var color = '#4CAF50';
  var iconCls = recordData.iconCls || 'x-fal fa-star';
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + '; margin-right: 3px;"\x3e\x3c/span\x3e';
  return recordData.value === '' ? '\x3chr\x3e ' + icon + ' ' + recordData.name : icon + ' ' + recordData.name;
}})}, displayTpl:new Ext.XTemplate('\x3ctpl for\x3d"."\x3e{[this.returnRenderer(values)]}\x3c/tpl\x3e', {returnRenderer:function(recordData) {
  return recordData.value === '' ? BpcCommon.Api.getTranslation('MONITOR_NO_VIEW_STATES_SELECTED') : recordData.name;
}}), initStore:function() {
  var me = this;
  var monitorViews = me.widget.getMonitorStateViews();
  me.emptyText = BpcCommon.Api.getTranslation('MONITOR_NO_VIEW_STATES_SELECTED');
  monitorViews.push({value:'', name:BpcCommon.Api.getTranslation('MONITOR_RESET_VIEW'), iconCls:'x-fal fa-undo'});
  this.store.loadRawData(monitorViews);
}, initComponent:function() {
  var me = this;
  me.widget = me.up('[widgetReference]').widgetReference;
  me.mon(me.widget, 'moduleIdChange', me.initStore, me);
  this.store = Ext.create('Ext.data.Store', {});
  me.initStore();
  this.callParent();
}});
Ext.define('monitor.widget.processDistribution.View', {extend:'Ext.container.Container', alias:'widget.monitorWidgetProcessDistributionView', mixins:['monitor.widget.abstract.mixins.ViewModeRendering'], layout:{type:'fit'}, controller:'monitorWidgetProcessDistributionViewController', chartCmp:'monitorWidgetProcessDistributionChart', gridCmp:'monitorWidgetProcessDistributionGrid', moduleId:undefined, viewObj:undefined, bpcGridViewName:null, initComponent:function() {
  var me = this;
  me.callParent();
  me.initStore();
  me.refreshWidget();
}, initStore:function() {
  var me = this;
  var $jscomp$destructuring$var7 = me.widgetReference.getWidgetConfiguration() || {};
  var excludedValues = $jscomp$destructuring$var7.excludedValues;
  var moduleId = $jscomp$destructuring$var7.moduleId;
  var column = $jscomp$destructuring$var7.column;
  var columnConfig = me.widgetReference.getMonitorColumnConfig(column);
  var suffix = 'analyzed';
  if (columnConfig && columnConfig.filter && columnConfig.filter.source) {
    suffix = columnConfig.filter.source;
  }
  monitor.assets.Helpers.getArrayFromString();
  var excludedValuesArr = monitor.assets.Helpers.getArrayFromString(excludedValues);
  var url = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + moduleId + '/distinctFieldValues/' + column;
  me.store = Ext.create('Ext.data.Store', {filters:[{property:'value', value:excludedValuesArr, operator:'notin'}], fields:[{name:'value'}, {name:'count'}], proxy:{type:'ajax', url:url, extraParams:{view:me.bpcGridViewName, queryId:me.queryId, source:suffix, timezoneOffset:'+0' + (new Date()).getTimezoneOffset() / 60 * -1 + ':00', timezoneName:BpcCommon.Util.getTimezoneName()}, reader:{rootProperty:'values', type:'json'}}});
  me.store.on('load', me.onStoreLoad, me);
}, onStoreLoad:function(store) {
  var me = this;
  var dataFieldsList = store.getRange().map(function(el) {
    return {name:el.get('value')};
  });
  me.widgetReference.fireEvent('successDataLoad', dataFieldsList);
  me.widgetReference.updateOverViewData.bind(me.widgetReference, me)();
  me.updateChartData();
}, refreshWidget:function() {
  var me = this;
  me.widgetReference.loadMonitorViewState().then(function() {
    me.loadGridWithMonitorView();
  });
}, loadGridWithMonitorView:function() {
  var me = this;
  var $jscomp$destructuring$var8 = me.widgetReference.getWidgetConfiguration() || {};
  var startDate = $jscomp$destructuring$var8.startDate;
  var endDate = $jscomp$destructuring$var8.endDate;
  var timeseriesColumn = $jscomp$destructuring$var8.timeseriesColumn;
  var aggregationLimit = $jscomp$destructuring$var8.aggregationLimit;
  var filter = me.widgetReference.getStoreFilterFromStateView(me.bpcGridViewName);
  var params = {timeseriesColumn:timeseriesColumn, startDate:me.widgetReference.getGlobalDateRangeStart() || startDate, endDate:me.widgetReference.getGlobalDateRangeEnd() || endDate, size:aggregationLimit, filter:JSON.stringify(filter)};
  me.store.reload({params:params, failure:function(records, operation) {
    var error = BpcCommon.Util.getErrorFromFailureResponse(operation);
    me.removeAll();
    me.add({xtype:'bpcCenterMessageCmp', message:error.message, messageIconCls:BpcCommon.Icons.ERROR, localized:{title:'CORE_ERROR'}});
  }.bind(me)});
}, updateChartData:function() {
  if (!this.chart) {
    this.renderContent();
  } else {
    this.chart.updateData(this.prepareChartData());
  }
}, prepareChartData:function() {
  var me = this;
  var data = [];
  var virtimoColors = monitor.assets.Colors.virtimoColors;
  var colorCounter = 0;
  var $jscomp$destructuring$var9 = me.widgetReference.getWidgetConfiguration() || {};
  var hiddenGraphs = $jscomp$destructuring$var9.hiddenGraphs;
  var column = $jscomp$destructuring$var9.column;
  try {
    me.store.each(function(record) {
      var fieldName = record.get('value');
      var fieldCount = record.get('count');
      var valueRendererConfig = me.widgetReference.getCustomRendererValueConfig(column, fieldName);
      var value = {original:fieldName, value:valueRendererConfig.label || fieldName, count:fieldCount};
      var hiddenGraphsStr = hiddenGraphs || '';
      var hiddenGraphsArr = monitor.assets.Helpers.getArrayFromString(hiddenGraphsStr);
      var hidden = false;
      if (hiddenGraphsArr.length > 0) {
        if (Ext.Array.contains(hiddenGraphsArr, value.original)) {
          hidden = true;
        }
      }
      value.hidden = hidden;
      if (!valueRendererConfig.color || valueRendererConfig.color === '' || valueRendererConfig.fallback) {
        value.color = virtimoColors[colorCounter];
        colorCounter++;
      } else {
        value.color = valueRendererConfig.color;
      }
      data.push(value);
    }, this);
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR prepareChartData', dump:e, stack:true, level:'error'});
  }
  return data;
}, createChartCmp:function() {
  var me = this;
  return {xtype:me.chartCmp, flex:1, config:{data:me.prepareChartData(), moduleId:me.moduleId, viewObj:me.viewObj}, tabConfig:{itemId:'chartTab'}};
}, createGridCmp:function() {
  var me = this;
  return {xtype:me.gridCmp, store:me.store, header:false, region:'south', tabConfig:{itemId:'dataTab'}};
}, getSetting:function(key) {
  var me = this;
  return me.widgetReference.getSetting(key);
}});
Ext.define('monitor.widget.processDistribution.ViewController', {extend:'monitor.widget.abstract.controller.WidgetViewController', alias:'controller.monitorWidgetProcessDistributionViewController', endpointName:'monitor_distinct', refreshData:function(force) {
  var me = this;
  var forceRefresh = force || false;
  var view = this.getView();
  if (!forceRefresh && !me.isWidgetVisible()) {
    me.refreshDataOnView = true;
    return;
  }
  if (Ext.isFunction(view.refreshWidget) && (forceRefresh || me.getAutoUpdateState())) {
    view.refreshWidget();
  }
}, isWidgetVisible:function() {
  var me = this;
  var view = me.getView();
  var overview = view.up('monitorWidgetProcessDistributionMain').down('monitorWidgetProcessDistributionOverview');
  if (!overview || !Ext.isFunction(overview.isVisible)) {
    return false;
  }
  return view.isVisible(true) || overview.isVisible(true);
}, onModuleShow:function() {
  var me = this;
  if (me.isWidgetVisible()) {
    if (me.refreshDataOnView) {
      me.refreshDataOnView = false;
      me.refreshData(true);
    }
  }
}});
Ext.define('monitor.widget.processDistribution.components.Chart', {extend:'Ext.Container', alias:'widget.monitorWidgetProcessDistributionChart', layout:'card', localized:{title:'MONITOR_CHARTS'}, data:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.listeners = {boxready:this.loadChart.bind(this), beforedestroy:function() {
    if ($jscomp$this.chart) {
      $jscomp$this.chart.dispose();
      delete $jscomp$this.chart;
    }
  }};
  this.chartViewCmp = Ext.create({layout:'fit', xtype:'component', itemId:'amChart'});
  this.noDataAvailableCmp = Ext.create({layout:'fit', xtype:'container', items:[{xtype:'bpcCenterMessageCmp', message:'MONITOR_NO_DATA_IN_TIME_SPAN'}]});
  this.items = [this.chartViewCmp, this.noDataAvailableCmp];
  this.callParent();
}, loadChart:function() {
  var me = this;
  lazyLoad.AmCharts().then(me.createChart.bind(me));
}, createChart:function() {
  var $jscomp$this = this;
  var $jscomp$destructuring$var10 = this.config || {};
  var data = $jscomp$destructuring$var10.data;
  var moduleId = $jscomp$destructuring$var10.moduleId;
  var viewId = this.up('[viewId]').viewId;
  var widgetReference = this.up('[widgetReference]').widgetReference;
  var config = widgetReference.getSetting('chartConfiguration') || {};
  var $jscomp$destructuring$var11 = config;
  var marginTop = $jscomp$destructuring$var11.marginTop;
  var exportConfig = config["export"] || {};
  if (window.am4core) {
    am4core.ready(function() {
      var chartIn3d = !!widgetReference.getSetting('3d');
      var chart = am4core.create($jscomp$this.chartViewCmp.getId(), am4charts[chartIn3d ? 'PieChart3D' : 'PieChart']);
      if (BpcCommon.Api.getSetting('_core', 'language') === 'de' && window.am4lang_de_DE) {
        chart.language.locale = window.am4lang_de_DE;
      }
      chart.marginTop = marginTop || 5;
      if (exportConfig.enabled) {
        chart.exporting.menu = new am4core.ExportMenu();
      }
      chart.data = $jscomp$this.data || data || [];
      var pieSeries = chart.series.push(chartIn3d ? new am4charts.PieSeries3D() : new am4charts.PieSeries());
      pieSeries.dataFields.value = 'count';
      pieSeries.dataFields.category = 'value';
      pieSeries.dataFields.hidden = 'hidden';
      pieSeries.slices.template.propertyFields.fill = 'color';
      chart.radius = am4core.percent(50);
      pieSeries.labels.template.text = "{category}\n{value.percent.formatNumber('#.0')}%";
      pieSeries.labels.template.maxWidth = 130;
      pieSeries.labels.template.wrap = true;
      pieSeries.slices.template.events.on('hit', function(ev) {
        try {
          BpcCommon.Api.showModule(moduleId, false, {storeState:{filters:[{operator:'in', property:widgetReference.getSetting('column'), value:[ev.target.dataItem._dataContext.original]}]}, viewId:viewId});
        } catch (e) {
        }
      });
      $jscomp$this.chart = chart;
      $jscomp$this.updateView();
    });
  }
}, updateData:function(chartData) {
  chartData = chartData === void 0 ? [] : chartData;
  if (this.chart) {
    this.chart.data = chartData;
  } else {
    this.data = chartData;
  }
  this.updateView();
}, updateView:function() {
  if (!Ext.isDefined(this.chart)) {
    return;
  }
  if (this.chart && Ext.isArray(this.chart.data) && this.chart.data.length < 1) {
    this.getLayout().setActiveItem(this.noDataAvailableCmp);
  } else {
    this.getLayout().setActiveItem(this.chartViewCmp);
  }
}});
Ext.define('monitor.widget.processDistribution.components.ConfigurationPanel', {extend:'monitor.widget.components.form.ConfigurationPanel', alias:'widget.monitorWidgetProcessDistributionConfigurationPanel', formComponents:[{xtype:'monitorWidgetFieldSet', title:'Monitor', fieldComponents:[{xtype:'monitorInstanceSelector', property:'moduleId', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_SOURCE_MONITOR', tooltip:'MONITOR_SOURCE_MONITOR'}, itemId:'moduleIdSelectorField', allowBlank:false}, 
{xtype:'monitorColumnSelector', property:'column', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_SETTING_WIDGET_AGGREGATION_COLUMN', tooltip:'MONITOR_SETTING_WIDGET_AGGREGATION_COLUMN_TOOLTIP'}, itemId:'columnSelectorField'}, {xtype:'bpcNumberField', property:'aggregationLimit', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_AGGREGATION_LIMIT', tooltip:'MONITOR_SETTING_WIDGET_AGGREGATION_LIMIT_TOOLTIP'}, itemId:'aggregationLimitSelectorField'}, {xtype:'bpcTextField', property:'startDate', 
localized:{fieldLabel:'MONITOR_SETTING_WIDGET_START_DATE', tooltip:'MONITOR_SETTING_WIDGET_START_DATE_TOOLTIP'}, itemId:'startDateSelectorField', allowBlank:false}, {xtype:'bpcTextField', property:'endDate', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_END_DATE', tooltip:'MONITOR_SETTING_WIDGET_END_DATE_TOOLTIP'}, itemId:'endDateSelectorField', allowBlank:false}, {xtype:'monitorColumnSelector', property:'timeseriesColumn', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_TIMESERIES_COLUMN', tooltip:'MONITOR_SETTING_WIDGET_TIMESERIES_COLUMN_TOOLTIP'}, 
itemId:'timeseriesColumnSelectorField', allowBlank:false}, {xtype:'monitorWidgetStateViewSelector', property:'selectedView', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_WIDGET_STATE_VIEW', tooltip:'MONITOR_WIDGET_STATE_VIEW_TOOLTIP'}, itemId:'selectedViewSelectorField'}, {xtype:'monitorWidgetMultiGridTagSelector', property:'views', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_SETTING_WIDGET_GRIDS', tooltip:'MONITOR_SETTING_WIDGET_GRIDS_TOOLTIP'}, itemId:'viewsSelectorField'}, 
{xtype:'monitorWidgetMultiGridComboSelector', property:'initialView', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_INITIAL_VIEW', tooltip:'MONITOR_SETTING_WIDGET_INITIAL_VIEW_TOOLTIP'}, itemId:'initialViewSelectorField'}]}, {xtype:'monitorWidgetFieldSet', title:'Widget', fieldComponents:[{xtype:'bpcIconPicker', property:'iconCls', localized:{fieldLabel:'CORE_SETTING_ICON_CLS', tooltip:'CORE_SETTING_ICON_CLS'}, itemId:'iconClsSelectorField'}, {xtype:'bpcTextField', property:'title', localized:{fieldLabel:'CORE_SETTING_WIDGET_TITLE', 
tooltip:'CORE_SETTING_WIDGET_TITLE_TOOLTIP'}, itemId:'titleSelectorField'}, {xtype:'monitorViewModeSelector', property:'viewMode', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_VIEWMODE', tooltip:'MONITOR_SETTING_WIDGET_VIEWMODE_TOOLTIP'}, itemId:'viewModeSelectorField'}]}, {xtype:'monitorWidgetFieldSet', title:'Chart', fieldComponents:[{xtype:'checkbox', property:'3d', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_3D', tooltip:'MONITOR_SETTING_WIDGET_3D_TOOLTIP'}, itemId:'threeDSelectorField'}, 
{xtype:'bpcExpandableJsonField', property:'chartConfiguration', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_CHART_CONFIGURATION', tooltip:'MONITOR_SETTING_WIDGET_CHART_CONFIGURATION_TOOLTIP'}, itemId:'chartConfigurationSelectorField'}, {xtype:'monitorWidgetDataFieldTagSelector', property:'hiddenGraphs', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_HIDDEN_GRAPHS', tooltip:'MONITOR_SETTING_WIDGET_HIDDEN_GRAPHS_TOOLTIP'}, itemId:'hiddenGraphsSelectorField'}]}, {xtype:'monitorWidgetFieldSet', title:'Data', 
fieldComponents:[{xtype:'bpcNumberField', property:'dataRefreshBuffer', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_LABEL_DATA_REFRESHBUFFER', tooltip:'MONITOR_SETTING_WIDGET_LABEL_DATA_REFRESHBUFFER_TOOLTIP'}, itemId:'dataRefreshBufferSelectorField'}, {xtype:'checkbox', property:'hideZeroValues', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_HIDE_ZERO_VALUES', tooltip:'MONITOR_SETTING_WIDGET_HIDE_ZERO_VALUES_TOOLTIP'}, itemId:'hideZeroValuesSelectorField'}, {xtype:'monitorWidgetDataFieldTagSelector', 
property:'excludedValues', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_EXCLUDED_VALUES', tooltip:'MONITOR_SETTING_WIDGET_EXCLUDED_VALUES_TOOLTIP'}, itemId:'excludedValuesSelectorField'}, {xtype:'monitorCellAlignSelector', property:'cellAlign', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_CELL_ALIGN', tooltip:'MONITOR_SETTING_WIDGET_CELL_ALIGN_TOOLTIP'}, itemId:'cellAlignSelectorField'}, {xtype:'monitorDockSummarySelector', property:'dockSummary', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_SUMMARY_DOCK', 
tooltip:'MONITOR_SETTING_WIDGET_SUMMARY_DOCK_TOOLTIP'}, itemId:'dockSummarySelectorField'}]}]});
Ext.define('monitor.widget.processDistribution.components.Grid', {extend:'Ext.grid.Panel', alias:'widget.monitorWidgetProcessDistributionGrid', features:[], forceFit:true, listeners:{cellclick:function(tableView, td, cellIndex, record) {
  try {
    var widget = tableView.up('[widgetReference]').widgetReference;
    var moduleId = widget.getSetting('moduleId');
    var widgetOverviewColumn = widget.getSetting('column');
    var filters = [{operator:'in', property:widgetOverviewColumn, value:[record.get('value')]}];
    BpcCommon.Api.showModule(moduleId, false, {storeState:{filters:filters}});
  } catch (e) {
    Ext.log({msg:'Fehler beim Sprung zum Monitor - cellClick', dump:e, stack:true, level:'error'});
    BpcCommon.Api.showNotification('Fehler beim Sprung zum Monitor');
  }
}}, initComponent:function() {
  var me = this;
  var widget = me.up('[widgetReference]').widgetReference;
  var $jscomp$destructuring$var12 = widget.getWidgetConfiguration() || {};
  var column = $jscomp$destructuring$var12.column;
  var dockSummary = $jscomp$destructuring$var12.dockSummary;
  me.title = BpcCommon.Api.getTranslation('MONITOR_DATA');
  me.columns = [{text:BpcCommon.Api.getTranslation('MONITOR_NAME'), dataIndex:'value', flex:2, summaryType:'sum', renderer:widget.getCustomRenderer(column), summaryRenderer:function() {
    return '\x3cb\x3eGesamt\x3c/b\x3e';
  }}, {text:BpcCommon.Api.getTranslation('MONITOR_COUNT'), dataIndex:'count', flex:1, summaryType:'sum', summaryRenderer:function(value) {
    return '\x3cb\x3e' + value + '\x3c/b\x3e';
  }}];
  me.features = [{ftype:'summary', dock:dockSummary}];
  me.callParent(arguments);
}});
Ext.define('monitor.widget.processDistribution.components.Overview', {extend:'Ext.grid.Panel', alias:'widget.monitorWidgetProcessDistributionOverview', localized:{title:'MONITOR_OVERVIEW', emptyText:'MONITOR_LOADING_DATA'}, forceFit:true, autoScroll:true, storeId:undefined, moduleId:undefined, widgetReference:undefined, createStore:function() {
  var me = this;
  return Ext.create('Ext.data.Store', {fields:[], storeId:this.storeId, data:[], listeners:{datachanged:me.createColumns.bind(me)}});
}, createColumns:function(store) {
  var me = this;
  var $jscomp$destructuring$var13 = me.widgetReference.getWidgetConfiguration() || {};
  var cellAlign = $jscomp$destructuring$var13.cellAlign;
  var column = $jscomp$destructuring$var13.column;
  var hideZeroValues = $jscomp$destructuring$var13.hideZeroValues;
  var storeCollection = store.getData();
  var targetColumnValues = [];
  storeCollection.each(function(item) {
    var viewObj = item.getData();
    delete viewObj.id;
    delete viewObj.view;
    targetColumnValues = targetColumnValues.concat(Object.keys(viewObj));
  });
  targetColumnValues = Ext.Array.unique(targetColumnValues);
  var columns = [{dataIndex:'view', text:BpcCommon.Api.getTranslation('MONITOR_VIEW'), align:cellAlign || 'center', minWidth:90, sortable:false, renderer:function(value) {
    return value;
  }}];
  Ext.Array.each(targetColumnValues, function(targetColumnValue) {
    var valueRendererConfig = me.widgetReference.getCustomRendererValueConfig(column, targetColumnValue);
    var customRenderer = me.widgetReference.getCustomRenderer(column);
    var columnHeaderText = customRenderer(targetColumnValue);
    columns.push({minWidth:60, dataIndex:targetColumnValue, text:columnHeaderText, align:cellAlign || 'center', sortable:false, renderer:function(value, metaData) {
      try {
        metaData.tdStyle = 'border: 1px solid ' + valueRendererConfig.color + ';';
        if (hideZeroValues && value === 0) {
          return '';
        }
        return value;
      } catch (e) {
        Ext.log({msg:': ERROR renderer', dump:e, stack:true, level:'error'});
      }
    }});
  });
  me.reconfigure(columns);
}, initComponent:function() {
  var me = this;
  me.store = me.createStore();
  this.callParent(arguments);
}});
Ext.define('monitor.widget.processPreview.components.Grid', {extend:'Ext.grid.Panel', alias:'widget.monitorWidgetProcessPreviewView', controller:'monitorWidgetProcessPreviewViewController', bodyStyle:'cursor: pointer', forceFit:true, height:100, plugins:'gridfilters', bbar:{xtype:'pagingtoolbar', displayInfo:true}, moduleId:undefined, viewObj:undefined, widgetId:undefined, queryId:undefined, stateful:true, stateId:undefined, initComponent:function() {
  var me = this;
  var moduleId = me.widgetReference.getSetting('moduleId');
  me.viewConfig = {localized:{emptyTexT:'MONITOR_NO_PROCESSES_FOUND'}, deferEmptyText:false, getRowClass:monitor.Module.rowStyleController.getRowClass.bind(me, moduleId, me.viewId)};
  me.emptyText = BpcCommon.Api.getTranslation('MONITOR_NO_DATA_AVAILABLE');
  me.stateId = 'Monitor_ProcessPreview_Widget_State_' + me.widgetId;
  me.store = me.createStore(me.bpcGridViewName);
  me.columns = me.createColumns();
  me.callParent(arguments);
  me.refreshWidget();
}, createColumns:function() {
  var me = this;
  var $jscomp$destructuring$var14 = me.widgetReference.getWidgetConfiguration() || {};
  var moduleId = $jscomp$destructuring$var14.moduleId;
  var columnList = $jscomp$destructuring$var14.columnList;
  var goTo = $jscomp$destructuring$var14.goTo;
  var widgetColumnConfig = monitor.assets.Helpers.getArrayFromString(columnList);
  var columns = [];
  Ext.Array.each(widgetColumnConfig, function(colName) {
    var columnName = colName.trim();
    var column = Ext.clone(me.widgetReference.getMonitorColumnConfig(columnName));
    if (column) {
      delete column.width;
      delete column.hidden;
      delete column.id;
      delete column.filter;
      column.sortable = false;
      column.renderer = me.widgetReference.getCustomRenderer(columnName);
      if (column.xtype === 'datecolumn') {
        column.format = Ext.isDefined(column.format) ? column.format : BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.dateFormat);
        delete column.renderer;
      }
      columns.push(column);
    }
  });
  if (goTo === true) {
    var moduleName = BpcCommon.Api.getSetting(moduleId, 'module_name');
    var tooltip = BpcCommon.Api.getTranslation('MONITOR_MONITOR_JUMP');
    if (Ext.isDefined(moduleName)) {
      tooltip += ' - ' + moduleName;
    }
    columns.push({xtype:'actioncolumn', align:'center', width:30, items:[{iconCls:BpcCommon.Icons.LINK, tooltip:tooltip, handler:me.monitorJumpHandler.bind(me)}]});
  }
  return columns;
}, monitorJumpHandler:function(grid, rowIndex) {
  var me = this;
  var $jscomp$destructuring$var15 = me.widgetReference.getWidgetConfiguration() || {};
  var moduleId = $jscomp$destructuring$var15.moduleId;
  var goTo = $jscomp$destructuring$var15.goTo;
  var timeseriesColumn = $jscomp$destructuring$var15.timeseriesColumn;
  var record = grid.getStore().getAt(rowIndex);
  if (goTo === true) {
    var id = me.widgetReference.getMonitorIdColumn();
    var filters = [{property:id, value:record.get(id)}];
    var sorters = [];
    var sorter = {property:timeseriesColumn, direction:'DESC'};
    sorters.push(sorter);
    BpcCommon.Api.showModule(moduleId, false, {storeState:{filters:filters, sorters:sorters}});
  }
}, createStore:function(viewName) {
  var me = this;
  try {
    var $jscomp$destructuring$var16 = me.widgetReference.getWidgetConfiguration() || {};
    var moduleId = $jscomp$destructuring$var16.moduleId;
    var pageSize = $jscomp$destructuring$var16.pageSize;
    var newModel = Ext.define(null, {extend:'Ext.data.Model', idProperty:me.widgetReference.getMonitorIdColumn()});
    return Ext.create('Ext.data.Store', {model:newModel, moduleId:moduleId, remoteFilter:true, pageSize:pageSize, proxy:{type:'ajax', url:BpcCommon.Api.getBackendUrl('monitor') + '/monitor/' + moduleId, reader:{type:'json', rootProperty:'data', totalProperty:'totalCount', keepRawData:true}, extraParams:{view:viewName, queryId:this.queryId, filter:JSON.stringify(me.getDefaultFilter()), sort:JSON.stringify(me.getDefaultSorter())}}});
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createStore', dump:e, stack:true, level:'error'});
  }
}, refreshWidget:function() {
  var me = this;
  me.widgetReference.loadMonitorViewState().then(function() {
    me.applyFiltersSorters();
  });
}, applyFiltersSorters:function() {
  var me = this;
  var defaultSorter = me.getDefaultSorter();
  var defaultFilter = me.getDefaultFilter();
  var storeStateFilter = me.widgetReference.getStoreFilterFromStateView(me.bpcGridViewName);
  var storeStateSorter = me.widgetReference.getStoreSortersFromStateView(me.bpcGridViewName);
  if (!me.getStore() || !Ext.isFunction(me.getStore().reload)) {
    return;
  }
  var appliedFilters = Ext.isArray(storeStateFilter) && storeStateFilter.length > 0 ? defaultFilter.concat(storeStateFilter) : defaultFilter;
  var appliedSorters = Ext.isArray(storeStateSorter) && storeStateSorter.length > 0 ? storeStateSorter : defaultSorter;
  me.getStore().reload({params:{sort:JSON.stringify(appliedSorters), filter:JSON.stringify(appliedFilters)}});
}, getDefaultFilter:function() {
  var me = this;
  var $jscomp$destructuring$var17 = me.widgetReference.getWidgetConfiguration() || {};
  var timeseriesColumn = $jscomp$destructuring$var17.timeseriesColumn;
  var startDate = $jscomp$destructuring$var17.startDate;
  var endDate = $jscomp$destructuring$var17.endDate;
  return [{id:'x-gridfilter-' + timeseriesColumn + '-start', property:timeseriesColumn, value:me.widgetReference.getGlobalDateRangeStart() || startDate, operator:'gte'}, {id:'x-gridfilter-' + timeseriesColumn + '-end', property:timeseriesColumn, value:me.widgetReference.getGlobalDateRangeEnd() || endDate, operator:'lte'}];
}, getDefaultSorter:function() {
  var me = this;
  var timeSeriesColumn = me.widgetReference.getSetting('timeseriesColumn');
  return [{property:timeSeriesColumn, direction:'DESC'}];
}});
Ext.define('monitor.widget.processPreview.ViewController', {extend:'monitor.widget.abstract.controller.WidgetViewController', alias:'controller.monitorWidgetProcessPreviewViewController', endpointName:'monitor', control:{'#':{rowdblclick:'onRowDblClick'}}, onRowDblClick:function(row, record) {
  var me = this;
  var view = me.getView();
  var widgetReference = view.widgetReference;
  var openDetailViewActive = widgetReference.getSetting('openDetailView');
  if (openDetailViewActive === true) {
    me.openMonitorDetailWindow(row, record);
  }
}, openMonitorDetailWindow:function(row, record) {
  var me = this;
  var view = me.getView();
  var widgetReference = view.widgetReference;
  var moduleId = widgetReference.getSetting('moduleId');
  var recordId = record.getId();
  var popupEnabled = BpcCommon.Api.getSetting(moduleId, monitor.assets.ConfigKeys.detailViewPopupEnabled);
  if (moduleId && recordId) {
    if (popupEnabled === true) {
      BpcCommon.Api.showModule(moduleId, true, {}, ['detailPopup', recordId]);
    }
  }
}});
Ext.define('monitor.widget.processPreview.view.config.ConfigurationPanel', {extend:'monitor.widget.components.form.ConfigurationPanel', alias:'widget.monitorWidgetProcessPreviewConfigurationPanel', formComponents:[{xtype:'monitorWidgetFieldSet', title:'Monitor', fieldComponents:[{xtype:'monitorInstanceSelector', property:'moduleId', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_SOURCE_MONITOR', tooltip:'MONITOR_SOURCE_MONITOR'}, itemId:'moduleIdSelectorField', allowBlank:false}, {xtype:'monitorColumnTagSelector', 
property:'columnList', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_COLUMN_LIST', tooltip:'MONITOR_SETTING_WIDGET_COLUMN_LIST_TOOLTIP'}, itemId:'columnListSelectorField', allowBlank:false}, {xtype:'checkbox', property:'goTo', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_JUMP_COLUMN', tooltip:'MONITOR_SETTING_WIDGET_JUMP_COLUMN_TOOLTIP'}, itemId:'goToSelectorField'}, {xtype:'bpcTextField', property:'startDate', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_START_DATE', tooltip:'MONITOR_SETTING_WIDGET_START_DATE_TOOLTIP'}, 
itemId:'startDateSelectorField', allowBlank:false}, {xtype:'bpcTextField', property:'endDate', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_END_DATE', tooltip:'MONITOR_SETTING_WIDGET_END_DATE_TOOLTIP'}, itemId:'endDateSelectorField', allowBlank:false}, {xtype:'monitorColumnSelector', property:'timeseriesColumn', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_TIMESERIES_COLUMN', tooltip:'MONITOR_SETTING_WIDGET_TIMESERIES_COLUMN_TOOLTIP'}, itemId:'timeseriesColumnSelectorField', allowBlank:false}, 
{xtype:'monitorWidgetStateViewSelector', property:'selectedView', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_WIDGET_STATE_VIEW', tooltip:'MONITOR_WIDGET_STATE_VIEW_TOOLTIP'}, itemId:'selectedViewSelectorField'}, {xtype:'monitorWidgetMultiGridTagSelector', property:'views', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_SETTING_WIDGET_GRIDS', tooltip:'MONITOR_SETTING_WIDGET_GRIDS_TOOLTIP'}, itemId:'viewsSelectorField'}, {xtype:'monitorWidgetMultiGridComboSelector', property:'initialView', 
localized:{fieldLabel:'MONITOR_SETTING_WIDGET_INITIAL_VIEW', tooltip:'MONITOR_SETTING_WIDGET_INITIAL_VIEW_TOOLTIP'}, itemId:'initialViewSelectorField'}]}, {xtype:'monitorWidgetFieldSet', title:'Widget', fieldComponents:[{xtype:'bpcIconPicker', property:'iconCls', localized:{fieldLabel:'CORE_SETTING_ICON_CLS', tooltip:'CORE_SETTING_ICON_CLS'}, itemId:'iconClsSelectorField'}, {xtype:'bpcTextField', property:'title', localized:{fieldLabel:'CORE_SETTING_WIDGET_TITLE', tooltip:'CORE_SETTING_WIDGET_TITLE_TOOLTIP'}, 
itemId:'titleSelectorField'}]}, {xtype:'monitorWidgetFieldSet', title:'Data', fieldComponents:[{xtype:'bpcNumberField', property:'pageSize', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_PAGE_SIZE', tooltip:'MONITOR_SETTING_WIDGET_PAGE_SIZE_TOOLTIP'}, itemId:'pageSizeSelectorField'}, {xtype:'bpcNumberField', property:'dataRefreshBuffer', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_LABEL_DATA_REFRESHBUFFER', tooltip:'MONITOR_SETTING_WIDGET_LABEL_DATA_REFRESHBUFFER_TOOLTIP'}, itemId:'dataRefreshBufferSelectorField'}, 
{xtype:'checkbox', property:'openDetailView', localized:{fieldLabel:'MONITOR_OPEN_DETAIL_ON_DBL_CLICK', tooltip:'MONITOR_OPEN_DETAIL_ON_DBL_CLICK_TOOLTIP'}, itemId:'openDetailViewSelectorField'}]}]});
Ext.define('monitor.widget.timeSeries.View', {extend:'Ext.container.Container', alias:'widget.monitorWidgetTimeSeriesView', mixins:['monitor.widget.abstract.mixins.ViewModeRendering'], layout:{type:'fit'}, isMonitorWidgetTab:true, controller:'monitorWidgetTimeSeriesViewController', chartCmp:'monitorWidgetTimeSeriesChart', gridCmp:'monitorWidgetTimeSeriesGrid', title:false, viewObj:undefined, widgetId:undefined, bpcGridViewName:null, widget:undefined, items:[{xtype:'box', style:'display: flex;justify-content: center;align-items: center;', 
localized:{html:{tpl:'\x3ci class\x3d"fal fa-spinner fa-spin fa-5x fa-fw"\x3e\x3c/i\x3e\x3cbr\x3e\x3cspan\x3e{CORE_LOADING}\x3c/span\x3e', keys:['CORE_LOADING']}}}], dataObj:{}, initComponent:function() {
  var me = this;
  me.callParent(arguments);
  me.initWidget();
}, initWidget:function() {
  this.loadData(this.renderContent);
}, refreshWidget:function() {
  this.loadData(this.updateItems);
}, createParams:function() {
  var me = this;
  var $jscomp$destructuring$var18 = me.widgetReference.getWidgetConfiguration() || {};
  var interval = $jscomp$destructuring$var18.interval;
  var startDate = $jscomp$destructuring$var18.startDate;
  var endDate = $jscomp$destructuring$var18.endDate;
  var aggregationColumn = $jscomp$destructuring$var18.aggregationColumn;
  var timeseriesColumn = $jscomp$destructuring$var18.timeseriesColumn;
  var showPreviousData = $jscomp$destructuring$var18.showPreviousData;
  var showFollowingData = $jscomp$destructuring$var18.showFollowingData;
  var groupedField = aggregationColumn, filterSuffix = null;
  var columnConfig = me.widgetReference.getMonitorColumnConfig(aggregationColumn);
  if (columnConfig && columnConfig.filter && columnConfig.filter.source) {
    filterSuffix = columnConfig.filter.source;
  }
  if (filterSuffix && filterSuffix !== 'analyzed') {
    groupedField = groupedField + '.' + filterSuffix;
  }
  return {filter:[], field:groupedField, interval:interval || 'day', startDate:me.widgetReference.getGlobalDateRangeStart() || startDate, endDate:me.widgetReference.getGlobalDateRangeEnd() || endDate, timezoneOffset:'+0' + (new Date()).getTimezoneOffset() / 60 * -1 + ':00', view:me.bpcGridViewName, queryId:me.queryId, timezoneName:BpcCommon.Util.getTimezoneName(), timeseriesColumn:timeseriesColumn, showPreviousData:showPreviousData, showFollowingData:showFollowingData};
}, loadData:function(callback) {
  var me = this;
  me.widgetReference.loadMonitorViewState().then(function() {
    var additionalFilters = me.widgetReference.getStoreFilterFromStateView(me.bpcGridViewName);
    me.sendRequest(callback, additionalFilters);
  });
}, sendRequest:function(callback, additionalFilters) {
  var me = this;
  additionalFilters = Ext.isArray(additionalFilters) ? additionalFilters : [];
  var url = BpcCommon.Api.getBackendUrl('monitor') + '/monitor/tsAggregation/';
  url += me.widgetReference.getSetting('moduleId');
  var ajaxParameters = Ext.apply(me.createParams(), {filter:JSON.stringify(additionalFilters)});
  Ext.Ajax.request({url:url, method:'GET', params:ajaxParameters, scope:this, success:function(response) {
    me.onSuccess(response, callback);
  }.bind(me), failure:function(response) {
    me.onFailure(response);
  }.bind(me)});
}, onSuccess:function(response, callback) {
  var me = this;
  var responseJson = Ext.JSON.decode(response.responseText);
  var responseData = {};
  var processedData = [];
  me.widgetReference.fireEvent('successDataLoad', Ext.clone(responseJson.fields));
  var excludedValuesStr = me.widgetReference.getSetting('excludedValues') || '';
  var excludedValuesArr = [];
  if (excludedValuesStr) {
    excludedValuesArr = monitor.assets.Helpers.getArrayFromString(excludedValuesStr);
  }
  if (Ext.isArray(excludedValuesArr) && excludedValuesArr.length > 0) {
    Ext.Array.each(responseJson.fields, function(item) {
      if (!Ext.Array.contains(excludedValuesArr, item.name)) {
        processedData.push(item);
      }
    });
  } else {
    processedData = responseJson.fields;
  }
  responseData.data = responseJson.data;
  responseData.fields = processedData;
  if (Ext.isFunction(callback)) {
    me.dataObj = responseData;
    callback.call(this, responseData);
  }
}, onFailure:function(response) {
  var me = this;
  if (response.status === 403) {
    me.removeAll();
    me.add({xtype:'bpcCenterMessageCmp', message:'CORE_ERROR_MISSING_RIGHT', messageIconCls:'x-fal fa-lock', localized:{title:'CORE_ERROR'}});
    return;
  }
  var error = BpcCommon.Util.getErrorFromFailureResponse(response);
  me.removeAll();
  me.add({xtype:'bpcCenterMessageCmp', message:error.message, messageIconCls:BpcCommon.Icons.ERROR, localized:{title:'CORE_ERROR'}});
}, updateItems:function(dataObj) {
  var me = this;
  try {
    var chart = me.down(me.chartCmp);
    if (chart) {
      chart.updateChartData(dataObj.data, me.getGraphConfig(dataObj));
    }
    var grid = me.down(me.gridCmp);
    if (grid) {
      grid.updateGridData(dataObj);
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateItems', dump:e, stack:true, level:'error'});
  }
}, getGraphConfig:function() {
  var me = this;
  var $jscomp$destructuring$var19 = me.widgetReference.widgetConfiguration || {};
  var aggregationColumn = $jscomp$destructuring$var19.aggregationColumn;
  var hiddenGraphs = $jscomp$destructuring$var19.hiddenGraphs;
  var graphOrder = $jscomp$destructuring$var19.graphOrder;
  var graphType = $jscomp$destructuring$var19.graphType;
  var timeseriesColumn = $jscomp$destructuring$var19.timeseriesColumn;
  var interval = $jscomp$destructuring$var19.interval;
  var localizedDateFormat = $jscomp$destructuring$var19.localizedDateFormat;
  var toolTipText = "{name}: {dateX.formatDate('" + localizedDateFormat + "')}: {valueY}";
  var graphConfig = [];
  try {
    var fields = me.dataObj.fields;
    if (!me.hiddenGraphs) {
      var hiddenGraphsStr = hiddenGraphs || '';
      me.hiddenGraphs = monitor.assets.Helpers.getArrayFromString(hiddenGraphsStr);
    }
    var virtimoColors = monitor.assets.Colors.virtimoColors;
    var colorCounter = 0;
    if (Ext.isArray(graphOrder) && graphOrder.length !== 0) {
      Ext.Array.each(graphOrder, function(attribute) {
        var cleanAttribute = attribute.trim();
        var index = fields.findIndex(function(el) {
          return el.name.toLowerCase() === cleanAttribute.toLowerCase();
        });
        if (index >= 0) {
          var element = fields.splice(index, 1)[0];
          fields.unshift(element);
        }
      }, this, true);
    }
    Ext.Array.each(fields, function(field) {
      if (colorCounter >= virtimoColors.length) {
        colorCounter = 0;
      }
      var valueConfig = me.widgetReference.getCustomRendererValueConfig(aggregationColumn, field.name);
      var hidden = false;
      if (me.hiddenGraphs.length > 0) {
        if (Ext.Array.contains(me.hiddenGraphs, field.name)) {
          hidden = true;
        }
      }
      var graph = {name:valueConfig.label || field.name, dataFields:{valueY:field.name, dateX:'date'}, hidden:hidden};
      var colorObject;
      if (!valueConfig.color || valueConfig.color === '' || valueConfig.fallback) {
        colorObject = {fill:virtimoColors[colorCounter], strokeColor:virtimoColors[colorCounter], lineColor:virtimoColors[colorCounter]};
        colorCounter++;
      } else {
        colorObject = {fill:valueConfig.color, strokeColor:valueConfig.color, lineColor:valueConfig.color};
      }
      Ext.apply(graph, colorObject);
      if (graphType === 'line') {
        graph = Ext.apply(graph, {type:'LineSeries', bullets:[{tooltipText:toolTipText, type:'CircleBullet'}], strokeWidth:2});
      } else {
        graph = Ext.apply(graph, {type:'ColumnSeries', stacked:true, columns:{tooltipText:toolTipText, 'events':{'hit':function(ev) {
          try {
            var dateField = timeseriesColumn;
            var startDate = new Date(ev.target.dataItem['_dataContext'].date);
            var endDate = BpcCommon.util.EsDateMath.addEsIntervalToDate(startDate, interval || 'day', 1);
            var filters = [{id:'x-gridfilter-' + dateField + '-start', operator:'gte', property:dateField, value:startDate}, {id:'x-gridfilter-' + dateField + '-end', operator:'lt', property:dateField, value:endDate}, {operator:'in', property:aggregationColumn, value:[ev.target.dataItem.component.dataFields.valueY]}];
            var sorters = [];
            var sorter = {property:timeseriesColumn, direction:'DESC'};
            sorters.push(sorter);
            BpcCommon.Api.showModule(me.moduleId, false, {storeState:{filters:filters, sorters:sorters}, viewId:me.viewObj && me.viewObj.id ? me.viewObj.id : 'mainGrid'});
          } catch (e) {
            Ext.log({msg:': ERROR clickGraphItem ', dump:e, stack:true, level:'error'});
          }
        }}}, strokeWidth:0});
      }
      graphConfig.push(graph);
    });
  } catch (e) {
    Ext.log({msg:me.self.getName() + ': ERROR getGraphConfig', dump:e, stack:true, level:'error'});
  }
  return graphConfig;
}, createChartCmp:function() {
  var me = this;
  var graphConfig = me.getGraphConfig();
  return {xtype:me.chartCmp, widgetReference:me.widgetReference, config:{data:me.dataObj.data, graphConfig:graphConfig, viewId:me.viewId}};
}, createGridCmp:function() {
  var me = this;
  return {xtype:me.gridCmp, widgetReference:me.widgetReference, viewId:me.viewId, config:{moduleId:me.moduleId, data:me.dataObj, viewObj:me.viewObj}};
}, getSetting:function(key) {
  var me = this;
  return me.widgetReference.getSetting(key);
}});
Ext.define('monitor.widget.timeSeriesViewController', {extend:'monitor.widget.abstract.controller.WidgetViewController', alias:'controller.monitorWidgetTimeSeriesViewController', endpointName:'monitor_ts_agg', control:{'monitorWidgetTimeSeriesChart':{boxready:'onChartBoxReady', beforedestroy:'onChartBeforeDestroy'}}, onChartBoxReady:function() {
  var me = this;
  var view = me.getView();
  var chartCmp = view.down(view.chartCmp);
  lazyLoad.AmCharts().then(chartCmp.renderChart.bind(chartCmp));
}, onChartBeforeDestroy:function() {
  var me = this;
  var view = me.getView();
  var chartCmp = view.down(view.chartCmp);
  if (chartCmp && chartCmp.chart) {
    chartCmp.chart.dispose();
    delete chartCmp.chart;
  }
}});
Ext.define('monitor.widget.timeSeries.components.Chart', {extend:'Ext.Container', alias:'widget.monitorWidgetTimeSeriesChart', localized:{title:'MONITOR_CHARTS'}, layout:'card', minHeight:100, hiddenGraphs:[], viewId:undefined, initComponent:function() {
  var me = this;
  lazyLoad.AmCharts();
  me.data = me.prepareChartData(me.config.data);
  me.itemId = me.viewId + '-chart';
  me.prepareComponents();
  me.callParent();
}, prepareComponents:function() {
  var me = this;
  me.chartViewCmp = Ext.create({layout:'fit', xtype:'component', itemId:'amChart'});
  me.noDataAvailableCmp = Ext.create({layout:'fit', xtype:'container', items:[{xtype:'bpcCenterMessageCmp', message:'MONITOR_NO_DATA_IN_TIME_SPAN'}]});
}, prepareChartData:function(data) {
  var me = this;
  var retData = me.filterData(data);
  me.hasAnyData = retData.some(function(item) {
    return item.totalCount !== 0;
  });
  return retData;
}, filterData:function(data) {
  return data.filter(function(item) {
    return item.date !== 'beforestart' && item.date !== 'afterend';
  });
}, getDataFieldsArray:function(seriesArray) {
  if (!Ext.isArray(seriesArray)) {
    return [];
  }
  var retArray = seriesArray.map(function(el) {
    return el.dataFields.valueY;
  });
  return retArray.sort();
}, updateChartData:function(chartData, graphConfig) {
  var me = this;
  chartData = chartData || [];
  graphConfig = graphConfig || [];
  if (!me.chart) {
    return;
  }
  me.data = me.prepareChartData(chartData);
  var oldSeriesConfig = me.getDataFieldsArray(me.config.graphConfig);
  var newSeriesConfig = me.getDataFieldsArray(graphConfig);
  if (!Ext.Array.equals(oldSeriesConfig, newSeriesConfig)) {
    me.config.graphConfig = me.prepareGraphConfig(graphConfig);
    me.chart.dispose();
    me.renderChart();
  } else {
    me.chart.data = me.data;
    me.updateView();
  }
}, prepareGraphConfig:function(graphConfig) {
  var me = this;
  var viewId = me.config.viewId;
  var hiddenGraphList = me.widgetReference.getStatefulHiddenGraphs(viewId);
  Ext.Array.each(graphConfig, function(serie) {
    var foundGraph = hiddenGraphList.find(function(graph) {
      return serie.dataFields.valueY === graph.dataIndex;
    });
    if (foundGraph) {
      serie.hidden = !foundGraph.activeState;
    }
  });
  return graphConfig;
}, onChangeLegendActiveState:function(dataIndex, activeState) {
  var me = this;
  var newElement = {dataIndex:dataIndex, activeState:activeState};
  var viewId = me.config.viewId;
  var hiddenGraphs = me.widgetReference.getStatefulHiddenGraphs(viewId);
  var listIndex = hiddenGraphs.findIndex(function(el) {
    return el.dataIndex === dataIndex;
  });
  if (listIndex !== -1) {
    hiddenGraphs[listIndex] = newElement;
  } else {
    hiddenGraphs.push(newElement);
  }
  me.widgetReference.setStatefulHiddenGraphs(viewId, hiddenGraphs);
}, getChartConfiguration:function() {
  var me = this;
  var dynamicConfig = me.widgetReference.getSetting('chartConfiguration');
  var legend = Ext.apply({}, dynamicConfig.legend);
  legend.itemContainers = Ext.apply({}, dynamicConfig.legend.itemContainers, {events:{'hit':function(ev) {
    try {
      var dataIndex = ev.target.dataItem.dataContext.dataFields.valueY;
      var isActive = ev.target.isActive;
      me.onChangeLegendActiveState(dataIndex, isActive);
    } catch (e) {
      Ext.log({msg:me.self.getName() + ': ERROR getting legend active-state', dump:e, stack:true, level:'error'});
    }
  }.bind(me)}});
  var stateFulGraphConfig = me.prepareGraphConfig(me.config.graphConfig);
  var chartConfig = {type:'serial', language:BpcCommon.Api.getSetting('_core', 'language') === 'de' ? {locale:'de_DE'} : null, data:me.data, exporting:dynamicConfig.exporting ? Ext.apply({}, dynamicConfig.exporting) : {menu:{}}, legend:legend, xAxes:[Ext.apply({type:'DateAxis'}, dynamicConfig.categoryAxis || {})], yAxes:[Ext.apply({type:'ValueAxis'}, dynamicConfig.valueAxis || {})], series:stateFulGraphConfig};
  return Ext.applyIf(chartConfig, dynamicConfig);
}, renderChart:function() {
  var me = this;
  if (window.am4core) {
    am4core.ready(function() {
      me.showChartCmp();
      me.chart = am4core.createFromConfig(me.getChartConfiguration(), me.chartViewCmp.getId(), am4charts.XYChart);
      me.updateView();
    });
  }
}, showChartCmp:function() {
  var me = this;
  me.getLayout().setActiveItem(me.chartViewCmp);
}, updateView:function() {
  var me = this;
  if (me.hasAnyData) {
    me.showChartCmp();
  } else {
    me.getLayout().setActiveItem(me.noDataAvailableCmp);
  }
}});
Ext.define('monitor.widget.processOverviewTime.view.config.ConfigurationPanel', {extend:'monitor.widget.components.form.ConfigurationPanel', alias:'widget.monitorWidgetProcessOverTimeConfigurationPanel', formComponents:[{xtype:'monitorWidgetFieldSet', title:'Monitor', fieldComponents:[{xtype:'monitorInstanceSelector', property:'moduleId', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_SOURCE_MONITOR', tooltip:'MONITOR_SOURCE_MONITOR'}, itemId:'moduleIdSelectorField', allowBlank:false}, {xtype:'monitorColumnSelector', 
property:'aggregationColumn', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_SETTING_WIDGET_AGGREGATION_COLUMN', tooltip:'MONITOR_SETTING_WIDGET_AGGREGATION_COLUMN_TOOLTIP'}, itemId:'aggregationColumnSelectorField', allowBlank:false}, {xtype:'bpcTextField', property:'startDate', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_START_DATE', tooltip:'MONITOR_SETTING_WIDGET_START_DATE_TOOLTIP'}, itemId:'startDateSelectorField', allowBlank:false}, {xtype:'bpcTextField', property:'endDate', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_END_DATE', 
tooltip:'MONITOR_SETTING_WIDGET_END_DATE_TOOLTIP'}, itemId:'endDateSelectorField', allowBlank:false}, {xtype:'monitorColumnSelector', property:'timeseriesColumn', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_TIMESERIES_COLUMN', tooltip:'MONITOR_SETTING_WIDGET_TIMESERIES_COLUMN_TOOLTIP'}, itemId:'timeseriesColumnSelectorField', allowBlank:false}, {xtype:'monitorWidgetStateViewSelector', property:'selectedView', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_WIDGET_STATE_VIEW', tooltip:'MONITOR_WIDGET_STATE_VIEW_TOOLTIP'}, 
itemId:'selectedViewSelectorField'}, {xtype:'monitorWidgetMultiGridTagSelector', property:'views', triggerChangeEvent:true, localized:{fieldLabel:'MONITOR_SETTING_WIDGET_GRIDS', tooltip:'MONITOR_SETTING_WIDGET_GRIDS_TOOLTIP'}, itemId:'viewsSelectorField'}, {xtype:'monitorWidgetInitialGridComboSelector', property:'initialView', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_INITIAL_VIEW', tooltip:'MONITOR_SETTING_WIDGET_INITIAL_VIEW_TOOLTIP'}, itemId:'initialViewSelectorField'}]}, {xtype:'monitorWidgetFieldSet', 
title:'Widget', fieldComponents:[{xtype:'bpcIconPicker', property:'iconCls', localized:{fieldLabel:'CORE_SETTING_ICON_CLS', tooltip:'CORE_SETTING_ICON_CLS'}, itemId:'iconClsSelectorField'}, {xtype:'bpcTextField', property:'title', localized:{fieldLabel:'CORE_SETTING_WIDGET_TITLE', tooltip:'CORE_SETTING_WIDGET_TITLE_TOOLTIP'}, itemId:'titleSelectorField'}, {xtype:'monitorViewModeSelector', property:'viewMode', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_VIEWMODE', tooltip:'MONITOR_SETTING_WIDGET_VIEWMODE_TOOLTIP'}, 
itemId:'viewModeSelectorField'}]}, {xtype:'monitorWidgetFieldSet', title:'Chart', fieldComponents:[{xtype:'monitorChartTypeSelector', property:'graphType', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_GRAPH_TYPE', tooltip:'MONITOR_SETTING_WIDGET_GRAPH_TYPE_TOOLTIP'}, itemId:'graphTypeSelectorField'}, {xtype:'bpcExpandableJsonField', property:'chartConfiguration', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_CHART_CONFIGURATION', tooltip:'MONITOR_SETTING_WIDGET_CHART_CONFIGURATION_TOOLTIP'}, itemId:'chartConfigurationSelectorField'}, 
{xtype:'monitorWidgetDataFieldTagSelector', property:'hiddenGraphs', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_HIDDEN_GRAPHS', tooltip:'MONITOR_SETTING_WIDGET_HIDDEN_GRAPHS_TOOLTIP'}, itemId:'hiddenGraphsSelectorField'}, {xtype:'monitorWidgetDataFieldTagSelector', property:'graphOrder', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_GRAPH_ORDER', tooltip:'MONITOR_SETTING_WIDGET_GRAPH_ORDER_TOOLTIP'}, itemId:'graphOrderSelectorField'}, {xtype:'monitorIntervalSelector', property:'interval', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_INTERVAL', 
tooltip:'MONITOR_SETTING_WIDGET_INTERVAL_TOOLTIP'}, itemId:'intervalSelectorField'}, {xtype:'bpcTextField', property:'localizedDateFormat', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_CHART_DATE_FORMAT', tooltip:'MONITOR_SETTING_WIDGET_CHART_DATE_FORMAT_TOOLTIP'}, itemId:'localizedDateFormatField'}]}, {xtype:'monitorWidgetFieldSet', title:'Data', fieldComponents:[{xtype:'bpcTextField', property:'dateFormat', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_DATE_FORMAT', tooltip:'MONITOR_SETTING_WIDGET_DATE_FORMAT_TOOLTIP'}, 
itemId:'dateFormatSelectorField'}, {xtype:'bpcNumberField', property:'dataRefreshBuffer', directSettingAssociation:true, localized:{fieldLabel:'MONITOR_SETTING_WIDGET_LABEL_DATA_REFRESHBUFFER', tooltip:'MONITOR_SETTING_WIDGET_LABEL_DATA_REFRESHBUFFER_TOOLTIP'}, itemId:'dataRefreshBufferSelectorField'}, {xtype:'checkbox', property:'showFollowingData', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_FOLLOWING_DATA', tooltip:'MONITOR_SETTING_WIDGET_FOLLOWING_DATA_TOOLTIP'}, itemId:'showFollowingDataSelectorField'}, 
{xtype:'checkbox', property:'showPreviousData', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_PREVIOUS_DATA', tooltip:'MONITOR_SETTING_WIDGET_PREVIOUS_DATA_TOOLTIP'}, itemId:'showPreviousDataSelectorField'}, {xtype:'checkbox', property:'hideZeroValues', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_HIDE_ZERO_VALUES', tooltip:'MONITOR_SETTING_WIDGET_HIDE_ZERO_VALUES_TOOLTIP'}, itemId:'hideZeroValuesSelectorField'}, {xtype:'checkbox', property:'hideEmptyRows', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_HIDE_EMPTY_ROWS', 
tooltip:'MONITOR_SETTING_WIDGET_HIDE_EMPTY_ROWS_TOOLTIP'}, itemId:'hideEmptyRowsSelectorField'}, {xtype:'monitorWidgetDataFieldTagSelector', property:'excludedValues', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_EXCLUDED_VALUES', tooltip:'MONITOR_SETTING_WIDGET_EXCLUDED_VALUES_TOOLTIP'}, itemId:'excludedValuesSelectorField'}, {xtype:'monitorCellAlignSelector', property:'cellAlign', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_CELL_ALIGN', tooltip:'MONITOR_SETTING_WIDGET_CELL_ALIGN_TOOLTIP'}, itemId:'cellAlignSelectorField'}, 
{xtype:'monitorDockSummarySelector', property:'dockSummary', localized:{fieldLabel:'MONITOR_SETTING_WIDGET_SUMMARY_DOCK', tooltip:'MONITOR_SETTING_WIDGET_SUMMARY_DOCK_TOOLTIP'}, itemId:'dockSummarySelectorField'}]}]});
Ext.define('monitor.widget.timeSeries.components.Grid', {extend:'Ext.grid.Panel', alias:'widget.monitorWidgetTimeSeriesGrid', localized:{title:'MONITOR_DATA'}, features:[], autoScroll:true, listeners:{el:{delegate:'.summary-cell', click:function(ev, element) {
  var grid = Ext.getCmp(this.id);
  var widgetReference = grid.widgetReference;
  var $jscomp$destructuring$var20 = widgetReference.getWidgetConfiguration() || {};
  var moduleId = $jscomp$destructuring$var20.moduleId;
  var timeseriesColumn = $jscomp$destructuring$var20.timeseriesColumn;
  var showPreviousData = $jscomp$destructuring$var20.showPreviousData;
  var showFollowingData = $jscomp$destructuring$var20.showFollowingData;
  var startDate = $jscomp$destructuring$var20.startDate;
  var endDate = $jscomp$destructuring$var20.endDate;
  var aggregationColumn = $jscomp$destructuring$var20.aggregationColumn;
  var statusValue = element.dataset['aggregationcolumn'];
  var viewId = grid.viewId;
  var filters = [];
  var startDateObject = monitor.assets.Helpers.transformDate(startDate);
  var endDateObject = monitor.assets.Helpers.transformDate(endDate);
  endDateObject.setHours(23, 59, 59);
  var filterStartDate = {id:'x-gridfilter-' + timeseriesColumn + '-start', operator:'gte', property:timeseriesColumn, value:startDateObject};
  var filterEndDate = {id:'x-gridfilter-' + timeseriesColumn + '-end', operator:'lte', property:timeseriesColumn, value:endDateObject};
  if (statusValue !== 'totalCount') {
    filters.push({operator:'in', property:aggregationColumn, value:[statusValue]});
    if (!showPreviousData && !showFollowingData) {
      filters.push([filterStartDate, filterEndDate]);
    } else if (showPreviousData && !showFollowingData) {
      filters.push(filterEndDate);
    } else if (!showPreviousData && showFollowingData) {
      filters.push(filterStartDate);
    }
  }
  if (element.classList.contains('range-summary')) {
    filters.push([filterStartDate, filterEndDate]);
  }
  BpcCommon.Api.showModule(moduleId, false, {storeState:{filters:filters}, viewId:viewId});
}}, cellclick:function(tableView, td, cellIndex, record) {
  var grid = Ext.getCmp(this.id);
  var widgetReference = grid.widgetReference;
  var $jscomp$destructuring$var21 = widgetReference.getWidgetConfiguration() || {};
  var moduleId = $jscomp$destructuring$var21.moduleId;
  var timeseriesColumn = $jscomp$destructuring$var21.timeseriesColumn;
  var aggregationColumn = $jscomp$destructuring$var21.aggregationColumn;
  var startDate = $jscomp$destructuring$var21.startDate;
  var endDate = $jscomp$destructuring$var21.endDate;
  var interval = $jscomp$destructuring$var21.interval;
  var clickedDataIndex = grid.headerCt.getHeaderAtIndex(cellIndex).dataIndex;
  var viewId = grid.up('[bpcGridViewName]').bpcGridViewName;
  var formattedStartDate;
  var formattedEndDate;
  if (record.get('flag') === 'before') {
    formattedEndDate = monitor.assets.Helpers.transformDate(startDate);
  } else if (record.get('flag') === 'after') {
    formattedStartDate = monitor.assets.Helpers.transformDate(endDate);
    formattedStartDate.setHours(23, 59, 59);
  } else {
    formattedStartDate = new Date(record.get('date'));
    formattedEndDate = BpcCommon.util.EsDateMath.addEsIntervalToDate(formattedStartDate, interval || 'day', 1);
  }
  var filters = [];
  if (formattedStartDate) {
    filters.push({id:'x-gridfilter-' + timeseriesColumn + '-start', operator:'gte', property:timeseriesColumn, value:formattedStartDate});
  }
  if (formattedEndDate) {
    filters.push({id:'x-gridfilter-' + timeseriesColumn + '-end', operator:'lt', property:timeseriesColumn, value:formattedEndDate});
  }
  if (clickedDataIndex !== 'date' && clickedDataIndex !== 'totalCount') {
    filters.push({operator:'in', property:aggregationColumn, value:[clickedDataIndex]});
  }
  var sorters = [{property:timeseriesColumn, direction:'DESC'}];
  BpcCommon.Api.showModule(moduleId, false, {storeState:{filters:filters, sorters:sorters}, viewId:viewId});
}}, viewConfig:{stripeRows:false, getRowClass:function(record) {
  if (record.get('flag') === 'before' || record.get('flag') === 'after') {
    return 'x-grid-item-alt';
  }
  return '';
}}, initComponent:function() {
  var me = this;
  try {
    var bpcGridViewName = this.up('[bpcGridViewName]')['bpcGridViewName'];
    this.itemId = bpcGridViewName + '-table';
    var $jscomp$destructuring$var22 = me.widgetReference.getWidgetConfiguration() || {};
    var dateFormat = $jscomp$destructuring$var22.dateFormat;
    var hideEmptyRows = $jscomp$destructuring$var22.hideEmptyRows;
    var dockSummary = $jscomp$destructuring$var22.dockSummary;
    var showPreviousData = $jscomp$destructuring$var22.showPreviousData;
    var showFollowingData = $jscomp$destructuring$var22.showFollowingData;
    this.columns = this.createColumns(this.config.data.fields);
    var fields = [{name:'date', type:'date', dateFormat:'Y-m-d H:i:s', displayFormat:dateFormat}];
    this.config.data.fields.forEach(function(fieldObj) {
      fields.push({name:fieldObj.name, defaultValue:0});
    });
    var preparedData = this.prepareData(this.config.data.data);
    this.store = Ext.create('Ext.data.Store', {data:preparedData, fields:fields});
    if (Ext.isDefined(hideEmptyRows) && hideEmptyRows === true) {
      this.store.filterBy(function(record) {
        return record.get('totalCount') !== 0;
      });
    }
    this.features = [{ftype:'summary', dock:dockSummary && dockSummary.length ? dockSummary : undefined}];
    if (showPreviousData || showFollowingData) {
      this.features.unshift({ftype:'summaryignorerows', dock:dockSummary && dockSummary.length ? dockSummary : undefined});
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent', dump:e, stack:true, level:'error'});
  }
  this.callParent(arguments);
}, createColumns:function(fields) {
  var me = this;
  var $jscomp$destructuring$var23 = me.widgetReference.getWidgetConfiguration() || {};
  var dateFormat = $jscomp$destructuring$var23.dateFormat;
  var showPreviousData = $jscomp$destructuring$var23.showPreviousData;
  var showFollowingData = $jscomp$destructuring$var23.showFollowingData;
  var startDate = $jscomp$destructuring$var23.startDate;
  var endDate = $jscomp$destructuring$var23.endDate;
  var cellAlign = $jscomp$destructuring$var23.cellAlign;
  var hideZeroValues = $jscomp$destructuring$var23.hideZeroValues;
  var aggregationColumn = $jscomp$destructuring$var23.aggregationColumn;
  var formattedStartDate = monitor.assets.Helpers.formatDate(startDate, dateFormat);
  var formattedEndDate = monitor.assets.Helpers.formatDate(endDate, dateFormat);
  var columns = [{dataIndex:'date', xtype:'datecolumn', format:dateFormat, text:BpcCommon.Api.getTranslation('MONITOR_DATE'), width:150, renderer:function(val, meta, rec) {
    if (!val && rec.get('flag') === 'before' && showPreviousData) {
      return '\x3c ' + formattedStartDate;
    }
    if (!val && rec.get('flag') === 'after' && showFollowingData) {
      return '\x3e ' + formattedEndDate;
    }
    return Ext.util.Format.date(val, dateFormat);
  }, summaryRenderer:function() {
    return '\x3cb\x3e' + BpcCommon.Api.getTranslation('MONITOR_TOTAL') + '\x3c/b\x3e';
  }, summaryIgnoreRowsRenderer:function() {
    return '\x3cb\x3e' + BpcCommon.Api.getTranslation('MONITOR_TOTAL') + ' (' + formattedStartDate + ' - ' + formattedEndDate + ')\x3c/b\x3e';
  }}];
  var defaultColumnConfig = {flex:1, align:cellAlign || 'center', summaryType:'sum', xtype:'numbercolumn', minWidth:60, renderer:function(value, meta) {
    meta.tdStyle = 'cursor: pointer';
    if (!value) {
      value = 0;
    }
    if (hideZeroValues && value === 0) {
      return '';
    }
    return value;
  }, summaryRenderer:function(value, summaryData, dataIndex) {
    if (isNaN(value)) {
      value = 0;
    }
    return '\x3cspan class\x3d"summary-cell" data-qtip\x3d"' + BpcCommon.Api.getTranslation('MONITOR_SHOW_ALL_STATUS') + '"' + (' data-aggregationcolumn\x3d"' + dataIndex + '" style\x3d"cursor: pointer"\x3e\x3cb\x3e' + value + '\x3c/b\x3e\x3cspan\x3e');
  }, summaryIgnoreRowsRenderer:function(value, summaryData, dataIndex) {
    if (isNaN(value)) {
      value = 0;
    }
    return '\x3cspan class\x3d"summary-cell range-summary" data-qtip\x3d"' + BpcCommon.Api.getTranslation('MONITOR_SHOW_ALL_STATUS') + '"' + (' data-aggregationcolumn\x3d"' + dataIndex + '" style\x3d"cursor: pointer"\x3e\x3cb\x3e' + value + '\x3c/b\x3e\x3cspan\x3e');
  }};
  var clonedFields = Ext.clone(fields);
  clonedFields.push({name:'totalCount'});
  var customRenderer = this.widgetReference.getCustomRenderer(aggregationColumn);
  Ext.Array.each(clonedFields, function(fieldObj) {
    var fieldName = fieldObj.name;
    var columnHeaderText;
    if (fieldName === 'totalCount') {
      columnHeaderText = BpcCommon.Api.getTranslation('MONITOR_TOTAL');
    } else {
      columnHeaderText = customRenderer(fieldName);
    }
    columns.push(Ext.applyIf({text:columnHeaderText, dataIndex:fieldName}, defaultColumnConfig));
  }, me);
  return columns;
}, updateGridData:function(data) {
  this.getStore().loadRawData(this.prepareData(data.data));
  this.reconfigure(this.store, this.createColumns(data.fields));
}, prepareData:function(data) {
  var shouldShowPreviousData = this.widgetReference.getSetting('showPreviousData');
  var shouldShowFollowingData = this.widgetReference.getSetting('showFollowingData');
  var result = data;
  if (!shouldShowPreviousData) {
    result = result.filter(function(item) {
      return item.date !== 'beforestart';
    });
  }
  if (!shouldShowFollowingData) {
    result = result.filter(function(item) {
      return item.date !== 'afterend';
    });
  }
  var calculateTotalCount = function(obj) {
    var totalCount = 0;
    Ext.Object.each(obj, function(key, value) {
      if (Ext.isNumber(value)) {
        totalCount += value;
      }
    });
    return totalCount;
  };
  result.forEach(function(item) {
    if (item.date === 'beforestart') {
      item.flag = 'before';
      item.ignoreSummary = true;
      item.totalCount = calculateTotalCount(item);
    }
    if (item.date === 'afterend') {
      item.flag = 'after';
      item.ignoreSummary = true;
      item.totalCount = calculateTotalCount(item);
    }
  });
  return result;
}});
